/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VOConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: AgentState.h,v 1.3 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_AGENTSTATE_H_
#define GLITE_DATA_TRANSFER_AGENTS_AGENTSTATE_H_

#include "glite/data/transfer/agent/model/Agent.h"
#include "glite/data/agents/dao/DAOContext.h"

#include <glite/data/config/service/Logger.hxx>
#include <boost/utility.hpp>
#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {

/**
 * The AgentState Class
 */   
template <class F>
class AgentState : boost::noncopyable {
public:
    
    /**
     * Constructor
     */
    AgentState(class agents::dao::DAOContext& ctx,
               const std::string& n,
               const std::string& t,
               const std::string& c = "",
               unsigned int heartbeat_interval = 60);
                                                                                                                           
    /**
     * Destructor
     */
    ~AgentState();

    /**
     * Set Agent State
     */
    void set(model::Agent::State s) /*throw (DAOException,LogicError)*/;

    /**
     * Check if an Agent is still running
     */
    bool check() /*throw (DAOException,LogicError)*/;

    /**
     * Update Agent State
     */
    void update(model::Agent::State s) /*throw(DAOException,LogicError)*/;
      
private:
    
    /**
     * Default Constructor. Not Implemented
     */
    AgentState();

private:
    /**
     * The Logger
     */
    glite::config::Logger m_logger;
                    
    /**
     * The DAO Context
     */
    class agents::dao::DAOContext& m_ctx;
        
    /**
     * The VO/Channel Name of the Agent
     */
    std::string  m_name;

    /**
     * The Type of the Agent
     */
    std::string  m_type;

    /**
     * The Name of the Contact for that Agent
     */
    std::string  m_contact;
    
    /**
     * The heartbeat Interval: specify the interval used to update the Agent 
     * lastActive time
     */
    unsigned int m_heartbeatInterval;
};

} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#include "glite/data/transfer/agent/AgentState.inl"

#endif //GLITE_DATA_TRANSFER_AGENTS_AGENTSTATE_H_
