/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Action Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Action.h,v 1.4 2006/08/14 08:37:40 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_ACTION_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_ACTION_H_

#include "glite/data/transfer/agent/action/ExecuteException.h"

#include <boost/utility.hpp>

namespace glite   {
namespace data    {
    
/*
 * Forward Declaration
 */   
namespace agents  {
namespace dao     {
class DAOContext;
}
}

namespace transfer{
namespace agent   {
namespace action  {
    
/**
 * Action Interface
 * @interface
 */
class Action : boost::noncopyable{
public:

    /**
     * Destructor
     */
    virtual ~Action(){}
       
    /**    
     * set DAOContext
     */
    virtual void context(const std::string& params, class glite::data::agents::dao::DAOContext& ctx) /*throw (LogicError, ExecuteException)*/ = 0;
    
    /**    
     * Get DAOContext
     */
    virtual class glite::data::agents::dao::DAOContext& context() /*throw (LogicError)*/ = 0;
            
    /**
     * Execute The Action
     */
    virtual void execute(const std::string& params = "") /*throw (LogicError, ExecuteException)*/ = 0;
private:

    /** @link dependency 
     * @label throws*/
    /*# ExecuteException lnkExecuteException; */
};

} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_ACTION_H_
