/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ActionFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ActionFactoryMethod.h,v 1.5 2006/08/14 08:37:40 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_ACTIONFACTORYMETHOD_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_ACTIONFACTORYMETHOD_H_

#include "glite/data/transfer/agent/action/ExecuteException.h"
#include "glite/data/transfer/agent/action/Action.h"
#include <map>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
    
/**
 * ActionFactoryMethod Class
 */
class ActionFactoryMethod {
public:
    
    /**
     * Destructor
     */
    virtual ~ActionFactoryMethod();

    /**
     * Create an instance of an action
     * @return the created instance
     */
    virtual Action * create() /*throw ()*/ = 0; 

    /**
     * Return Action Type
     * @return the Action Type
     */
    const std::string& type() const{ return m_type;}
    
protected:
    
    /**
     * Constructor
     */
    ActionFactoryMethod(const std::string& type);

private:
    
    /**
     * Default Constructor Not Implemented
     */
    ActionFactoryMethod();
    
private:

    /**
     * Action Type
     */
    std::string m_type;

    /** @link dependency 
     * @label throws*/
    /*# ExecuteException lnkExecuteException; */

    /** @link dependency 
     * @label creates*/
    /*# Action lnkAction; */
};

#define DECLARE_FACTORY_METHOD( Class , Type) \
    private: \
    class ThisActionFactoryMethod : public glite::data::transfer::agent::action::ActionFactoryMethod { \
    public: \
        virtual glite::data::transfer::agent::action::Action * create() /*throw ()*/ { \
            return new Class(); \
        } \
    private: \
        friend class Class; \
        ThisActionFactoryMethod():glite::data::transfer::agent::action::ActionFactoryMethod(Type){} \
        static ThisActionFactoryMethod * s_instance; \
    }; \
    public: \
    static glite::data::transfer::agent::action::ActionFactoryMethod* factory(){return ThisActionFactoryMethod::s_instance;}
    
#define REGISTER_FACTORY_METHOD( Class ) \
    Class::ThisActionFactoryMethod* Class::ThisActionFactoryMethod::s_instance = \
        new Class::ThisActionFactoryMethod();

} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_ACTIONFACTORYMETHOD_H_
