/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CatalogServiceException Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CatalogServiceException.h,v 1.4 2006/08/14 08:37:42 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICEEXCEPTION_H_
#define GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICEEXCEPTION_H_

#include "glite/data/agents/AgentExceptions.h"
#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace catalog {
    
/**
 * The Exception is thrown when an Logical error happens on CatalogService
 */
class CatalogLogicError : public glite::data::agents::LogicError {
public:

    /**
     * Constructor
     * @param reason [IN] the reason of the error
     */
    CatalogLogicError(const std::string& reason):glite::data::agents::LogicError(reason){}

    /**
     * Destructor
     */
    virtual ~CatalogLogicError() throw() {}
    
private:

    /**
     * Default Constructor Not Implemented
     */
    CatalogLogicError();
};

/**
 * The Exception is thrown when an error happens on contacting the CatalogService
 */
class CatalogRemoteError : public glite::data::agents::RuntimeError {
public:

    /**
     * Constructor
     * @param reason [IN] the reason of the error
     */
    CatalogRemoteError(const std::string& reason):glite::data::agents::RuntimeError(reason){}

    /**
     * Destructor
     */
    virtual ~CatalogRemoteError() throw() {}
    
private:

    /**
     * Default Constructor Not Implemented
     */
    CatalogRemoteError();
};

} // end namespace catalog
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICEEXCEPTION_H_
