/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: AgentDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: AgentDAO.h,v 1.2 2006/08/14 08:37:43 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_AGENTDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_AGENTDAO_H_

#include "glite/data/transfer/agent/model/Agent.h"
#include "glite/data/agents/dao/DAOException.h"

#include <string>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {

/**
 * Data Access Object For the model::Agent class
 * @interface
 */
class AgentDAO : boost::noncopyable{
public:
    /**
     * Destructor
     */
    virtual ~AgentDAO(){}
        
    /**
     * Return the Channel with the given Name
     */
    virtual model::Agent * get(bool lock = false) /*throw (DAOLogicError, DAOException)*/ = 0;

    /**
     * Update a Agent object
     */
    virtual void update(const model::Agent& obj) /*throw (DAOLogicError, DAOException)*/ = 0;

    /**
     * Insert a new Agent instance
     */
    virtual void create(const model::Agent& obj) /*throw (DAOLogicError, DAOException)*/ = 0;

private:

    /** @link dependency 
     * @label creates/updates*/
    /*# model::Agent lnkAgent; */
};

} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite


#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_AGENTDAO_H_
