/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FileDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: FileDAO.h,v 1.8 2007/07/04 13:08:13 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_FILEDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_FILEDAO_H_

#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/dao-types.h"

#include <string>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {

/**
 * Data Access Object For the model::File class
 * @interface
 */
class FileDAO : boost::noncopyable {
public:
    /**
     * Destructor
     */
    virtual ~FileDAO(){}

    /**
     * Return the File with the given Id
     */
    virtual model::File * get(const std::string& id) /*throw (DAOLogicError,DAOException)*/ = 0;

    /**
     * Return the File with the given id searching all jobs/files (also where the  
     * "job_finished" flag is set).
     * This function should be used only for tests.
     */
    virtual model::File * getFullSearch(const std::string& id) /*throw (DAOLogicError,DAOException)*/ = 0;    
        
    /**
     * Update a File instance
     */
    virtual void update(const model::File& obj) /*throw (DAOLogicError,DAOException)*/ = 0;

    /**
     * Return the list of File instances for the given job Id
     */
    virtual void getByJobId(const std::string& job_id, agents::dao::StringArray& ids) /*throw (DAOException)*/ = 0;

private:

    /** @link dependency 
     * @label creates/updates*/
    /*# glite::data::transfer::agent::model::File lnkFile; */
};

} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_FILEDAO_H_
