/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: TransferDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TransferDAO.h,v 1.6 2007/07/04 13:08:13 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_TRANSFERDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_TRANSFERDAO_H_

#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/dao-types.h"

#include <string>
#include <vector>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {

/**
 * Data Access Object For the model::Transfer class
 * @interface
 */
class TransferDAO : boost::noncopyable {
public:
    /**
     * Destructor
     */
    virtual ~TransferDAO(){}

    /**
     * Return the Transfer with the given FileId - RequestId
     */
    virtual model::Transfer * get(const std::string& file_id, const std::string& req_id) /*throw (DAOLogicError,DAOException)*/ = 0;
        
    /**
     * Return the Transfer with the given FileId - RequestId searching all 
     * jobs/files/transfers  (also where the "job_finished" flag is set).
     * This function should be used only for tests.
     */
    virtual model::Transfer * getFullSearch(const std::string& id, const std::string& req_id) /*throw (DAOLogicError,DAOException)*/ = 0;    

    /**
     * Return the list of Transfer instances for the given File Id, ordered by 
     * transfer time descending
     */
    virtual void getByFileId(const std::string& file_id, agents::dao::StringArray& ids, unsigned long limit = 0) /*throw (DAOLogicError,DAOException)*/ = 0;

    /**
     * Return the list of Transfer instances for the given Request Id
     */
    virtual void getByRequestId(const std::string& request_id, agents::dao::StringArray& ids) /*throw (DAOLogicError,DAOException)*/ = 0;

private:

    /** @link dependency 
     * @label creates/updates*/
    /*# glite::data::transfer::agent::model::Transfer lnkTransfer; */
};

} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_TRANSFERDAO_H_
