/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FileDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: FileDAO.h,v 1.7 2007/03/12 15:03:19 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_CHANNEL_FILEDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_CHANNEL_FILEDAO_H_

#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/dao/FileDAO.h"
#include "glite/data/transfer/agent/dao/StringArrayTable.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace channel {

/**
 * Data Access Object For the model::File class filtered with a Channel View
 * @interface
 */
class FileDAO : public glite::data::transfer::agent::dao::FileDAO {
public:

    /**
     * Destructor
     */
    virtual ~FileDAO(){}

    /**
     * Return the next job (and file) Ids to transfer for the given 
     * channel following the rule: files in ready state, which job 
     * state is ready or active, not in canceling, order by job_priority, 
     * job submission time, job_id for the given VO
     */
    virtual void getNextJobToTransfer(const std::string& vo_name, std::string& job_id, agents::dao::StringArray& file_ids, unsigned int limit = 0) /*throw (DAOException,DAOLogicError)*/ = 0;

    /**
     * Return the next jobs' and files' ids, groups by SEs' pair, to transfer 
     * for the given 
     * channel following the rule: files in pending or waiting state, which job 
     * state is pending, order by job_priority, job submission time, job_id for 
     * the given VO
     */
    virtual void getJobsToTransferPerSE(const std::string& vo_name, dao::StringArrayTable& ids, unsigned int se_limit, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/ = 0;
        
    /**
     * Return the job and file Ids to cancel for the given 
     * channel following the rule: files in Ready, Done or Hold state, which job 
     * state is ready or active, with canceljob flag set to true, order by 
     * job_priority, job submission time, job id 
     */
    virtual void getFilesToCancel(StringArrayTable& ids, unsigned int limit = 0) /*throw (DAOException,DAOLogicError)*/ = 0;
};

} // End namespace channel
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_CHANNEL_FILEDAO_H_
