/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VOViewChannelDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ChannelDAO.h,v 1.7 2007/03/21 14:15:50 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_VO_CHANNELDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_VO_CHANNELDAO_H_

#include "glite/data/transfer/agent/model/Channel.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/transfer/agent/dao/ChannelDAO.h"

#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace vo      {

/**
 * Data Access Object For the model::Channel filtered on VOView
 * @interface
 */
class ChannelDAO : public glite::data::transfer::agent::dao::ChannelDAO {
public:

    /**
     * Destructor
     */
    virtual ~ChannelDAO(){}
      
    /**
     * Return the name of the Channel based on the sites' endpoint and channel 
     * share. In case no channel could be found, returns and empty string
     */
    virtual std::string getBySites(const std::string& ssite, const std::string& dsite) /*throw (DAOLogicError,DAOException)*/ = 0;
        
    /**
     * Returns true if the VO has a share for the given channel, false otherwise
     * @param channel_name [IN] the name of the requested channel
     * @return true if the VO has a share for the given channel, false otherwise
     * @throws DAOException in case of error
     */
    virtual bool hasShare(const std::string& channel_name) /*throw (DAOLogicError, DAOException)*/ = 0;
        
    /**
     * Return true if the SE or the site is backlisted. If true, also return the reason
     * @param host [IN] the SE's host name
     * @param site [IN] the SE's site name
     * @return true if the SE is blacklisted
     * @throws DAOException in case of error
     */
    virtual std::pair<bool,std::string> isBlacklisted(const std::string& host, const std::string& site) /*throw (DAOLogicError,DAOException)*/ = 0;
};

} // End namespace vo
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite


#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_VO_CHANNELDAO_H_
