/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VOViewJobDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: JobDAO.h,v 1.7 2007/06/22 09:26:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_VO_JOBDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_VO_JOBDAO_H_

#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/dao/JobDAO.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace vo      {

/**
 * Data Access Object For the model::Job class
 * @interface
 */
class JobDAO : public glite::data::transfer::agent::dao::JobDAO {
public:

    /**
     * Destructor
     */
    virtual ~JobDAO(){}

    /**
     * Return the list of Submitted Job identifiers that have File sin Submitted State.
     */
    virtual void getSubmittedJobs(agents::dao::StringArray& jobIds, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/ = 0;

    /**
     * Return the list of Job identifiers that are in Pending or Active state 
     * with at least one file in Pending state or that hould be canceled and have
     * at least on file in Hold state
     */
    virtual void getPendingJobs(agents::dao::StringArray& jobIds, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/ = 0;

    /**
     * Return the list of Job identifiers that are in Done or DoneWithErrors state
     */
    virtual void getDoneJobs(agents::dao::StringArray& jobIds, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/ = 0;

    /**
     * Return the list of Job identifiers that are in Finishing state
     */
    virtual void getFinishingJobs(agents::dao::StringArray& jobIds, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/ = 0;
};

} // End namespace vo
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite


#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_VO_JOBDAO_H_
