/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: StageReqDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: StageReqDAO.h,v 1.2 2006/08/14 08:37:48 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_STAGEREQDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_STAGEREQDAO_H_

#include "glite/data/transfer/agent/model/StageReq.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/dao-types.h"

#include <string>
#include <vector>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace vo      {

/**
 * Data Access Object For the model::StageReq class
 * @interface
 */
class StageReqDAO : boost::noncopyable {
public:
    /**
     * Destructor
     */
    virtual ~StageReqDAO(){}

    /**
     * Return the Transfer with the given FileId - RequestId
     */
    virtual model::StageReq * get(const std::string& file_id, const std::string& req_id) /*throw (DAOLogicError,DAOException)*/ = 0;

    /**
     * Return the list of Transfer instances for the given File Id, ordered by 
     * transfer time descending
     */
    virtual void getByFileId(const std::string& file_id, agents::dao::StringArray& ids, unsigned long limit = 0) /*throw (DAOLogicError,DAOException)*/ = 0;

    /**
     * Return the list of Transfer instances for the given Request Id
     */
    virtual void getByRequestId(const std::string& request_id, agents::dao::StringArray& ids) /*throw (DAOLogicError,DAOException)*/ = 0;

    /**
     * Update a StageReq instance
     */
    virtual void update(const model::StageReq& obj) /*throw (DAOLogicError,DAOException)*/ = 0;

    /**
     * Create a StageReq instance
     */
    virtual void create(const model::StageReq& obj) /*throw (DAOLogicError,DAOException)*/ = 0;
        
    /**
     * Count the active (Received & Processing) Stage Requests instances for the VO
     */
    virtual unsigned long countActiveRequests() /*throw (DAOLogicError,DAOException)*/ = 0;

    /**
     * Count the active (Received & Processing) Stage Requests instances for the VO
     * for the files allocated to a given channel
     */
    virtual unsigned long countActiveRequests(const std::string& channel_name) /*throw (DAOLogicError,DAOException)*/ = 0;

private:

    /** @link dependency 
     * @label creates/updates*/
    /*# glite::data::transfer::agent::model::StageReq lnkStageReq; */
};

} // End namespace vo
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_STAGEREQDAO_H_
