/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FileEventListener interface
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: FileEventListener.h,v 1.2 2006/08/14 08:37:48 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_FSM_FILEEVENTLISTENER_H_
#define GLITE_DATA_TRANSFER_AGENTS_FSM_FILEEVENTLISTENER_H_

#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"

#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace fsm     {

/**
 * FileEventListener Interface
 * @interface
 */
class FileEventListener : boost::noncopyable {
public:
    
    // -------------------------------------------------------------------------
    // -------------------------------------------------------------------------
    // Generic Event Handler
    // -------------------------------------------------------------------------
    // -------------------------------------------------------------------------

    /**
     * This method is used to notify a file state transition. This method 
     * should not throw any exceptions
     * @param file [IN] the file instance that has been modified
     * @param job [IN] the job instance which the file belongs to
     * @param old_state [IN] the previous file's state
     */
    virtual void onStateChanged(const model::File& file, const model::Job& job, model::File::State old_state) throw(){};

    // -------------------------------------------------------------------------
    // -------------------------------------------------------------------------
    // Specific Event Handlers: Listeners may be interested in just relevant 
    // events; these specific event handlers are provided in orer to avoid
    // the listener implementations to filter the generic events.
    // Please note that the generic event handler is called in any case
    // -------------------------------------------------------------------------
    // -------------------------------------------------------------------------

    /**
     * This method is used to notify that a file is Ready, i.e. inserted to 
     * the ChannelAgent queue. 
     * @param file [IN] the file instance that has been modified
     * @param job [IN] the job instance which the file belongs to
     */
    virtual void onReady(const model::File& file, const model::Job& job) throw(){};

    /**
     * This method is used to notify that a file is Done, i.e. the transfer 
     * completes successfully
     * @param file [IN] the file instance that has been modified
     * @param job [IN] the job instance which the file belongs to
     */
    virtual void onDone(const model::File& file, const model::Job& job) throw(){};

    /**
     * This method is used to notify that a file is Finished
     * @param file [IN] the file instance that has been modified
     * @param job [IN] the job instance which the file belongs to
     */
    virtual void onFinished(const model::File& file, const model::Job& job) throw(){};

protected:
    /**
     * Constructor
     */
    FileEventListener();

    /**
     * Destructor
     */
    virtual ~FileEventListener();
};

} // End namespace fsm
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_FSM_FILEEVENTLISTENER_H_
