/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Agent Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Agent.h,v 1.4 2006/08/14 08:37:49 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_MODEL_AGENT_H_
#define GLITE_DATA_TRANSFER_AGENTS_MODEL_AGENT_H_

#include "glite/data/transfer/agent/model/Property.h"
#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace model   {

/**
 * Agent Class
 * @stereotype ValueObject
 */
struct Agent {

    typedef enum{ // Define values so they could combined in OR
        S_UNDEF        = 0x0000,
        S_STARTED      = 0x0001,
        S_STOPPED      = S_STARTED    << 1  //0x0002
    } State;

    /**
     * Default Constructor
     */
    Agent():state(S_UNDEF),lastActive((time_t)-1){}

    /**
     * Constructor
     */
    Agent(const std::string& t,         // agent_type
                State        s,         // agent_state
          const std::string& v,         // agent_version
          const std::string& host,      // agent_host
          const std::string& c = "",    // contact
                time_t l = (time_t)-1): // last_active
            state(s),type(t),version(v),
            hostName(host),contact(c),lastActive(l){}

    /**
     * State Property
     */
    Property<State> state;

    /**
     * Type Property
     */
    Property<std::string> const type;

    /**
     * Version Property
     */
    Property<std::string> const version;

    /**
     * HostName Property
     */
    Property<std::string> const hostName;

    /**
     * Contact Property
     */
    Property<std::string> const contact;

    /**
     * LastActive Property
     */
    Property<time_t> lastActive;

public:
    /**
     * Reset All R/W Properties
     */
    void reset(){
        state.reset();
        lastActive.reset();
    }        
};

} // End namespace model
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_MODEL_AGENT_H_
