/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Property Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Property.h,v 1.2 2006/08/14 08:37:50 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_MODEL_PROPERTY_H_
#define GLITE_DATA_TRANSFER_AGENTS_MODEL_PROPERTY_H_

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace model   {
    
/**
 * Property
 */
template<typename T>
class Property{
public:
    /**
     * Constructor
     */
    Property():m_isSet(false){}
    /**
     * Constructor
     */
    Property(const T& t):m_data(t),m_isSet(false){}
    /**
     * Constructor
     */
    Property(const Property<T>& o):m_data(o.m_data),m_isSet(false){}

    /**
     * Call Operator
     */
    const T& operator()() const{ return m_data ;}
    /**
     * Call Operator
     */
    T& operator()(){ return m_data ;}
    /**
     * Cast to the Embedded value
     */
    //operator const T&() const { const T& d = m_data; return d ; }
    /**
     * Return Embedded value
     */
    T& data() { return m_data ; }
    /**
     * Return Embedded value
     */
    const T& data() const { return m_data ; }

    /**
     * Assignment Operator
     */
    Property& operator=(const T& t){
        m_data  = t;
        m_isSet = true;
        return *this; 
    }
    /**
     * Conversion Operator
     */
    Property& operator=(const Property<T>& o){
        m_data  = o.m_data;
        m_isSet = true;
        return *this; 
    }
    /**
     * Comparison Operator
     */
    bool operator==(const T& t) const {
        return (m_data  == t)?true:false;
    }
    /**
     * Comparison Operator
     */
    bool operator==(const Property<T>& o) const {
        return (m_data  == o.m_data)?true:false;
    }
    /**
     * Return true if the Property is Set
     */
    bool isSet() const {return m_isSet; }
    /**
     * Reset the isSet flag
     */
    void reset() {m_isSet = false; }
    
private:
    /**
     * The Data
     */
    T    m_data;
    /**
     * Has data been modified
     */
    bool m_isSet;
};

} // end namespace model
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_MODEL_PROPERTY_H_
