/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: JobInternalFlags Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: JobFlags.cpp,v 1.6 2008/01/08 09:35:25 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "JobFlags.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
    
const char * const JOB_PARAM_DISABLE_DELEGATION           = "--disable-delegation";
const char * const JOB_PARAM_SKIP_RESOLUTION              = "--skip-resolution";
const char * const JOB_PARAM_SKIP_REGISTRATION            = "--skip-registration";
const char * const JOB_PARAM_MOVE_MASTER_REPLICA          = "--move-master-replica";
const char * const JOB_PARAM_SUBMIT_HOST                  = "--submit-host";
const char * const JOB_PARAM_ALLOCATE_IN_DIFFERENT_CYCLE  = "--allocate-in-different-cycle";
const char * const FILE_PARAM_OVERWRITE                   = "--overwrite-file";

/*
 * get_param_value
 *
 * Extract a value from a Param List
 */
std::string get_param_value(const std::string& tag, const std::string& params){
    std::string value;
    
    // Try to find the tag
    std::string::size_type pos = params.find(tag);
    if(std::string::npos != pos){
        std::string val = params.substr(pos + tag.length());
        
        char const* delims = " \t\r\n";
         // trim leading whitespace
        pos = val.find_first_not_of(delims);
        val.erase(0,pos);

        // trim trailing whitespace
        pos = val.find_first_of(delims);
        if(std::string::npos != pos){
            val.erase(pos);
        }
        // Set endpoint
        value = val;
    }
    return value;
}

} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite
