/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ActionsConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ActionsConfig.cpp,v 1.12 2008/12/17 15:56:45 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "ActionsConfig.h"
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/config/service/ParamReader.hxx>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::config;
using glite::data::agents::ConfigParamException;
using glite::data::agents::ConfigurationException;
using glite::data::agents::LogicError;
                                                                                                                          
namespace {    
const char * const CHANNEL_ACTION_NAME            = "glite-transfer-channel-actions";

const char * const MAXFILESTOCANCEL_PROPERTY_NAME       = "MaxFilesToCancel";
const unsigned int DEFAULT_MAXFILESTOCANCEL             = 500;
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return &(ActionsConfig::instance());
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
}
                                                                                
} // End extern "C"
    

/*
 * instance
 *
 * Return Singleton Instance
 */
ActionsConfig& ActionsConfig::instance(){
    static ActionsConfig singleton;
    return singleton;
}

/*
 * ActionsConfig
 *
 * Constructor
 */
ActionsConfig::ActionsConfig():BaseActionsConfig(CHANNEL_ACTION_NAME),
    m_maxFilesToCancel(DEFAULT_MAXFILESTOCANCEL){
}
                                                                                                                          
/*
 * ~ActionsConfig
 *
 * Destructor
 */
ActionsConfig::~ActionsConfig(){
    
}

/*
 * config
 *
 * Configure the Component
 */
int ActionsConfig::config(const Params& params){
    
    try{
        ParamReader reader(getName(),params);
        reader.getParameter(MAXFILESTOCANCEL_PROPERTY_NAME,m_maxFilesToCancel)
            .ge(1);
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Logic Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        throw;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        throw;
    }

    // Log Configuration Values
    m_log_info(getName() << " Configured. Configuration Parameters are");
    m_log_info("MaxFilesToCancel         : " << m_maxFilesToCancel);
    return 0;
}

/*
 * stop
 *
 * Stop the Component
 */
int ActionsConfig::stop(){
         
    m_activeTxCache.reset();
    m_log_info(getName() << " Stopped");
    return 0;
}
