/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Channel ActionsConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ActionsConfig.h,v 1.6 2007/09/14 10:51:50 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_ACTIONSCONFIG_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_ACTIONSCONFIG_H_

#include "BaseActionsConfig.h"
#include "glite/data/transfer/agent/action/channel/ActiveTxCache.h"

#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace channel {

/**
 * The ActionsConfig Class is used to retrieve the Configuration Parameters from 
 * the ServiceConfigurator
 * @stereotype singleton
 */    
class ActionsConfig :  public BaseActionsConfig {
public:
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                      
    /**
     * Finailize the Component
     */
    virtual int stop();
    
    /**
     * Return Singleton Instance
     */
    static ActionsConfig& instance();

    // =========================================================================
    // Configuration Properties
    // =========================================================================
   
    // =========================================================================
    // Active Tx Cache
    // =========================================================================
    ActiveTxCache& activeTxCache() {return m_activeTxCache;}
    
    /**
     * The maximum number of files to be canceled returned by a query to Cancel
     * operation
     */
    unsigned int maxFilesToCancel() const {return m_maxFilesToCancel;}

private:
    /**
     * Constructor
     */
    ActionsConfig();
                                                                                                                           
    /**
     * Destructor
     */
    virtual ~ActionsConfig(); 

private:
    
    /**
     * The maximum number of files to be canceled returned by a query to Cancel
     * operation
     */
    unsigned int m_maxFilesToCancel;

    /**
     * Active Transfer Cache
     */
    ActiveTxCache m_activeTxCache;
};

} // end namespace channel
} // end namespace action
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_ACTIONSCONFIG_H_
