/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Agent Common: ActiveTxCache Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ActiveTxCache.cpp,v 1.7 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/transfer/agent/action/channel/ActiveTxCache.h"
#include "ActiveTxCacheImpl.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;

/*
 * ActiveTxCache
 *
 * Constructor
 */
ActiveTxCache::ActiveTxCache():
    m_impl(new ActiveTxCacheImpl()),
    m_isSet(false){
}
    
/*
 * ~ActiveTxCache
 *
 * Destructor
 */
ActiveTxCache::~ActiveTxCache(){
}

/*
 * insert
 *
 * Insert a Transfer into the Cache
 */
void ActiveTxCache::insert(TransferPtr tx, FilePtr file, JobPtr job) /*throw (LogicError)*/{
    m_impl->insert(tx,file,job);
    m_isSet = true;
}

/*
 * get
 *
 * Get a Transfer cache entry
 */
ActiveTxCache::TransferEntry ActiveTxCache::get(const std::string& file_id) const /*throw (RuntimeError)*/{
    return m_impl->get(file_id);
}

/**
 * getTransferEntries
 * Get all the Transfer cache entries
 */
ActiveTxCache::TransferEntryArray ActiveTxCache::getTransferEntries() const /*throw ()*/{
    return m_impl->getTransferEntries();
}

/*
 * getJob
 *
 * Get a Job Ptr (if exists)
 */
ActiveTxCache::JobPtr ActiveTxCache::getJob(const std::string& job_id) const /*throw (RuntimeError)*/{
    return m_impl->getJob(job_id);
}

/*
 * getFile
 *
 * Get a File Ptr (if exists)
 */
ActiveTxCache::FilePtr ActiveTxCache::getFile(const std::string& file_id) const /*throw (RuntimeError)*/{
    return m_impl->getFile(file_id);
}

/*
 * getRequest
 *
 * Get the Transfer cache entry that refers to a request
 */
ActiveTxCache::RequestEntry ActiveTxCache::getRequest(const std::string& req_id) const /*throw (RuntimeError)*/{
    return m_impl->getRequest(req_id);
}


/*
 * getRequests
 *
 * Get the request entries
 */
ActiveTxCache::RequestEntryArray ActiveTxCache::getRequests() const /*throw ()*/{
    return m_impl->getRequests();
}

/*
 * remove
 *
 * Remove a Transfer from the cache
 */
void ActiveTxCache::remove(const std::string& file_id) /*throw ()*/{
    m_impl->remove(file_id);
}

/*
 * reset
 *
 * Reset the Cache
 */
void ActiveTxCache::reset() /*throw ()*/{
    m_impl->reset();
    m_isSet = false;
}

/*
 * count
 *
 * Count the number of entries in the cache
 */
unsigned int ActiveTxCache::count() const /*throw ()*/{
    return m_impl->count();
}

/*
 * count
 *
 * Count the number of entries in the cache for a given VO
 */
unsigned int ActiveTxCache::count(const std::string& vo_name) const /*throw ()*/{
    return m_impl->count(vo_name);
}

/*
 * countRequests
 *
 * Count the number of requests in the cache
 */
unsigned int ActiveTxCache::countRequests() const /*throw ()*/{
    return m_impl->countRequests();
}

/*
 * countRequests
 *
 * Count the number of requests in the cache for a given VO
 */
unsigned int ActiveTxCache::countRequests(const std::string& vo_name) const /*throw ()*/{
    return m_impl->countRequests(vo_name);
}

/*
 * countPerSourceSE
 *
 * Count the number of transfer with the given source SE in the cache
 */
unsigned int ActiveTxCache::countPerSourceSE(const std::string& src_se) const /*throw ()*/{
    return m_impl->countPerSourceSE(src_se);
}

/*
 * countPerDestSE
 *
 * Count the number of transfer with the given destination SE in the cache
 */
unsigned int ActiveTxCache::countPerDestSE(const std::string& dst_se) const /*throw ()*/{
    return m_impl->countPerDestSE(dst_se);
}

/**
 * get all the transfer entries where the transfer is in one of the given 
 * states
 */
ActiveTxCache::TransferEntryArray ActiveTxCache::getTransfersByState(
    model::Transfer::State states) /*throw()*/ const{
    return m_impl->getTransfersByState(states);
}

/**
 * get the number of transfer entries where the transfer is in one of the 
 * given states
 */
unsigned int ActiveTxCache::countTransfersByState(
    model::Transfer::State states) /*throw()*/ const{
    return m_impl->countTransfersByState(states);
}

