/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Cancel Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Cancel.h,v 1.6 2007/09/12 12:29:22 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_CANCEL_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_CANCEL_H_

#include "glite/data/transfer/agent/action/channel/ChannelAction.h"
#include "glite/data/transfer/agent/action/ActionFactoryMethod.h"

#include <boost/shared_ptr.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace channel {

/**
 * The purpose of that Action is to cancel Active File Requests
 */
class Cancel : public ChannelAction {
    /*
     * Declare Action Factory Method
     */
    DECLARE_FACTORY_METHOD( Cancel , "glite:CancelActive");

public:
    /**
     * Constructor
     */
    Cancel();

    /**
     * Destructor
     */
    virtual ~Cancel();

protected:

    /**
     * Execute The Action
     */
    virtual void exec(const std::string& params) /*throw (DAOException, ExecuteException, LogicError)*/;
    
    /**
     * clean the active transfers cache
     */
    virtual void beforeExec();

private:
    
    /**
     * Cancel a Active Transfer Requests
     */
    void cancelActiveTx() /*throw (DAOException, ExecuteException, LogicError)*/;

    /**
     * Cancel a Queue File and Job in Ready State
     */
    void cancelTxInQueue() /*throw (DAOException, ExecuteException, LogicError)*/;

    /**
     * Update File & Transfer instances of an Active Transfer Request
     */
    void updateRequest(ActiveTxCache::RequestEntry& req, std::set<std::string>& jobIds) /* throw(DAOException,LogicError)*/;

    /**
     * Cancel a Queue File
     */
    void cancelFile(const std::string& file_id, const std::string& job_id) /*throw (DAOException, LogicError)*/;
};


} // End namespace channel
} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_CANCEL_H_
