/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CheckState Channel Action Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CheckState.h,v 1.7 2008/04/23 15:10:13 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_CHECKSTATE_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_CHECKSTATE_H_

#include "glite/data/transfer/agent/action/channel/ChannelAction.h"
#include "glite/data/transfer/agent/action/channel/ActiveTxCache.h"
#include "glite/data/transfer/agent/action/ActionFactoryMethod.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace channel {

/**
 * The purpose of that Action is to check the State of all the Active Transfers
 * against the TransferService and then update the Data Source
 */
class CheckState : public ChannelAction {
    /*
     * Declare Action Factory Method
     */
    DECLARE_FACTORY_METHOD( CheckState , "glite:CheckState");
    
public:
    /**
     * Constructor
     */
    CheckState();

    /**
     * Destructor
     */
    virtual ~CheckState();

protected:

    /**
     * Execute The Action
     */
    virtual void exec(const std::string& params) /*throw (LogicError, DAOException, ExecuteException)*/;

private:

    /**
     * Check Request's State
     */
    void checkRequestState(ActiveTxCache::RequestEntry& req, const std::string& proxy_file, std::set<std::string>& jobIds) /* throw(DAOException)*/;

    /**
     * Check if there are active jobs with files done or in terminal state. This may 
     * happen since it's the VO agent that is responsible for the file transition 
     * from Waiting and Pending (including Prestating): in case of retries, the job's 
     * state is Active, so the VO Agent can't update the job state
     */
    void checkIntegrity(std::set<std::string>& jobIds) /* throw(DAOException)*/;
    
    /**
     * Ensure that the appropriate number of transfers is running (if the
     * transfer service is configured to stop before transfers)
     */
    void ensureRunningTransfers();
};

} // End namespace vo
} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_CHECKSTATE_H_
