/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VOActionsConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VOActionsConfig.h,v 1.3 2009/10/06 15:40:53 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_PYTHON_VOACTIONSCONFIG_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_PYTHON_VOACTIONSCONFIG_H_

#include "glite/data/config/service/ComponentConfiguration.hxx"
#include <string>

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace vo      {

/**
 * The ActionsConfig Class is used to retrieve the Configuration Parameters from 
 * the ServiceConfigurator
 */    
class VOActionsConfig : public glite::config::ComponentConfiguration{
public:

    /**
     * Configure the Component
     */
    virtual int init(const Params& params);

    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
    
    /**
     * Start the Component
     */
    virtual int start(){return 0;}
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}
                                                                                                                           
    /**
     * Finailize the Component
     */
    virtual int fini(){return 0;}

    // =========================================================================
    // Configuration Properties
    // =========================================================================
    
protected:
    /**
     * Constructor
     */
    VOActionsConfig();
                                                                                                                           
    /**
     * Destructor
     */
    virtual ~VOActionsConfig();
       
private:

    /*
     * Declare factory methods as friends
     */
    friend glite::config::ComponentConfiguration * ::create_glite_component();
    friend void ::destroy_glite_component(glite::config::ComponentConfiguration*);

    /**
     * Copy Constructor. Not Implemented
     */
    VOActionsConfig(const VOActionsConfig&);

    /**
     * Assignment Operator. Not Implemented
     */
    VOActionsConfig& operator=(const VOActionsConfig&);
    
    /**
     * Read a string parameter value from the config. The method bypasses the
     * parameter reader, configurator, etc.
     *
     * @param paramName the parameter name
     * @param params the map containing the parameters
     *
     * @return the parameter value or empty string if the parameter is not found.
     */
    std::string _getStringParamValue(const std::string& paramName, const Params& params);

    glite::config::Logger m_logger;
};

} // end namespace vo
} // end namespace action
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_PYTHON_VOACTIONSCONFIG_H_
