/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VOActionsConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ActionsConfig.h,v 1.10 2007/02/23 14:38:22 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_ACTIONSCONFIG_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_ACTIONSCONFIG_H_

#include "BaseActionsConfig.h"
#include "glite/data/transfer/agent/action/vo/ChannelCache.h"

#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace vo      {

/**
 * The ActionsConfig Class is used to retrieve the Configuration Parameters from 
 * the ServiceConfigurator
 * @stereotype singleton
 */    
class ActionsConfig :  public BaseActionsConfig {
public:
    
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                      
    /**
     * Finailize the Component
     */
    virtual int stop();

    /**
     * Return Singleton Instance
     */
    static ActionsConfig& instance();
                                                                                                                           
    // =========================================================================
    // Configuration Properties
    // =========================================================================
    
    /**
     * Get Max Failures Property
     * @return Max Failures  Property
     */
    unsigned int maxFailures() const { return m_maxFailures;}

    /**
     * Get Resubmit Delay Property
     * @return Resubmit Delay Property
     */
    unsigned int resubmitDelay() const { return m_resubmitDelay;}

    /**
     * Get Catalog Retry Delay Property
     * @return Catalog Retry Delay Property
     */
    unsigned int catalogRetryDelay() const { return m_catalogRetryDelay;}
         
    /**
     * Returns true is the VO can transfer from an unknown source
     * @return true is the VO can transfer from an unknown source
     */
    bool unknownSourceEnabled() const { return m_unknownSrcEnabled;}

    /**
     * Returns true is the VO can transfer to an unknown destination
     * @return true is the VO can transfer to an unknown destination
     */
    bool unknownDestEnabled() const { return m_unknownDstEnabled;}
    
    /**
     * Returns true if Catalog interation is enabled
     * @return true if Catalog interation is enabled
     */
    bool catalogEnabled() const { return m_catalogEnabled;}

    // =========================================================================
    // Channel Cache
    // =========================================================================
    ChannelCache& channelCache() {return m_channelCache;}
    
protected:
    /**
     * Constructor
     */
    ActionsConfig();
                                                                                                                           
    /**
     * Destructor
     */
    virtual ~ActionsConfig(){}
       
private:
    /**
     * Max Failures Property
     */
    unsigned int m_maxFailures;
    
    /**
     * Resubmit Delay Property
     */
    unsigned int m_resubmitDelay;
    
    /**
     * Catalog Delay Property
     */
    unsigned int m_catalogRetryDelay;
    
    /**
     * Unknwon Source Enable Flag
     */
    bool m_unknownSrcEnabled;

    /**
     * Unknwon Destination Enable Flag
     */
    bool m_unknownDstEnabled;

    /**
     * Treu if Catalog interation is enabled
     */
    bool m_catalogEnabled;

    /**
     * Channel Cache
     */
    ChannelCache m_channelCache;
};

} // end namespace vo
} // end namespace action
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_ACTIONSCONFIG_H_
