/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ChannelCache class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ChannelCache.cpp,v 1.3 2007/03/21 14:15:50 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/transfer/agent/action/vo/ChannelCache.h"
#include "glite/data/agents/AgentExceptions.h"
#include "ChannelCacheImpl.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;
using namespace glite::data::agents;

/*
 * ChannelCache
 *
 * Constructor
 */
ChannelCache::ChannelCache():
    m_impl(new ChannelCacheImpl()),
    m_isSet(false){
}
    
/*
 * ~ChannelCache
 *
 * Destructor
 */
ChannelCache::~ChannelCache(){
}

/*
 * ttl
 *
 * Set the Validity of an entry in the Cache
 */
void ChannelCache::ttl(unsigned int v){
    m_impl->ttl(v);
}

/*
 * purgeDelay
 *
 * Set the min delay beween two purge
 */
void ChannelCache::purgeDelay(unsigned int d){
    m_impl->purgeDelay(d);
}

/*
 * insert
 *
 * Insert a Channel into the Cache
 */
void ChannelCache::insert(const std::string& ssite, const std::string& dsite, ChannelPtr c) /*throw (LogicError)*/{
    m_impl->insert(ssite,dsite,c);
    m_isSet = true;
}

/*
 * get
 *
 * Get a Channel cache entry
 */
ChannelCache::ChannelPtr ChannelCache::get(const std::string& channel_name) const /*throw (RuntimeError)*/{
    return m_impl->get(channel_name);
}

/*
 * getFromSites
 *
 * Get a Channel cache entry (if exists)
 */
ChannelCache::ChannelPtr ChannelCache::getFromSites(const std::string& ssite, const std::string& dsite) const /*throw (RuntimeError)*/{
    return m_impl->getFromSites(ssite,dsite);
}

/*
 * remove
 *
 * Remove a Channel from the cache
 */
void ChannelCache::remove(const std::string& ssite, const std::string& dsite) /*throw ()*/{
    m_impl->remove(ssite,dsite);
}

/*
 * count
 * 
 * Count the number of entries in the cache
 */
unsigned int ChannelCache::count() const /*throw ()*/{
    return m_impl->count();
}

/*
 * rememberMissingChannel
 *
 * Rememner missing Channel
 */
void ChannelCache::rememberMissingChannel(const std::string& ssite, const std::string& dsite) /*throw ()*/{
    m_impl->rememberMissingChannel(ssite,dsite);
    m_isSet = true;
}

/*
 * isChannelMissing
 *
 * Is Channel missing
 */
bool ChannelCache::isChannelMissing(const std::string& ssite, const std::string& dsite) const /*throw ()*/{
    return m_impl->isChannelMissing(ssite,dsite);
}

/*
 * removeMissingChannel
 *
 * remove missing Channel
 */
void ChannelCache::removeMissingChannel(const std::string& ssite, const std::string& dsite) /*throw ()*/{
    m_impl->removeMissingChannel(ssite,dsite);
}

/*
 * rememberSiteGroups
 *
 * Remember site's groups
 */
void ChannelCache::rememberSiteGroups(const std::string& site, const std::vector<std::string>& groups) /*throw ()*/{
    m_impl->rememberSiteGroups(site,groups);
}

/*
 * rememberSiteGroup
 *
 * Remember site's group
 */
void ChannelCache::rememberSiteGroup(const std::string& site, const std::string& group) /*throw ()*/{
    m_impl->rememberSiteGroup(site,group);
}

/*
 * getSiteGroups
 *
 * Is Channel missing
 */
void ChannelCache::getSiteGroups(const std::string& site, std::vector<std::string>& groups) const /*throw ()*/{
    m_impl->getSiteGroups(site,groups);
}

/*
 * removeSiteGroups
 *
 * Remove Site Groups
 */
void ChannelCache::removeSiteGroups(const std::string& site, const std::vector<std::string>& groups) /*throw ()*/{
    m_impl->removeSiteGroups(site,groups);
}

/*
 * removeSiteGroup
 *
 * Remove Site Group
 */
void ChannelCache::removeSiteGroup(const std::string& site, const std::string& group) /*throw ()*/{
    m_impl->removeSiteGroup(site,group);
}

/*
 * clear
 *
 * clear the Cache
 */
void ChannelCache::clear() /*throw ()*/{
    m_impl->clear();
    m_isSet = false;
}

/*
 * purge
 *
 * Purge invalid entries
 */
void ChannelCache::purge(bool use_delay){
    m_impl->purge(use_delay);
}
