/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CheckReadiness Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CheckReadiness.h,v 1.2 2006/08/14 08:38:26 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_CHECKREADINESS_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_CHECKREADINESS_H_

#include "glite/data/transfer/agent/action/vo/VOAction.h"
#include "glite/data/transfer/agent/action/ActionFactoryMethod.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace vo      {

/**
 * The purpose of that Action is to cancel Pending File Requests
 */
class CheckReadiness : public VOAction {
    /*
     * Declare Action Factory Method
     */
    DECLARE_FACTORY_METHOD( CheckReadiness , "glite:CheckReadiness");

public:
    /**
     * Constructor
     */
    CheckReadiness();

    /**
     * Destructor
     */
    virtual ~CheckReadiness();

protected:

    /**
     * Execute The Action
     */
    virtual void exec(const std::string& params) /*throw (LogicError,DAOException)*/;

private:
    
    typedef std::vector<std::string> StringArray;

    /**
     * Check Readiness of a Job
     */
    void checkReadiness(const std::string& job_id, StringArray& jobIds) /*throw (LogicError,DAOException)*/;

};

} // End namespace vo
} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_CHECKREADINESS_H_
