/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Finalize Action Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Finalize.h,v 1.2 2006/08/14 08:38:28 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_FINALIZE_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_FINALIZE_H_

#include "glite/data/transfer/agent/action/vo/VOAction.h"
#include "glite/data/transfer/agent/action/ActionFactoryMethod.h"
#include "glite/data/transfer/agent/catalog/CatalogService.h"

#include <map>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
               
/*
 * Forward Declarations
 */
namespace model{
class Job;
}

namespace action  {
namespace vo      {

/**
 * The purpose of that Action is to finalize a Job request, by registering when
 * needed the new replicas after the transfer is completed
 */
class Finalize : public VOAction {
    /*
     * Declare Action Factory Method
     */
    DECLARE_FACTORY_METHOD( Finalize , "glite:Finalize");
    
public:
    /**
     * Constructor
     */
    Finalize();

    /**
     * Destructor
     */
    virtual ~Finalize();

    /**
     * Execute The Action. Overrid ethe default since the transaction are 
     * executed per Job
     */
    virtual void execute(const std::string& params = "") /*throw (LogicError,ExecuteException)*/;

private:

    typedef std::vector<std::string> StringArray;


    /**
     * Empty Implementation
     */
    virtual void exec(const std::string& params) /*throw (DAOException,LogicError)*/;

    /**
     * Prepare to finalize the jobs
     */
    void prepareToFinish(bool disable_delegation) /*throw (DAOException,LogicError)*/;

    /**
     * Finalize the jobs
     */
    void finish(bool disable_delegation) /*throw (DAOException,LogicError)*/;

    /**
     * Prepare Job To Finish
     */
    void prepareJobToFinishing(const std::string& job_id, bool disable_delegation) /*throw (DAOException,LogicError)*/;

    /**
     * Finalize a Job
     */
    void finishJob(const std::string& job_id, bool disable_delegation) /*throw (DAOException,LogicError)*/;

    /**
     * Finish a Job
     */
    void finishingJob(const std::string& job_id, bool disable_delegation) /*throw (DAOException,LogicError)*/;

    /**
     * Register Files in bulk mode
     */
    void registerFiles(const model::Job& job, const StringArray& files, const std::string& proxy_file) /*throw (DAOException,LogicError)*/;

    /**
     * Finalize a file
     */
    void finishFile(const std::string& file_id, const model::Job& job) /*throw(DAOException,LogicError)*/;
    
    /**
     * Add the Source Stat to the Replicas Array
     */
    void getSourceStats(const StringArray& files,
              std::map<std::string,std::string>& logical_id_mapping,
              catalog::CatalogService::StringPairArray& src_names,
              catalog::CatalogService::ReplicaStatArray& replicas,
        const std::string& src_catalog_endpoint,
        const std::string& src_catalog_type,
        const std::string& proxy_file) /*throw (DAOException,LogicError)*/;

};

} // End namespace vo
} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_FINALIZE_H_
