/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Define the Utils Module
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CatalogModule.h,v 1.4 2006/08/23 14:31:44 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_CATALOG_PYTHON_CATALOGMODULE_H_
#define GLITE_DATA_TRANSFER_AGENTS_CATALOG_PYTHON_CATALOGMODULE_H_

#include <boost/python.hpp>

#include "glite/data/transfer/agent/catalog/CatalogService.h"
#include "glite/data/transfer/agent/catalog/CatalogServiceException.h"
#include "PythonCatalog.h"

/*
 * Forward Declaration of module name
 */
#ifdef __cplusplus
extern "C" {
#endif //__cplusplus
void init_catalog(void);
#ifdef __cplusplus
}
#endif //__cplusplus

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace catalog {

/*
 * Python Module Name
 */    
extern const char * const CATALOG_MODULE_NAME;

/*
 * CatalogRemoteError Excetion Name 
 */    
extern const char * const REMOTE_CATALOG_EXCEPTION;
    
/**
 * ListSurlsResult Class
 */
struct ListSurlsResult : CatalogService::CatalogResult{
    
    /**
     * List Of Surls
     */
    std::vector<CatalogService::StringArray> surls;
};

/**
 * GetStatsResult Class
 */
struct GetStatsResult: CatalogService::CatalogResult{
    
    /**
     * List Of Stats
     */
    boost::python::list stats;
};

/**
 * Replica Wrapper Class
 */
struct ReplicaStatWrapper{
    /**
     * Logical Name
     */
    std::string logical;
    /**
     * SURL
     */
    std::string surl;
    /**
     * Stat
     */
    boost::python::object stat;
    /**
     * Constructors
     */
    ReplicaStatWrapper(){}
    ReplicaStatWrapper(const std::string& l, const std::string& s):logical(l),surl(s){}
    ReplicaStatWrapper(const std::string& l, const std::string& s, boost::python::object o):
            logical(l),surl(s),stat(o){}
    ReplicaStatWrapper(const ReplicaStatWrapper& o):logical(o.logical),surl(o.surl),stat(o.stat){}
    ReplicaStatWrapper(const CatalogService::ReplicaStat& o):logical(o.logical),surl(o.surl){
        PythonCatalog::PythonFileStat * p = dynamic_cast<PythonCatalog::PythonFileStat*>(o.stat.get());
        if(0 != p){
            stat = p->embedded;
        }
    }
    /**
     * Assignment Operator
     */
    ReplicaStatWrapper& operator=(const ReplicaStatWrapper& o){
        logical = o.logical;
        surl    = o.surl;
        stat    = o.stat;
        return *this;
    }
    /**
     * Comparison Operator
     */
    bool operator==(const ReplicaStatWrapper& o) const{
        return ((logical == o.logical) &&
                (surl    == o.surl) &&
                (stat    == o.stat))?true:false;
    }
};

} // end namespace catalog
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_CATALOG_PYTHON_CATALOGMODULE_H_
