/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: PythonCatalog Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: PythonCatalog.h,v 1.3 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_CATALOG_PYTHON_PYTHONCATALOG_H_
#define GLITE_DATA_TRANSFER_AGENTS_CATALOG_PYTHON_PYTHONCATALOG_H_

#include "glite/data/transfer/agent/catalog/CatalogService.h"
#include <vector>
#include <string>
#include <boost/python.hpp>
#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace catalog {

/**
 * PythonCatalog Class. Implements the CatalogService using a python script
 */
class PythonCatalog : public CatalogService{
public:
    /**
     * PythonFileStat 
     */
    struct PythonFileStat : public FileStat{
        /**
         * Constructor
         */
        PythonFileStat(const std::string& l, boost::python::object obj):
            embedded(obj){logical = l;}
    
        /**
         * Desturctor
         */
        virtual ~PythonFileStat(){}
        
        /**
         * The Embedded Python Stats Object
         */
        boost::python::object embedded;
    };

    /**
     * Constructor
     * @param e [IN] Catalog endpoint
     * @param type [IN] The Catalog type
     * @param cred [IN] Credentials to use when contacting the catalog
     */
    PythonCatalog(const std::string& e,const std::string& type,const std::string& cred);

    /**
     * Destructor
     */
    virtual ~PythonCatalog();
     
    /**
     * Check that the user permission is allowed to read and replicate the 
     * requested files
     */
    virtual void checkPermissions(const StringArray& logicals, CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/;
    
    /**
     * List the names of all the replicas associated to the given Logical Name
     */
    virtual void listSurls(const StringArray& logicals, const std::string& site,
                                 std::vector<StringArray>& surls, CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/;
    
    /**
     * Check that the given SURLs don't already exist 
     */
    virtual void checkSurls(const StringPairArray& names, CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/;
    
    /**
     * Get the Stats info of the given files. This information would the be 
     * passed to the destination catalog in the registerSurls method. The purpose
     * of this method is to synchronize permisisons information between two 
     * catalogs. This method may not be called in case the source and 
     * destination catalog endpoint are the same
     */
    virtual void getStats(const StringPairArray& names, FileStatArray& stats, CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/;

    /**
     * Register the name of a new replica 
     */
    virtual void registerSurls(const ReplicaStatArray& replicas, CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/;
     
private:

    /**
     * Constructor
     */
    PythonCatalog();

private:
    glite::config::Logger m_logger;
    /** 
     * The Catalog Endpoint
     */
    std::string m_endpoint;

    /** 
     * The Catalog Type
     */
    std::string m_type;
    
    /** 
     * The Credential to be used
     */
    std::string m_cred;
};

} // End namespace catalog
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_CATALOG_PYTHON_PYTHONCATALOG_H_
