/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agent: DelegCred Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: DelegCred.h,v 1.2 2006/08/14 08:38:30 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENT_CRED_DELEG_DELEGCRED_H_
#define GLITE_DATA_TRANSFER_AGENT_CRED_DELEG_DELEGCRED_H_

#include "glite/data/agents/cred/CredService.h"
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace cred    {
namespace deleg   {

/**
 * DelegCred CredService
 */
class DelegCred : public glite::data::agents::cred::CredService, boost::noncopyable{
public:
    /**
     * Constructor
     */
    DelegCred(const class DelegCredFactory& f);
        
    /**
     * Destructor
     */
    virtual ~DelegCred();

protected:

    /**
     * Generate a Name for the Certificate Proxy File
     */
    virtual std::string getFileName(const std::string& userDn, const std::string& passphrase) /* throw(LogicError) */; 

    /**
     * Get the Cerificate From the MyProxy Server
     */
    virtual void getNewCertificate(const std::string& userDn, const std::string& passphrase, const std::string& filename) /*throw (MyProxyClientException)*/;

    /**
     * Returns the validity time that the cached copy of the certificate should 
     * have. 
     */
    virtual unsigned long minValidityTime() /* throw() */;

private:
    
    /**
     * Encode a name
     */
    std::string encodeName(const std::string& str)/*throw()*/;

private:
    
    /**
     * Default Constructor. Not Implemented
     */
    DelegCred();

private:
    /**
     * The factory that creates the object
     */
    const class DelegCredFactory& m_factory;

   /** @link dependency 
     * @label throws*/
    /*# DelegCredException lnkDelegCredException; */
};

} // end namespace deleg
} // end namespace cred
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENT_CRED_DELEG_DELEGCRED_H_
