/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agent: DelegCredFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: DelegCredFactory.h,v 1.4 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENT_CRED_DELEG_DELEGCREDFACTORY_H_
#define GLITE_DATA_TRANSFER_AGENT_CRED_DELEG_DELEGCREDFACTORY_H_

#include "glite/data/agents/cred/CredServiceFactory.h"
#include "glite/data/config/service/ComponentConfiguration.hxx"
#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace cred    {
namespace deleg   {

/**
 * Configure DelegCred library
 */
class DelegCredFactory : public glite::data::agents::cred::CredServiceFactory, 
                         public glite::config::ComponentConfiguration{
public:

    /**
     * Constructor
     */
    DelegCredFactory();
        
    /**
     * Destructor
     */
    virtual ~DelegCredFactory();

    // =========================================================================
    // CertProxyServiceFactory Methods
    // =========================================================================

    /**
     * Return the CredService Type.
     * @return the type of the CredService
     */
    virtual const std::string& getType() /*throw ()*/; 

    /**
     * Return the CertProxy Service Type to be used to query ServiceDiscovery.
     * @return the CertProxy Service Type to be used to query ServiceDiscovery 
     */
    virtual const std::string& getServiceType() /*throw ()*/;

    /**
     * Create a CredService instance
     * @param endpoint [IN] the service hostname and port to contact. If this 
     * parameter is empty, ServiceDiscovery will be used to find the endpoint 
     * of the Server to contact
     * @return a new instance of a CredService
     * @throws LogicError in case of error
     */
    virtual glite::data::agents::cred::CredService * create(const std::string& endpoint) /*throw (LogicError)*/; 

    // =========================================================================
    // ComponentConfiguration Methods
    // =========================================================================

    /**
     * Initialize the Component
     */
    virtual int init(const Params& params);
                                                                                                                           
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start(){return 0;}
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}
                                                                                                                           
    /**
     * Finalize the Component
     */
    virtual int fini();

    // =========================================================================
    // Properties
    // =========================================================================
    
    /**
     * Get Minimum Valibity Time for a Proxy Certificate 
     */
    unsigned long minValidityTime() const { return m_minValidityTime;}
    
    /**
     * Get Proxy  Certificates Repository Property
     */
    const std::string& repository() const { return m_repository;}
    
    /**
     * Get the logger object
     */
    const glite::config::Logger& logger() const {return m_logger;}
    
private:
    glite::config::Logger m_logger;
    /**
     * the Deleg Service Type
     */
    std::string m_delegType;

    /**
     * the dummy Service Type 
     */
    std::string m_fakeServiceType;
    
    /**
     * Minimum Valibity Time for a Proxy Certificate 
     */
    unsigned long m_minValidityTime;
    
    /**
     * Proxy Certificates Repository
     */
    std::string m_repository;

    /** @link dependency 
     * @label creates*/
    /*# DelegCred lnkDelegCred; */
};

} // end namespace deleg
} // end namespace cred
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENT_CRED_DELEG_DELEGCREDFACTORY_H_
