/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: StringArrayTable Type
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: StringArrayTable.cpp,v 1.2 2006/08/14 08:38:30 badino Exp $
 *  Release: $Name:  $
 *
 */

#include "glite/data/transfer/agent/dao/StringArrayTable.h"
#include <algorithm>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
    
class find_key {
public:		
   find_key(const std::string& k) : m_key(k) {}		
   int operator() (const std::pair<std::string, std::vector<std::string> >& v) { return v.first == m_key; }	
private:
    find_key();
private:		
   const std::string& m_key;
};

/*
 * insertFile
 *
 * Insert a new File/Job Entry
 */
void insert_entry(StringArrayTable& l, const std::string& val, const std::string& key){
    StringArrayTable::iterator it = std::find_if(l.begin(),l.end(),find_key(key));
    if(it == l.end()){
        std::pair<std::string, std::vector<std::string> > entry;
        entry.first = key;
        it = l.insert(l.end(),entry);
    }
    (*it).second.push_back(val);
}

} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite
