/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: DAO utilities Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: dao-utility.h,v 1.12 2008/12/11 13:24:44 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_DAO_UTILITY_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_DAO_UTILITY_H_

#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/StageReq.h"
#include "glite/data/transfer/agent/model/Channel.h"
#include "glite/data/transfer/agent/model/Agent.h"
#include "glite/data/transfer/agent/model/Error.h"

#include "glite/data/agents/dao/dao-types.h"
#include "glite/data/agents/dao/DAOContext.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
        
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Job
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
    
// Constraints
extern const unsigned int MAX_T_JOB_REASON_LEN;

/**
 * Translate Job State into a String
 */
const char * translateJobState(model::Job::State state) /*throw (DAOLogicError)*/;

/**
 * Translate a String into a Job State
 */
model::Job::State translateJobState(const char * state) /*throw (DAOLogicError)*/;

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// File
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------    

// Constraints
extern const unsigned int MAX_T_FILE_REASON_LEN;

/**
 * Translate File State into a String
 */
const char * translateFileState(model::File::State state) /*throw (DAOLogicError)*/;
    
/**
 * Translate a String into a File State
 */
model::File::State translateFileState(const char * str_state) /*throw (DAOLogicError)*/;
  
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Error
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------    

/**
 * Translate Error Category into a String
 */
const char * translateErrorCategory(model::Error::Category c) /*throw (DAOLogicError)*/;

/**
 * Translate a String into an Error Category
 */
model::Error::Category translateErrorCategory(const char * str_category) /*throw (DAOLogicError)*/;

/**
 * Translate Error Scope into a String
 */
const char * translateErrorScope(model::Error::Scope s) /*throw (DAOLogicError)*/;

/**
 * Translate a String into an Error Scope
 */
model::Error::Scope translateErrorScope(const char * str_scope) /*throw (DAOLogicError)*/;

/**
 * Translate Error Phase into a String
 */
const char * translateErrorPhase(model::Error::Phase p) /*throw (DAOLogicError)*/;

/**
 * Translate a String into an Error Phase
 */
model::Error::Phase translateErrorPhase(const char * str_phase) /*throw (DAOLogicError)*/;

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Transfer
// -----------------------------------------------------------------------------    
// -----------------------------------------------------------------------------

// Constraints
extern const unsigned int MAX_T_TRANSFER_REASON_LEN;

/**
 * Translate TransferState into a String
 */
const char * translateTransferState(model::Transfer::State state) /*throw (DAOLogicError)*/;

/**
 * Translate a String into a TransferState
 */
model::Transfer::State translateTransferState(const char * state) /*throw (DAOLogicError)*/;

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// StageReq
// -----------------------------------------------------------------------------    
// -----------------------------------------------------------------------------

// Constraints
extern const unsigned int MAX_T_STAGEREQ_REASON_LEN;

/**
 * Translate StageReq State into a String
 */
const char * translateStageReqState(model::StageReq::State state) /*throw (DAOLogicError)*/;

/**
 * Translate a String into a StageReq State
 */
model::StageReq::State translateStageReqState(const char * state) /*throw (DAOLogicError)*/;

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Channel
// -----------------------------------------------------------------------------    
// -----------------------------------------------------------------------------

/**
 * Translate Channel State into a String
 */
const char * translateChannelState(model::Channel::State state) /*throw (DAOLogicError)*/;

/**
 * Translate a String into a Channel State
 */
model::Channel::State translateChannelState(const char * state) /*throw (DAOLogicError)*/;

/**
 * Translate Channel VOState into a String
 */
const char * translateChannelVOState(model::Channel::VOState state) /*throw (DAOLogicError)*/;

/**
 * Translate a String into a Channel VOState
 */
model::Channel::VOState translateChannelVOState(const char * state) /*throw (DAOLogicError)*/;

/**
 * Translate Channel Type into a String
 */
const char * translateChannelType(model::Channel::Type t) /*throw (DAOLogicError)*/;

/**
 * Translate a String into a Channel Type
 */
model::Channel::Type translateChannelType(const char * t) /*throw (DAOLogicError)*/;

/**
 * Translate Channel LogLevel to a string
 */
const char * translateChannelLogLevel(model::Channel::LogLevel l) /*throw (DAOLogicError)*/;

/**
 * Translate a string to a Channel LogLevel 
 */
model::Channel::LogLevel translateChannelLogLevel(const char * l) /*throw (DAOLogicError)*/;

/**
 * Translate Channel SrmCopyDirection to a string
 */
const char * translateChannelSrmCopyDirection(model::Channel::SrmCopyDirection d) /*throw (DAOLogicError)*/;

/**
 * Translate a string to a Channel SrmCopyDirection
 */
model::Channel::SrmCopyDirection translateChannelSrmCopyDirection(const char * l) /*throw (DAOLogicError)*/;

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Agent
// -----------------------------------------------------------------------------    
// -----------------------------------------------------------------------------

// Agent State Values
extern const char * const AGENT_STATE_STARTED;
extern const char * const AGENT_STATE_STOPPED;

// Agent Name Prefix
extern const char * const VO_PREFIX;
extern const char * const CHANNEL_PREFIX;
extern const char * const CRED_PREFIX;
    
/**
 * Translate AgentState into a String
 */
const char * translateAgentState(model::Agent::State state) /*throw (DAOLogicError)*/;

/**
 * Translate a String into a AgentState
 */
model::Agent::State translateAgentState(const char * state) /*throw (DAOLogicError)*/;

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Schema Version
// -----------------------------------------------------------------------------    
// -----------------------------------------------------------------------------
extern const char * const T_SCHEMA_VERS;
extern const char * const T_CREDENTIAL_VERS;

extern const unsigned int SCHEMA_VERS_MAJOR;
extern const unsigned int SCHEMA_VERS_MINOR;
extern const unsigned int SCHEMA_VERS_PATCH;

/**
 * Check Schema Version
 */
void check_schema(agents::dao::DAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

/**
 * Check Credential Schema Version
 */
void check_cred_schema(agents::dao::DAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Helpers Methods
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------

/**
 * Update Job State
 */
bool update_job_state(model::Job& job, const agents::dao::StringArray& file_states) /*throw (DAOLogicError)*/;

} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_JOBSTATEMENTS_H_
