/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Oracle AgentStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: AgentStatements.h,v 1.3 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_AGENTSTATEMENTS_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_AGENTSTATEMENTS_H_

#include "glite/data/transfer/agent/model/Agent.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"
#include <glite/data/config/service/Logger.hxx>

#include <vector>
#include <string>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Statemets For the model::Agent class
 */
class AgentStatements : boost::noncopyable{
public:

    /**
     * Return Singleton instance 
     * @return Singleton instance 
     */
    static AgentStatements& instance() /*throw ()*/ ;

    // -------------------------------------------------------------------------
    // Methods
    // -------------------------------------------------------------------------

    /*
     * dispose
     *
     * dispose Statements for OracleDAOContext
     */
    void dispose(agents::dao::oracle::OracleDAOContext& ctx) /*throw ()*/;
      
    /**
     * Return the statement for the AgentDAO::get
     *
     * Bind Parameters:
     *  :1 - Agent Name
     */
    ::oracle::occi::Statement* createGet(agents::dao::oracle::OracleDAOContext& ctx, bool lock = false) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Return the statement for the AgentDAO::update
     */
    ::oracle::occi::Statement* createUpdate(agents::dao::oracle::OracleDAOContext& ctx, const model::Agent& agent, const std::string& agent_name) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the AgentDAO::create
     */
    ::oracle::occi::Statement* createCreate(agents::dao::oracle::OracleDAOContext& ctx, const model::Agent& agent, const std::string& agent_name) /*throw (DAOLogicError, DAOException)*/;
    
private:
    glite::config::Logger m_logger;
    /**
     * Default Constructor
     */
    AgentStatements();

    /**
     * Destructor
     */
    ~AgentStatements(){}
        
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_AGENTSTATEMENTS_H_
