/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Channel_AgentDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Channel_AgentDAO.cpp,v 1.4 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Channel_AgentDAO.h"
#include "OracleAgentDAO.h"
#include "AgentStatements.h"
#include "dao-utility.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using namespace ::oracle::occi;
using glite::data::agents::dao::oracle::OracleDAOContext;

/*
 * Channel_AgentDAO
 *
 * Constructor
 */
Channel_AgentDAO::Channel_AgentDAO(OracleDAOContext& ctx, const std::string& channel_name, const std::string& category):
        m_logger(category),
        m_agentDaoImpl(new OracleAgentDAO(ctx,(std::string)CHANNEL_PREFIX + channel_name, category)){
}

/*
 * get
 *
 * Return the Agent with the given Name
 */
model::Agent * Channel_AgentDAO::get(bool lock) /*throw (DAOLogicError,DAOException)*/{
    return m_agentDaoImpl->get(lock);
}

/*
 * update
 *
 * Update a Agent object
 */
void Channel_AgentDAO::update(const model::Agent& obj) /*throw (DAOLogicError,DAOException)*/{
    return m_agentDaoImpl->update(obj);
}

/*
 * create
 *
 * Insert a new Agent Entry
 */
void Channel_AgentDAO::create(const model::Agent& obj) /*throw (DAOLogicError,DAOException)*/{
    return m_agentDaoImpl->create(obj);
}
