/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Channel_ChannelDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Channel_ChannelDAO.h,v 1.10 2008/12/04 08:53:30 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNEL_CHANNELDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNEL_CHANNELDAO_H_

#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/dao/channel/ChannelDAO.h"
#include "glite/data/agents/dao/DAOException.h"
#include "OracleChannelDAO.h"

#include <boost/scoped_ptr.hpp>

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object For the model::Channel class, Channel View
 */
class Channel_ChannelDAO : public channel::ChannelDAO{
public:

    /**
     * Constructor
     */
    Channel_ChannelDAO(agents::dao::oracle::OracleDAOContext& ctx, const std::string& channel_name, const std::string& category);

    /**
     * Destructor
     */
    virtual ~Channel_ChannelDAO(){}

    /**
     * Return the Channel instance with the given name
     */
    virtual model::Channel * get(const std::string& name) /*throw (DAOLogicError,DAOException)*/;
    
    /**
     * Update a Channel LastActive Property
     */
    virtual void updateLastActive(const model::Channel& obj) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Update a Channel TransferType Property
     */
    virtual void updateTransferType(const model::Channel& obj) /*throw (DAOLogicError,DAOException)*/;
    
    /**
     * Return the list of VO that are allowed to submit request to the Channel
     */
    virtual void getVONames(agents::dao::StringArray& vo_names, unsigned int limit = 0, unsigned int offset = 0) /*throw (DAOLogicError,DAOException)*/;
        
    /**
     * Return the configured share for the given VO on the Channel
     */
    virtual double getShare(const std::string& vo_name, bool normalized = false, bool only_vo_with_active_jobs = false) /*throw (DAOLogicError,DAOException)*/;
    
    /**
     * Return the hard limit (as absolute value) for the given VO on 
     * the channel.
     */
    virtual std::pair<int,model::Channel::VOState> getLimit(const std::string& vo_name) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the list of groups which a site belong to
     */
    virtual void getSiteGroups(const std::string& site, agents::dao::StringArray& groups) /*throw (DAOLogicError,DAOException)*/;

private:
    /**
     * Default Constructor. Not Implemented
     */
    Channel_ChannelDAO();
    
private:
    glite::config::Logger m_logger;
    /**
     * The Helper
     * @link dependency
     */
     /*#OracleChannelDAO lnkOracleChannelDAO;*/
    boost::scoped_ptr<class OracleChannelDAO> m_channelDaoImpl;
    
    /**
     * The Name of the Channel
     */
    std::string m_channelName;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNEL_CHANNELDAO_H_
