/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Oracle Channel DAO Factory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Channel_DAOFactory.h,v 1.4 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNEL_DAOFACTORY_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNEL_DAOFACTORY_H_

#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"
#include "glite/data/agents/dao/DAOException.h"

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * DataAccessObject Factory For Oracle, Channel View
 */
class Channel_DAOFactory : public channel::DAOFactory{
public:

    /**
     * Constructor
     */
    Channel_DAOFactory();

    /**
     * Destructor
     */
    virtual ~Channel_DAOFactory(){}
    
    // =========================================================================
    // DAO Factory Method
    // =========================================================================

    /**
     * Validate The DB Schema
     * @param ctx [IN] The DAO Context to use
      */
    virtual void checkSchema(agents::dao::DAOContext& ctx) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Returns DataAccessObject for model:File instances
     * @param channel_name [IN] The name of the channel to view on
     * @param ctx [IN] The DAO Context to use
     */
    virtual channel::FileDAO * createFileDAO(const std::string& channel_name, agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

    /**
     * Returns DataAccessObject for model:Job instances
     * @paramchannel_name [IN] The name of the channel to view on
     * @param ctx [IN] The DAO Context to use
     */
    virtual channel::JobDAO * createJobDAO(const std::string& channel_name, agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

    /**
     * Returns DataAccessObject for model:Transfer instances
     * @param channel_name [IN] The name of the channel to view on
     * @param ctx [IN] The DAO Context to use
     */
    virtual channel::TransferDAO * createTransferDAO(const std::string& channel_name, agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

    /**
     * Returns DataAccessObject for model:Channel instances
     * @param channel_name [IN] The name of the channel to view on
     * @param ctx [IN] The DAO Context to use
     */
    virtual channel::ChannelDAO * createChannelDAO(const std::string& channel_name, agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

    /**
     * Returns DataAccessObject for model:Agent instances
     * @param channel_name [IN] The name of the channel to view on
     * @param ctx [IN] The DAO Context to use
     */
    virtual channel::AgentDAO * createAgentDAO(const std::string& channel_name, agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

    /**
     * Returns DataAccessObject for model:Cred instances
     * @param channel_name [IN] The name of the channel to view on
     * @param ctx [IN] The DAO Context to use
     */
    virtual channel::CredDAO * createCredDAO(const std::string& channel_name, agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

private:
    glite::config::Logger m_logger;
    /** 
     * @link dependency 
     * @label creates
	 */
    /*# Channel_JobDAO lnkChannel_JobDAO; */

    /** 
     * @link dependency 
     * @label creates
	 */
    /*# Channel_FileDAO lnkChannel_FileDAO; */

    /** 
     * @link dependency 
     * @label creates
	 */
    /*# Channel_TransferDAO lnkChannel_TransferDAO; */

    /**
     * @link dependency 
     * @label creates
	 */
    /*# Channel_ChannelDAO lnkChannel_ChannelDAO; */
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNEL_DAOFACTORY_H_
