/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Channel_FileDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Channel_FileDAO.h,v 1.9 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNEL_FILEDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNEL_FILEDAO_H_

#include <glite/data/config/service/Logger.hxx>
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/dao/channel/FileDAO.h"
#include "OracleFileDAO.h"

#include <boost/scoped_ptr.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object For the File class, Channel View
 */
class Channel_FileDAO : public channel::FileDAO{
public:

    /**
     * Constructor
     */
    Channel_FileDAO(agents::dao::oracle::OracleDAOContext& ctx, const std::string& vo_name, const std::string& category);

    /**
     * Destructor
     */
    virtual ~Channel_FileDAO(){}

    // -------------------------------------------------------------------------
    // VO File DAO Methods
    // -------------------------------------------------------------------------

    /**
     * Return the File with the given Id
     */
    virtual model::File * get(const std::string& id) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Return the File with the given Id
     */
    virtual model::File * getFullSearch(const std::string& id) /*throw (DAOLogicError, DAOException)*/;
        
    /**
     * Update a File instance
     */
    virtual void update(const model::File& obj) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Return the list of Fileids for the given job Id
     */
    virtual void getByJobId(const std::string& job_id, agents::dao::StringArray& files) /*throw (DAOLogicError, DAOException)*/;
    
    /**
     * Return the next job (and file Ids) files to transfer for the given 
     * channel following the rule: files in pending or waiting state, which job 
     * state is pending, order by job_priority, job submission time, job_id for 
     * the given VO
     */
    virtual void getNextJobToTransfer(const std::string& vo_name, std::string& job_id, agents::dao::StringArray& file_ids, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Return the next jobs' and files' ids, groups by SEs' pair, to transfer 
     * for the given 
     * channel following the rule: files in pending or waiting state, which job 
     * state is pending, order by job_priority, job submission time, job_id for 
     * the given VO
     */
    virtual void getJobsToTransferPerSE(const std::string& vo_name, dao::StringArrayTable& ids, unsigned int se_limit, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Return the next job (and file Ids) files to cancel for the given 
     * channel following the rule: files in ready state, which job 
     * state is ready or active, with canceljob flag set to true, order by 
     * job_priority, job submission time, job id 
     */
    virtual void getFilesToCancel(StringArrayTable& ids, unsigned int limit = 0) /*throw (DAOException,DAOLogicError)*/;

private:
    
    /**
     * Default Constructor. Not Implemented
     */
    Channel_FileDAO();

private:
    glite::config::Logger m_logger;
    /**
     * The DAO Context
     * @link dependency
     */
     /*#OracleFileDAO lnkOracleFileDAO;*/
    boost::scoped_ptr<class OracleFileDAO> m_fileDaoImpl;
    
    /**
     * The Name of the Channel
     */
    std::string m_channelName;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNEL_FILEDAO_H_
