/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Channel_TransferDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Channel_TransferDAO.h,v 1.10 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNEL_TRANSFERDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNEL_TRANSFERDAO_H_

#include <glite/data/config/service/Logger.hxx>
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/dao/channel/TransferDAO.h"
#include "glite/data/agents/dao/DAOException.h"
#include "OracleTransferDAO.h"

#include <boost/scoped_ptr.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object For the Transfer class, Channel View
 */
class Channel_TransferDAO : public channel::TransferDAO{
public:

    /**
     * Constructor
     */
    Channel_TransferDAO(agents::dao::oracle::OracleDAOContext& ctx, const std::string& channel_name, const std::string& category);

    /**
     * Destructor
     */
    virtual ~Channel_TransferDAO(){}

    /**
     * Return the Transfer with the given FileId - RequestId
     */
    virtual model::Transfer * get(const std::string& file_id, const std::string& req_id) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the Transfer with the given FileId - RequestId
     */
    virtual model::Transfer * getFullSearch(const std::string& file_id, const std::string& req_id) /*throw (DAOLogicError,DAOException)*/;
        
    /**
     * Update a File instance
     */
    virtual void update(const model::Transfer& obj) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the list of request_ids for the given File Id
     */
    virtual void getByFileId(const std::string& file_id, agents::dao::StringArray& ids, unsigned long limit = 0) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the list of Transfer instances for the given File Id, ordered by 
     * transfer time descending
     */
    virtual void getByRequestId(const std::string& request_id, agents::dao::StringArray& ids) /*throw (DAOLogicError,DAOException)*/;

    // Channel Transfer DAO Methods
    // -------------------------------------------------------------------------
    
    /**
     * Create a Transfer instance
     */
    virtual void create(const model::Transfer& obj) /*throw (DAOLogicError,DAOException)*/;
        
    /**
     * Count the active (Received & Processing) Transfer instances on the channel
     */
    virtual unsigned long countActiveTransfers() /*throw (DAOLogicError,DAOException)*/;
        
    /**
     * Count the active (Received & Processing) Transfer instances the have been
     * submitted by a VO
     */
    virtual unsigned long countActiveTransfers(const std::string& vo_name) /*throw (DAOLogicError,DAOException)*/;
        
    /**
     * Return the list of next active transfers for the given channel following 
     * the rule: transfer in received or processing state.
     */
    virtual void getActiveTransfers(StringPairArray& ids) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the list of the active (Received and Processing) Transfers which 
     * Job state is Canceling and File State is Active
     */
    //virtual void getTransfersToCancel(FilesPerRequestList& ids, bool lock = false, unsigned int limit = 0, unsigned int offset = 0) /*throw (DAOException)*/;

private:
    
    /**
     * Return true if the file id is valid
     */
    bool isValidFile(const std::string& file_id) /*throw (DAOLogicError,DAOException)*/;
 
private:
    
    /**
     * Default Constructor. Not Implemented
     */
    Channel_TransferDAO();

private:
    glite::config::Logger m_logger;
    /**
     * The Helper
     * @link dependency
     */
     /*#OracleTransferDAO lnkOracleTransferDAO;*/
    boost::scoped_ptr<class OracleTransferDAO> m_transferDaoImpl;
    
    /**
     * The Name of the Channel
     */
    std::string m_channelName;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNEL_TRANSFERDAO_H_
