/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Oracle CredStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: CredStatements.h,v 1.3 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CREDSTATEMENTS_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CREDSTATEMENTS_H_

#include <glite/data/config/service/Logger.hxx>
#include "glite/data/transfer/agent/model/Cred.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"

#include <vector>
#include <string>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Statemets For the model::Cred class
 */
class CredStatements : boost::noncopyable{
public:

    /**
     * Return Singleton instance 
     * @return Singleton instance 
     */
    static CredStatements& instance() /*throw ()*/ ;

    // -------------------------------------------------------------------------
    // Methods
    // -------------------------------------------------------------------------

    /*
     * dispose
     *
     * dispose Statements for OracleDAOContext
     */
    void dispose(agents::dao::oracle::OracleDAOContext& ctx) /*throw ()*/;

    /**
     * Return the statement for the CredDAO::get, Cred and Channel View
     *
     * Bind Parameters:
     *  :1 - Cred ID
     *  :2 - User Dn
     */
    ::oracle::occi::Statement* createGet(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the CredDAO::get, VO View
     *
     * Bind Parameters:
     *  :1 - Cred ID
     *  :2 - User Dn
     *  :2 - VO Name
     */
    ::oracle::occi::Statement* createGet_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Prepare Statement For CredDAO::update, Cred View. Returns 0 if cred doesn't need to be updated
     */
    ::oracle::occi::Statement * createUpdate(agents::dao::oracle::OracleDAOContext& ctx, const model::Cred& c) /*throw (DAOLogicError,DAOException)*/;
        
    /*
     * Return the statement for the CredDAO::getExpiringCred
     *
     * Bind Parameters:
     *  :1 - Interval in second
     *  :2 - Limit
     *  :3 - Limit
     */
    ::oracle::occi::Statement* createGetExpiringCreds(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

private:
    glite::config::Logger m_logger;
    /**
     * Default Constructor
     */
    CredStatements();

    /**
     * Destructor
     */
    ~CredStatements(){}

};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CREDSTATEMENTS_H_
