/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Cred_CredDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Cred_CredDAO.h,v 1.3 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CRED_CREDDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CRED_CREDDAO_H_

#include "glite/data/transfer/agent/model/Cred.h"
#include "glite/data/transfer/agent/dao/cred/CredDAO.h"
#include "OracleCredDAO.h"
#include <boost/scoped_ptr.hpp>
#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object For the Cred class, Cred View
 */
class Cred_CredDAO : public cred::CredDAO{
public:

    /**
     * Constructor
     */
    Cred_CredDAO(agents::dao::oracle::OracleDAOContext& ctx, const std::string& category);

    /**
     * Destructor
     */
    virtual ~Cred_CredDAO(){}

    // -------------------------------------------------------------------------
    // Cred DAO Methods
    // -------------------------------------------------------------------------

    /**
     * Return the Cred with the given Id
     */
    virtual model::Cred * get(const std::string& id, const std::string& user_dn) /*throw (DAOLogicError,DAOException)*/;
        
    /**
     * Update the Cred instance
     */
    virtual void update(model::Cred& cred) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Get the identifiers of the Credentials that are going to expire and 
     * have Pending or Active jobs
     */
    virtual void getExpiringCreds(StringPairArray& ids, unsigned int validity, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/;

private:
    /**
     * Default Constructor. Not Implemented
     */
    Cred_CredDAO();

private:
    glite::config::Logger m_logger;
    /**
     * The Helper
     * @link dependency
     */
     /*#OracleCredDAO lnkOracleCredDAO;*/
    boost::scoped_ptr<class OracleCredDAO> m_credDaoImpl;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CRED_CREDDAO_H_
