/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Oracle Cred DAO Factory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Cred_DAOFactory.h,v 1.3 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CRED_DAOFACTORY_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CRED_DAOFACTORY_H_

#include "glite/data/transfer/agent/dao/cred/DAOFactory.h"
#include "glite/data/agents/dao/DAOException.h"

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * DataAccessObject Factory For Oracle, Cred View
 */
class Cred_DAOFactory : public cred::DAOFactory{
public:

    /**
     * Constructor
     */
    Cred_DAOFactory();

    /**
     * Destructor
     */
    virtual ~Cred_DAOFactory(){}
    
    // =========================================================================
    // DAO Factory Method
    // =========================================================================

    /**
     * Validate The DB Schema
     * @param ctx [IN] The DAO Context to use
      */
    virtual void checkSchema(agents::dao::DAOContext& ctx) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Returns DataAccessObject for model::Cred instances
     * @param ctx [IN] The DAO Context to use
     */
    virtual cred::CredDAO * createCredDAO(agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;
        
    /**
     * Returns DataAccessObject for model::Job instances
     * @param ctx [IN] The DAO Context to use
     */
    virtual cred::JobDAO * createJobDAO(agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

    /**
     * Returns DataAccessObject for model:Agent instances
     * @param name [IN] The Agent name
     * @param ctx [IN] The DAO Context to use
     */
    virtual cred::AgentDAO * createAgentDAO(const std::string& name,agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

private:
    glite::config::Logger m_logger;
    /** 
     * @link dependency 
     * @label creates
	 */
    /*# Cred_CredDAO lnkCred_CredDAO; */
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CRED_DAOFACTORY_H_
