/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Cred_JobDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Cred_JobDAO.cpp,v 1.4 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Cred_JobDAO.h"
#include "OracleJobDAO.h"
#include "JobStatements.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;

using namespace ::oracle::occi;

/**
 * Cred_JobDAO
 * Constructor
 */
Cred_JobDAO::Cred_JobDAO(OracleDAOContext& ctx,const std::string& category):
        m_logger(category),
        m_jobDaoImpl(new OracleJobDAO(ctx,category)){
}

/**
 * get
 * Return the last Job submitted with the given cred_Id-user_dn
 */
model::Job * Cred_JobDAO::getLast(const std::string& cred_id, const std::string& user_dn) /*throw (DAOLogicError,DAOException)*/{

    model::Job * job = 0;
    
    m_log_trace("Calling JobDAO::get");
    
    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createGetLast_Cred(ctx));
    try{
        // Set Parameters
        stmt->setString(1,cred_id);
        stmt->setString(2,user_dn);
        
        // Get Data
        job = m_jobDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Job - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("JobDAO::get Called");
    
    return job;
}
