/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Oracle Statements Dumper method
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: DumpStatements.h,v 1.5 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_DUMPSTATEMENTS_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_DUMPSTATEMENTS_H_


#include "glite/data/config/service/ComponentConfiguration.hxx"
#include "glite/data/agents/AgentExceptions.h"

#include <string>

#include <glite/data/config/service/Logger.hxx>    

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {
    
/**
 * DataAccessObject Configuration For Oracle
 */
class StatementsDumper : public glite::config::ComponentConfiguration{
public:

protected:

    // =========================================================================
    // ComponentConfiguration Methods
    // =========================================================================

    /**
     * Initialize the Component
     */
    virtual int init(const Params& params) { return 0; }
                                                                                                                           
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start(){return 0;}
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}
                                                                                                                           
    /**
     * Finailize the Component
     */
    virtual int fini(){return 0;}
    
private:
    glite::config::Logger m_logger;
    /*
     * Declare factory methods as friends
     */
    friend glite::config::ComponentConfiguration * ::create_glite_component();
    friend void ::destroy_glite_component(glite::config::ComponentConfiguration*);

    /**
     * Constructor
     */
    StatementsDumper();

    /**
     * Destructor
     */
    virtual ~StatementsDumper();
    
    /**
     * Initialize all the statements aand dump the related sql to files into the 
     * specified path
     * @param path [IN] the directory where to store the statements
     * @param type [IN] dump only statements of the given type. Can be one fo the 
     *   following values: 'agent';'channel';'file';'job';'transfer'
     *   An empty string will dump all the statements
     */
    void dump() /* throw () */;

private:
     
    /**
     * The registered Instance
     */
    static StatementsDumper * s_instance;

    /**
     * Dump file repository
     */
    std::string m_path;

    /**
     * Dump type
     */
    std::string m_type;
    
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_DUMPSTATEMENTS_H_
