/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Oracle FileStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: FileStatements.h,v 1.12 2007/06/22 09:26:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_FILESTATEMENTS_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_FILESTATEMENTS_H_

#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"
#include "Statements.h"

#include <vector>
#include <string>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Statemets For the model::File class
 */
class FileStatements : boost::noncopyable, public Statements {
public:

    /**
     * Return Singleton instance 
     * @return Singleton instance 
     */
    static FileStatements& instance() /*throw ()*/ ;

    // -------------------------------------------------------------------------
    // Methods
    // -------------------------------------------------------------------------

    /*
     * dispose
     *
     * dispose Statements for OracleDAOContext
     */
    void dispose(agents::dao::oracle::OracleDAOContext& ctx) /*throw ()*/;

    /**
     * Return the statement for the FileDAO::get, VO View
     *
     * Bind Parameters:
     *  :1 - File ID
     *  :2 - VO Name
     */
    ::oracle::occi::Statement* createGet_VO(
        agents::dao::oracle::OracleDAOContext&  ctx,
        const bool&                             finished = false) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the FileDAO::get, Channel View
     *
     * Bind Parameters:
     *  :1 - File ID
     *  :2 - Channel Name
     */
    ::oracle::occi::Statement* createGet_Channel(
        agents::dao::oracle::OracleDAOContext&  ctx,
        const bool&                             finished = false) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the FileDAO::update, VO View
     */
    ::oracle::occi::Statement* createUpdate_VO(agents::dao::oracle::OracleDAOContext& ctx, const model::File& file, const std::string& vo_name) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the FileDAO::update, Channel View
     */
    ::oracle::occi::Statement* createUpdate_Channel(agents::dao::oracle::OracleDAOContext& ctx, const model::File& file, const std::string& ch_name) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the FileDAO::getByJobId, VO View
     *
     * Bind Parameters:
     *  :1 - Job ID
     *  :2 - VO Name
     */
    ::oracle::occi::Statement* createGetByJobId_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the FileDAO::getByJobId, Channel View
     *
     * Bind Parameters:
     *  :1 - Job ID
     *  :2 - Channel Name
     */
    ::oracle::occi::Statement* createGetByJobId_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the FileDAO::getWaitingFiles, VO View
     *
     * Bind Parameters:
     *  :1 - VO Name
     *  :2 - Limit
     *  :3 - Limit
     */
    ::oracle::occi::Statement* createGetWaitingFiles_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the FileDAO::getCatalogWaitingFiles, VO View
     *
     * Bind Parameters:
     *  :1 - VO Name
     *  :2 - Limit
     *  :3 - Limit
     */
    ::oracle::occi::Statement* createGetCatalogWaitingFiles_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the FileDAO::getNextJobToTransfer, Channel View
     *
     * Bind Parameters:
     *  :1 - Channel Name
     *  :2 - VO Name
     *  :3 - Limit
     *  :4 - Limit
     */
    ::oracle::occi::Statement* createGetNextJobToTransfer_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the FileDAO::getJobsToTransferPerSE
     *
     * Bind Parameters:
     *  :1 - Channel Name
     *  :2 - VO Name
     *  :3 - Limit
     *  :4 - Limit
     *  :5 - SE Limit
     *  :6 - SE Limit
     */
    ::oracle::occi::Statement* createGetJobsToTransferPerSE_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;
     
    /**
     * Return the statement for the FileDAO::getFilesToCancel, VO View
     *
     * Bind Parameters:
     *  :1 - VO Name
     *  :2 - Limit
     *  :3 - Limit
     */
    ::oracle::occi::Statement* createGetFilesToCancel_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

private:
    /**
     * Default Constructor
     */
    FileStatements();

    /**
     * Destructor
     */
    ~FileStatements(){}

};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_FILESTATEMENTS_H_
