/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: JobStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: JobStatements.cpp,v 1.25 2009/07/08 10:14:12 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "JobStatements.h"
#include "dao-utility.h"
#include "glite/data/agents/dao/oracle/StatementPtr.h"

#include <sstream>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;

using namespace ::oracle::occi;

namespace {
const char * const ORACLE_JOB_STATEMENTS_NAME = "transfer-agent-dao-oracle-job-stmt";

const char * const GET_STATEMENT_VO_TAG             = "t_job_get_vo";
const char * const GET_FULL_SEARCH_STATEMENT_VO_TAG = "t_job_get_full_search_vo";
const char * const GET_STATEMENT_CH_TAG             = "t_job_get_ch";
const char * const GET_FULL_SEARCH_STATEMENT_CH_TAG = "t_job_get_full_search_ch";
const char * const GET_LAST_STATEMENT_CRED_TAG      = "t_job_get_last_cred";
const char * const UPDATE_STATEMENT_VO_TAG          = "t_job_update_vo";
const char * const UPDATE_STATEMENT_CH_TAG          = "t_job_update_ch";
const char * const GET_SUBMITTED_STATEMENT_VO_TAG   = "t_job_get_submitted_vo";
const char * const GET_PENDING_STATEMENT_VO_TAG     = "t_job_get_pending_vo";
const char * const GET_DONE_STATEMENT_VO_TAG        = "t_job_get_done_vo";
const char * const GET_FINISHING_STATEMENT_VO_TAG   = "t_job_get_finishing_vo";
const char * const GET_FILES_STATE_STATEMENT_VO_TAG = "t_job_get_files_state_vo";
const char * const GET_FILES_STATE_STATEMENT_CH_TAG = "t_job_get_files_state_ch";
const char * const GET_JOBS_TO_FIX_CH_TAG           = "t_job_get_to_fix_ch";
}

/*
 * instance
 *
 * Return Singleton instance
 */
JobStatements& JobStatements::instance() /*throw ()*/{
    static JobStatements singleton;
    return singleton;
}

/*
 * JobStatements
 *
 * Constructor
 */
JobStatements::JobStatements():
    Statements(ORACLE_JOB_STATEMENTS_NAME){
}

/*
 * dispose
 *
 * dispose Statements For OracleJobDAO
 */
void JobStatements::dispose(OracleDAOContext& ctx) {
#warning "TODO : terminate Statements"
}

/*
 * createGet_VO
 *
 * Prepare Statement For JobDAO::get
 *
 * Bind Parameters:
 *  :1 - Job ID
 *  :2 - VO Name
 */
namespace{
const std::string& select_statement()
{
	static const std::string statement =
		"SELECT t_job.job_id"
		", t_job.job_state"
		", t_job.vo_name"
		", t_job.channel_name"
		", t_job.priority"
		", t_job.cancel_job"
		", t_job.source"
		", t_job.dest"
		", t_job.user_dn"
		", t_job.job_params"
		", t_job.reason"
		", t_job.submit_time"
		", t_job.agent_dn"
		", t_job.submit_host"
		", t_job.max_time_in_queue"
		", t_job.finish_time"
		", t_job.myproxy_server"
		", t_job.space_token"
		", t_job.storage_class"
		", t_job.src_catalog"
		", t_job.src_catalog_type"
		", t_job.dest_catalog"
		", t_job.dest_catalog_type"
		", t_job.user_cred"
		", t_job.cred_id"
		", t_job.voms_cred"
		", t_job.internal_job_params"
		", t_job.source_se"
		", t_job.dest_se"
		", t_job.overwrite_flag"
		//--------------------------------------------------------------
		// Added with schema version 3.2.0
		", t_job.source_space_token"
		", t_job.source_token_description"
		//--------------------------------------------------------------
		// Added with schema version 3.3.0
		", t_job.copy_pin_lifetime"
		", t_job.lan_connection"
		", t_job.fail_nearline"
		// Added with schema version 3.4.1
		", t_job.checksum_method";

	return statement;
}

const char * create_get_vo(){
    static const std::string s =
    	select_statement() +
        //--------------------------------------------------------------
        " FROM t_job WHERE t_job.job_id = :1"
        " AND t_job.vo_name = :2"
        " AND t_job.job_finished is NULL";

    return s.c_str();
}

const char * create_get_full_search_vo(){
    static const std::string s =
    	select_statement() +
        //--------------------------------------------------------------
        " FROM t_job WHERE t_job.job_id = :1"
        " AND t_job.vo_name = :2";

    return s.c_str();
}

/*
 * createGet_Channel
 *
 * Prepare Statement For JobDAO::get
 *
 * Bind Parameters:
 *  :1 - Job ID
 *  :2 - Channel Name
 */
const char * create_get_channel(){
    static const std::string s =
    	select_statement() +
        //--------------------------------------------------------------
        " FROM t_job WHERE t_job.job_id = :1"
        " AND t_job.channel_name = :2"
        " AND t_job.job_finished is NULL";

    return s.c_str();
}

const char * create_get_full_search_channel(){
    static const std::string s =
    	select_statement() +
        //--------------------------------------------------------------
        " FROM t_job WHERE t_job.job_id = :1"
        " AND t_job.channel_name = :2";

    return s.c_str();
}

} // namespace

Statement * JobStatements::createGet_VO(
    OracleDAOContext&   ctx,
    const bool&         finished) /*throw (DAOLogicError, DAOException)*/{

    if (finished){
        return create_statement(ctx,GET_FULL_SEARCH_STATEMENT_VO_TAG,create_get_full_search_vo,"Get Full Search (VO)");
    } else {
        return create_statement(ctx,GET_STATEMENT_VO_TAG,create_get_vo,"Get (VO)");
    }
}

Statement * JobStatements::createGet_Channel(
    OracleDAOContext&   ctx,
    const bool&         finished) /*throw (DAOLogicError,DAOException)*/{

    if (finished){
        return create_statement(ctx,GET_FULL_SEARCH_STATEMENT_CH_TAG,create_get_full_search_channel,"Get Full Search (Channel)");
    } else {
        return create_statement(ctx,GET_STATEMENT_CH_TAG,create_get_channel,"Get (Channel)");
    }
}

/*
 * createGetLast_Cred
 *
 * Return the statement for the JobDAO::getLast, Cred View
 *
 * Bind Parameters:
 *  :1 - Cred ID
 *  :2 - User DN
 */
Statement* JobStatements::createGetLast_Cred(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    // Get tag
    const char * const tag = GET_LAST_STATEMENT_CRED_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
        	static const std::string query_stmt =
                std::string("SELECT * FROM (") +
				   select_statement() +
                    //----------------------------------------------------------
                   " FROM t_job WHERE t_job.cred_id = :1"
                   " AND t_job.user_dn = :2"
                   //-----------------------------------------------------------
                   // added with schema version 3.1.0
                   " AND t_job.job_finished is NULL"
                   //-----------------------------------------------------------
                   " ORDER BY SYS_EXTRACT_UTC(t_job.submit_time) DESC"
                ") WHERE ROWNUM <= 1";
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt.c_str(),tag);
        } else {
            // Create Statement
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Get Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare statement - Nullpointer");
    }
    return stmt;
}

/*
 * createUpdate_VO
 *
 * Prepare Statement For JobDAO::update. Returns 0 if job doesn't need to be updated
 */
Statement * JobStatements::createUpdate_VO(OracleDAOContext& ctx, const model::Job& job, const std::string& vo_name) /*throw (DAOLogicError,DAOException)*/ {

    // Get tag
    Statement * stmt = 0;
    try {
        // Update Job Properties
        // ----------------------------
        // | Column           | RO/RW |
        // ----------------------------
        // | job_id           |  RO   |
        // | job_state        |  RW   |
        // | vo_name          |  RO   |
        // | channel_name     |  RW   |
        // | priority         |  RO   |
        // | cancel_job       |  RO   |
        // | source           |  RW   |
        // | dest             |  RW   |
        // | user_dn          |  RO   |
        // | job_params       |  RO   |
        // | reason           |  RW   |
        // | submit_time      |  RO   |
        // | agent_dn         |  RW   |
        // | submit_host      |  RO   |
        // | max_time_in_queue|  RO   |
        // | finish_time      |  RW   |
        // | myproxy_server   |  RW   |
        // | space_token      |  RO   |
        // | storage_class    |  RO   |
        // | src_catalog      |  RW   |
        // | src_catalog_type |  RW   |
        // | dest_catalog     |  RW   |
        // | dest_catalog_type|  RW   |
        // | user_cred        |  RO   |
        // | cred_id          |  RO   |
        // | voms_cred        |  RO   |
        // | internal_params  |  RW   |
        // | source_se        |  RW   |
        // | dest_se          |  RW   |
        // | overwrite_flag   |  RO   |
        // ----------------------------

        // Prepare Statement String
        unsigned int count = 1;
        std::stringstream query_stmt;
        query_stmt << "UPDATE t_job SET ";

        add_property_update(query_stmt,count,1,job.state            ,"job_state");
        add_property_update(query_stmt,count,1,job.channelName      ,"channel_name");
        add_property_update(query_stmt,count,1,job.source           ,"source");
        add_property_update(query_stmt,count,1,job.dest             ,"dest");
        add_property_update(query_stmt,count,1,job.reason           ,"reason");
        add_property_update(query_stmt,count,1,job.agentDn          ,"agent_dn");
        add_property_update(query_stmt,count,1,job.finishTime       ,"finish_time");
        add_property_update(query_stmt,count,1,job.myproxyServer    ,"myproxy_server");
        add_property_update(query_stmt,count,1,job.srcCatalog       ,"src_catalog");
        add_property_update(query_stmt,count,1,job.srcCatalogType   ,"src_catalog_type");
        add_property_update(query_stmt,count,1,job.destCatalog      ,"dest_catalog");
        add_property_update(query_stmt,count,1,job.destCatalogType  ,"dest_catalog_type");
        add_property_update(query_stmt,count,1,job.internalParams   ,"internal_job_params");
        add_property_update(query_stmt,count,1,job.sourceSE         ,"source_se");
        add_property_update(query_stmt,count,1,job.destSE           ,"dest_se");

        query_stmt << " WHERE job_id = :" << count << " AND vo_name = :" << count + 1;

        //-----------------------------------------------------------
        // added with schema version 3.1.0
        query_stmt << " AND job_finished is NULL";
        //-----------------------------------------------------------

        if(count > 1){

            m_log_trace("Preparing statement: " << query_stmt);

            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt.str());

            // RAII
            StatementPtr stmt_ptr(ctx.connection(),stmt);

            // Set the value of the properties
            count = 1;
            // State
            if(job.state.isSet()){
                stmt->setString(count++,translateJobState(job.state()));
            }
            // Channel
            if(job.channelName.isSet()){
                stmt->setString(count++,job.channelName());
            }
            // Source
            if(job.source.isSet()){
                stmt->setString(count++,job.source());
            }
            // Dest
            if(job.dest.isSet()){
                stmt->setString(count++,job.dest());
            }
            // Reason
            if(job.reason.isSet()){
                if(job.reason().length() < MAX_T_JOB_REASON_LEN){
                    stmt->setString(count++,job.reason());
                } else {
                    stmt->setString(count++,job.reason().substr(0,MAX_T_JOB_REASON_LEN - 1));
                }
            }
            // agent_dn
            if(job.agentDn.isSet()){
                stmt->setString(count++,job.agentDn());
            }
            // finish_time
            if(job.finishTime.isSet()){
                if((time_t)-1 != job.finishTime()){
                    stmt->setTimestamp(count++,ctx.toTimestamp(job.finishTime()));
                } else {
                    stmt->setNull(count++ ,OCCITIMESTAMP);
                }
            }
            // myproxy_server
            if(job.myproxyServer.isSet()){
                stmt->setString(count++,job.myproxyServer());
            }
            // src_catalog
            if(job.srcCatalog.isSet()){
                stmt->setString(count++,job.srcCatalog());
            }
            // src_catalog_type
            if(job.srcCatalogType.isSet()){
                stmt->setString(count++,job.srcCatalogType());
            }
            // dest_catalog
            if(job.destCatalog.isSet()){
                stmt->setString(count++,job.destCatalog());
            }
            // dest_catalog_type
            if(job.destCatalogType.isSet()){
                stmt->setString(count++,job.destCatalogType());
            }
            // internal_job_params
            if(job.internalParams.isSet()){
                stmt->setString(count++,job.internalParams());
            }
            // source_se
            if(job.sourceSE.isSet()){
                stmt->setString(count++,job.sourceSE());
            }
            // dest_se
            if(job.destSE.isSet()){
                stmt->setString(count++,job.destSE());
            }

            // set clause
            stmt->setString(count,job.id());
            stmt->setString(count+1,vo_name);

            // Release the Pointer
            stmt_ptr.release();
        } else {
            m_log_trace("Nothing to update");
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Update statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Don't Check Postcondition (Null is allowed)
    return stmt;
}

/*
 * createUpdate_Channel
 *
 * Prepare Statement For JobDAO::update. Returns 0 if job doesn't need to be updated
 */
Statement * JobStatements::createUpdate_Channel(OracleDAOContext& ctx, const model::Job& job, const std::string& ch_name) /*throw (DAOLogicError,DAOException)*/ {

    Statement * stmt = 0;
    try {
        // Update Job Properties
        // ----------------------------
        // | Column           | RO/RW |
        // ----------------------------
        // | job_id           |  RO   |
        // | job_state        |  RW   |
        // | vo_name          |  RO   |
        // | channel_name     |  RO   |
        // | priority         |  RO   |
        // | cancel_job       |  RO   |
        // | source           |  RO   |
        // | dest             |  RO   |
        // | user_dn          |  RO   |
        // | job_params       |  RO   |
        // | reason           |  RW   |
        // | submit_time      |  RO   |
        // | agent_dn         |  RW   |
        // | submit_host      |  RO   |
        // | max_time_in_queue|  RO   |
        // | finish_time      |  RW   |
        // | myproxy_server   |  RO   |
        // | space_token      |  RO   |
        // | storage_class    |  RO   |
        // | src_catalog      |  RW   |
        // | src_catalog_type |  RW   |
        // | dest_catalog     |  RW   |
        // | dest_catalog_type|  RW   |
        // | user_cred        |  RO   |
        // | cred_id          |  RO   |
        // | voms_cred        |  RO   |
        // | internal_params  |  RW   |
        // | source_se        |  RO   |
        // | dest_se          |  RO   |
        // | overwrite_flag   |  RO   |
        // ----------------------------

        // Prepare Statement String
        unsigned int count = 1;
        std::stringstream query_stmt;
        query_stmt << "UPDATE t_job SET ";
        // State
        if(job.state.isSet()){
            query_stmt << ((1 == count)?"":",") << " job_state = :" << count;
            ++count;
        }
        // Reason
        if(job.reason.isSet()){
            query_stmt << ((1 == count)?"":",") << " reason = :" << count;
            ++count;
        }
        // agent_dn
        if(job.agentDn.isSet()){
            query_stmt << ((1 == count)?"":",") << " agent_dn = :" << count;
            ++count;
        }
        // finish_time
        if(job.finishTime.isSet()){
            query_stmt << ((1 == count)?"":",") << " finish_time = :" << count;
            ++count;
        }
        // internal_job_params
        if(job.internalParams.isSet()){
            query_stmt << ((1 == count)?"":",") << " internal_job_params = :" << count;
            ++count;
        }
        query_stmt << " WHERE job_id = :" << count << " AND channel_name = :" << count + 1;

        //-----------------------------------------------------------
        // added with schema version 3.1.0
        query_stmt << " AND job_finished is NULL";
        //-----------------------------------------------------------

        if(count > 1){
            m_log_trace("Preparing statement: " << query_stmt);

            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt.str());

            // RAII
            StatementPtr stmt_ptr(ctx.connection(),stmt);

            // Set already the value of the properties
            count = 1;
            // State
            if(job.state.isSet()){
                stmt->setString(count++,translateJobState(job.state()));
            }
            // Reason
            if(job.reason.isSet()){
                if(job.reason().length() < MAX_T_JOB_REASON_LEN){
                    stmt->setString(count++,job.reason());
                } else {
                    stmt->setString(count++,job.reason().substr(0,MAX_T_JOB_REASON_LEN - 1));
                }
            }
            // agent_dn
            if(job.agentDn.isSet()){
                stmt->setString(count++,job.agentDn());
            }
            // finish_time
            if(job.finishTime.isSet()){
                if((time_t)-1 != job.finishTime()){
                    stmt->setTimestamp(count++,ctx.toTimestamp(job.finishTime()));
                } else {
                    stmt->setNull(count++ ,OCCITIMESTAMP);
                }
            }
            // internal_job_params
            if(job.internalParams.isSet()){
                stmt->setString(count++,job.internalParams());
            }

            // set clause
            stmt->setString(count,job.id());
            stmt->setString(count+1,ch_name);

            // Release the Pointer
            stmt_ptr.release();
        } else {
            m_log_trace("Nothing to update");
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Update statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Don't Check Postcondition (Null is allowed)
    return stmt;
}

/*
 * createGetSubmittedJobs_VO
 *
 * Prepare Statements For JobDAO::getSubmittedJobs, VO View
 *
 * Bind Parameters:
 *  :1 - VO Name
 *  :2 - Limit
 *  :3 - Limit
 */
Statement * JobStatements::createGetSubmittedJobs_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/ {

    // Get tag
    const char * const tag = GET_SUBMITTED_STATEMENT_VO_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT id FROM ("
                    "SELECT DISTINCT t_job.job_id id"
                    " , DENSE_RANK() OVER ("
                      " ORDER BY t_job.priority DESC"
                      " , SYS_EXTRACT_UTC(t_job.submit_time) ) TopJob"
                    " FROM t_job, t_file"
                    " WHERE t_job.job_id = t_file.job_id"
                    " AND t_job.job_state = 'Submitted'"
                    " AND t_file.file_state = 'Submitted'"
                    " AND t_job.vo_name = :1"
                    //-----------------------------------------------------------
                    // added with schema version 3.1.0
                    " AND t_job.job_finished is NULL"
                    " AND t_file.job_finished is NULL"
                    //-----------------------------------------------------------
                " ) WHERE :2=0 OR TopJob<=:3";

            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating GetSubmittedJobs Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetPendingJobs_VO
 *
 * Prepare Statements For JobDAO::getPendingJobs, VO View
 *
 * Bind Parameters:
 *  :1 - VO Name
 *  :2 - Limit
 *  :3 - Limit
 */
Statement * JobStatements::createGetPendingJobs_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/ {

    // Get tag
    const char * const tag = GET_PENDING_STATEMENT_VO_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT id FROM ("
                    "SELECT DISTINCT t_job.job_id id"
                    " , DENSE_RANK() OVER ("
                      " ORDER BY t_job.priority DESC"
                      " , SYS_EXTRACT_UTC(t_job.submit_time) ) TopJob"
                    " FROM t_job, t_file"
                    " WHERE t_job.job_id = t_file.job_id"
                    //-----------------------------------------------------------
                    // added with schema version 3.1.0
                    " AND t_job.job_finished is NULL"
                    " AND t_file.job_finished is NULL"
                    //-----------------------------------------------------------
                    " AND ("
                        "( t_job.job_state IN ('Pending','Active')"
                        "  AND t_file.file_state = 'Pending'"
                        ") OR ("
                        " t_job.job_state = 'Pending'"
                        " AND t_file.file_state = 'Hold'"
                        " AND t_job.cancel_job = 'Y')"
                    ") AND t_job.vo_name = :1"
                " ) WHERE :2=0 OR TopJob<=:3";
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating GetPendingJobs Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetDoneJobs_VO
 *
 * Prepare Statements For JobDAO::getDoneJobs, VO View
 *
 * Bind Parameters:
 *  :1 - VO Name
 *  :2 - Limit
 *  :3 - Limit
 */
Statement * JobStatements::createGetDoneJobs_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/ {

    // Get tag
    const char * const tag = GET_DONE_STATEMENT_VO_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT id FROM ("
                    "SELECT t_job.job_id id"
                    " , ROW_NUMBER() OVER ("
                      " ORDER BY t_job.priority DESC"
                      " , SYS_EXTRACT_UTC(t_job.submit_time) ) TopJob"
                    " FROM t_job"
                    " WHERE t_job.job_state IN ('Done','DoneWithErrors')"
                    " AND t_job.vo_name = :1"
                    //-----------------------------------------------------------
                    // added with schema version 3.1.0
                    " AND t_job.job_finished is NULL"
                    //-----------------------------------------------------------
                " ) WHERE :2=0 OR TopJob<=:3";
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating GetDoneJobs Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetFinishingJobs_VO
 *
 * Prepare Statements For JobDAO::getFinishingJobs, VO View
 *
 * Bind Parameters:
 *  :1 - VO Name
 *  :2 - VO Name
 *  :3 - Limit
 *  :4 - Limit
 */
Statement * JobStatements::createGetFinishingJobs_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/ {

    // Get tag
    const char * const tag = GET_FINISHING_STATEMENT_VO_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT id FROM ("
                    "SELECT DISTINCT t_job.job_id id"
                    " , DENSE_RANK() OVER ("
                      " ORDER BY t_job.priority DESC"
                      " , SYS_EXTRACT_UTC(t_job.submit_time) ) TopJob"
                    " FROM t_job, t_file"
                    " WHERE t_job.job_id = t_file.job_id"
                    " AND t_job.job_state = 'Finishing'"
                    " AND t_job.vo_name = :1"
                    //-----------------------------------------------------------
                    // added with schema version 3.1.0
                    " AND t_job.job_finished is NULL"
                    " AND t_file.job_finished is NULL"
                    //-----------------------------------------------------------
                    " AND ("
                      "t_file.file_state = 'Finishing'"
                      "OR t_job.job_id NOT IN ("
                        "SELECT DISTINCT t_file.job_id"
                        " FROM t_file,t_job"
                        " WHERE t_file.job_id = t_job.job_id"
                        " AND t_job.job_state = 'Finishing'"
                        " AND t_job.vo_name = :2"
                        " AND t_file.file_state = 'WaitingCatalogRegistration'"
                        //-----------------------------------------------------------
                        // added with schema version 3.1.0
                        " AND t_job.job_finished is NULL"
                        " AND t_file.job_finished is NULL"
                        //-----------------------------------------------------------
                      ")"
                    ")"
                " ) WHERE :3=0 OR TopJob<=:4";
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating GetDoneJobs Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetFileStates_VO
 *
 * Return the state of all the files belonging to a given job
 *
 * Bind Parameters:
 *  :1 - Job ID
 *  :2 - VO Name
 */
Statement* JobStatements::createGetFileStates_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    // Get tag
    const char * const tag = GET_FILES_STATE_STATEMENT_VO_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT DISTINCT t_file.file_state"
                " FROM t_job, t_file"
                " WHERE t_job.job_id = t_file.job_id"
                " AND t_job.job_id = :1"
                " AND t_job.vo_name = :2"
                //-----------------------------------------------------------
                // added with schema version 3.1.0
                " AND t_job.job_finished is NULL"
                " AND t_file.job_finished is NULL";
                //-----------------------------------------------------------
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating GetFileStates Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetFileStates_Channel
 *
 * Return the state of all the files belonging to a given job
 *
 * Bind Parameters:
 *  :1 - Job ID
 *  :2 - Channel Name
 */
Statement* JobStatements::createGetFileStates_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    // Get tag
    const char * const tag = GET_FILES_STATE_STATEMENT_CH_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){

            // Prepare Statement String
            const char * const query_stmt =
                "SELECT DISTINCT t_file.file_state"
                " FROM t_job, t_file"
                " WHERE t_job.job_id = t_file.job_id"
                " AND t_job.job_id = :1"
                " AND t_job.channel_name = :2"
                //--------------------------------------------------------------
                // added with schema version 3.1.0
                " AND t_job.job_finished is NULL"
                " AND t_file.job_finished is NULL";
                //--------------------------------------------------------------
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating GetFileStates Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare statement - Nullpointer");
    }
    return stmt;
}


/*
 * createGetJobsToFix_Channel
 *
 * Return the statement for the getJobsToFix, Channel View
 *
 * Bind Parameters:
 *  :1 - Channel Name
 *  :2 - Limit
 *  :3 - Limit
 */
Statement* JobStatements::createGetJobsToFix_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    // Get tag
    const char * const tag = GET_JOBS_TO_FIX_CH_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){

            // Prepare Statement String
            const char * const query_stmt =
                "SELECT jid FROM ("
                    "SELECT t_job.job_id AS jid"
                    ", ROW_NUMBER() OVER (ORDER BY t_job.priority DESC"
                        " , SYS_EXTRACT_UTC(t_job.submit_time)"
                        " , t_job.job_id) JobCount"
                    " FROM t_job"
                    " WHERE t_job.job_state = 'Active'"
                    " AND t_job.channel_name = :1"
                    //----------------------------------------------------------
                    // added with schema version 3.1.0
                    " AND t_job.job_finished is NULL"
                    //----------------------------------------------------------
                    " AND NOT EXISTS( "
                        "SELECT NULL FROM t_file"
                        " WHERE t_file.job_id = t_job.job_id"
                        " AND t_file.file_state NOT IN ("
                                "'Done','Canceled','Hold','Failed')"
                        //------------------------------------------------------
                        // added with schema version 3.1.0
                        " AND t_file.job_finished is NULL"
                        //------------------------------------------------------
                    ")"
                ") WHERE :2=0 OR JobCount<=:3";
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating GetJobsToFix Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare statement - Nullpointer");
    }
    return stmt;
}
