/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleAgentDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: OracleAgentDAO.h,v 1.3 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLETAGENTDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLETAGENTDAO_H_

#include "glite/data/transfer/agent/model/Agent.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/dao-types.h"
#include "OracleDAO.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object For the Agent class
 */
class OracleAgentDAO : public OracleDAO {
public:

    /**
     * Constructor
     */
    OracleAgentDAO(OracleDAOContext& ctx, const std::string& agent_name, const std::string& category);

    /**
     * Destructor
     */
    virtual ~OracleAgentDAO(){}

    /**
     * Return the Agent instance
     */
    virtual model::Agent * get(bool lock = false) /*throw (DAOException)*/;

    /**
     * Update a Agent instance
     */
    virtual void update(const model::Agent& obj) /*throw (DAOException)*/;
 
    /**
     * Insert a new Agent instance
     */
    virtual void create(const model::Agent& obj) /*throw (DAOException)*/;

private:
    /**
     * Default Constructor. Not Implemented
     */
    OracleAgentDAO();

private:
    /**
     * The Agent Name
     */
    std::string m_agentName;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite


#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLETAGENTDAO_H_
