/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleCredDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: OracleCredDAO.cpp,v 1.4 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "OracleCredDAO.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/ResultSetPtr.h>

#include <memory>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using namespace ::oracle::occi;
using glite::data::agents::dao::oracle::ResultSetPtr;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;

/*
 * OracleCredDAO
 *
 * Constructor
 */
OracleCredDAO::OracleCredDAO(OracleDAOContext& ctx, const std::string& category):
        OracleDAO(ctx,category){
}

/*
 * get
 *
 * Return the Cred instance with the given cred_id:user_dn
 */
model::Cred * OracleCredDAO::get(Statement& stmt) /*throw (DAOLogicError,DAOException)*/{
    
    std::auto_ptr<model::Cred> c;

    m_log_trace("Calling OracleCredDAO::get");
    try{
        // Execute Query
        ResultSetPtr rs(stmt,stmt.executeQuery());
        if(0 != rs->next()){
            std::string certificate, voms_attr;
            // Read the proxy from a CLOB
            toString(rs->getClob(3),certificate);
            // Read the VOMS attributes from a CLOB
            toString(rs->getClob(5),voms_attr);
            // Get the cred
            c.reset(new model::Cred(
                rs->getString(1),               // cred_id
                rs->getString(2),               // user_dn
                certificate,                    // certificate
                toTimeT(rs->getTimestamp(4)),   // expiration_time
                voms_attr));                    // VOMS attributes
        } else {
            throw DAOException("No Cred found for the given Id");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Retrieving Cred: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Get Cred - SQL Exception");
    }
    m_log_trace("OracleCredDAO::get Called");
    
    return c.release();
}

/*
 * update
 *
 * Update a Cred object
 */
void OracleCredDAO::update(Statement& stmt) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling OracleCredDAO::update");
    
    try {
        // Execute Update
        unsigned int res = stmt.executeUpdate();
        m_log_trace(res << " row(s) affected");
        if(res <= 0){
            throw DAOException("Cred instance hasn't been updated");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Update Cred - SQL Exception");
    }
    m_log_trace("Cred instance updated");
}
