/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleCredDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: OracleCredDAO.h,v 1.3 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLECREDDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLECREDDAO_H_

#include "glite/data/transfer/agent/model/Cred.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/dao-types.h"
#include "OracleDAO.h"

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object Helper for the Cred class
 */
class OracleCredDAO : public OracleDAO {
public:

    /**
     * Constructor
     */
    OracleCredDAO(OracleDAOContext& ctx, const std::string& category);

    // -------------------------------------------------------------------------
    // File Helper Methods
    // -------------------------------------------------------------------------

    /**
     * Return the Cred instance that correspond to the given statement
     */
    model::Cred * get(::oracle::occi::Statement& stmt) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Update a Cred instance
     */
    void update(::oracle::occi::Statement& stmt) /*throw (DAOLogicError,DAOException)*/;

private:
    /**
     * Default Constructor. Not Implemented
     */
    OracleCredDAO();
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLECREDDAO_H_
