/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: OracleDAO.h,v 1.5 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLEDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLEDAO_H_

#include <glite/data/config/service/Logger.hxx>
#include "glite/data/agents/glite-types.h"
#include "glite/data/agents/dao/dao-types.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"
#include <string>
#include <boost/utility.hpp>

#include "occi.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object Base class
 */
class OracleDAO : 
    boost::noncopyable {
public:

    typedef glite::data::agents::dao::oracle::OracleDAOContext OracleDAOContext;

    /**
     * Return the DAO Context
     */
    OracleDAOContext& ctx() {return m_ctx;}
    
    /**
     * Convert a Timestamp into a time value
     */
    inline time_t toTimeT(const ::oracle::occi::Timestamp& timestamp){
        return m_ctx.toTimeT(timestamp);
    }

    /**
     * Convert a time value into a Timestamp 
     */
    inline ::oracle::occi::Timestamp toTimestamp(time_t t){
        return m_ctx.toTimestamp(t);
    }

    /**
     * Convert a Number into a longlong
     */
    inline agents::longlong toLongLong(const ::oracle::occi::Number& number){
        return m_ctx.toLongLong(number);
    }

    /**
     * Convert a longlong into a Number
     */
    inline ::oracle::occi::Number toNumber(agents::longlong n){
        return m_ctx.toNumber(n);
    }

    /**
     * Convert a Char into a boolean
     * @param str [IN] the value as string
     * @param defaultValue [IN] the value to use if the string is empty
     * @return boolean
     */
    inline bool toBoolean(const std::string& str,bool defaultValue) const /*throw ()*/{
        if (str.empty()){
            return defaultValue;
        } else {
            return m_ctx.toBoolean(str);
        }
    }

    /**
     * Convert a Boolean into a String
     */
    inline const std::string& toBoolean(bool b) const /*throw ()*/{
        return m_ctx.toBoolean(b);
    }

    /**
     * Read a Clob into a String
     */
    inline void toString(::oracle::occi::Clob clob, std::string& str){
        m_ctx.toString(clob,str);
    }
    
protected:

    /**
     * Constructor
     */
    OracleDAO(OracleDAOContext& ctx, const std::string& category):
        m_logger(category),
        m_ctx(ctx){
    }

private:
    /**
     * Default Constructor. Not Implemented
     */
    OracleDAO();

protected:
    glite::config::Logger m_logger;
    /**
     * The Connection Object
     */
    OracleDAOContext& m_ctx;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLEDAO_H_
