/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleDAOConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: OracleDAOConfig.h,v 1.5 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLEDAOCONFIG_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLEDAOCONFIG_H_

#include "glite/data/config/service/ComponentConfiguration.hxx"

#include <boost/scoped_ptr.hpp>

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * DataAccessObject Configuration For Oracle
 */
class OracleDAOConfig : public glite::config::ComponentConfiguration{
public:

    // =========================================================================
    // ComponentConfiguration Methods
    // =========================================================================

    /**
     * Initialize the Component
     */
    virtual int init(const Params& params);
                                                                                                                           
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start(){return 0;}
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}
                                                                                                                           
    /**
     * Finailize the Component
     */
    virtual int fini();
    
private:

    /*
     * Declare factory methods as friends
     */
    friend glite::config::ComponentConfiguration * ::create_glite_component();
    friend void ::destroy_glite_component(glite::config::ComponentConfiguration*);

    /**
     * Constructor
     */
    OracleDAOConfig();

    /**
     * Destructor
     */
    virtual ~OracleDAOConfig();

private:
    glite::config::Logger m_logger; 
    /**
     * VO DAO Factory
     * @link dependency
     * @label creates
     */
     /*#VO_DAOFactory lnkVO_DAOFactory;*/
    boost::scoped_ptr<class VO_DAOFactory> m_voDAOFactory;
    /**
     * Channel DAO Factory
     * @link dependency
     * @label creates
     */
     /*#Channel_DAOFactory lnkChannel_DAOFactory;*/
    boost::scoped_ptr<class Channel_DAOFactory> m_channelDAOFactory;
    /**
     * Cred DAO Factory
     * @link dependency
     * @label creates
     */
     /*#Cred_DAOFactory lnkCred_DAOFactory;*/
    boost::scoped_ptr<class Cred_DAOFactory> m_credDAOFactory;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLEDAOCONFIG_H_
