/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleTransferDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: OracleTransferDAO.h,v 1.4 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLETRANSFERDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLETRANSFERDAO_H_

#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/agents/dao/DAOException.h"
//#include "glite/data/transfer/agent/dao/StringArrayTable.h"
#include "OracleDAO.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object For the Transfer class
 */
class OracleTransferDAO : public OracleDAO {
public:

    /**
     * Constructor
     */
    OracleTransferDAO(OracleDAOContext& ctx, const std::string& category);

    /**
     * Return the Transfer instance with the given name
     */
    model::Transfer * get(::oracle::occi::Statement& stmt) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Update a Transfer object
     */
    void update(::oracle::occi::Statement& stmt) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Create a Transfer object
     */
    void create(::oracle::occi::Statement& stmt) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the a list of id for the given Statement
     */
    void getIdList(::oracle::occi::Statement& stmt, agents::dao::StringArray& ids) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the id-id list for the given statement
     */
    void getIdPairArray(::oracle::occi::Statement& stmt, std::vector<std::pair<std::string,std::string> >& ids) /*throw (DAOLogicError,DAOException)*/;

private:
    /**
     * Default Constructor. Not Implemented
     */
    OracleTransferDAO();
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite


#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_ORACLETRANSFERDAO_H_
