/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Oracle StageReqStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: StageReqStatements.h,v 1.3 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_STAGEREQSTATEMENTS_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_STAGEREQSTATEMENTS_H_

#include <glite/data/transfer/agent/model/StageReq.h>
#include <glite/data/agents/dao/DAOException.h>
#include <glite/data/agents/dao/oracle/OracleDAOContext.h>
#include <glite/data/config/service/Logger.hxx>
#include <string>
#include <vector>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Statemets For the model::StageReq class
 */
class StageReqStatements : 
    boost::noncopyable{
public:

    /**
     * Return Singleton instance 
     * @return Singleton instance 
     */
    static StageReqStatements& instance() /*throw ()*/ ;

    // -------------------------------------------------------------------------
    // Methods
    // -------------------------------------------------------------------------

    /*
     * dispose
     *
     * dispose Statements for OracleDAOContext
     */
    void dispose(agents::dao::oracle::OracleDAOContext& ctx) /*throw ()*/;
        
    /**
     * Return the statement for the StageReqDAO::get, VO View
     *
     * Bind Parameters:
     *  :1 - File ID
     *  :2 - Request ID
     *  :3 - VO Name
     */
    ::oracle::occi::Statement* createGet_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the StageReqDAO::update, VO  View
     */
    ::oracle::occi::Statement* createUpdate_VO(agents::dao::oracle::OracleDAOContext& ctx, const model::StageReq& tx, const std::string& ch_name) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the StageReqDAO::getByFileId, VO View
     *
     * Bind Parameters:
     *  :1 - File ID
     *  :2 - VO Name
     *  :3 - Limit
     *  :4 - Limit
     */
    ::oracle::occi::Statement* createGetByFileId_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the StageReqDAO::getByRequestId, VO View
     *
     * Bind Parameters:
     *  :1 - Request ID
     *  :2 - VO Name
     */
    ::oracle::occi::Statement* createGetByRequestId_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the StageReqDAO::create, Channel View
     */
    ::oracle::occi::Statement* createCreate_VO(agents::dao::oracle::OracleDAOContext& ctx, const model::StageReq& req) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the StageReqDAO::countActiveStageReqs, Channel View
     *
     * Bind Parameters:
     *  :1 - VO Name
     *  :1 - Channel Name (only if per_channel is set to true)
     */
    ::oracle::occi::Statement* createCountActiveRequests_VO(agents::dao::oracle::OracleDAOContext& ctx, bool per_channel) /*throw (DAOLogicError,DAOException)*/;

private:
    /**
     * Default Constructor
     */
    StageReqStatements();

    /**
     * Destructor. Not Implemented
     */
    ~StageReqStatements(){}
    
    glite::config::Logger m_logger;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_STAGEREQSTATEMENTS_H_
