/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Oracle TransferStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TransferStatements.h,v 1.8 2007/06/22 09:26:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_TRANSFERSTATEMENTS_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_TRANSFERSTATEMENTS_H_

#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"
#include "Statements.h"

#include <string>
#include <vector>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Statemets For the model::Transfer class
 */
class TransferStatements : boost::noncopyable, public Statements {
public:

    /**
     * Return Singleton instance 
     * @return Singleton instance 
     */
    static TransferStatements& instance() /*throw ()*/ ;

    // -------------------------------------------------------------------------
    // Methods
    // -------------------------------------------------------------------------

    /*
     * dispose
     *
     * dispose Statements for OracleDAOContext
     */
    void dispose(agents::dao::oracle::OracleDAOContext& ctx) /*throw ()*/;
        
    /**
     * Return the statement for the TransferDAO::get, VO View
     *
     * Bind Parameters:
     *  :1 - File ID
     *  :2 - Request ID
     *  :3 - VO Name
     */
    ::oracle::occi::Statement* createGet_VO(
        agents::dao::oracle::OracleDAOContext&  ctx,
        const bool&                             finished = false) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the TransferDAO::get, Channel View
     *
     * Bind Parameters:
     *  :1 - File ID
     *  :2 - Request ID
     *  :2 - Channel Name
     */
    ::oracle::occi::Statement* createGet_Channel(
        agents::dao::oracle::OracleDAOContext&  ctx,
        const bool&                             finished = false) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the TransferDAO::update, Channel View
     */
    ::oracle::occi::Statement* createUpdate_Channel(agents::dao::oracle::OracleDAOContext& ctx, const model::Transfer& tx, const std::string& ch_name) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the TransferDAO::getByFileId, VO View
     *
     * Bind Parameters:
     *  :1 - File ID
     *  :2 - VO Name
     *  :3 - Limit
     *  :4 - Limit
     */
    ::oracle::occi::Statement* createGetByFileId_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the TransferDAO::getByFileId, Channel View
     *
     * Bind Parameters:
     *  :1 - File ID
     *  :2 - Channel Name
     *  :3 - Limit
     *  :4 - Limit
     */
    ::oracle::occi::Statement* createGetByFileId_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the TransferDAO::getByRequestId, VO View
     *
     * Bind Parameters:
     *  :1 - Request ID
     *  :2 - VO Name
     */
    ::oracle::occi::Statement* createGetByRequestId_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the TransferDAO::getByRequestId, Channel View
     *
     * Bind Parameters:
     *  :1 - Request ID
     *  :2 - Channel Name
     */
    ::oracle::occi::Statement* createGetByRequestId_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the TransferDAO::create, Channel View
     */
    ::oracle::occi::Statement* createCreate_Channel(agents::dao::oracle::OracleDAOContext& ctx, const model::Transfer& tx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the TransferDAO::countActiveTransfers, Channel View
     *
     * Bind Parameters:
     *  :1 - Channel Name
     *  :2 - VO Name (only if per_vo is true)
     */
    ::oracle::occi::Statement* createCountActiveTransfers_Channel(agents::dao::oracle::OracleDAOContext& ctx,bool per_vo) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the TransferDAO::getActiveTransfers, Channel View
     *
     * Bind Parameters:
     *  :1 - Channel Name
     */
    ::oracle::occi::Statement* createGetActiveTransfers_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

private:
    /**
     * Default Constructor
     */
    TransferStatements();

    /**
     * Destructor. Not Implemented
     */
    ~TransferStatements(){}

};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_TRANSFERSTATEMENTS_H_
