/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO_AgentDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VO_AgentDAO.h,v 1.4 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_AGENTDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_AGENTDAO_H_

#include "glite/data/transfer/agent/model/Agent.h"
#include "glite/data/transfer/agent/dao/vo/AgentDAO.h"
#include "glite/data/agents/dao/DAOException.h"
#include "OracleAgentDAO.h"
#include <glite/data/config/service/Logger.hxx>
#include <boost/scoped_ptr.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object For the Agent class, VO View
 */
class VO_AgentDAO : public vo::AgentDAO{
public:

    /**
     * Constructor
     */
    VO_AgentDAO(agents::dao::oracle::OracleDAOContext& ctx, const std::string& vo_name, const std::string& category);

    /**
     * Destructor
     */
    virtual ~VO_AgentDAO(){}

    /**
     * Return the Agent instance
     */
    virtual model::Agent * get(bool lock = false) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Update a Agent instance
     */
    virtual void update(const model::Agent& obj) /*throw (DAOLogicError,DAOException)*/;
 
    /**
     * Insert a new Agent instance
     */
    virtual void create(const model::Agent& obj) /*throw (DAOLogicError,DAOException)*/;

private:
    /**
     * Default Constructor. Not Implemented
     */
    VO_AgentDAO();

private:
    glite::config::Logger m_logger;
    /**
     * The Helper
     * @link dependency
     */
    /*#OracleAgentDAO lnkOracleAgentDAO;*/
    boost::scoped_ptr<class OracleAgentDAO> m_agentDaoImpl;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_AGENTDAO_H_
