/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO_ChannelDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VO_ChannelDAO.h,v 1.8 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_CHANNELDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_CHANNELDAO_H_

#include <glite/data/transfer/agent/model/Job.h>
#include <glite/data/transfer/agent/dao/vo/ChannelDAO.h>
#include <glite/data/agents/dao/DAOException.h>
#include "OracleChannelDAO.h"

#include <boost/scoped_ptr.hpp>

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object For the model::Channel class, VO View
 */
class VO_ChannelDAO : public vo::ChannelDAO{
public:

    /**
     * Constructor
     */
    VO_ChannelDAO(agents::dao::oracle::OracleDAOContext& ctx, const std::string& vo_name, const std::string& category);

    /**
     * Destructor
     */
    virtual ~VO_ChannelDAO(){}

    /**
     * Return the Channel instance with the given name
     */
    virtual model::Channel * get(const std::string& name) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the name of the Channel instance based on the provided site endpoints
     */
    virtual std::string getBySites(const std::string& ssite, const std::string& dsite) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Returns true if the VO has a share for the given channel, false otherwise
     */
    virtual bool hasShare(const std::string& channel_name) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return true if the SE or the site is backlisted. If true, also return the reason
     */
    virtual std::pair<bool,std::string> isBlacklisted(const std::string& host, const std::string& site) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the list of groups which a site belong to
     */
    virtual void getSiteGroups(const std::string& site, agents::dao::StringArray& groups) /*throw (DAOLogicError,DAOException)*/;
   
private:
    /**
     * Default Constructor. Not Implemented
     */
    VO_ChannelDAO();
    
private:
    glite::config::Logger m_logger;
    /**
     * The Helper
     * @link dependency
     */
     /*#OracleChannelDAO lnkOracleChannelDAO;*/
    boost::scoped_ptr<class OracleChannelDAO> m_channelDaoImpl;
    
    /**
     * The Name of the VO
     */
    std::string m_voName;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_CHANNELDAO_H_
