/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO_CredDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VO_CredDAO.cpp,v 1.4 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "VO_CredDAO.h"
#include "OracleCredDAO.h"
#include "CredStatements.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;
using namespace ::oracle::occi;

/*
 * VO_CredDAO
 *
 * Constructor
 */
VO_CredDAO::VO_CredDAO(OracleDAOContext& ctx, const std::string& vo_name,const std::string& category):
        m_logger(category),
        m_credDaoImpl(new OracleCredDAO(ctx,category)),
        m_voName(vo_name){
}

/*
 * get
 *
 * Return the Cred with the given Id
 */
model::Cred * VO_CredDAO::get(const std::string& id, const std::string& user_dn) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling CredDAO::get");
    
    model::Cred * c = 0;

    // Get Context
    OracleDAOContext& ctx = m_credDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),CredStatements::instance().createGet_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,id);
        stmt->setString(2,user_dn);
        stmt->setString(3,m_voName);
        
        // Get Data
        c = m_credDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Get Cred - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("CredDAO::get Called");
    
    return c;
}
