/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Oracle VO DAO Factory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VO_DAOFactory.h,v 1.4 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_DAOFACTORY_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_DAOFACTORY_H_

#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"
#include "glite/data/agents/dao/DAOException.h"

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * DataAccessObject Factory For Oracle, VO View
 */
class VO_DAOFactory : public vo::DAOFactory{
public:

    /**
     * Constructor
     */
    VO_DAOFactory();

    /**
     * Destructor
     */
    virtual ~VO_DAOFactory(){}
    
    // =========================================================================
    // DAO Factory Method
    // =========================================================================

    /**
     * Validate The DB Schema
     * @param ctx [IN] The DAO Context to use
      */
    virtual void checkSchema(agents::dao::DAOContext& ctx) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Returns DataAccessObject for model:File instances
     * @param vo_name [IN] The name of the VO to view on
     * @param ctx [IN] The DAO Context to use
     */
    virtual vo::FileDAO * createFileDAO(const std::string& vo_name, agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

    /**
     * Returns DataAccessObject for model:Job instances
     * @param vo_name [IN] The name of the VO to view on
     * @param ctx [IN] The DAO Context to use
     */
    virtual vo::JobDAO * createJobDAO(const std::string& vo_name, agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

    /**
     * Returns DataAccessObject for model:Transfer instances
     * @param vo_name [IN] The name of the VO to view on
     * @param ctx [IN] The DAO Context to use
     */
    virtual vo::TransferDAO * createTransferDAO(const std::string& vo_name, agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

    /**
     * Returns DataAccessObject for model:StageReq instances
     * @param vo_name [IN] The name of the VO to view on
     * @param ctx [IN] The DAO Context to use
     */
    virtual vo::StageReqDAO * createStageReqDAO(const std::string& vo_name, agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

    /**
     * Returns DataAccessObject for model:Channel instances
     * @param vo_name [IN] The name of the VO to view on
     * @param ctx [IN] The DAO Context to use
     */
    virtual vo::ChannelDAO * createChannelDAO(const std::string& vo_name, agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

    /**
     * Returns DataAccessObject for model:Agent instances
     * @param vo_name [IN] The name of the VO to view on
     * @param ctx [IN] The DAO Context to use
     */
    virtual vo::AgentDAO * createAgentDAO(const std::string& vo_name, agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

    /**
     * Returns DataAccessObject for model:Cred instances
     * @param vo_name [IN] The name of the VO to view on
     * @param ctx [IN] The DAO Context to use
     */
    virtual vo::CredDAO * createCredDAO(const std::string& vo_name, agents::dao::DAOContext& ctx) /*throw (DAOLogicError)*/;

private:
    glite::config::Logger m_logger;
    /** 
     * @link dependency 
     * @label creates
	 */
    /*# VO_JobDAO lnkVO_JobDAO; */

    /** 
     * @link dependency 
     * @label creates
	 */
    /*# VO_FileDAO lnkVO_FileDAO; */

    /** 
     * @link dependency 
     * @label creates
	 */
    /*# VO_TransferDAO lnkVO_TransferDAO; */

    /**
     * @link dependency 
     * @label creates
	 */
    /*# VO_ChannelDAO lnkVO_ChannelDAO; */
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_DAOFACTORY_H_
