/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO_FileDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: VO_FileDAO.h,v 1.9 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_FILEDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_FILEDAO_H_

#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/dao/vo/FileDAO.h"
#include "OracleFileDAO.h"

#include <boost/scoped_ptr.hpp>

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object For the File class, VO View
 */
class VO_FileDAO : public vo::FileDAO{
public:

    /**
     * Constructor
     */
    VO_FileDAO(agents::dao::oracle::OracleDAOContext& ctx, const std::string& vo_name, const std::string& category);

    /**
     * Destructor
     */
    virtual ~VO_FileDAO(){}

    // -------------------------------------------------------------------------
    // VO File DAO Methods
    // -------------------------------------------------------------------------

    /**
     * Return the File with the given Id
     */
    virtual model::File * get(const std::string& id) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the File with the given Id
     */
    virtual model::File * getFullSearch(const std::string& id) /*throw (DAOLogicError,DAOException)*/;
        
    /**
     * Update a File instance
     */
    virtual void update(const model::File& obj) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the list of File instances for the given job Id
     */
    virtual void getByJobId(const std::string& job_id, agents::dao::StringArray& files) /*throw (DAOLogicError,DAOException)*/;
    
    /**
     * Return the list of identifiers of the file which state is WAITING and 
     * the related job state is ACTIVE
     */
    virtual void getWaitingFiles(agents::dao::StringArray& fileIds, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Return the list of the job-files identifiers for the files which state 
     * is WAITING_CATALOG_RESOLUTION or WAITING_CATALOG_REGISTRATION 
     */
    virtual void getCatalogWaitingFiles(FilesPerJobList& ids, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/;
    
private:
    /**
     * Default Constructor. Not Implemented
     */
    VO_FileDAO();

private:
    glite::config::Logger m_logger;
    /**
     * The Helper
     * @link dependency
     */
     /*#OracleFileDAO lnkOracleFileDAO;*/
    boost::scoped_ptr<class OracleFileDAO> m_fileDaoImpl;
    
    /**
     * The Name of the VO
     */
    std::string m_voName;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_FILEDAO_H_
