/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO_JobDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: VO_JobDAO.h,v 1.8 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_JOBDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_JOBDAO_H_

#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/dao/vo/JobDAO.h"
#include "OracleJobDAO.h"

#include <boost/scoped_ptr.hpp>

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object For the model::Job class, VO View
 */
class VO_JobDAO : public vo::JobDAO{
public:

    /**
     * Constructor
     */
    VO_JobDAO(agents::dao::oracle::OracleDAOContext& ctx, const std::string& vo_name, const std::string& category);

    /**
     * Destructor
     */
    virtual ~VO_JobDAO(){}

    /**
     * Return the TransferJob with the given JobId
     */
    virtual model::Job * get(const std::string& id) /*throw (DAOLogicError,DAOException)*/;
    
    /**
     * Return the Job with the given JobId searching all jobs 
     */
    virtual model::Job * getFullSearch(const std::string& id) /*throw (DAOLogicError,DAOException)*/;    
        
    /**
     * Update a TransferJob object
     */
    virtual void update(const model::Job& obj) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Update the state of all the transfer jobs in the given list
     * If an empty list is provided, this method fails with a LogicError
     */
    virtual void updateStates(const agents::dao::StringArray& ids) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the list of Submitted Job instances that have files in Submitted 
     * state
     */
    virtual void getSubmittedJobs(agents::dao::StringArray& jobIds, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Return the list of Job identifiers that are in Pending state
     */
    virtual void getPendingJobs(agents::dao::StringArray& jobIds, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Return the list of Job identifiers that are in Done or DoneWithErrors state
     */
    virtual void getDoneJobs(agents::dao::StringArray& jobIds, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Return the list of Job identifiers that are in Finishing state
     */
    virtual void getFinishingJobs(agents::dao::StringArray& jobIds, unsigned int limit = 0) /*throw (DAOLogicError, DAOException)*/;

private:
    /**
     * Default Constructor. Not Implemented
     */
    VO_JobDAO();
    
private:
    glite::config::Logger m_logger;
    /**
     * The Helper
     * @link dependency
     */
     /*#OracleJobDAO lnkOracleJobDAO;*/
    boost::scoped_ptr<class OracleJobDAO> m_jobDaoImpl;
    
    /**
     * The Name of the VO
     */
    std::string m_voName;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_JOBDAO_H_
