/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO_StageReqDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VO_StageReqDAO.h,v 1.3 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_STAGEREQDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_STAGEREQDAO_H_

#include <glite/data/config/service/Logger.hxx>
#include "glite/data/transfer/agent/model/StageReq.h"
#include "glite/data/transfer/agent/dao/vo/StageReqDAO.h"
#include "glite/data/agents/dao/DAOException.h"
#include "OracleStageReqDAO.h"

#include <boost/scoped_ptr.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object For the StageReq class, VO View
 */
class VO_StageReqDAO : public vo::StageReqDAO{
public:

    /**
     * Constructor
     */
    VO_StageReqDAO(agents::dao::oracle::OracleDAOContext& ctx, const std::string& vo_name, const std::string& category);

    /**
     * Destructor
     */
    virtual ~VO_StageReqDAO(){}

    /**
     * Return the StageReq with the given FileId - RequestId
     */
    virtual model::StageReq * get(const std::string& file_id, const std::string& req_id) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Return the list of StageReq instances for the given File Id, ordered by 
     * transfer time descending
     */
    virtual void getByFileId(const std::string& file_id, agents::dao::StringArray& ids, unsigned long limit = 0) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Return the list of File_ids instances for the given Request Id
     */
    virtual void getByRequestId(const std::string& request_id, agents::dao::StringArray& ids) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Update a StageReq instance
     */
    virtual void update(const model::StageReq& obj) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Create a StageReq instance
     */
    virtual void create(const model::StageReq& obj) /*throw (DAOLogicError,DAOException)*/;
        
    /**
     * Count the active (Received & Processing) Stage Requests instances for the VO
     */
    virtual unsigned long countActiveRequests() /*throw (DAOLogicError,DAOException)*/;

    /**
     * Count the active (Received & Processing) Stage Requests instances for the VO
     * allocated to a given channel
     */
    virtual unsigned long countActiveRequests(const std::string& channel_name) /*throw (DAOLogicError,DAOException)*/;
   
private:
    
    /**
     * Return true if the file id is valid
     */
    bool isValidFile(const std::string& file_id) /*throw (DAOLogicError,DAOException)*/;
 
private:
    /**
     * Default Constructor. Not Implemented
     */
    VO_StageReqDAO();

private:
    glite::config::Logger m_logger;
    /**
     * The Helper
     * @link dependency
     */
     /*#OracleStageReqDAO lnkOracleStageReqDAO;*/
    boost::scoped_ptr<class OracleStageReqDAO> m_stagereqDaoImpl;
    
    /**
     * The Name of the VO
     */
    std::string m_voName;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_STAGEREQDAO_H_
