/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO_TransferDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: VO_TransferDAO.h,v 1.9 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_TRANSFERDAO_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_TRANSFERDAO_H_

#include <glite/data/config/service/Logger.hxx>
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/dao/vo/TransferDAO.h"
#include "glite/data/agents/dao/DAOException.h"
#include "OracleTransferDAO.h"

#include <boost/scoped_ptr.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object For the Transfer class, VO View
 */
class VO_TransferDAO : public vo::TransferDAO{
public:

    /**
     * Constructor
     */
    VO_TransferDAO(agents::dao::oracle::OracleDAOContext& ctx, const std::string& vo_name,const std::string& category);

    /**
     * Destructor
     */
    virtual ~VO_TransferDAO(){}

    /**
     * Return the Transfer with the given FileId - RequestId
     */
    virtual model::Transfer * get(const std::string& file_id, const std::string& req_id) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Return the Transfer with the given FileId - RequestId
     */
    virtual model::Transfer * getFullSearch(const std::string& file_id, const std::string& req_id) /*throw (DAOLogicError, DAOException)*/;
        
    /**
     * Return the list of Transfer instances for the given File Id, ordered by 
     * transfer time descending
     */
    virtual void getByFileId(const std::string& file_id, agents::dao::StringArray& ids, unsigned long limit = 0) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Return the list of File_ids instances for the given Request Id
     */
    virtual void getByRequestId(const std::string& request_id, agents::dao::StringArray& ids) /*throw (DAOLogicError, DAOException)*/;

private:
    
    /**
     * Return true if the file id is valid
     */
    bool isValidFile(const std::string& file_id) /*throw (DAOLogicError,DAOException)*/;
 
private:
    /**
     * Default Constructor. Not Implemented
     */
    VO_TransferDAO();

private:
    glite::config::Logger m_logger;
    /**
     * The Helper
     * @link dependency
     */
     /*#OracleTransferDAO lnkOracleTransferDAO;*/
    boost::scoped_ptr<class OracleTransferDAO> m_transferDaoImpl;
    
    /**
     * The Name of the VO
     */
    std::string m_voName;
};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_VO_TRANSFERDAO_H_
