/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ErrorMatch Class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ErrorMatch.h,v 1.2 2007/07/06 14:35:00 ptedesco Exp $
 *  Release: $Name:  $
 */

#ifndef GLITE_DATA_TRANSFER_AGENTS_FSM_ERRORMATCH_H_
#define GLITE_DATA_TRANSFER_AGENTS_FSM_ERRORMATCH_H_

#include "glite/data/transfer/agent/model/Error.h"
#include <vector>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace fsm     {

/**
 * interface for error matching objects
 */    
class IErrorMatch{
public:
    IErrorMatch(){}
    virtual ~IErrorMatch(){}
    virtual bool isMatch(
        const glite::data::transfer::agent::model::Error::Category&  category, 
        const glite::data::transfer::agent::model::Error::Scope&     scope, 
        const glite::data::transfer::agent::model::Error::Phase&     phase) const = 0;
};
    
/**
 * ErrorMatch class
 * utility for state transitions on error conditions
 */
class ErrorMatch{
    
public:
    /**
     * constructor
     */
    ErrorMatch(){}
    /**
     * destructor
     */
    virtual ~ErrorMatch();
    /**
     * add a match on the phase
     */
    void add(
        const glite::data::transfer::agent::model::Error::Phase& phase);
    /**
     * add a match on phase and category
     */
    void add(
        const glite::data::transfer::agent::model::Error::Phase&    phase,
        const glite::data::transfer::agent::model::Error::Category& category);
    /**
     * add a match on the category
     */
    void add(
        const glite::data::transfer::agent::model::Error::Category& category);
    /**
     * add a match on phase, category and scope
     */
    void add(
        const glite::data::transfer::agent::model::Error::Phase&    phase,
        const glite::data::transfer::agent::model::Error::Category& category,
        const glite::data::transfer::agent::model::Error::Scope&    scope);
    /**
     * return true if category, scope and phase match any of the internal 
     * criteria
     */
    bool isMatch(
        const glite::data::transfer::agent::model::Error::Category&  category, 
        const glite::data::transfer::agent::model::Error::Scope&     scope, 
        const glite::data::transfer::agent::model::Error::Phase&     phase) const;
private:
    std::vector<IErrorMatch*> m_matches;
};

} // End namespace model
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif // GLITE_DATA_TRANSFER_AGENTS_FSM_ERRORMATCH_H_
