/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FileEventListener interface
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: EventDispatcher.h,v 1.2 2006/08/14 08:38:42 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_FSM_EVENTDISPATCHER_H_
#define GLITE_DATA_TRANSFER_AGENTS_FSM_EVENTDISPATCHER_H_

#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"

#include "glite/data/transfer/agent/fsm/FileEventListener.h"
#include "glite/data/transfer/agent/fsm/JobEventListener.h"

#include <boost/utility.hpp>
#include <vector>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace fsm     {

/**
 * FileEventListener Interface
 * @interface
 */
class EventDispatcher : boost::noncopyable {
public:
    /**
     * Return singleton instance
     */
    static EventDispatcher& instance();

    // -------------------------------------------------------------------------
    // -------------------------------------------------------------------------
    // File Events
    // -------------------------------------------------------------------------
    // -------------------------------------------------------------------------

    /**
     * Register a File Event Listener
     */
    void registerListener(FileEventListener* l) throw();
    /**
     * Deregister a File Event Listener
     */
    void deregisterListener(FileEventListener* l) throw();
    
    /**
     * Dispatch Event File State Changed
     */
    void fileStateChanged(const model::File& file, const model::Job& job, model::File::State old_state) throw();
    /**
     * Dispatch Event File Ready
     */
    void fileReady(const model::File& file, const model::Job& job) throw();
    /**
     * Dispatch Event File Done
     */
    void fileDone(const model::File& file, const model::Job& job) throw();
    /**
     * Dispatch Event File Finished
     */
    void fileFinished(const model::File& file, const model::Job& job) throw();

    // -------------------------------------------------------------------------
    // -------------------------------------------------------------------------
    // Job Events
    // -------------------------------------------------------------------------
    // -------------------------------------------------------------------------

    /**
     * Register a Job Event Listener
     */
    void registerListener(JobEventListener* l) throw();
    /**
     * Deregister a Job Event Listener
     */
    void deregisterListener(JobEventListener* l) throw();
    /**
     * Dispatch Event Job State Changed
     */
    void jobStateChanged(const model::Job& job, model::Job::State old_state) throw();
    /**
     * Dispatch Event Job Pending
     */
    void jobPending(const model::Job& job) throw();
    /**
     * Dispatch Event Job Finished (or Finished Dirty)
     */
    void jobFinished(const model::Job& job) throw();

    
private:
    /**
     * Constructor
     */
    EventDispatcher();

    /**
     * Destructor
     */
    ~EventDispatcher();

private:
    typedef std::vector<FileEventListener*> FileEventListeners;
    typedef std::vector<JobEventListener* > JobEventListeners;

    /**
     * The file's events Listeners
     */
    FileEventListeners m_fileListeners;
    /**
     * The job's events Listeners
     */
    JobEventListeners  m_jobListeners;
};

} // End namespace fsm
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_FSM_EVENTDISPATCHER_H_
