/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FSMConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: FSMConfig.h,v 1.4 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_FSM_FSMCONFIG_H_
#define GLITE_DATA_TRANSFER_AGENTS_FSM_FSMCONFIG_H_

#include "glite/data/config/service/ComponentConfiguration.hxx"
#include <string>

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace fsm     {

/**
 * The FSMConfig Class is used to retrieve the Configuration Parameters from 
 * the ServiceConfigurator
 * @streotype singleton
 */    
class FSMConfig :  public glite::config::ComponentConfiguration{
public:                                                                                          
    /**
     * Initialize the Component
     */
    virtual int init(const Params& params);
                                                                                                                           
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start(){return 0;}
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}
                                                                                                                           
    /**
     * Finailize the Component
     */
    virtual int fini(){return 0;}
                                                                                                                           
    /**
     * Return Singleton Instance
     */
    static FSMConfig& instance();
                                                                                                                           
    /**
     * Finalize Singleton Instance
     */
    static void finalize();
        
    /**
     * Return the Logger
     */
    
                                                                                                                           
    // =========================================================================
    // Configuration Properties
    // =========================================================================
               
    /**
     * Get Hold State Enabled Property
     * @return Hold State Enabled Property
     */
    bool holdEnabled() const { return m_holdEnabled;}
                                                                                                                           
   
protected:
    /**
     * Constructor
     */
    FSMConfig();
                                                                                                                           
    /**
     * Destructor
     */
    virtual ~FSMConfig();
                                                                                                                           
private:
    glite::config::Logger m_logger;
    /**
     * Singleton Instance
     */
    static FSMConfig * s_instance;
                                                                                                                           
    /**
     * The Hold State Enabled Property
     */
    bool  m_holdEnabled;
};

} // end namespace fsm
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_FSM_FSMCONFIG_H_
