/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: TransferJobFSM Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: JobFSM.h,v 1.2 2006/08/14 08:38:43 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_FSM_JOBFSM_H_
#define GLITE_DATA_TRANSFER_AGENTS_FSM_JOBFSM_H_

#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/File.h"
#include "FSMException.h"
#include <string>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace fsm     {
/**
 * JobFSM class. Implements the logic to handle the transitions between 
 * the Job's states. It operates as a Visitor on the Job Object. This logic 
 * could be implemented also on the Job class, but then it would violate the 
 * ValueObject pattern
 */
class JobFSM : boost::noncopyable{
public:

    /**
     * Constructor
     */
    JobFSM(glite::data::transfer::agent::model::Job& j):m_job(j){}

    /**
     * Update The Job State concording the States of all the related 
     * Files. 
     * @param states [IN] an OR of all the states of the related Files
     */
    void updateState(glite::data::transfer::agent::model::File::State states) /*throw (FSMException)*/;

private:
    
    /**
     * Dispatch the Job's State Changed event, if needed
     */
    void eventStateChanged(glite::data::transfer::agent::model::Job::State old_state) throw ();

private:
    /**
     * Default Constructor. Not Implemented
     */
    JobFSM();

private:
    /**
     * TransferJob Property
     */
    glite::data::transfer::agent::model::Job& m_job;

    /** @link dependency 
     * @label throws*/
    /*# FSMException lnkFSMException; */
};

} // end namespace fsm
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_FSM_JOBFSM_H_
