/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: NameGenerationFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: NameGenerationFactory.cpp,v 1.2 2006/08/14 08:38:50 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/transfer/agent/namegen/NameGenerationFactory.h"
#include "DefaultNameGeneration.h"

using namespace glite::data::transfer::agent::namegen;

/*
 * Initialize Static Attirbutes
 */
NameGenerationFactory *  NameGenerationFactory::s_instance = 0;

/*
 * getInstance
 *
 * Return the registered Instance
 */
NameGenerationFactory& NameGenerationFactory::instance() /*throw (LogicError)*/ {
    if(0 == s_instance){
        // No Name Generation Factory configure: instantiate the Default one
        new NameGenerationFactory();
    }
    return *s_instance;
}

/*
 * NameGenerationFactory
 *
 * Constructor
 */
NameGenerationFactory::NameGenerationFactory(){
    // Register the instance
    if(0 == s_instance){
        s_instance = this;
    }
}
                                                                                                                          
/*
 * ~NameGenerationFactory
 *
 * Destructor
 */
NameGenerationFactory::~NameGenerationFactory(){
    if(this != s_instance){
        s_instance = 0;
    }
}

/*
 * create
 *
 * Create a NameGeneration instance
 */
NameGeneration * NameGenerationFactory::create() /*throw ()*/{
    return new DefaultNameGeneration();
}
