/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: NameGenConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: NameGenConfig.h,v 1.3 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_NAMEGEN_PYTHON_NAMEGENCONFIG_H_
#define GLITE_DATA_TRANSFER_AGENTS_NAMEGEN_PYTHON_NAMEGENCONFIG_H_

#include <string>
#include <boost/python.hpp>
#include <glite/data/config/service/Logger.hxx>

namespace glite    {
namespace data     {
namespace transfer {
namespace agent    {
namespace namegen  {

/**
 * NameGenConfig Class
 */
class NameGenConfig{
public:

    /**
     * Return the NameGenConfig singleton instance
     * @return the NameGenConfig singleton instance
     */
    static NameGenConfig& instance() /*throw ()*/; 

    // =========================================================================
    // NameGen Static Constants
    // =========================================================================
    static const char * const NAMEGEN_VERSION_FUNCTION_NAME;
    static const char * const NAMEGEN_V_1_0_GENERATE_FUNCTION_NAME;
    static const char * const NAMEGEN_V_1_0_INIT_FUNCTION_NAME;
    static const char * const NAMEGEN_V_1_0;

    // =========================================================================
    // PythonNameGen Properties
    // =========================================================================

    /**
     * Get NameGen Module Name 
     * @return the NameGen Module Name
     */
    const std::string& module() const {return m_namegenModule;} 

    /**
     * Get NameGen Module Interface Version
     * @return the NameGen Module Interface Version
     */
    const std::string& version() const {return m_namegenVersion;}

    /**
     * Get the module NameGen Generate Function Object
     * @return the NameGen Generate Function Object
     */
    boost::python::object generateFunction() { return m_generate;}

protected:
        
    /**
     * Set NameGenModule 
     */
    void module(const std::string& m) {m_namegenModule = m;} 

    // =========================================================================
    // VOActionsData Methods
    // =========================================================================
    
    /**
     * Verify that the module is  correct and get the module's version  
     */
    bool validate() /*throw()*/;

    /**
     * Initialize the module
     */
    bool init(const std::string& params) /*throw()*/;
    
private:

    /*
     * Declare factory methods as friends
     */
    friend class PythonNameGenFactory;

    /**
     * Constructor
     */
    NameGenConfig () : m_logger("glite-transfer-namegen-python"){
    }
        
    /**
     * Destructor
     */
    ~NameGenConfig (){}

private:
    glite::config::Logger m_logger;
    /**
     * The NameGen Python Module Name
     */
    std::string m_namegenModule;

    /**
     * The NameGen Python Module Interface Version
     */
    std::string m_namegenVersion;

    /**
     * The module Object
     */
    boost::python::object m_module;

    /**
     * The Generate Function Object
     */
    boost::python::object m_generate;
};

} // end namespace namegen
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_NAMEGEN_PYTHON_NAMEGENCONFIG_H_
