/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: PythonCatalog Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: PythonNameGen.h,v 1.2 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_NAMEGEN_PYTHON_PYTHONNAMEGEN_H_
#define GLITE_DATA_TRANSFER_AGENTS_NAMEGEN_PYTHON_PYTHONNAMEGEN_H_

#include "glite/data/transfer/agent/namegen/NameGeneration.h"
#include "glite/data/transfer/agent/namegen/NameGenerationException.h"
#include <string>
#include <boost/python.hpp>
#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace namegen {

/**
 * PythonNameGen Class. Implements the NameGenerator using a python script
 */
class PythonNameGen : public NameGeneration{
public:

    /**
     * Constructor
     */
    PythonNameGen();

    /**
     * Destructor
     */
    virtual ~PythonNameGen();
     
    /**
     * Generate a destination Name SURL starting from the logical name and the 
     * Se path
     */
    virtual std::string generate(const std::string& logical,
                                 const std::string& endpoint,
                                 const std::string& se_path, 
                                 const std::string& storage_class) /*throw (NameGenerationException)*/;
private:
    glite::config::Logger m_logger;
};

} // End namespace namegen
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_NAMEGEN_PYTHON_PYTHONNAMEGEN_H_
