/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: NameGeneratorFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: PythonNameGenFactory.h,v 1.2 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_NAMEGEN_PYTHONNAMEGENFACTORY_H_
#define GLITE_DATA_TRANSFER_AGENTS_NAMEGEN_PYTHONNAMEGENFACTORY_H_

#include <glite/data/transfer/agent/namegen/NameGenerationFactory.h>
#include <glite/data/config/service/ComponentConfiguration.hxx>
#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace namegen {

/**
 * PythonNameGenFactory class. Implement the factory for the Python 
 * NameGenerator Client 
 */
class PythonNameGenFactory : public NameGenerationFactory, 
                             public glite::config::ComponentConfiguration{
public:

    /**
     * Constructor
     */ 
    PythonNameGenFactory();

    /**
     * Destructor
     */ 
    virtual ~PythonNameGenFactory();

    // =========================================================================
    // NameGeneratorFactory Methods
    // =========================================================================

    /**
     * Create a NameGenerator instance
     */
    virtual NameGeneration * create() /*throw ()*/; 

    // =========================================================================
    // ComponentConfiguration Methods
    // =========================================================================

    /**
     * Initialize the Component
     */
    virtual int init(const Params& params);
                                                                                                                           
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start(){return 0;}
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}
                                                                                                                           
    /**
     * Finalize the Component
     */
    virtual int fini();

private:
    
    glite::config::Logger m_logger;
    
    /** @link dependency 
     * @label creates*/
    /*# PythonNameGenFactory lnkPythonNameGen; */
};

} // End namespace namegen
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_NAMEGEN_PYTHONNAMEGENFACTORY_H_
