/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  Python wrapper for Error class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: FtsModule_Error.cpp,v 1.2 2008/12/11 13:24:44 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#include "FtsModule.h"
#include "PropertyConverters.h"

#include <glite/data/transfer/agent/model/Error.h>

#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
#include <boost/shared_ptr.hpp>
#include <vector>
#include <string>

using namespace glite::data::transfer::agent::python;
namespace py = boost::python;

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace python  {
    
/**
 * Create and return the wrapper for the Error::Category enum
 */
void ErrorCategory_enum() {

    // Expose Error::Category as ErrorCategory
    static py::object s_ErrorCategory_enum(
      py::enum_<model::Error::Category>("ErrorCategory")
        .value("Undef"                  , model::Error::UNDEF)
        .value("Transfer"               , model::Error::TRANSFER)
        .value("Security"               , model::Error::SECURITY)
        .value("Network"                , model::Error::NETWORK)
        .value("Configuration"          , model::Error::CONFIGURATION)
        .value("Consistency"            , model::Error::CONSISTENCY)
        .value("Catalog"                , model::Error::CATALOG)
        // Added with schema version 3.1.0
        .value("User"                   , model::Error::USER)
        .value("FileExists"             , model::Error::FILE_EXISTS)
        .value("InvalidPath"            , model::Error::INVALID_PATH)
        .value("NoSpaceLeft"            , model::Error::NO_SPACE_LEFT)
        .value("InvalidSize"            , model::Error::INVALID_SIZE)
        .value("RequestFailure"         , model::Error::REQUEST_FAILURE)
        .value("Connection"             , model::Error::CONNECTION)
        .value("HttpTimeout"            , model::Error::HTTP_TIMEOUT)
        .value("RequestTimeout"         , model::Error::REQUEST_TIMEOUT)
        .value("TransferTimeout"        , model::Error::TRANSFER_TIMEOUT)
        .value("InternalError"          , model::Error::INTERNAL_ERROR)
        .value("Aborted"                , model::Error::ABORTED)
        .value("GeneralFailure"         , model::Error::GENERAL_FAILURE)
        .value("Lost"                   , model::Error::LOST)
        .value("StorageInternalError"   , model::Error::STORAGE_INTERNAL_ERROR)
        .value("GridFtp"                , model::Error::GRIDFTP)
        .value("MarkersTimeout"         , model::Error::MARKERS_TIMEOUT)
        .value("SrmCopy"                , model::Error::SRM_COPY)
        .value("CatalogInternalError"   , model::Error::CATALOG_INTERNAL_ERROR)
        // Added with schema version 3.3.0
        .value("Locality"               , model::Error::LOCALITY)
        .value("FirstMarkerTimeout"     , model::Error::FIRST_MARKER_TIMEOUT)
        .value("NoProgress"             , model::Error::NO_PROGRESS)
        // deprecated
        .value("Agent"                  , model::Error::AGENT)
        .value("Source"                 , model::Error::SOURCE)
        .value("SourcePreparation"      , model::Error::SOURCE_PREP)
        .value("SourceFinalization"     , model::Error::SOURCE_FINAL)
        .value("Destination"            , model::Error::DESTINATION)
        .value("DestinationPreparation" , model::Error::DESTINATION_PREP)
        .value("DestinationFinalization", model::Error::DESTINATION_FINAL)
        .value("Remote"                 , model::Error::REMOTE)
        .value("CatalogSource"          , model::Error::CATALOG_SRC)
        .value("CatalogSrcResolution"   , model::Error::CATALOG_SRC_RES)
        .value("CatalogDestination"     , model::Error::CATALOG_DST)
        .value("CatalogDestResolution"  , model::Error::CATALOG_DST_RES)
        .value("CatalogDestRegistration", model::Error::CATALOG_DST_REG)
        .export_values()
    );
    
    // Error Category property converter
    py::to_python_converter<
      model::Property<model::Error::Category>,
      PropertyToPython<model::Error::Category> >();
    py::implicitly_convertible<model::Error::Category, model::Property<model::Error::Category> >();

}

/**
 * Create and return the wrapper for the Error::Scope enum
 */
void ErrorScope_enum() {

    // Expose Error::Scope as ErrorScope
    static py::object s_ErrorScope_enum(
      py::enum_<model::Error::Scope>("ErrorScope")
        .value("Undef"          , model::Error::S_UNDEF)
        .value("Source"         , model::Error::S_SOURCE)
        .value("Destination"    , model::Error::S_DESTINATION)
        .value("Transfer"       , model::Error::S_TRANSFER)
        .value("Agent"          , model::Error::S_AGENT)
        .export_values()
    );
    
    // Error Scope property converter
    py::to_python_converter<
      model::Property<model::Error::Scope>,
      PropertyToPython<model::Error::Scope> >();
    py::implicitly_convertible<model::Error::Scope, model::Property<model::Error::Scope> >();
}

/**
 * Create and return the wrapper for the Error::Phase enum
 */
void ErrorPhase_enum() {

    // Expose Error::Phase as ErrorPhase
    static py::object s_ErrorPhase_enum(
      py::enum_<model::Error::Phase>("ErrorPhase")
        .value("Undef"                  , model::Error::P_UNDEF)
        .value("TxPreparation"          , model::Error::P_TX_PREPARATION)
        .value("TxFinalization"         , model::Error::P_TX_FINALIZATION)
        .value("Transfer"               , model::Error::P_TRANSFER)
        .value("Allocation"             , model::Error::P_ALLOCATION)
        .value("CatalogResolution"      , model::Error::P_CATALOG_RESOLUTION)
        .value("CatalogRegistration"    , model::Error::P_CATALOG_REGISTRATION)
        .value("Prestaging"             , model::Error::P_PRESTAGING)
        .value("Finalization"           , model::Error::P_FINALIZATION)
        .value("Waiting"                , model::Error::P_WAITING)
        .value("WaitingCatalog"         , model::Error::P_WAITING_CATALOG)
        .value("Canceling"              , model::Error::P_CANCELING)
        .value("TxService"              , model::Error::P_TX_SERVICE)
        .value("PreTransfer"            , model::Error::P_PRE_TRANSFER)
        .value("PostTransfer"           , model::Error::P_POST_TRANSFER)
        .value("CheckReadiness"         , model::Error::P_CHECK_READINESS)
        .export_values()
    );
    
    // Error Phase property converter
    py::to_python_converter<
      model::Property<model::Error::Phase>,
      PropertyToPython<model::Error::Phase> >();
    py::implicitly_convertible<model::Error::Phase, model::Property<model::Error::Phase> >();
    
}
    
    
} // namespace python
} // namespace agent
} // namespace transfer
} // namespace data
} // namespace glite

