/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  Python wrapper for the File class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: FtsModule_File.cpp,v 1.1 2008/11/13 14:20:28 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#include "FtsModule.h"
#include "PropertyConverters.h"

#include <glite/data/transfer/agent/model/File.h>

#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
#include <boost/shared_ptr.hpp>
#include <vector>
#include <string>

using namespace glite::data::transfer::agent::python;
namespace py = boost::python;

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace python  {

/**
 * Create and return the wrapper for the File::State enum
 */
void FileState_enum() {

    // Expose File::State as FileState
    static py::object s_FileState_enum(
      py::enum_<model::File::State>("FileState")
        .value("Undef"                     , model::File::S_UNDEF)
        .value("Submitted"                 , model::File::S_SUBMITTED)
        .value("Pending"                   , model::File::S_PENDING)
        .value("Ready"                     , model::File::S_READY)
        .value("Active"                    , model::File::S_ACTIVE)
        .value("Done"                      , model::File::S_DONE)
        .value("Waiting"                   , model::File::S_WAITING)
        .value("Hold"                      , model::File::S_HOLD)
        .value("Failed"                    , model::File::S_FAILED)
        .value("Finishing"                 , model::File::S_FINISHING)
        .value("Finished"                  , model::File::S_FINISHED)
        .value("AwaitingPrestage"          , model::File::S_AWAITING_PRESTAGE)
        .value("Prestaging"                , model::File::S_PRESTAGING)
        .value("WaitingPrestage"           , model::File::S_WAITING_PRESTAGE)
        .value("WaitingCatalogResolution"  , model::File::S_WAITING_CATALOG_RESOLUTION)
        .value("WaitingCatalogRegistration", model::File::S_WAITING_CATALOG_REGISTRATION)
        .value("Canceled"                  , model::File::S_CANCELED)
        .export_values()
    );
    
    // File::State property converter
    py::to_python_converter<
      model::Property<model::File::State>,
      PropertyToPython<model::File::State> >();
    py::implicitly_convertible<model::File::State, model::Property<model::File::State> >();

    
}

/**
 * Create and return the wrapper for the File class
 */
void File_class() {

    // Create the Python type object for File Class
    static py::object s_File_class(
      py::class_<model::File, boost::shared_ptr<model::File> >("File")
        .def(py::init<const std::string&              // file_id
                ,const std::string&                   // job_id
                ,model::File::State                   // File state
             >())
        .def(py::init<const std::string&              // file_id
                ,const std::string&                   // job_id
                ,model::File::State                   // File state
                ,const std::string&                   // Logical Name
                ,const std::string&                   // Source Surl
                ,const std::string&                   // Dest Surl
                ,model::Error::Category               // Reason Class
                ,const std::string&                   // Reason
                ,py::optional<
                  unsigned int                        // Failures
                 ,unsigned int                        // Current Failures
                 ,unsigned int                        // Catalog Failures
                 ,unsigned int                        // Prestage Failures
// skip it since it doesn't compile (This constructor is needed oly for some tests)
//                 ,agents::longlong                    // Filesize                  
//                 ,const std::string&                  // Checksum
//                 ,time_t                              // Finish Time
//                 ,const std::string&                  // Agent Dn
                 >
                >())
       .def(py::init<const model::File&>())
       .add_property("id"              , py::make_getter(&model::File::id              , py::return_value_policy<py::return_by_value>()))
       .add_property("jobId"           , py::make_getter(&model::File::jobId           , py::return_value_policy<py::return_by_value>()))
       .add_property("state"           , py::make_getter(&model::File::state           , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::state           , py::return_value_policy<py::return_by_value>()))
       .add_property("logical"         , py::make_getter(&model::File::logical         , py::return_value_policy<py::return_by_value>()))
       .add_property("sourceSurl"      , py::make_getter(&model::File::sourceSurl      , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::sourceSurl      , py::return_value_policy<py::return_by_value>()))
       .add_property("destSurl"        , py::make_getter(&model::File::destSurl        , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::destSurl        , py::return_value_policy<py::return_by_value>()))
       .add_property("reasonClass"     , py::make_getter(&model::File::reasonClass     , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::reasonClass     , py::return_value_policy<py::return_by_value>()))
       .add_property("reason"          , py::make_getter(&model::File::reason          , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::reason          , py::return_value_policy<py::return_by_value>()))
       .add_property("failures"        , py::make_getter(&model::File::failures        , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::failures        , py::return_value_policy<py::return_by_value>()))
       .add_property("currentFailures" , py::make_getter(&model::File::currentFailures , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::currentFailures , py::return_value_policy<py::return_by_value>()))
       .add_property("catalogFailures" , py::make_getter(&model::File::catalogFailures , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::catalogFailures , py::return_value_policy<py::return_by_value>()))
       .add_property("prestageFailures", py::make_getter(&model::File::prestageFailures, py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::prestageFailures, py::return_value_policy<py::return_by_value>()))
       .add_property("fileSize"        , py::make_getter(&model::File::fileSize        , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::fileSize        , py::return_value_policy<py::return_by_value>()))
       .add_property("checksum"        , py::make_getter(&model::File::checksum        , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::checksum        , py::return_value_policy<py::return_by_value>()))
       .add_property("finishTime"      , py::make_getter(&model::File::finishTime      , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::finishTime      , py::return_value_policy<py::return_by_value>()))
       .add_property("agentDn"         , py::make_getter(&model::File::agentDn         , py::return_value_policy<py::return_by_value>()))
       // added with 3.1.0            
       .add_property("errorScope"      , py::make_getter(&model::File::errorScope      , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::errorScope      , py::return_value_policy<py::return_by_value>()))
       .add_property("errorPhase"      , py::make_getter(&model::File::errorPhase      , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::File::errorPhase      , py::return_value_policy<py::return_by_value>()))
    );
    
    //register converted for const pointer
    py::to_python_converter<
        boost::shared_ptr<const model::File>,
        copy_shared_const_ptr<model::File> >();
}
    
/**
 * Create and return the wrapper for the FileArray class
 */
void FileArray_class() {

    typedef std::vector<boost::shared_ptr<model::File> > FileArray;
    typedef std::vector<boost::shared_ptr<const model::File> > FileConstArray;
   
    static py::object s_FileArray_class(
      py::class_<FileArray>("FileArray")
       .def(py::vector_indexing_suite<FileArray,true >())
    );
    
    static py::object s_FileCArray_class(
      py::class_<FileConstArray>("FileConstArray")
       .def(py::vector_indexing_suite<FileConstArray,true >())
    );
}
    
} // namespace python
} // namespace agent
} // namespace transfer
} // namespace data
} // namespace glite
    
    