/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  Python wrapper for Job class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: FtsModule_Job.cpp,v 1.2 2008/11/17 10:17:39 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "FtsModule.h"
#include "PropertyConverters.h"

#include <glite/data/transfer/agent/model/Job.h>

#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
#include <boost/shared_ptr.hpp>
#include <vector>
#include <string>

using namespace glite::data::transfer::agent::python;
namespace py = boost::python;

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace python  {

/**
 * Create and return the wrapper for the Job class
 */
void Job_class() {

    // Create the Python type object for Job Class
    static py::object s_Job_class(
      py::class_<model::Job, boost::shared_ptr<model::Job> >("Job")
        .def(py::init<const std::string&            // Job Id
                ,model::Job::State                  // state
                ,const std::string&                 // VO Name
                ,const std::string&                 // Channel Name
             >())                  
        .def(py::init<const std::string&            // Job Id
                ,model::Job::State                  // state
                ,const std::string&                 // VO Name
                ,const std::string&                 // Channel Name
                ,unsigned int                       // priority
                ,bool                               // cancel Job
                ,const std::string&                 // source
                ,const std::string&                 // dest
                ,const std::string&                 // user Dn
                ,const std::string&                 // job params
                ,const std::string&                 // reason
                ,py::optional< 
                  time_t                             // submit time
                 ,const std::string&                 // agent DN
                 ,const std::string&                 // submit_host
// skip it since it doesn't compile (This constructor is needed oly for some tests)
//                 ,unsigned int                       // max time in queue
//                 ,time_t                             // finish time
//                 ,const std::string&                 // myproxy_server
//                 ,const std::string&                 // space token
//                 ,const std::string&                 // storage_class
                 >
               >())
        .def(py::init<const model::Job&>())
        .add_property("id"              , py::make_getter(&model::Job::id               , py::return_value_policy<py::return_by_value>()))
        .add_property("state"           , py::make_getter(&model::Job::state            , py::return_value_policy<py::return_by_value>())
                                        , py::make_setter(&model::Job::state            , py::return_value_policy<py::return_by_value>()))
        .add_property("voName"          , py::make_getter(&model::Job::voName           , py::return_value_policy<py::return_by_value>()))
        .add_property("channelName"     , py::make_getter(&model::Job::channelName      , py::return_value_policy<py::return_by_value>()))
        .add_property("priority"        , py::make_getter(&model::Job::priority         , py::return_value_policy<py::return_by_value>()))
        .add_property("cancelJob"       , py::make_getter(&model::Job::cancelJob        , py::return_value_policy<py::return_by_value>()))
        .add_property("source"          , py::make_getter(&model::Job::source           , py::return_value_policy<py::return_by_value>()))
        .add_property("dest"            , py::make_getter(&model::Job::dest             , py::return_value_policy<py::return_by_value>()))
        .add_property("userDn"          , py::make_getter(&model::Job::userDn           , py::return_value_policy<py::return_by_value>()))
        .add_property("params"          , py::make_getter(&model::Job::params           , py::return_value_policy<py::return_by_value>()))
        .add_property("reason"          , py::make_getter(&model::Job::reason           , py::return_value_policy<py::return_by_value>())
                                        , py::make_setter(&model::Job::reason           , py::return_value_policy<py::return_by_value>()))
        .add_property("submitTime"      , py::make_getter(&model::Job::submitTime       , py::return_value_policy<py::return_by_value>()))
        .add_property("agentDn"         , py::make_getter(&model::Job::agentDn          , py::return_value_policy<py::return_by_value>()))
        .add_property("submitHost"      , py::make_getter(&model::Job::submitHost       , py::return_value_policy<py::return_by_value>()))
        .add_property("maxTimeInQueue"  , py::make_getter(&model::Job::maxTimeInQueue   , py::return_value_policy<py::return_by_value>()))
        .add_property("finishTime"      , py::make_getter(&model::Job::finishTime       , py::return_value_policy<py::return_by_value>())
                                        , py::make_setter(&model::Job::finishTime       , py::return_value_policy<py::return_by_value>()))
        .add_property("myproxyServer"   , py::make_getter(&model::Job::myproxyServer    , py::return_value_policy<py::return_by_value>()))
        .add_property("spaceToken"      , py::make_getter(&model::Job::spaceToken       , py::return_value_policy<py::return_by_value>()))
        .add_property("storageClass"    , py::make_getter(&model::Job::storageClass     , py::return_value_policy<py::return_by_value>()))
        .add_property("srcCatalog"      , py::make_getter(&model::Job::srcCatalog       , py::return_value_policy<py::return_by_value>()))
        .add_property("srcCatalogType"  , py::make_getter(&model::Job::srcCatalogType   , py::return_value_policy<py::return_by_value>()))
        .add_property("destCatalog"     , py::make_getter(&model::Job::destCatalog      , py::return_value_policy<py::return_by_value>()))
        .add_property("destCatalogType" , py::make_getter(&model::Job::destCatalogType  , py::return_value_policy<py::return_by_value>()))
        .add_property("sourceSE"        , py::make_getter(&model::Job::sourceSE         , py::return_value_policy<py::return_by_value>()))
        .add_property("destSE"          , py::make_getter(&model::Job::destSE           , py::return_value_policy<py::return_by_value>()))
        .add_property("overwriteFlag"   , py::make_getter(&model::Job::overwriteFlag    , py::return_value_policy<py::return_by_value>()))
        .add_property("sourceSpaceToken", py::make_getter(&model::Job::sourceSpaceToken , py::return_value_policy<py::return_by_value>()))
        // new in 3.3.0
        .add_property("copyPinLifetime" , py::make_getter(&model::Job::copyPinLifetime  , py::return_value_policy<py::return_by_value>()))
        .add_property("lanConnection"   , py::make_getter(&model::Job::lanConnection    , py::return_value_policy<py::return_by_value>()))
        .add_property("failNearline"    , py::make_getter(&model::Job::failNearline     , py::return_value_policy<py::return_by_value>()))
   );
    
    //register converted for const pointer
    py::to_python_converter<
        boost::shared_ptr<const model::Job>,
        copy_shared_const_ptr<model::Job> >();
}
    
/**
 * Create and return the wrapper for the Job::State enum
 */
void JobState_enum() {

    // Expose Job::State as JobState
    static py::object s_JobState_enum(
      py::enum_<model::Job::State>("JobState")
        .value("Undef"         , model::Job::S_UNDEF)
        .value("Submitted"     , model::Job::S_SUBMITTED)
        .value("Pending"       , model::Job::S_PENDING)
        .value("Ready"         , model::Job::S_READY)
        .value("Active"        , model::Job::S_ACTIVE)
        .value("Done"          , model::Job::S_DONE)
        .value("DoneWithErrors", model::Job::S_DONE_WITH_ERRORS)
        .value("Hold"          , model::Job::S_HOLD)
        .value("Canceled"      , model::Job::S_CANCELED)
        .value("Failed"        , model::Job::S_FAILED)
        .value("Finishing"     , model::Job::S_FINISHING)
        .value("Finished"      , model::Job::S_FINISHED)
        .value("FinishedDirty" , model::Job::S_FINISHED_DIRTY)
        .export_values()
    );
    
    // Job::State converter
    py::to_python_converter<
            model::Property<model::Job::State>,
    PropertyToPython<model::Job::State> >();
    py::implicitly_convertible<model::Job::State, model::Property<model::Job::State> >();
    
}

} // namespace python
} // namespace agent
} // namespace transfer
} // namespace data
} // namespace glite
    

