/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  Python wrapper for the StageReq class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: FtsModule_StageReq.cpp,v 1.1 2008/11/13 14:20:28 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#include "FtsModule.h"
#include "PropertyConverters.h"

#include <glite/data/transfer/agent/model/StageReq.h>

#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
#include <boost/shared_ptr.hpp>
#include <vector>
#include <string>

using namespace glite::data::transfer::agent::python;
namespace py = boost::python;

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace python  {

/**
 * Create the wrapper for the StageReq::State enum
 */
void StageReqState_enum() {
                                                                                                                                            
    // Expose StageReq::State as StageReqState
    static py::object s_StageReqState_enum(
      py::enum_<model::StageReq::State>("StageReqState")
        .value("Undef"      , model::StageReq::S_UNDEF)
        .value("Submitted"  , model::StageReq::S_SUBMITTED)
        .value("Processing" , model::StageReq::S_PROCESSING)
        .value("Staged"     , model::StageReq::S_STAGED)
        .value("Finalizing" , model::StageReq::S_FINALIZING)
        .value("Completed"  , model::StageReq::S_COMPLETED)
        .value("Failed"     , model::StageReq::S_FAILED)
        .value("Aborted"    , model::StageReq::S_ABORTED)
        .export_values()
    );
    
    // StageReq State property converter
    py::to_python_converter<
      model::Property<model::StageReq::State>,
      PropertyToPython<model::StageReq::State> >();
    py::implicitly_convertible<model::StageReq::State, model::Property<model::StageReq::State> >();

    
}

/**
 * Create the wrapper for the Transfer class
 */
void StageReq_class() {
                                                                                                                                            
    // Create the Python type object for StageReq Class
    static py::object s_StageReq_class(
      py::class_<model::StageReq, boost::shared_ptr<model::StageReq> >("StageReq")
        .def(py::init<const std::string&              // request_id
                ,const std::string&                   // file_id
                ,unsigned int                         // stage_id
                ,const std::string&                   // Job id
                ,model::StageReq::State               // StageReq state
             >())
       .def(py::init<const model::StageReq&>())
       .add_property("requestId"    , py::make_getter(&model::StageReq::requestId    , py::return_value_policy<py::return_by_value>()))
       .add_property("fileId"       , py::make_getter(&model::StageReq::fileId       , py::return_value_policy<py::return_by_value>()))
       .add_property("stageId"      , py::make_getter(&model::StageReq::stageId      , py::return_value_policy<py::return_by_value>()))
       .add_property("jobId"        , py::make_getter(&model::StageReq::jobId        , py::return_value_policy<py::return_by_value>()))
       .add_property("state"        , py::make_getter(&model::StageReq::state        , py::return_value_policy<py::return_by_value>())
                                    , py::make_setter(&model::StageReq::state        , py::return_value_policy<py::return_by_value>()))
       .add_property("turl"         , py::make_getter(&model::StageReq::turl         , py::return_value_policy<py::return_by_value>())
                                    , py::make_setter(&model::StageReq::turl         , py::return_value_policy<py::return_by_value>()))
       .add_property("requestTime"  , py::make_getter(&model::StageReq::requestTime  , py::return_value_policy<py::return_by_value>())
                                    , py::make_setter(&model::StageReq::requestTime  , py::return_value_policy<py::return_by_value>()))
       .add_property("duration"     , py::make_getter(&model::StageReq::duration     , py::return_value_policy<py::return_by_value>())
                                    , py::make_setter(&model::StageReq::duration     , py::return_value_policy<py::return_by_value>()))
       .add_property("finalDuration", py::make_getter(&model::StageReq::finalDuration, py::return_value_policy<py::return_by_value>())
                                    , py::make_setter(&model::StageReq::finalDuration, py::return_value_policy<py::return_by_value>()))
       .add_property("reasonClass"  , py::make_getter(&model::StageReq::reasonClass  , py::return_value_policy<py::return_by_value>())
                                    , py::make_setter(&model::StageReq::reasonClass  , py::return_value_policy<py::return_by_value>()))
       .add_property("reason"       , py::make_getter(&model::StageReq::reason       , py::return_value_policy<py::return_by_value>())
                                    , py::make_setter(&model::StageReq::reason       , py::return_value_policy<py::return_by_value>()))
       .add_property("fileSize"     , py::make_getter(&model::StageReq::fileSize     , py::return_value_policy<py::return_by_value>())
                                    , py::make_setter(&model::StageReq::fileSize     , py::return_value_policy<py::return_by_value>()))
       // added with 3.1.0            
       .add_property("errorScope"   , py::make_getter(&model::StageReq::errorScope      , py::return_value_policy<py::return_by_value>())
                                    , py::make_setter(&model::StageReq::errorScope      , py::return_value_policy<py::return_by_value>()))
       .add_property("errorPhase"   , py::make_getter(&model::StageReq::errorPhase      , py::return_value_policy<py::return_by_value>())
                                    , py::make_setter(&model::StageReq::errorPhase      , py::return_value_policy<py::return_by_value>()))
       .add_property("finishTime"   , py::make_getter(&model::StageReq::finishTime      , py::return_value_policy<py::return_by_value>())
                                    , py::make_setter(&model::StageReq::finishTime      , py::return_value_policy<py::return_by_value>()))
    );
    
    //register converted for const pointer
    py::to_python_converter<
        boost::shared_ptr<const model::StageReq>,
        copy_shared_const_ptr<model::StageReq> >();
    
}
    
    
} // namespace python
} // namespace agent
} // namespace transfer
} // namespace data
} // namespace glite
    
