/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  Python wrapper for the Transfer class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: FtsModule_Transfer.cpp,v 1.1 2008/11/13 14:20:28 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#include "FtsModule.h"
#include "PropertyConverters.h"

#include <glite/data/transfer/agent/model/Transfer.h>

#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
#include <boost/shared_ptr.hpp>
#include <vector>
#include <string>

using namespace glite::data::transfer::agent::python;
namespace py = boost::python;

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace python  {

/**
 * Create the wrapper for the Transfer::State enum
 */
void TransferState_enum() {
                                                                                                                                            
    // Expose Transfer::State as TransferState
    static py::object s_TransferState_enum(
      py::enum_<model::Transfer::State>("TransferState")
        .value("Undef"      , model::Transfer::S_UNDEF)
        .value("Received"   , model::Transfer::S_RECEIVED)
        .value("Preparing"  , model::Transfer::S_PREPARING)
        .value("Ready"      , model::Transfer::S_READY)
        .value("Submitted"  , model::Transfer::S_SUBMITTED)
        .value("Processing" , model::Transfer::S_PROCESSING)
        .value("Transferred", model::Transfer::S_TRANSFERRED)
        .value("Completed"  , model::Transfer::S_COMPLETED)
        .value("Failed"     , model::Transfer::S_FAILED)
        .value("Aborted"    , model::Transfer::S_ABORTED)
        .export_values()
    );
    
    // Transfer State property converter
    py::to_python_converter<
      model::Property<model::Transfer::State>,
      PropertyToPython<model::Transfer::State> >();
    py::implicitly_convertible<model::Transfer::State, model::Property<model::Transfer::State> >();

    
}

/**
 * Create the wrapper for the Transfer class
 */
void Transfer_class() {
                                                                                                                                            
    // Create the Python type object for Transfer Class
    static py::object s_Transfer_class(
      py::class_<model::Transfer,boost::shared_ptr<model::Transfer>, boost::shared_ptr<const model::Transfer> >("Transfer")
        .def(py::init<const std::string&              // request_id
                ,const std::string&                   // file_id
                ,unsigned int                         // transfer_id
                ,const std::string&                   // Job id
                ,model::Transfer::State               // Transfer state
                ,const std::string&                   // transfer type
             >())
       .def(py::init<const model::Transfer&>())
       .add_property("requestId"       , py::make_getter(&model::Transfer::requestId       , py::return_value_policy<py::return_by_value>()))
       .add_property("fileId"          , py::make_getter(&model::Transfer::fileId          , py::return_value_policy<py::return_by_value>()))
       .add_property("transferId"      , py::make_getter(&model::Transfer::transferId      , py::return_value_policy<py::return_by_value>()))
       .add_property("jobId"           , py::make_getter(&model::Transfer::jobId           , py::return_value_policy<py::return_by_value>()))
       .add_property("state"           , py::make_getter(&model::Transfer::state           , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::state           , py::return_value_policy<py::return_by_value>()))
       .add_property("sourceTurl"      , py::make_getter(&model::Transfer::sourceTurl      , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::sourceTurl      , py::return_value_policy<py::return_by_value>()))
       .add_property("destTurl"        , py::make_getter(&model::Transfer::destTurl        , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::destTurl        , py::return_value_policy<py::return_by_value>()))
       .add_property("transferType"    , py::make_getter(&model::Transfer::transferType    , py::return_value_policy<py::return_by_value>()))
       .add_property("transferTime"    , py::make_getter(&model::Transfer::transferTime    , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::transferTime    , py::return_value_policy<py::return_by_value>()))
       .add_property("prepareTime"     , py::make_getter(&model::Transfer::prepareTime     , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::prepareTime     , py::return_value_policy<py::return_by_value>()))
       .add_property("duration"        , py::make_getter(&model::Transfer::duration        , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::duration        , py::return_value_policy<py::return_by_value>()))
       .add_property("srcPrepDuration" , py::make_getter(&model::Transfer::srcPrepDuration , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::srcPrepDuration , py::return_value_policy<py::return_by_value>()))
       .add_property("dstPrepDuration" , py::make_getter(&model::Transfer::dstPrepDuration , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::dstPrepDuration , py::return_value_policy<py::return_by_value>()))
       .add_property("txDuration"      , py::make_getter(&model::Transfer::txDuration      , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::txDuration      , py::return_value_policy<py::return_by_value>()))
       .add_property("srcFinalDuration", py::make_getter(&model::Transfer::srcFinalDuration, py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::srcFinalDuration, py::return_value_policy<py::return_by_value>()))
       .add_property("dstFinalDuration", py::make_getter(&model::Transfer::dstFinalDuration, py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::dstFinalDuration, py::return_value_policy<py::return_by_value>()))
       .add_property("bytesWritten"    , py::make_getter(&model::Transfer::bytesWritten    , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::bytesWritten    , py::return_value_policy<py::return_by_value>()))
       .add_property("throughput"      , py::make_getter(&model::Transfer::throughput      , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::throughput      , py::return_value_policy<py::return_by_value>()))
       .add_property("reasonClass"     , py::make_getter(&model::Transfer::reasonClass     , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::reasonClass     , py::return_value_policy<py::return_by_value>()))
       .add_property("reason"          , py::make_getter(&model::Transfer::reason          , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::reason          , py::return_value_policy<py::return_by_value>()))
       .add_property("fileSize"        , py::make_getter(&model::Transfer::fileSize        , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::fileSize        , py::return_value_policy<py::return_by_value>()))
       .add_property("srcSrmToken"     , py::make_getter(&model::Transfer::srcSrmToken     , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::srcSrmToken     , py::return_value_policy<py::return_by_value>()))
       .add_property("destSrmToken"    , py::make_getter(&model::Transfer::destSrmToken    , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::destSrmToken    , py::return_value_policy<py::return_by_value>()))
       .add_property("srcHost"         , py::make_getter(&model::Transfer::srcHost         , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::srcHost         , py::return_value_policy<py::return_by_value>()))
       .add_property("destHost"        , py::make_getter(&model::Transfer::destHost        , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::destHost        , py::return_value_policy<py::return_by_value>()))
       // added with 3.1.0            
       .add_property("errorScope"      , py::make_getter(&model::Transfer::errorScope      , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::errorScope      , py::return_value_policy<py::return_by_value>()))
       .add_property("errorPhase"      , py::make_getter(&model::Transfer::errorPhase      , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::errorPhase      , py::return_value_policy<py::return_by_value>()))
       .add_property("finishTime"      , py::make_getter(&model::Transfer::finishTime      , py::return_value_policy<py::return_by_value>())
                                       , py::make_setter(&model::Transfer::finishTime      , py::return_value_policy<py::return_by_value>()))
    );
    
    //register converted for const pointer
    py::to_python_converter<
        boost::shared_ptr<const model::Transfer>,
        copy_shared_const_ptr<model::Transfer> >();
}

/**
 * Create and return the wrapper for the TransferArray class
 */
void TransferArray_class() {

    typedef std::vector<boost::shared_ptr<model::Transfer> > TransferArray;
    typedef std::vector<boost::shared_ptr<const model::Transfer> > TransferConstArray;
    
    static py::object s_TransferArray_class(
      py::class_<TransferArray>("TransferArray")
       .def(py::vector_indexing_suite<TransferArray,true >())
    );
    static py::object s_TransferCArray_class(
      py::class_<TransferConstArray>("TransferConstArray")
       .def(py::vector_indexing_suite<TransferConstArray,true >())
    );
}

    
    
} // namespace python
} // namespace agent
} // namespace transfer
} // namespace data
} // namespace glite
    
