/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Define the FTS Proeprty Converters
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: PropertyConverters.h,v 1.3 2008/11/13 14:20:28 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_PYTHON_PROPERTYCONVERTERS_H_
#define GLITE_DATA_TRANSFER_AGENTS_PYTHON_PROPERTYCONVERTERS_H_
 
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/StageReq.h"
#include "glite/data/transfer/agent/model/Error.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace python  {

/**
 * Property to python Custom converter
 */
template <typename T>
struct copy_shared_const_ptr {
    static PyObject* convert(boost::shared_ptr<const T> const& p) {
        T t = (*p);
        return boost::python::incref(boost::python::object(t).ptr());
    }
};

    
/**
 * Property to python Custom converter
 */
template <typename T>
struct PropertyToPython {
    static PyObject* convert(model::Property<T> const& s) {
        return boost::python::incref(boost::python::object(s.data()).ptr());
    }
};

} // end namespace python
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_PYTHON_PROPERTYCONVERTERS_H_
