/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ActionScheduler Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: SchedulerConfig.h,v 1.4 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_SCHEDULERCONFIG_H_
#define GLITE_DATA_TRANSFER_AGENTS_SCHEDULERCONFIG_H_

#include "glite/data/config/service/ServiceConfiguration.hxx"


#include <glite/data/config/service/Logger.hxx>    

namespace glite    {
namespace data     {
namespace transfer {
namespace agent    {
    
/**
 * ActionScheduler Class: Executes FTA Actions 
 */
class SchedulerConfig : public glite::config::ComponentConfiguration{
public:

    //--------------------------------------------------------------------------
    // ServiceConfiguration interface
    //--------------------------------------------------------------------------
    
    /**
     * Initialize the Component
     */
    virtual int init(const Params& params);

    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
    
    /**
     * Start the Component
     */
    virtual int start(){return 0;}

    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}

    /**
     * Finailize the Component
     */
    virtual int fini();
    
    /**
     * Return Singleton Instance
     */
    static SchedulerConfig * instance();
                                                                                                                           
    /**
     * Finalize Singleton Instance
     */
    static void finalize();

    // =========================================================================
    // Configuration Properties
    // =========================================================================
    
    /**
     * Get the maximum number of failures after that the action should be disabled
     */
    unsigned long maxFailures() const {return m_maxFailures;}

    /**
     * Get the interval, in second, an action should stay disabled
     */
    unsigned long disableTime() const {return m_disableTime;}      
   
    /**
     * Get the DAO context check interval, in second
     */
    unsigned long checkInterval() const {return m_checkInterval;}      
   
    /**
     * Get the scheduler stop timeout interval, in second
     */
    unsigned long stopTimeout() const {return m_stopTimeout;}
       
private:
    
    /**
     * Constructor
     */
    SchedulerConfig();
    
    /**
     * Destructor
     */
    virtual ~SchedulerConfig();
    
    glite::config::Logger m_logger;
    /**
     * Singleton Instance
     */
    static SchedulerConfig * s_instance;
     
    /**
     * Maximum number of failures after that the action should be disabled
     */
    unsigned long m_maxFailures;

    /**
     * The interval, in second, an action should stay disabled
     */
    unsigned long m_disableTime;
    
    /**
     * The DAO context check interval, in second
     */
    unsigned long m_checkInterval;
    
    /**
     * The scheduler stop timeout interval, in second
     */
    unsigned long m_stopTimeout;
};

} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_SCHEDULERCONFIG_H_
