/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: TransferServiceFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TransferServiceFactory.cpp,v 1.5 2008/02/29 08:59:03 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/transfer/agent/ts/TransferServiceFactory.h"
#include "glite/data/agents/AgentExceptions.h"

using namespace glite::data::transfer::agent::ts;
using namespace glite::data::transfer::agent::model;

/*
 * Initialize Static Attirbutes
 */
TransferServiceFactory *  TransferServiceFactory::s_instance = 0;


/*
 * getInstance
 *
 * Return the registered Instance
 */
TransferServiceFactory& TransferServiceFactory::instance() /*throw (LogicError)*/ {
    if(0 == s_instance){
        throw glite::data::agents::LogicError("no TransferServiceFactory configured");
    }
    return *s_instance;
}

/*
 * TransferServiceFactory
 *
 * Constructor
 */
TransferServiceFactory::TransferServiceFactory(){
    // Register the instance
    if(0 == s_instance){
        s_instance = this;
    }
}
                                                                                                                          
/*
 * ~TransferServiceFactory
 *
 * Destructor
 */
TransferServiceFactory::~TransferServiceFactory(){
    if(this != s_instance){
        s_instance = 0;
    }
}
                                                                                                                         
/**
 * TransferService::TransferRequest
 * Constructor
 */
TransferService::TransferRequest::TransferRequest() : 
    streams(0),
    tcpbs(0),
    blocksize(0),
    overwrite_flag(false),
    // new in 3.1.0
    http_timeout(-1),
    log_level(Channel::LL_UNDEF),
    urlcopy_put_timeout(-1),
    urlcopy_putdone_timeout(-1),
    urlcopy_get_timeout(-1),
    urlcopy_getdone_timeout(-1),
    urlcopy_tx_timeout(-1),
    urlcopy_tx_markers_timeout(-1),
    srmcopy_direction(Channel::SCD_UNDEF),
    srmcopy_timeout(-1),
    srmcopy_refresh_timeout(-1),
    // new in 3.2.0
    enable_dir_check(true),
    urlcopy_first_tx_marker_timeout(-1){
}
