/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: UrlCopyTransferFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: UrlCopyTransferFactory.h,v 1.14 2009/12/11 11:27:04 molnarzs Exp $ 
 *  Release: $Name:  $
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_TS_URLCOPY_URLCOPYTRANSFERFACTORY_H_
#define GLITE_DATA_TRANSFER_AGENTS_TS_URLCOPY_URLCOPYTRANSFERFACTORY_H_

#include "glite/data/config/service/ComponentConfiguration.hxx"

#include "glite/data/transfer/agent/ts/TransferServiceFactory.h"
#include "glite/data/transfer/agent/ts/TransferException.h"
#include "glite/data/transfer/agent/ts/TransferService.h"
#include "SrmCopyTransferService.h"
#include "Properties.h"

#include <glite/data/config/service/Logger.hxx>

namespace glite   {

namespace config {
    class ParamReader;
} // End namespace config

namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {

class TestUrlCopyTransferFactory;

/**
 * Implements TransferServiceFactory for the Stork Proixies.
 * Note: Maybe it will be cleaner if there was a different Factory for StorkExec 
 * and StorkAPI, but I don't want to flood with the number of classes
 * @factory
 */
class UrlCopyTransferFactory : public TransferServiceFactory , 
                              public glite::config::ComponentConfiguration
{
    // Access point for whitebox tests
    friend class TestUrlCopyTransferFactory;
public:
    /**
     * Constructor
     */
    UrlCopyTransferFactory();
        
    /**
     * Destructor
     */
    virtual ~UrlCopyTransferFactory();

    // =========================================================================
    // Transfer Service Method
    // =========================================================================

    /**
     * Create a TransferService
     * @param channel_name [IN] the name of the channel the TransferService is
     * responsible for
     * @return a new instance of a TransferService
     * @throws TransferException in case of error
     */
    virtual TransferService * create(const std::string& channel_name) /*throw (TransferException)*/;

    // =========================================================================
    // ComponentConfiguration Methods
    // =========================================================================

    /**
     * Initialize the Component
     */
    virtual int init(const Params& params);
                                                                                                                           
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start(){return 0;}
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}
                                                                                                                           
    /**
     * Finailize the Component
     */
    virtual int fini(){return 0;}

private:
    /** Read the syslog configuration parameters and set up logger accordingly */
    void _setupSysLog(config::ParamReader& reader);

    /** Write the configuration parameters into the log */
    void _logConfigValues();

    glite::config::Logger m_logger;
    Properties m_properties;
};

} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_TS_URLCOPY_URLCOPYTRANSFERFACTORY_H_
