/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Heartbeat Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Heartbeat.h,v 1.2 2006/08/14 08:38:53 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_HEARTBEAT_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_HEARTBEAT_H_

#include "glite/data/transfer/agent/action/vo/VOAction.h"
#include "glite/data/transfer/agent/action/ActionFactoryMethod.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace vo      {

/**
 * The purpose of that Action is to cancel Pending File Requests
 */
class Heartbeat : public VOAction {
    /*
     * Declare Action Factory Method
     */
    DECLARE_FACTORY_METHOD( Heartbeat , "glite:VOHeartbeat");

public:
    /**
     * Constructor
     */
    Heartbeat();

    /**
     * Destructor
     */
    virtual ~Heartbeat();

    /**
     * Execute The Action
     */
    virtual void execute(const std::string& params = "") /*throw (ExecuteException, LogicError)*/;

protected:
    
    /**
     * This code should never be called.
     */
    virtual void exec(const std::string& params) /*throw (LogicError)*/{
        throw agents::LogicError("Not Implemented");
    }
};

} // End namespace vo
} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_HEARTBEAT_H_
