/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ActiveTxCache Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ActiveTxCacheTestCase.cpp,v 1.7 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/transfer/agent/action/channel/ActiveTxCache.h"
#include "DaoHelper.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/dao/channel/FileDAO.h"
#include "glite/data/transfer/agent/dao/channel/JobDAO.h"
#include "glite/data/transfer/agent/dao/channel/TransferDAO.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"
#include "glite/data/agents/AgentExceptions.h"

#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

#include <sstream>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::action::channel;

using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;

using boost::scoped_ptr;
using boost::shared_ptr;

/**
 * CPPUnit TestCase
 */
class ActiveTxCacheTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( ActiveTxCacheTestCase);
        CPPUNIT_TEST( testGet );
        CPPUNIT_TEST( testGetJob );
        CPPUNIT_TEST( testGetFile );
        CPPUNIT_TEST( testGetRequest );
        CPPUNIT_TEST( testGetTransfers );
        CPPUNIT_TEST( testGetRequests );
        CPPUNIT_TEST( testCount );
        CPPUNIT_TEST( testCount_VO );
        CPPUNIT_TEST( testCountRequests );
        CPPUNIT_TEST( testCountRequests_VO );
        CPPUNIT_TEST( testCountPerSourceSE );
        CPPUNIT_TEST( testCountPerDestSE );
        CPPUNIT_TEST( testGetTransfersByState );
        CPPUNIT_TEST( countTransfersByState );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the Get Method
     */
    void testGet();
    /**
     * Test the GetFile Method
     */
    void testGetFile();
    /**
     * Test the GetJob Method
     */
    void testGetJob();
    /**
     * Test the GetRequest Method
     */
    void testGetRequest();
    /**
     * Test the Get Method
     */
    void testGetTransfers();
    /**
     * Test the Get Requests Method
     */
    void testGetRequests();
    /**
     * Test the Count method
     */
    void testCount();
    /**
     * Test the Count Method for a VO
     */
    void testCount_VO();
    /**
     * Test the CountRequests Method
     */
    void testCountRequests();
    /**
     * Test the CountRequests Method for a VO
     */
    void testCountRequests_VO();
    /**
     * Test the Count x Source SE method
     */
    void testCountPerSourceSE();
    /**
     * Test the Count x Dest SE Method
     */
    void testCountPerDestSE();
    
    /**
     * Test the getTransfersByState method
     */
    void testGetTransfersByState();
    
    /**
     * Test the countTransfersByState method
     */
    void countTransfersByState();
    
private:

    ActiveTxCache m_cache;
    ActiveTxCache m_cache_2; // use cache 2 to test only new features
    ActiveTxCache m_cache_3; // use cache 3 to test only testGetTransfersByState method
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ActiveTxCacheTestCase, "ActiveTxCacheTestCase" );

namespace {
const char * const JOB1_ID             = "70000000-0000-0000-0000-000000000001";
const char * const JOB2_ID             = "70000000-0000-0000-0000-000000000002";
const char * const JOB3_ID             = "70000000-0000-0000-0000-000000000003";
const char * const JOB4_ID             = "70000000-0000-0000-0000-000000000004";
const char * const JOB5_ID             = "70000000-0000-0000-0000-000000000005";
const char * const JOB6_ID             = "70000000-0000-0000-0000-000000000006";
const char * const JOB7_ID             = "70000000-0000-0000-0000-000000000007";
const char * const JOB8_ID             = "70000000-0000-0000-0000-000000000008";
const char * const JOB9_ID             = "70000000-0000-0000-0000-000000000009";

const char * const JOB10_ID            = "70000000-0000-0000-0000-000000000010";
const char * const JOB20_ID            = "70000000-0000-0000-0000-000000000020";
const char * const JOB30_ID            = "70000000-0000-0000-0000-000000000030";

const char * const FILE1_ID            = "70000001";
const char * const FILE2_ID            = "70000002";
const char * const FILE3_ID            = "70000003";
const char * const FILE4_ID            = "70000004";
const char * const FILE5_ID            = "70000005";
const char * const FILE6_ID            = "70000006";
const char * const FILE7_ID            = "70000007";
const char * const FILE8_ID            = "70000008";
const char * const FILE9_ID            = "70000009";
const char * const FILE10_ID           = "70000010";
const char * const FILE11_ID           = "70000011";
const char * const FILE12_ID           = "70000012";
const char * const FILE13_ID           = "70000013";
const char * const FILE14_ID           = "70000014";
const char * const FILE15_ID           = "70000015";
const char * const FILE16_ID           = "70000016";
const char * const FILE17_ID           = "70000017";
const char * const FILE18_ID           = "70000018";
const char * const FILE19_ID           = "70000019";
const char * const FILE20_ID           = "70000020";

const char * const FILE111_ID          = "70000111";
const char * const FILE112_ID          = "70000112";
const char * const FILE113_ID          = "70000113";
const char * const FILE121_ID          = "70000121";
const char * const FILE122_ID          = "70000122";
const char * const FILE123_ID          = "70000123";
const char * const FILE131_ID          = "70000131";
const char * const FILE132_ID          = "70000132";
const char * const FILE133_ID          = "70000133";

const char * const REQ1_ID             = "req_7_1";
const char * const REQ2_ID             = "req_7_2";
const char * const REQ3_ID             = "req_7_3";
const char * const REQ4_ID             = "req_7_4";
const char * const REQ5_ID             = "req_7_5";
const char * const REQ6_ID             = "req_7_6";
const char * const REQ7_ID             = "req_7_7";
const char * const REQ8_ID             = "req_7_8";
const char * const REQ9_ID             = "req_7_9";
const char * const REQ10_ID            = "req_7_10";
const char * const REQ11_ID            = "req_7_11";
const char * const REQ12_ID            = "req_7_12";

const char * const REQ111_ID           = "req_7_111";
const char * const REQ112_ID           = "req_7_112";
const char * const REQ113_ID           = "req_7_113";
const char * const REQ121_ID           = "req_7_121";
const char * const REQ122_ID           = "req_7_122";
const char * const REQ123_ID           = "req_7_123";
const char * const REQ131_ID           = "req_7_131";
const char * const REQ132_ID           = "req_7_132";
const char * const REQ133_ID           = "req_7_133";

const char * const VO1                 = "VO_1";
const char * const VO2                 = "VO_2";
const char * const CHANNEL_NAME        = "ACTIVETXCACHE_channel_1";
}

/*
 * setUp
 *
 * set-up test case
 */
void ActiveTxCacheTestCase::setUp(){

    // Get DAOs
    
    // Create FileDAO instance
    scoped_ptr<dao::channel::FileDAO> fileDAO(dao::channel::DAOFactory::instance().createFileDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
    
    // Create JobDAO instance
    scoped_ptr<dao::channel::JobDAO> jobDAO(dao::channel::DAOFactory::instance().createJobDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);
    
    // Create TransferDAO instance
    scoped_ptr<dao::channel::TransferDAO> transferDAO(dao::channel::DAOFactory::instance().createTransferDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null TransferDAO", transferDAO.get() != 0);
    
    // Insert Valus into the Cache
    
    // Req 7_1:0
    shared_ptr<Job> job(jobDAO->get(JOB1_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Job instance (JOB1)", job.get() != 0);
    shared_ptr<File> file(fileDAO->get(FILE1_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE1)", file.get() != 0);
    shared_ptr<Transfer> tx(transferDAO->get(FILE1_ID,REQ1_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req1:0)", tx.get() != 0);
    m_cache.insert(tx,file,job);
    // Req 7_1:1
    file.reset(fileDAO->get(FILE2_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE2)", file.get() != 0);
    tx.reset(transferDAO->get(FILE2_ID,REQ1_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req1:1)", tx.get() != 0);
    m_cache.insert(tx,file,job);
    // Req 7_2:0
    job.reset(jobDAO->get(JOB2_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Job instance (JOB1)", job.get() != 0);
    file.reset(fileDAO->get(FILE3_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE3)", file.get() != 0);
    tx.reset(transferDAO->get(FILE3_ID,REQ2_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req2:0)", tx.get() != 0);
    m_cache.insert(tx,file,job);
    // Req 7_2:1
    file.reset(fileDAO->get(FILE4_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE4)", file.get() != 0);
    tx.reset(transferDAO->get(FILE4_ID,REQ2_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req2:1)", tx.get() != 0);
    m_cache.insert(tx,file,job);
    // Req 7_3:0
    job.reset(jobDAO->get(JOB3_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Job instance (JOB3)", job.get() != 0);
    file.reset(fileDAO->get(FILE5_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE5)", file.get() != 0);
    tx.reset(transferDAO->get(FILE5_ID,REQ3_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req3:0)", tx.get() != 0);
    m_cache.insert(tx,file,job);
    // Req 7_4:0
    file.reset(fileDAO->get(FILE6_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE6)", file.get() != 0);
    tx.reset(transferDAO->get(FILE6_ID,REQ4_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req4:0)", tx.get() != 0);
    m_cache.insert(tx,file,job);
    // Req 7_5:0
    job.reset(jobDAO->get(JOB4_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Job instance (JOB4)", job.get() != 0);
    file.reset(fileDAO->get(FILE7_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE7)", file.get() != 0);
    tx.reset(transferDAO->get(FILE7_ID,REQ5_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req5:0)", tx.get() != 0);
    m_cache.insert(tx,file,job);
    // Req 7_5:1
    file.reset(fileDAO->get(FILE8_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE8)", file.get() != 0);
    tx.reset(transferDAO->get(FILE8_ID,REQ5_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req5:1)", tx.get() != 0);
    m_cache.insert(tx,file,job);
    // Req 7_6:0
    job.reset(jobDAO->get(JOB5_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Job instance (JOB5)", job.get() != 0);
    file.reset(fileDAO->get(FILE9_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE9)", file.get() != 0);
    tx.reset(transferDAO->get(FILE9_ID,REQ6_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req6:0)", tx.get() != 0);
    m_cache.insert(tx,file,job);
    // Req 7_6:1
    file.reset(fileDAO->get(FILE10_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE10)", file.get() != 0);
    tx.reset(transferDAO->get(FILE10_ID,REQ6_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req6:1)", tx.get() != 0);
    m_cache.insert(tx,file,job);
    // Req 7_7:0
    job.reset(jobDAO->get(JOB6_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Job instance (JOB6)", job.get() != 0);
    file.reset(fileDAO->get(FILE11_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE11)", file.get() != 0);
    tx.reset(transferDAO->get(FILE11_ID,REQ7_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req7:0)", tx.get() != 0);
    m_cache.insert(tx,file,job);
    // Req 7_7:1
    file.reset(fileDAO->get(FILE12_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE12)", file.get() != 0);
    tx.reset(transferDAO->get(FILE12_ID,REQ7_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req7:1)", tx.get() != 0);
    m_cache.insert(tx,file,job);
    // Req 7_7:2
    file.reset(fileDAO->get(FILE13_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE13)", file.get() != 0);
    tx.reset(transferDAO->get(FILE13_ID,REQ7_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req3:2)", tx.get() != 0);
    m_cache.insert(tx,file,job);
    
    // Fill Cache2
    // ---------------------------------------------------------------------

    // Req 7_8:0
    job.reset(jobDAO->get(JOB7_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Job instance (JOB7)", job.get() != 0);
    file.reset(fileDAO->get(FILE14_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE14)", file.get() != 0);
    tx.reset(transferDAO->get(FILE14_ID,REQ8_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req8:0)", tx.get() != 0);
    m_cache_2.insert(tx,file,job);
    // Req 7_9:0
    file.reset(fileDAO->get(FILE15_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE15)", file.get() != 0);
    tx.reset(transferDAO->get(FILE15_ID,REQ9_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req9:0)", tx.get() != 0);
    m_cache_2.insert(tx,file,job);
    // Req 7_10:0
    job.reset(jobDAO->get(JOB8_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Job instance (JOB8)", job.get() != 0);
    file.reset(fileDAO->get(FILE16_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE16)", file.get() != 0);
    tx.reset(transferDAO->get(FILE16_ID,REQ10_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req10:0)", tx.get() != 0);
    m_cache_2.insert(tx,file,job);
    // Req 7_10:1
    file.reset(fileDAO->get(FILE17_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE17)", file.get() != 0);
    tx.reset(transferDAO->get(FILE17_ID,REQ10_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req10:1)", tx.get() != 0);
    m_cache_2.insert(tx,file,job);
    // Req 7_11:0
    job.reset(jobDAO->get(JOB9_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Job instance (JOB9)", job.get() != 0);
    file.reset(fileDAO->get(FILE18_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE18)", file.get() != 0);
    tx.reset(transferDAO->get(FILE18_ID,REQ11_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req11:0)", tx.get() != 0);
    m_cache_2.insert(tx,file,job);
    // Req 7_11:1
    file.reset(fileDAO->get(FILE19_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE19)", file.get() != 0);
    tx.reset(transferDAO->get(FILE19_ID,REQ11_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req11:1)", tx.get() != 0);
    m_cache_2.insert(tx,file,job);
    // Req 7_12:0
    file.reset(fileDAO->get(FILE20_ID));
    CPPUNIT_ASSERT_MESSAGE("Null File instance (FILE20)", file.get() != 0);
    tx.reset(transferDAO->get(FILE20_ID,REQ12_ID));
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (Req12:0)", tx.get() != 0);
    m_cache_2.insert(tx,file,job);
    
    // Fill Cache 3
    //--------------------------------------------------------------------------
#define GET_JOB(j) \
    job.reset(jobDAO->get(j##_ID));\
    CPPUNIT_ASSERT_MESSAGE("Null Job instance (" #j ")",job.get() != 0);
#define GET_FILE(f) \
    file.reset(fileDAO->get(f##_ID));\
    CPPUNIT_ASSERT_MESSAGE("Null File instance (" #f ")", file.get() != 0);
#define GET_TX(f,r) \
    tx.reset(transferDAO->get(f##_ID,r##_ID));\
    CPPUNIT_ASSERT_MESSAGE("Null Tx instance (" #r ":0)", tx.get() != 0);
    
    // Job10
    GET_JOB(JOB10);
    // File111
    GET_FILE(FILE111);
    GET_TX(FILE111,REQ111);
    m_cache_3.insert(tx,file,job);
    // File112
    GET_FILE(FILE112);
    GET_TX(FILE112,REQ112);
    m_cache_3.insert(tx,file,job);
    // File113
    GET_FILE(FILE113);
    GET_TX(FILE113,REQ113);
    m_cache_3.insert(tx,file,job);

    // Job20
    GET_JOB(JOB20);
    // File111
    GET_FILE(FILE121);
    GET_TX(FILE121,REQ121);
    m_cache_3.insert(tx,file,job);
    // File112
    GET_FILE(FILE122);
    GET_TX(FILE122,REQ122);
    m_cache_3.insert(tx,file,job);
    // File113
    GET_FILE(FILE123);
    GET_TX(FILE123,REQ123);
    m_cache_3.insert(tx,file,job);
    
    // Job30
    GET_JOB(JOB30);
    // File131
    GET_FILE(FILE131);
    GET_TX(FILE131,REQ131);
    m_cache_3.insert(tx,file,job);
    // File132
    GET_FILE(FILE132);
    GET_TX(FILE132,REQ132);
    m_cache_3.insert(tx,file,job);
    // File133
    GET_FILE(FILE133);
    GET_TX(FILE133,REQ133);
    m_cache_3.insert(tx,file,job);
    
}

/*
 * Test testGet Method
 */
void ActiveTxCacheTestCase::testGet(){

    // Get The Transfers from the cache
    
    // Req_1:0
    ActiveTxCache::TransferEntry entry = m_cache.get(FILE1_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Tx   (1)", entry.tx.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned File (1)", entry.file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Job  (1)", entry.job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Req  ID Tx (1)", entry.tx->requestId  == REQ1_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Tx   ID Tx (1)", entry.tx->transferId == 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID Tx (1)", entry.tx->fileId     == FILE1_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID Tx (1)", entry.tx->jobId      == JOB1_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID F  (1)", entry.file->id       == FILE1_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID J  (1)", entry.job->id        == JOB1_ID);
    
    // Req_1:1
    entry = m_cache.get(FILE2_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Tx   (2)", entry.tx.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned File (2)", entry.file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Job  (2)", entry.job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Req  ID Tx (2)", entry.tx->requestId  == REQ1_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Tx   ID Tx (2)", entry.tx->transferId == 1);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID Tx (2)", entry.tx->fileId     == FILE2_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID Tx (2)", entry.tx->jobId      == JOB1_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID F  (2)", entry.file->id       == FILE2_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID J  (2)", entry.job->id        == JOB1_ID);

    // Req_2:0        
    entry = m_cache.get(FILE3_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Tx   (3)", entry.tx.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned File (3)", entry.file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Job  (3)", entry.job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Req  ID Tx (3)", entry.tx->requestId  == REQ2_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Tx   ID Tx (3)", entry.tx->transferId == 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID Tx (3)", entry.tx->fileId     == FILE3_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID Tx (3)", entry.tx->jobId      == JOB2_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID F  (3)", entry.file->id       == FILE3_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID J  (3)", entry.job->id        == JOB2_ID);
    
    // Req_2:1
    entry = m_cache.get(FILE4_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Tx   (4)", entry.tx.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned File (4)", entry.file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Job  (4)", entry.job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Req  ID Tx (4)", entry.tx->requestId  == REQ2_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Tx   ID Tx (4)", entry.tx->transferId == 1);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID Tx (4)", entry.tx->fileId     == FILE4_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID Tx (4)", entry.tx->jobId      == JOB2_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID F  (4)", entry.file->id       == FILE4_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID J  (4)", entry.job->id        == JOB2_ID);

    // Req_3:0        
    entry = m_cache.get(FILE5_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Tx   (5)", entry.tx.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned File (5)", entry.file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Job  (5)", entry.job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Req  ID Tx (5)", entry.tx->requestId  == REQ3_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Tx   ID Tx (5)", entry.tx->transferId == 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID Tx (5)", entry.tx->fileId     == FILE5_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID Tx (5)", entry.tx->jobId      == JOB3_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID F  (5)", entry.file->id       == FILE5_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID J  (5)", entry.job->id        == JOB3_ID);
    
    // Req_4:0
    entry = m_cache.get(FILE6_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Tx   (6)", entry.tx.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned File (6)", entry.file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Job  (6)", entry.job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Req  ID Tx (6)", entry.tx->requestId  == REQ4_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Tx   ID Tx (6)", entry.tx->transferId == 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID Tx (6)", entry.tx->fileId     == FILE6_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID Tx (6)", entry.tx->jobId      == JOB3_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID F  (6)", entry.file->id       == FILE6_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID J  (6)", entry.job->id        == JOB3_ID);
    
    // Req_5:0        
    entry = m_cache.get(FILE7_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Tx   (7)", entry.tx.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned File (7)", entry.file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Job  (7)", entry.job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Req  ID Tx (7)", entry.tx->requestId  == REQ5_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Tx   ID Tx (7)", entry.tx->transferId == 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID Tx (7)", entry.tx->fileId     == FILE7_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID Tx (7)", entry.tx->jobId      == JOB4_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID F  (7)", entry.file->id       == FILE7_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID J  (7)", entry.job->id        == JOB4_ID);
    
    // Req_5:1
    entry = m_cache.get(FILE8_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Tx   (8)", entry.tx.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned File (8)", entry.file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Job  (8)", entry.job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Req  ID Tx (8)", entry.tx->requestId  == REQ5_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Tx   ID Tx (8)", entry.tx->transferId == 1);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID Tx (8)", entry.tx->fileId     == FILE8_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID Tx (8)", entry.tx->jobId      == JOB4_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID F  (8)", entry.file->id       == FILE8_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID J  (8)", entry.job->id        == JOB4_ID);

    // Req_6:0        
    entry = m_cache.get(FILE9_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Tx   (9)", entry.tx.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned File (9)", entry.file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Job  (9)", entry.job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Req  ID Tx (9)", entry.tx->requestId  == REQ6_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Tx   ID Tx (9)", entry.tx->transferId == 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID Tx (9)", entry.tx->fileId     == FILE9_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID Tx (9)", entry.tx->jobId      == JOB5_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID F  (9)", entry.file->id       == FILE9_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID J  (9)", entry.job->id        == JOB5_ID);
    
    // Req_6:1
    entry = m_cache.get(FILE10_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Tx   (10)", entry.tx.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned File (10)", entry.file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Job  (10)", entry.job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Req  ID Tx (10)", entry.tx->requestId  == REQ6_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Tx   ID Tx (10)", entry.tx->transferId == 1);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID Tx (10)", entry.tx->fileId     == FILE10_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID Tx (10)", entry.tx->jobId      == JOB5_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID F  (10)", entry.file->id       == FILE10_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID J  (10)", entry.job->id        == JOB5_ID);

    // Req_7:0
    entry = m_cache.get(FILE11_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Tx   (11)", entry.tx.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned File (11)", entry.file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Job  (11)", entry.job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Req  ID Tx (11)", entry.tx->requestId  == REQ7_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Tx   ID Tx (11)", entry.tx->transferId == 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID Tx (11)", entry.tx->fileId     == FILE11_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID Tx (11)", entry.tx->jobId      == JOB6_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID F  (11)", entry.file->id       == FILE11_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID J  (11)", entry.job->id        == JOB6_ID);
    
    // Req_7:1
    entry = m_cache.get(FILE12_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Tx   (12)", entry.tx.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned File (12)", entry.file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Job  (12)", entry.job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Req  ID Tx (12)", entry.tx->requestId  == REQ7_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Tx   ID Tx (12)", entry.tx->transferId == 1);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID Tx (12)", entry.tx->fileId     == FILE12_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID Tx (12)", entry.tx->jobId      == JOB6_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID F  (12)", entry.file->id       == FILE12_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID J  (12)", entry.job->id        == JOB6_ID);

    // Req_7:2
    entry = m_cache.get(FILE13_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Tx   (13)", entry.tx.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned File (13)", entry.file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned Job  (13)", entry.job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Req  ID Tx (13)", entry.tx->requestId  == REQ7_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Tx   ID Tx (13)", entry.tx->transferId == 2);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID Tx (13)", entry.tx->fileId     == FILE13_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID Tx (13)", entry.tx->jobId      == JOB6_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID F  (13)", entry.file->id       == FILE13_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID J  (13)", entry.job->id        == JOB6_ID);

}

/*
 * Test testGetJob Method
 */
void ActiveTxCacheTestCase::testGetJob(){

    // Get The Transfers from the cache
    
    // Job 1
    ActiveTxCache::JobPtr job = m_cache.getJob(JOB1_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (1)", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID (1)", job->id  == JOB1_ID);
    
    // Job 2
    job = m_cache.getJob(JOB2_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (2)", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID (2)", job->id  == JOB2_ID);

    // Job 3
    job = m_cache.getJob(JOB3_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (3)", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID (3)", job->id  == JOB3_ID);

    // Job 4
    job = m_cache.getJob(JOB4_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (4)", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID (4)", job->id  == JOB4_ID);

    // Job 5
    job = m_cache.getJob(JOB5_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (5)", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID (5)", job->id  == JOB5_ID);

    // Job 6
    job = m_cache.getJob(JOB6_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (6)", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job  ID (6)", job->id  == JOB6_ID);

    try{
        ActiveTxCache::JobPtr job = m_cache.getJob("none");
        CPPUNIT_FAIL("Invalid Job ID");
    } catch(const RuntimeError& exc){
        // OK
    }
}

/*
 * Test testGetFile Method
 */
void ActiveTxCacheTestCase::testGetFile(){

    // Get The Transfers from the cache
    
    // File 1
    ActiveTxCache::FilePtr file = m_cache.getFile(FILE1_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (1)", file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID (1)", file->id  == FILE1_ID);
    
    // File 2
    file = m_cache.getFile(FILE2_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (2)", file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID (2)", file->id  == FILE2_ID);

    // File 3
    file = m_cache.getFile(FILE3_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (3)", file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID (3)", file->id  == FILE3_ID);

    // File 4
    file = m_cache.getFile(FILE4_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (4)", file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID (4)", file->id  == FILE4_ID);

    // File 5
    file = m_cache.getFile(FILE5_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (5)", file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID (5)", file->id  == FILE5_ID);

    // File 6
    file = m_cache.getFile(FILE6_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (6)", file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID (6)", file->id  == FILE6_ID);

    // File 7
    file = m_cache.getFile(FILE7_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (7)", file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID (7)", file->id  == FILE7_ID);

    // File 8
    file = m_cache.getFile(FILE8_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (8)", file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID (8)", file->id  == FILE8_ID);

    // File 9
    file = m_cache.getFile(FILE9_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (9)", file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID (9)", file->id  == FILE9_ID);

    // File 10
    file = m_cache.getFile(FILE10_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (10)", file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID (10)", file->id  == FILE10_ID);

    // File 11
    file = m_cache.getFile(FILE11_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (11)", file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID (11)", file->id  == FILE11_ID);

    // File 12
    file = m_cache.getFile(FILE12_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (12)", file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID (12)", file->id  == FILE12_ID);

    // File 13
    file = m_cache.getFile(FILE13_ID);
    CPPUNIT_ASSERT_MESSAGE("Null Pointer Returned (13)", file.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid File ID (13)", file->id  == FILE13_ID);

    try{
        ActiveTxCache::FilePtr file = m_cache.getFile("none");
        CPPUNIT_FAIL("Invalid FILE ID");
    } catch(const RuntimeError& exc){
        // OK
    }
}

/*
 * Test testGetRequest Method
 */
void ActiveTxCacheTestCase::testGetRequest(){

    // Get The Transfers from the cache
    
    // Req1
    ActiveTxCache::RequestEntry entry = m_cache.getRequest(REQ1_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Request ID (1)", entry.id  == REQ1_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Num of Tx (1)", entry.transfers.size()  == 2);
    
    // Req2
    entry = m_cache.getRequest(REQ2_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Request ID (2)", entry.id  == REQ2_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Num of Tx (1)", entry.transfers.size()  == 2);

    // Req3
    entry = m_cache.getRequest(REQ3_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Request ID (3)", entry.id  == REQ3_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Num of Tx (3)", entry.transfers.size()  == 1);

    // Req4
    entry = m_cache.getRequest(REQ4_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Request ID (4)", entry.id  == REQ4_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Num of Tx (4)", entry.transfers.size()  == 1);

    // Req5
    entry = m_cache.getRequest(REQ5_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Request ID (5)", entry.id  == REQ5_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Num of Tx (5)", entry.transfers.size()  == 2);

    // Req6
    entry = m_cache.getRequest(REQ6_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Request ID (6)", entry.id  == REQ6_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Num of Tx (6)", entry.transfers.size()  == 2);

    // Req7
    entry = m_cache.getRequest(REQ7_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Request ID (7)", entry.id  == REQ7_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Num of Tx (7)", entry.transfers.size()  == 3);

    try{
        ActiveTxCache::RequestEntry entry = m_cache.getRequest("none");
        CPPUNIT_FAIL("Invalid Request ID");
    } catch(const RuntimeError& exc){
        // OK
    }
}

/*
 * Test Get Method
 */
void ActiveTxCacheTestCase::testGetTransfers(){

    std::vector<ActiveTxCache::TransferEntry> entries = m_cache.getTransferEntries();
    CPPUNIT_ASSERT_MESSAGE("Empty", entries.empty() == false);
    CPPUNIT_ASSERT_MESSAGE("Invalid size", entries.size() == 13);

}

/*
 * Test GetRequests Method
 */
void ActiveTxCacheTestCase::testGetRequests(){

    ActiveTxCache::RequestEntryArray entries = m_cache.getRequests();
    CPPUNIT_ASSERT_MESSAGE("Empty", entries.empty() == false);
    CPPUNIT_ASSERT_MESSAGE("Invalid size", entries.size() == 7);

}

/*
 * Test testCount Method
 */
void ActiveTxCacheTestCase::testCount(){

    unsigned int c = m_cache.count();
    CPPUNIT_ASSERT_MESSAGE("Invalid count", c == 13);

}

/*
 * Test testCount Method For a Given VO
 */
void ActiveTxCacheTestCase::testCount_VO(){

    unsigned int c = m_cache.count(VO1);
    CPPUNIT_ASSERT_MESSAGE("Invalid count (VO1)", c == 6);
    
    c = m_cache.count(VO2);
    CPPUNIT_ASSERT_MESSAGE("Invalid count (VO2)", c == 7);

}

/*
 * Test the testCountRequests Method
 */
void ActiveTxCacheTestCase::testCountRequests(){

    unsigned int c = m_cache.countRequests();
    CPPUNIT_ASSERT_MESSAGE("Invalid count", c == 7);

}
/*
 * testCountRequests_VO Method
 */
void ActiveTxCacheTestCase::testCountRequests_VO(){

    unsigned int c = m_cache.countRequests(VO1);
    CPPUNIT_ASSERT_MESSAGE("Invalid count (VO1)", c == 4);
    
    c = m_cache.countRequests(VO2);
    CPPUNIT_ASSERT_MESSAGE("Invalid count (VO2)", c == 3);

}

/*
 * testCountPerSourceSE Method
 */
void ActiveTxCacheTestCase::testCountPerSourceSE(){

    unsigned int c = m_cache_2.countPerSourceSE("host1.cern.ch");
    CPPUNIT_ASSERT_MESSAGE("Invalid count (HOST1)", c == 4);
    
    c = m_cache_2.countPerSourceSE("host2.cern.ch");
    CPPUNIT_ASSERT_MESSAGE("Invalid count (HOST2)", c == 3);

}

/*
 * testCountPerDestSE Method
 */
void ActiveTxCacheTestCase::testCountPerDestSE(){

    unsigned int c = m_cache_2.countPerDestSE("host1.ral.ac.uk");
    CPPUNIT_ASSERT_MESSAGE("Invalid count (HOST1)", c == 5);
    
    c = m_cache_2.countPerDestSE("host2.ral.ac.uk");
    CPPUNIT_ASSERT_MESSAGE("Invalid count (HOST2)", c == 2);

}

#define CHECK_VALUE(val,exp)\
    if (val != exp){\
        std::stringstream ss;\
        ss << "Wrong value for " << #val << ": got " << val << " expected " << exp;\
        CPPUNIT_FAIL(ss.str().c_str());\
    }
    
/**
 * Test the getTransfersByState method
 */
void ActiveTxCacheTestCase::testGetTransfersByState(){
    ActiveTxCache::TransferEntryArray txs;
    
    txs = m_cache_3.getTransfersByState(Transfer::S_PROCESSING);
    CHECK_VALUE(txs.size(),3);
    
    txs = m_cache_3.getTransfersByState((Transfer::State)(Transfer::S_RECEIVED | Transfer::S_PREPARING));
    CHECK_VALUE(txs.size(),2);
}

/**
 * Test the countTransfersByState method
 */
void ActiveTxCacheTestCase::countTransfersByState(){
    
    unsigned int processing_tx = m_cache_3.countTransfersByState(Transfer::S_PROCESSING);
    CHECK_VALUE(processing_tx,3);
    
    unsigned int received_or_preparing_tx = m_cache_3.countTransfersByState((Transfer::State)(Transfer::S_RECEIVED | Transfer::S_PREPARING));
    CHECK_VALUE(received_or_preparing_tx,2);
}
