/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: SrmaterRetry Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: SmarterRetryTestCase.cpp,v 1.7 2008/04/11 11:02:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "PythonRetry.h"
#include "DaoHelper.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/dao/vo/FileDAO.h"
#include "glite/data/transfer/agent/dao/vo/JobDAO.h"
#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

#include "dao_test_utils.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;
using namespace glite::data::transfer::agent::model;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class SmarterRetryTestCase : public CppUnit::TestCase{
public:
    /** 
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( SmarterRetryTestCase);
        CPPUNIT_TEST( testExecute );
        CPPUNIT_TEST( testExecute_101 );
        CPPUNIT_TEST( testExecute_102 );
        CPPUNIT_TEST( testExecute_103 );
        CPPUNIT_TEST( testExecute_104 );
        CPPUNIT_TEST( testExecute_105 );
        CPPUNIT_TEST( testExecute_106 );
        CPPUNIT_TEST( testExecute_107 );
        CPPUNIT_TEST( testExecute_108 );
        CPPUNIT_TEST( testExecute_109 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Execute Method
     */
    void testExecute();
    
    void testExecute_101();
    void testExecute_102();
    void testExecute_103();
    void testExecute_104();
    void testExecute_105();
    void testExecute_106();
    void testExecute_107();
    void testExecute_108();
    void testExecute_109();

private:

    scoped_ptr<dao::vo::JobDAO> jobDAO;
    scoped_ptr<dao::vo::FileDAO> fileDAO;

};

/*
 * Declare Constants. This unit test foresee at beginning
 *
 * - 1 ACTIVE Job with:                   => 
 *        job_id      = 8-...-01
 *        files       = 80000001 (WAITING)      PENDING
 *                      80000002 (WAITING)              (delay not elapsed)
 *                      80000003 (WAITING)      PENDING
 *                      80000004 (WAITING)              (delay not elapsed)
 *                      80000005 (WAITING)      PENDING
 *                      80000006 (WAITING)              (delay not elapsed)
 *                      80000007 (WAITING)      FAILED  (source doesn't exist)
 *                      80000008 (WAITING)      PENDING
 *                      80000009 (WAITING)              (delay not elapsed)
 *                      80000010 (WAITING)      PENDING
 *                      80000011 (WAITING)              (delay not elapsed)
 *                      80000012 (WAITING)      PENDING
 *                      80000013 (WAITING)              (delay not elapsed)
 *                      80000014 (WAITING)      HOLD    (security error)
 *                      80000015 (WAITING)      HOLD    (configuration error)
 *                      80000016 (WAITING)      PENDING (with multiple failures)
 *                      80000017 (WAITING)      PENDING (with multiple failures)
 *                      80000018 (WAITING)      HOLD    (too many failures)
 *                      80000019 (WAITING)      PENDING
 *                      80000020 (WAITING)              (delay not elapsed)
 *                      80000021 (WAITING)      PENDING (with multiple failures)
 *                      80000022 (WAITING)      PENDING 
 *
 * - 1 ACTIVE Job with:                   => 
 *        job_id      = 8-...-02
 *        files       = 80000101 (WAITING)      HOLD    (security error)
 *                      80000102 (WAITING)      FAILED  (source file does not exist)
 *                      80000103 (WAITING)      FAILED  (file exists)
 *                      80000104 (WAITING)      HOLD    (too many failures)
 *                      80000105 (WAITING)      PENDING (source request timeout  
 *                          during transfer preparation; file is being staged?)
 *                      80000106 (WAITING)      PENDING (same as 105, but no transfer
 *                          reason is present. Check that file reason is used)
 *                      80000107 (WAITING)      PENDING (same as 105, but no transfer
 *                          is present. Check that file reason is used)
 *                      80000108 (WAITING)      FAILED  (unexpected reason class)
 *                      80000109 (WAITING)      FAILED  (deprecated reason class)
 *                      
 */
namespace {
const char * const JOB1_ID   = "80000000-0000-0000-0000-000000000001";
const char * const FILE1_ID  = "80000001";
const char * const FILE2_ID  = "80000002";
const char * const FILE3_ID  = "80000003";
const char * const FILE4_ID  = "80000004";
const char * const FILE5_ID  = "80000005";
const char * const FILE6_ID  = "80000006";
const char * const FILE7_ID  = "80000007";
const char * const FILE8_ID  = "80000008";
const char * const FILE9_ID  = "80000009";
const char * const FILE10_ID = "80000010";
const char * const FILE11_ID = "80000011";
const char * const FILE12_ID = "80000012";
const char * const FILE13_ID = "80000013";
const char * const FILE14_ID = "80000014";
const char * const FILE15_ID = "80000015";
const char * const FILE16_ID = "80000016";
const char * const FILE17_ID = "80000017";
const char * const FILE18_ID = "80000018";
const char * const FILE19_ID = "80000019";
const char * const FILE20_ID = "80000020";
const char * const FILE21_ID = "80000021";
const char * const FILE22_ID = "80000022";
const char * const VO1_NAME  = "SMARTERRETRY_VO_1";

const char * const JOB2_ID   = "80000000-0000-0000-0000-000000000002";
const char * const FILE101_ID  = "80000101";
const char * const FILE102_ID  = "80000102";
const char * const FILE103_ID  = "80000103";
const char * const FILE104_ID  = "80000104";
const char * const FILE105_ID  = "80000105";
const char * const FILE106_ID  = "80000106";
const char * const FILE107_ID  = "80000107";
const char * const FILE108_ID  = "80000108";
const char * const FILE109_ID  = "80000109";
}

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( SmarterRetryTestCase,  "SmarterRetryTestCase" );

/*
 * setUp
 *
 * set-up test case
 */
void SmarterRetryTestCase::setUp(){
    static bool run_action = true;
    
    if (run_action){
        run_action = false;
        scoped_ptr<PythonRetry> action;
        // Create Action instance
        std::auto_ptr<Action> a(ActionFactory::instance().create(PythonRetry::factory()->type()));
        CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<PythonRetry*>(a.get()) != 0);

        // Store Action        
        action.reset(dynamic_cast<PythonRetry*>(a.release()));
        CPPUNIT_ASSERT_MESSAGE("Null Action", action.get() != 0);

        // Set Context
        action->context(VO1_NAME,DaoHelper::ctx());
        
        // Execute Action 
        action->execute();
        
    }
    
    // Create FileDAO instance
    fileDAO.reset(dao::vo::DAOFactory::instance().createFileDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
    
    // Create JobDAO instance
    jobDAO.reset(dao::vo::DAOFactory::instance().createJobDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);
        
}

/*
 * Test Execute Method
 */
void SmarterRetryTestCase::testExecute(){
    // check files
    check_file(fileDAO,FILE1_ID,"FILE1",File::S_PENDING);
    check_file(fileDAO,FILE2_ID,"FILE2",File::S_WAITING);
    check_file(fileDAO,FILE3_ID,"FILE3",File::S_PENDING);
    check_file(fileDAO,FILE4_ID,"FILE4",File::S_WAITING);
    check_file(fileDAO,FILE5_ID,"FILE5",File::S_PENDING);
    check_file(fileDAO,FILE6_ID,"FILE6",File::S_WAITING);
    check_file(fileDAO,FILE7_ID,"FILE7",File::S_FAILED);
    check_file(fileDAO,FILE8_ID,"FILE8",File::S_PENDING);
    check_file(fileDAO,FILE9_ID,"FILE9",File::S_WAITING);
    check_file(fileDAO,FILE10_ID,"FILE10",File::S_PENDING);
    check_file(fileDAO,FILE11_ID,"FILE11",File::S_WAITING);
    check_file(fileDAO,FILE12_ID,"FILE12",File::S_PENDING);
    check_file(fileDAO,FILE13_ID,"FILE13",File::S_WAITING);
    check_file(fileDAO,FILE14_ID,"FILE14",File::S_HOLD);
    check_file(fileDAO,FILE15_ID,"FILE15",File::S_HOLD);
    check_file(fileDAO,FILE16_ID,"FILE16",File::S_PENDING);
    check_file(fileDAO,FILE17_ID,"FILE17",File::S_PENDING);
    check_file(fileDAO,FILE18_ID,"FILE18",File::S_HOLD);
    check_file(fileDAO,FILE19_ID,"FILE19",File::S_PENDING);
    check_file(fileDAO,FILE20_ID,"FILE20",File::S_WAITING);
    check_file(fileDAO,FILE21_ID,"FILE21",File::S_PENDING);
    check_file(fileDAO,FILE22_ID,"FILE22",File::S_PENDING);
    // check job
    check_job(jobDAO,JOB1_ID,"JOB1",Job::S_ACTIVE);
}

void SmarterRetryTestCase::testExecute_101(){
    check_file(fileDAO,FILE101_ID,"FILE101",File::S_HOLD);
}

void SmarterRetryTestCase::testExecute_102(){
    check_file(fileDAO,FILE102_ID,"FILE102",File::S_FAILED);
}

void SmarterRetryTestCase::testExecute_103(){
    check_file(fileDAO,FILE103_ID,"FILE103",File::S_FAILED);
}

void SmarterRetryTestCase::testExecute_104(){
    check_file(fileDAO,FILE104_ID,"FILE104",File::S_HOLD);
}

void SmarterRetryTestCase::testExecute_105(){
    check_file(fileDAO,FILE105_ID,"FILE105",File::S_PENDING);
}

void SmarterRetryTestCase::testExecute_106(){
    check_file(fileDAO,FILE106_ID,"FILE106",File::S_PENDING);
}

void SmarterRetryTestCase::testExecute_107(){
    check_file(fileDAO,FILE107_ID,"FILE107",File::S_PENDING);
}

void SmarterRetryTestCase::testExecute_108(){
    check_file(fileDAO,FILE108_ID,"FILE108",File::S_FAILED);
}

void SmarterRetryTestCase::testExecute_109(){
    check_file(fileDAO,FILE109_ID,"FILE109",File::S_FAILED);
}

