/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: AllocateTransfers Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: AgentsTestCase.cpp,v 1.16 2008/12/04 08:53:30 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "DaoHelper.h"
#include <glite/data/transfer/agent/dao/vo/FileDAO.h>
#include <glite/data/transfer/agent/dao/vo/JobDAO.h>
#include <glite/data/transfer/agent/dao/vo/DAOFactory.h>
#include <glite/data/transfer/agent/dao/channel/DAOFactory.h>

#include "dao_test_utils.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;

using namespace glite::data::agents::dao;

using boost::scoped_ptr;
using boost::shared_ptr;

/**
 * CPPUnit TestCase
 */
class AgentsTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:

    scoped_ptr<dao::vo::FileDAO> fileDAO;
    scoped_ptr<dao::vo::JobDAO>  jobDAO; 

    shared_ptr<File>    file; 
    shared_ptr<Job>     job;

    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( AgentsTestCase);
        CPPUNIT_TEST( test01 );
        CPPUNIT_TEST( test02 );
        CPPUNIT_TEST( test03 );
        CPPUNIT_TEST( test04 );
        CPPUNIT_TEST( test05 );
        CPPUNIT_TEST( test06 );
        CPPUNIT_TEST( test07 );
        CPPUNIT_TEST( test08 );
        CPPUNIT_TEST( test09 );
        CPPUNIT_TEST( test10 );
        CPPUNIT_TEST( test11 );
        CPPUNIT_TEST( test12 );
        CPPUNIT_TEST( test13 );
        CPPUNIT_TEST( test14 );
        CPPUNIT_TEST( test15 );
        CPPUNIT_TEST( test16 );
        CPPUNIT_TEST( test17 );
        CPPUNIT_TEST( test18 );
        CPPUNIT_TEST( test19 );
        CPPUNIT_TEST( test20 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    void test01();
    void test02();
    void test03();
    void test04();
    void test05();
    void test06();
    void test07();
    void test08();
    void test09();
    void test10();
    void test11();
    void test12();
    void test13();
    void test14();
    void test15();
    void test16();
    void test17();
    void test18();
    void test19();
    void test20();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( AgentsTestCase, "AgentsTestCase" );

/*
 * Declare Constants. This unit test foresee at beginning
 * - 1 SUBMITTED Job with:                    => FINISHED
 *        job_id      = 1-...-01
 *        channel     = [NULL]                   AGENTS_channel_1
 *        files       = 10000001 (SUBMITTED)     FINISHED
 *                      10000002 (SUBMITTED)     FINISHED
 *                      10000003 (SUBMITTED)     FINISHED
 *
 * - 1 SUBMITTED Job with:                    => FINISHED
 *        job_id      = 1-...-02
 *        channel     = [NULL]                   AGENTS_channel_1
 *        files       = 10000004 (SUBMITTED)     FINISHED
 *                      10000005 (SUBMITTED)     FINISHED
 *
 * - 1 SUBMITTED Job with:                    => FINISHED
 *        job_id      = 1-...-03
 *        channel     = [NULL]                   AGENTS_channel_1
 *        files       = 10000006 (SUBMITTED)     FINISHED
 *                      10000007 (SUBMITTED)     FINISHED
 *
 * - 1 SUBMITTED Job with:                    => FINISHED_DIRTY
 *        job_id      = 1-...-04
 *        channel     = [NULL]                   AGENTS_channel_1
 *        files       = 10000008 (SUBMITTED)     FINISHED
 *                      10000009 (SUBMITTED)     FAILED
 *
 * - 1 SUBMITTED Job with:                    => CANCELED
 *        job_id      = 1-...-05
 *        channel     = [NULL]                   
 *        files       = 10000010 (SUBMITTED)     CANCELED
 *                      10000011 (SUBMITTED)     CANCELED
 *
 * - 1 SUBMITTED Job with:                    => FAILED (Invalid User DN with myproxy)
 *        job_id      = 1-...-06
 *        channel     = [NULL]                   
 *        files       = 10000012 (SUBMITTED)     FAILED
 *                      10000013 (SUBMITTED)     FAILED
 *
 * - 1 SUBMITTED Job with:                    => FINISHED (With Delegated Credentials)
 *        job_id      = 1-...-07
 *        channel     = [NULL]                   AGENTS_channel_1
 *        files       = 10000014 (SUBMITTED)     FINISHED
 *                      10000015 (SUBMITTED)     FINISHED
 *
 * - 1 SUBMITTED Job with:                    => FAILED (Invalid Deleg Id)
 *        job_id      = 1-...-08
 *        channel     = [NULL]                   
 *        files       = 10000016 (SUBMITTED)     FAILED
 *                      10000017 (SUBMITTED)     FAILED
 *
 * - 1 SUBMITTED Job with:                    => FAILED (Invalid User DN with deleg)
 *        job_id      = 1-...-09
 *        channel     = [NULL]                   
 *        files       = 10000018 (SUBMITTED)     FAILED
 *                      10000019 (SUBMITTED)     FAILED
 *
 * - 1 SUBMITTED Job with:                    => FINISHED
 *        job_id      = 1-...-10
 *        channel     = [NULL]                   AGENTS_channel_2
 *        files       = 10000020 (SUBMITTED)     FINISHED
 *                      10000021 (SUBMITTED)     FINISHED
 *                      10000022 (SUBMITTED)     FINISHED
 *
 * - 1 SUBMITTED Job with:                    => FINISHED (With Delegated Credentials)
 *        job_id      = 1-...-11
 *        channel     = [NULL]                   AGENTS_channel_2
 *        files       = 10000023 (SUBMITTED)     FINISHED
 *                      10000024 (SUBMITTED)     FINISHED
 *
 * - 1 SUBMITTED Job with:                    => FINISHED_DIRTY
 *        job_id      = 1-...-12
 *        channel     = [NULL]                   AGENTS_channel_2
 *        files       = 10000025 (SUBMITTED)     FINISHED
 *                      10000026 (SUBMITTED)     FAILED
 *
 * - 1 SUBMITTED Job with:                    => FINISHED
 *        job_id      = 1-...-13
 *        channel     = [NULL]                   AGENTS_channel_3
 *        files       = 10000027 (SUBMITTED)     FINISHED
 *                      10000028 (SUBMITTED)     FINISHED
 *
 * - 1 SUBMITTED Job with:                    => FINISHED
 *        job_id      = 1-...-14
 *        channel     = [NULL]                   AGENTS_channel_3
 *        files       = 10000029 (SUBMITTED)     FINISHED
 *                      10000030 (SUBMITTED)     FINISHED
 *                      10000031 (SUBMITTED)     FINISHED
 *                      10000032 (SUBMITTED)     FINISHED
 *
 * - 1 SUBMITTED Job with:                    => FINISHED
 *        job_id      = 1-...-15
 *        channel     = [NULL]                   AGENTS_channel_3
 *        files       = 10000033 (SUBMITTED)     FINISHED
 *                      10000034 (SUBMITTED)     FINISHED
 *                      10000035 (SUBMITTED)     FINISHED
 *
 * - 1 SUBMITTED Job with:                    => FINISHED
 *        job_id      = 1-...-16
 *        channel     = [NULL]                   AGENTS_channel_3
 *        files       = 10000036 (SUBMITTED)     FINISHED
 *                      10000037 (SUBMITTED)     FINISHED
 *                      10000038 (SUBMITTED)     FINISHED
 */
namespace {
const char * const JOB1_ID         = "10000000-0000-0000-0000-000000000001";
const char * const JOB2_ID         = "10000000-0000-0000-0000-000000000002";
const char * const JOB3_ID         = "10000000-0000-0000-0000-000000000003";
const char * const JOB4_ID         = "10000000-0000-0000-0000-000000000004";
const char * const JOB5_ID         = "10000000-0000-0000-0000-000000000005";
const char * const JOB6_ID         = "10000000-0000-0000-0000-000000000006";
const char * const JOB7_ID         = "10000000-0000-0000-0000-000000000007";
const char * const JOB8_ID         = "10000000-0000-0000-0000-000000000008";
const char * const JOB9_ID         = "10000000-0000-0000-0000-000000000009";
const char * const JOB10_ID        = "10000000-0000-0000-0000-000000000010";
const char * const JOB11_ID        = "10000000-0000-0000-0000-000000000011";
const char * const JOB12_ID        = "10000000-0000-0000-0000-000000000012";
const char * const JOB13_ID        = "10000000-0000-0000-0000-000000000013";
const char * const JOB14_ID        = "10000000-0000-0000-0000-000000000014";
const char * const JOB15_ID        = "10000000-0000-0000-0000-000000000015";
const char * const JOB16_ID        = "10000000-0000-0000-0000-000000000016";
const char * const FILE1_ID        = "10000001";
const char * const FILE2_ID        = "10000002";
const char * const FILE3_ID        = "10000003";
const char * const FILE4_ID        = "10000004";
const char * const FILE5_ID        = "10000005";
const char * const FILE6_ID        = "10000006";
const char * const FILE7_ID        = "10000007";
const char * const FILE8_ID        = "10000008";
const char * const FILE9_ID        = "10000009";
const char * const FILE10_ID       = "10000010";
const char * const FILE11_ID       = "10000011";
const char * const FILE12_ID       = "10000012";
const char * const FILE13_ID       = "10000013";
const char * const FILE14_ID       = "10000014";
const char * const FILE15_ID       = "10000015";
const char * const FILE16_ID       = "10000016";
const char * const FILE17_ID       = "10000017";
const char * const FILE18_ID       = "10000018";
const char * const FILE19_ID       = "10000019";
const char * const FILE20_ID       = "10000020";
const char * const FILE21_ID       = "10000021";
const char * const FILE22_ID       = "10000022";
const char * const FILE23_ID       = "10000023";
const char * const FILE24_ID       = "10000024";
const char * const FILE25_ID       = "10000025";
const char * const FILE26_ID       = "10000026";
const char * const FILE27_ID       = "10000027";
const char * const FILE28_ID       = "10000028";
const char * const FILE29_ID       = "10000029";
const char * const FILE30_ID       = "10000030";
const char * const FILE31_ID       = "10000031";
const char * const FILE32_ID       = "10000032";
const char * const FILE33_ID       = "10000033";
const char * const FILE34_ID       = "10000034";
const char * const FILE35_ID       = "10000035";
const char * const FILE36_ID       = "10000036";
const char * const FILE37_ID       = "10000037";
const char * const FILE38_ID       = "10000038";
const char * const VO1_NAME        = "AGENTS_VO_1";
const char * const CHANNEL1_NAME   = "AGENTS_channel_1";
const char * const CHANNEL2_NAME   = "AGENTS_channel_2";
const char * const CHANNEL3_NAME   = "AGENTS_channel_3";
const char * const CHANNEL4_NAME   = "AGENTS_channel_4";
}

/*
 * setUp
 *
 * set-up test case
 */
void AgentsTestCase::setUp(){
    static bool s_sleep = true;
    if (s_sleep){
        s_sleep = false;
        // Wait that for a while: agents should finish all the transfers requests
        sleep(240);
    }

    // Create FileDAO instance
    fileDAO.reset(dao::vo::DAOFactory::instance().createFileDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
    
    // Create JobDAO instance
    jobDAO.reset(dao::vo::DAOFactory::instance().createJobDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);

}

//------------------------------------------------------------------------------
// test methods
//------------------------------------------------------------------------------
void AgentsTestCase::test01(){

    // Get File instance (1) and Check it
    file = check_file(fileDAO,FILE1_ID,"FILE1",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE1)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE1)", file->finishTime() != (time_t)-1);
    // Get File instance (2) and Check it
    file = check_file(fileDAO,FILE2_ID,"FILE2",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE2)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE2)", file->finishTime() != (time_t)-1);
    // Get File instance (3) and Check it
    file = check_file(fileDAO,FILE3_ID,"FILE3",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE3)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE3)", file->finishTime() != (time_t)-1);
    // Get Job instance (1) and Check it
    job = check_job(jobDAO,JOB1_ID,"JOB1",Job::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB1)", job->channelName == CHANNEL1_NAME);

}

void AgentsTestCase::test02(){

    // Get File instance (4) and Check it
    file = check_file(fileDAO,FILE4_ID,"FILE4",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE4)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE4)", file->finishTime() != (time_t)-1);
    // Get File instance (5) and Check it
    file = check_file(fileDAO,FILE5_ID,"FILE5",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE5)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE5)", file->finishTime() != (time_t)-1);
    // Get Job instance (2) and Check it
    job = check_job(jobDAO,JOB2_ID,"JOB2",Job::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB2)", job->channelName == CHANNEL1_NAME);

}

void AgentsTestCase::test03(){

    // Get File instance (6) and Check it
    file = check_file(fileDAO,FILE6_ID,"FILE6",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE6)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE6)", file->finishTime() != (time_t)-1);
    // Get File instance (7) and Check it
    file = check_file(fileDAO,FILE7_ID,"FILE7",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE7)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE7)", file->finishTime() != (time_t)-1);
    // Get Job instance (3) and Check it
    job = check_job(jobDAO,JOB3_ID,"JOB3",Job::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB3)", job->channelName == CHANNEL1_NAME);

}

void AgentsTestCase::test04(){

    // Get File instance (8) and Check it
    file = check_file(fileDAO,FILE8_ID,"FILE8",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE8)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE8)", file->finishTime() != (time_t)-1);
    // Get File instance (9) and Check it
    file = check_file(fileDAO,FILE9_ID,"FILE9",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE9)", file->finishTime() != (time_t)-1);
    // Get Job instance (4) and Check it
    job = check_job(jobDAO,JOB4_ID,"JOB4",Job::S_FINISHED_DIRTY);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB4)", job->channelName == CHANNEL1_NAME);

}

void AgentsTestCase::test05(){

    // Get File instance (10) and Check it
    file = check_file(fileDAO,FILE10_ID,"FILE10",File::S_CANCELED);
    // Get File instance (11) and Check it
    file = check_file(fileDAO,FILE11_ID,"FILE11",File::S_CANCELED);
    // Get Job instance (5) and Check it
    job = check_job(jobDAO,JOB5_ID,"JOB5",Job::S_CANCELED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB5)", job->channelName().empty() == true);

}

void AgentsTestCase::test06(){

    // Get File instance (12) and Check it
    file = check_file(fileDAO,FILE12_ID,"FILE12",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE12)", file->finishTime() != (time_t)-1);
    // Get File instance (13) and Check it
    file = check_file(fileDAO,FILE13_ID,"FILE13",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE13)", file->finishTime() != (time_t)-1);
    // Get Job instance (6) and Check it
    job = check_job(jobDAO,JOB6_ID,"JOB6",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB6)", job->channelName().empty() == true);

}

void AgentsTestCase::test07(){

    // Get File instance (14) and Check it
    file = check_file(fileDAO,FILE14_ID,"FILE14",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE14)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE14)", file->finishTime() != (time_t)-1);
    // Get File instance (15) and Check it
    file = check_file(fileDAO,FILE15_ID,"FILE15",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE15)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE15)", file->finishTime() != (time_t)-1);
    // Get Job instance (7) and Check it
    job = check_job(jobDAO,JOB7_ID,"JOB7",Job::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB7)", job->channelName == CHANNEL1_NAME);

}

void AgentsTestCase::test08(){

    // Get File instance (16) and Check it
    file = check_file(fileDAO,FILE16_ID,"FILE16",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE16)", file->finishTime() != (time_t)-1);
    // Get File instance (17) and Check it
    file = check_file(fileDAO,FILE17_ID,"FILE17",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE17)", file->finishTime() != (time_t)-1);
    // Get Job instance (8) and Check it
    job = check_job(jobDAO,JOB8_ID,"JOB8",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB8)", job->channelName().empty() == true);

}

void AgentsTestCase::test09(){

    // Get File instance (18) and Check it
    file = check_file(fileDAO,FILE18_ID,"FILE18",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE18)", file->finishTime() != (time_t)-1);
    // Get File instance (19) and Check it
    file = check_file(fileDAO,FILE19_ID,"FILE19",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE19)", file->finishTime() != (time_t)-1);
    // Get Job instance (9) and Check it
    job = check_job(jobDAO,JOB9_ID,"JOB9",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB9)", job->channelName().empty() == true);

}

void AgentsTestCase::test10(){

    // Get File instance (20) and Check it
    file = check_file(fileDAO,FILE20_ID,"FILE20",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE20)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE20)", file->finishTime() != (time_t)-1);
    // Get File instance (21) and Check it
    file = check_file(fileDAO,FILE21_ID,"FILE21",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE21)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE21)", file->finishTime() != (time_t)-1);
    // Get File instance (22) and Check it
    file = check_file(fileDAO,FILE22_ID,"FILE22",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE22)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE22)", file->finishTime() != (time_t)-1);
    // Get Job instance (10) and Check it
    job = check_job(jobDAO,JOB10_ID,"JOB10",Job::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB10)", job->channelName == CHANNEL2_NAME);

}

void AgentsTestCase::test11(){

    // Get File instance (23) and Check it
    file = check_file(fileDAO,FILE23_ID,"FILE23",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE23)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE23)", file->finishTime() != (time_t)-1);
    // Get File instance (24) and Check it
    file = check_file(fileDAO,FILE24_ID,"FILE24",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE24)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE24)", file->finishTime() != (time_t)-1);
    // Get Job instance (11) and Check it
    job = check_job(jobDAO,JOB11_ID,"JOB11",Job::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB11)", job->channelName == CHANNEL2_NAME);

}

void AgentsTestCase::test12(){

    // Get File instance (25) and Check it
    file = check_file(fileDAO,FILE25_ID,"FILE25",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE25)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE25)", file->finishTime() != (time_t)-1);
    // Get File instance (26) and Check it
    file = check_file(fileDAO,FILE26_ID,"FILE26",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE26)", file->finishTime() != (time_t)-1);
    // Get Job instance (12) and Check it
    job = check_job(jobDAO,JOB12_ID,"JOB12",Job::S_FINISHED_DIRTY);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB12)", job->channelName == CHANNEL2_NAME);

}

void AgentsTestCase::test13(){

    // Get File instance (27) and Check it
    file = check_file(fileDAO,FILE27_ID,"FILE27",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE27)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE27)", file->finishTime() != (time_t)-1);
    // Get File instance (28) and Check it
    file = check_file(fileDAO,FILE28_ID,"FILE28",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE28)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE28)", file->finishTime() != (time_t)-1);
    // Get Job instance (13) and Check it
    job = check_job(jobDAO,JOB13_ID,"JOB13",Job::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB13)", job->channelName == CHANNEL3_NAME);

}

void AgentsTestCase::test14(){

    // Get File instance (29) and Check it
    file = check_file(fileDAO,FILE29_ID,"FILE29",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE29)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE29)", file->finishTime() != (time_t)-1);
    // Get File instance (30) and Check it
    file = check_file(fileDAO,FILE30_ID,"FILE30",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE30)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE30)", file->finishTime() != (time_t)-1);
    // Get File instance (31) and Check it
    file = check_file(fileDAO,FILE31_ID,"FILE31",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE31)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE31)", file->finishTime() != (time_t)-1);
    // Get File instance (32) and Check it
    file = check_file(fileDAO,FILE32_ID,"FILE32",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE32)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE32)", file->finishTime() != (time_t)-1);
    // Get Job instance (14) and Check it
    job = check_job(jobDAO,JOB14_ID,"JOB14",Job::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB14)", job->channelName == CHANNEL3_NAME);

}

void AgentsTestCase::test15(){

    // Get File instance (33) and Check it
    file = check_file(fileDAO,FILE33_ID,"FILE33",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE33)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE33)", file->finishTime() != (time_t)-1);
    // Get File instance (34) and Check it
    file = check_file(fileDAO,FILE34_ID,"FILE34",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE34)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE34)", file->finishTime() != (time_t)-1);
    // Get File instance (35) and Check it
    file = check_file(fileDAO,FILE35_ID,"FILE35",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE35)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE35)", file->finishTime() != (time_t)-1);
    // Get Job instance (15) and Check it
    job = check_job(jobDAO,JOB15_ID,"JOB15",Job::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB15)", job->channelName == CHANNEL3_NAME);

}

void AgentsTestCase::test16(){

    // Get File instance (36) and Check it
    file = check_file(fileDAO,FILE36_ID,"FILE36",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE36)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE36)", file->finishTime() != (time_t)-1);
    // Get File instance (37) and Check it
    file = check_file(fileDAO,FILE37_ID,"FILE37",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE37)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE37)", file->finishTime() != (time_t)-1);
    // Get File instance (38) and Check it
    file = check_file(fileDAO,FILE38_ID,"FILE38",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE38)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE38)", file->finishTime() != (time_t)-1);
    // Get Job instance (16) and Check it
    job = check_job(jobDAO,JOB16_ID,"JOB16",Job::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB16)", job->channelName == CHANNEL3_NAME);

}

void AgentsTestCase::test17(){
    // SRMCopy Channel: this job is expected to finish correctly
    const char * const JOB401_ID = "40000000-0000-0000-0000-000000000001";
    const char * const FILE40101_ID = "40000101";
    
    file = check_file(fileDAO,FILE40101_ID,"FILE40101",File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Size  (FILE40101)", file->fileSize() > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (FILE40101)", file->finishTime() != (time_t)-1);
    
    job = check_job(jobDAO,JOB401_ID,"JOB401",Job::S_FINISHED);
    CHECK_MODEL_OBJECT_VALUE(job,channelName,CHANNEL4_NAME);
}

void AgentsTestCase::test18(){
    // SRMCopy Channel: this job is expected to fail
    const char * const JOB402_ID = "40000000-0000-0000-0000-000000000002";
    const char * const FILE40201_ID = "40000201";
    
    file = check_file(fileDAO,FILE40201_ID,"FILE40201",File::S_FAILED);
    check_error(file,"FILE40201",Error::FILE_EXISTS,Error::S_DESTINATION,Error::P_TRANSFER);
    
    job = check_job(jobDAO,JOB402_ID,"JOB402",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB402)", job->channelName == CHANNEL4_NAME);
}

void AgentsTestCase::test19(){
    // SRMCopy Channel: this job is expected to fail
    const char * const JOB403_ID = "40000000-0000-0000-0000-000000000003";
    const char * const FILE40301_ID = "40000301";
    const char * const FILE40302_ID = "40000302";
    const char * const FILE40303_ID = "40000303";
    
    file = check_file(fileDAO,FILE40301_ID,"FILE40301",File::S_FINISHED);
    file = check_file(fileDAO,FILE40302_ID,"FILE40302",File::S_FAILED);
    check_error(file,"FILE40302",Error::FILE_EXISTS,Error::S_DESTINATION,Error::P_TRANSFER);
    file = check_file(fileDAO,FILE40303_ID,"FILE40303",File::S_FINISHED);
    
    job = check_job(jobDAO,JOB403_ID,"JOB403",Job::S_FINISHED_DIRTY);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB403)", job->channelName == CHANNEL4_NAME);
}

void AgentsTestCase::test20(){
    // check that the agents set the transfer_type in the database
    scoped_ptr<dao::channel::ChannelDAO> channel_dao;
    scoped_ptr<model::Channel> channel;
    
    channel_dao.reset(dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null ChannelDAO (1)", channel_dao.get() != 0);
    channel.reset(channel_dao->get(CHANNEL1_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL1)", channel.get() != 0);
    CHECK_MODEL_OBJECT_VALUE(channel,transferType,"URLCOPY");
    
    channel_dao.reset(dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL2_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null ChannelDAO (2)", channel_dao.get() != 0);
    channel.reset(channel_dao->get(CHANNEL2_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL2)", channel.get() != 0);
    CHECK_MODEL_OBJECT_VALUE(channel,transferType,"URLCOPY");

    channel_dao.reset(dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL3_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null ChannelDAO (3)", channel_dao.get() != 0);
    channel.reset(channel_dao->get(CHANNEL3_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL3)", channel.get() != 0);
    CHECK_MODEL_OBJECT_VALUE(channel,transferType,"URLCOPY");

    channel_dao.reset(dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL4_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null ChannelDAO (4)", channel_dao.get() != 0);
    channel.reset(channel_dao->get(CHANNEL4_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL4)", channel.get() != 0);
    CHECK_MODEL_OBJECT_VALUE(channel,transferType,"SRMCOPY");
        
}


