/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ChannelDAO Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ChannelDaoTestCase.cpp,v 1.36 2008/12/04 08:53:30 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "DaoHelper.h"
#include <glite/data/transfer/agent/model/Channel.h>
#include <glite/data/transfer/agent/dao/vo/ChannelDAO.h>
#include <glite/data/transfer/agent/dao/channel/ChannelDAO.h>
#include <glite/data/agents/dao/DAOException.h>
#include <glite/data/transfer/agent/dao/vo/DAOFactory.h>
#include <glite/data/transfer/agent/dao/channel/DAOFactory.h>
#include <glite/data/agents/dao/DAOContextFactory.h>
#include "dao-utility.h"

#include "dao_test_utils.h"
#include <glite/data/agents/extra/utility.h>

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using glite::data::agents::dao::DAOContextFactory;
using glite::data::agents::dao::DAOException;
using glite::data::agents::longlong;
using glite::data::agents::dao::StringArray;

using boost::scoped_ptr;

/**
 * CPPUnit ChannelDaoTestCase
 */
class ChannelDaoTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( ChannelDaoTestCase);
        CPPUNIT_TEST( testGet_VO );
        CPPUNIT_TEST( testGet_VO_1 );
        CPPUNIT_TEST( testGet_VO_2 );
        CPPUNIT_TEST( testGet_VO_3 );
        CPPUNIT_TEST( testGet_VO_Error );
        CPPUNIT_TEST( testGetBySites_VO );
        CPPUNIT_TEST( testGetBySites_VO_2 );
        CPPUNIT_TEST( testHasShare_VO );
        CPPUNIT_TEST( testIsBlacklisted_VO );
        CPPUNIT_TEST( testGetSiteGroups_VO );
        CPPUNIT_TEST( testGet_Ch );
        CPPUNIT_TEST( testGet_Ch_Error );
        CPPUNIT_TEST( testGet_Ch_noshare );
        CPPUNIT_TEST( testGet_Ch_2 );
        CPPUNIT_TEST( testGet_Ch_3 );
        CPPUNIT_TEST( testGet_Ch_4 );
        CPPUNIT_TEST( testGet_Ch_5 );
        CPPUNIT_TEST( testUpdateLastActive_Ch );
        CPPUNIT_TEST( testGetVONames_Ch );
        CPPUNIT_TEST( testGetShare_Ch );
        CPPUNIT_TEST( testGetShare_Ch_2 );
        CPPUNIT_TEST( testGetShare_Ch_3 );
        CPPUNIT_TEST( testGetShare_Ch_Error );
        CPPUNIT_TEST( testGetLimit_Ch );
        CPPUNIT_TEST( testGetLimit_Ch_Error );
        CPPUNIT_TEST( testGet_Ch_TargetDirCheck_Default );
        CPPUNIT_TEST( testGet_Ch_Defaults );
        CPPUNIT_TEST( testGet_Ch_Defaults2 );
        // schema version 3.3.0
        CPPUNIT_TEST( testGet_VO_schema_3_3_0 );
        CPPUNIT_TEST( testGet_Ch_schema_3_3_0 );
    CPPUNIT_TEST_SUITE_END();

    // VO Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_VO();
    void testGet_VO_1();
    void testGet_VO_2();
    void testGet_VO_3();
    void testGet_VO_Error();
    /**
     * Test GetBySites Method
     */
    void testGetBySites_VO();
    void testGetBySites_VO_2();    
    
    /**
     * Test HasShare Method
     */
    void testHasShare_VO();    
    
    /**
     * Test IsBlacklisted Method
     */
    void testIsBlacklisted_VO();
    
    /**
     * Test GetSiteGroups Method
     */
    void testGetSiteGroups_VO();
    
    // Channel Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_Ch();
    void testGet_Ch_Error();
    void testGet_Ch_noshare();
    void testGet_Ch_2();
    void testGet_Ch_3();
    void testGet_Ch_4();
    void testGet_Ch_5();
    
    /**
     * Test UpdateLastActive Method
     */
    void testUpdateLastActive_Ch();
    /**
     * Test GetVONames Method
     */
    void testGetVONames_Ch();
    /**
     * Test GetShare Method
     */
    void testGetShare_Ch(); 
    void testGetShare_Ch_2(); 
    void testGetShare_Ch_3(); 
    /**
     * Test GetShare Method
     */
    void testGetShare_Ch_Error(); 
    /**
     * Test GetLimit Method
     */
    void testGetLimit_Ch(); 
    /**
     * Test GetLimit Method
     */
    void testGetLimit_Ch_Error(); 
    
    void testGet_Ch_TargetDirCheck_Default();
    void testGet_Ch_Defaults();
    void testGet_Ch_Defaults2();
    
    /**
     * Schema 3.3.0
     */
    void testGet_VO_schema_3_3_0();
    void testGet_Ch_schema_3_3_0();
    
private:

    /**
     * The VO FileDAO Instance
     */
    scoped_ptr<dao::vo::ChannelDAO> m_voChannelDAO;

    /**
     * The Channel FileDAO Instance
     */
    scoped_ptr<dao::channel::ChannelDAO> m_chChannelDAO;
    scoped_ptr<dao::channel::ChannelDAO> m_chChannelDAO_3_1_0;

};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ChannelDaoTestCase, "ChannelDaoTestCase" );

namespace {
const char * const CHANNEL1_NAME  = "channel_1";
const char * const CHANNEL2_NAME  = "channel_2";
const char * const CHANNEL3_NAME  = "channel_3";
const char * const CHANNEL4_NAME  = "channel_4";
const char * const CHANNEL5_NAME  = "channel_5";
const char * const CHANNEL6_NAME  = "channel_6";
const char * const CHANNEL7_NAME  = "channel_7";
const char * const CHANNEL8_NAME  = "channel_8";
const char * const CHANNEL9_NAME  = "channel_9";
const char * const CHANNEL10_NAME = "channel_10";
const char * const CHANNEL11_NAME = "channel_11";
const char * const CHANNEL12_NAME = "channel_12";
const char * const VO1_NAME       = "CHANNEL_VO_1";
const char * const VO2_NAME       = "CHANNEL_VO_2";
const char * const VO3_NAME       = "CHANNEL_VO_3";
const char * const VO4_NAME       = "CHANNEL_VO_4";
const char * const VO5_NAME       = "CHANNEL_VO_5";
const char * const VO6_NAME       = "CHANNEL_VO_6";
const int    CHANNEL1_NOS   = 1;
const int    CHANNEL1_NOF   = 100;
const int    VO1_SHARE      = 10;
const int    VO2_SHARE      = 10;
const int    VO4_LIMIT      = 22;
const int    VO5_LIMIT      = 0;
const int    VO6_LIMIT      = 5;
const char * const BLACKLISTED_HOST1 = "bad.host.cern.ch";
const char * const BLACKLISTED_SITE1 = "BAD_SITE";
const char * const GOOD_HOST1        = "host.cern.ch";
const char * const SITE_WITH_GROUPS1   = "site_1";
const char * const SITE_WITHOUT_GROUPS = "site_2";
const char * const GROUP_1             = "[group_1]";
const char * const GROUP_2             = "[group_2]";

// schema 3.1.0
const char * const  CHANNEL20_NAME                  = "channel_20";
const char * const  CHANNEL23_NAME                  = "channel_23";
const char * const  CHANNEL20_blocksize             = "1 MB"; // VARCHAR2(24)
const int           CHANNEL20_http_to               = 10; // INTEGER
const Channel::LogLevel CHANNEL20_tx_loglevel= Channel::LL_DEBUG; // VARCHAR2(12)
const int           CHANNEL20_urlcopy_put_to        = 20; // INTEGER
const int           CHANNEL20_urlcopy_putdone_to    = 30; // INTEGER
const int           CHANNEL20_urlcopy_get_to        = 40; // INTEGER
const int           CHANNEL20_urlcopy_getdone_to    = 50; // INTEGER
const int           CHANNEL20_urlcopy_tx_to         = 60; // INTEGER
const int           CHANNEL20_urlcopy_txmarks_to    = 70; // INTEGER
const Channel::SrmCopyDirection CHANNEL20_srmcopy_direction     = Channel::SCD_PUSH; // VARCHAR2(4)
const int           CHANNEL20_srmcopy_to            = 80; // INTEGER
const int           CHANNEL20_srmcopy_refresh_to    = 90; // INTEGER

// schema 3.2.0
const char * const  CHANNEL21_NAME                  = "channel_21";
const bool          CHANNEL21_TARGET_DIR_CHECK      = false;
const int           CHANNEL21_FIRST_TXMARK_TO       = 5;

const char * const  CHANNEL22_NAME                  = "channel_22";
const bool          CHANNEL22_TARGET_DIR_CHECK      = true;
const int           CHANNEL22_FIRST_TXMARK_TO       = 15;
}

/*
 * setUp
 *
 * set-up test case
 */
void ChannelDaoTestCase::setUp(){

    // Create VO ChannelDAO instance
    m_voChannelDAO.reset(
        dao::vo::DAOFactory::instance().createChannelDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null VO ChannelDAO", m_voChannelDAO.get() != 0);
    // Create Channel ChannelDAO instance
    m_chChannelDAO.reset(
        dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null Channel ChannelDAO", m_chChannelDAO.get() != 0);

    m_chChannelDAO_3_1_0.reset(
        dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL20_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null Channel ChannelDAO", m_chChannelDAO_3_1_0.get() != 0);

}

/*
 * testGet
 *
 * Test Get Method
 */
void ChannelDaoTestCase::testGet_VO(){

    // Get Channel instance
    scoped_ptr<model::Channel> channel(m_voChannelDAO->get(CHANNEL1_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL1)", channel.get() != 0);
    
    // Check that name is the same
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Name (CHANNEL1)", channel->name == CHANNEL1_NAME);
    
    // Check that the State is Active (Precondition)
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid State (CHANNEL1)", channel->state == Channel::S_ACTIVE);
    
    // Check that DomainA is cern.ch (Precondition)
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid SourceSite (CHANNEL1)", channel->sourceSite == "cern.ch");

    // Check that DomainB is ral.ac.uk (Precondition)
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid DestSite (CHANNEL1)", channel->destSite == "ral.ac.uk");

    // Check that Streams is 1
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Streams (CHANNEL1)", channel->streams == CHANNEL1_NOS);

    // Check that Files is 100
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Files (CHANNEL1)", channel->files == CHANNEL1_NOF);

    // Get Channel instance
    channel.reset(m_voChannelDAO->get(CHANNEL4_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL4)", channel.get() != 0);

}

/*
 * testGet
 *
 * Test Get Method
 */
void ChannelDaoTestCase::testGet_VO_1(){

    // Get Channel instance
    scoped_ptr<model::Channel> channel(m_voChannelDAO->get(CHANNEL9_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL9)", channel.get() != 0);
    // Check that the State is Active (Precondition)
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid State (CHANNEL9)", channel->state == Channel::S_ARCHIVED);

}

/*
 * testGet
 *
 * Test Get Method
 */
void ChannelDaoTestCase::testGet_VO_2(){

    // Get Channel instance
    scoped_ptr<model::Channel> channel(m_voChannelDAO->get(CHANNEL11_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL11)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL11)", channel->name == CHANNEL11_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid SE Limit (CHANNEL11)", channel->seLimit == 12);
    CPPUNIT_ASSERT_MESSAGE("Invalid Type (CHANNEL11)", channel->type == Channel::T_DEDICATED);

    // Get Channel instance
    channel.reset(m_voChannelDAO->get(CHANNEL12_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL12)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL12)", channel->name == CHANNEL12_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid SE Limit (CHANNEL12)", channel->seLimit == 5);
    CPPUNIT_ASSERT_MESSAGE("Invalid Type (CHANNEL12)", channel->type == Channel::T_NON_DEDICATED);

    // Get Channel instance
    channel.reset(m_voChannelDAO->get(CHANNEL1_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL1)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL1)", channel->name == CHANNEL1_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid SE Limit (CHANNEL1)", channel->seLimit == 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Type (CHANNEL1)", channel->type == Channel::T_UNDEF);

}

void ChannelDaoTestCase::testGet_VO_3(){

    // Get Channel instance
    scoped_ptr<model::Channel> channel(m_voChannelDAO->get(CHANNEL21_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL21)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL21)", channel->name == CHANNEL21_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid targetDirCheck (CHANNEL21)", channel->targetDirCheck == CHANNEL21_TARGET_DIR_CHECK);
    CPPUNIT_ASSERT_MESSAGE("Invalid first tx mark to (CHANNEL21)", channel->urlcopyFirstTxmarkTimeout == CHANNEL21_FIRST_TXMARK_TO);

    // Get Channel instance
    channel.reset(m_voChannelDAO->get(CHANNEL22_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL22)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL22)", channel->name == CHANNEL22_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid targetDirCheck (CHANNEL22)", channel->targetDirCheck == CHANNEL22_TARGET_DIR_CHECK);
    CPPUNIT_ASSERT_MESSAGE("Invalid first tx mark to (CHANNEL22)", channel->urlcopyFirstTxmarkTimeout == CHANNEL22_FIRST_TXMARK_TO);

}

/*
 * testGet
 *
 * Test Get Method
 */
void ChannelDaoTestCase::testGet_VO_Error(){
    try{
        // Get Channel instance
        scoped_ptr<model::Channel> channel(m_voChannelDAO->get(CHANNEL6_NAME));
        CPPUNIT_FAIL("This Channel shouldn't be reachable (CHANNEL6)");
    } catch(const DAOException& exc){
        // OK
    }
}

/*
 * testGetBySites
 *
 * Test GetByDomains Method
 */
void ChannelDaoTestCase::testGetBySites_VO(){
    
    // Get By Sites

    std::string channel = m_voChannelDAO->getBySites("cern.ch","ral.ac.uk");
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Name (CH1)", channel == CHANNEL1_NAME);

    // the channel is Drain => it should go to the catch all channel 
    channel = m_voChannelDAO->getBySites("cern.ch","nikhef.nl");
    CPPUNIT_ASSERT_MESSAGE("Channel Not empty (CH2)", channel.empty() == true);
    channel = m_voChannelDAO->getBySites("cern.ch","*");
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Name (CH5)", channel == CHANNEL5_NAME);

    channel = m_voChannelDAO->getBySites("cern.ch","infn.it");
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Name (CH3)", channel == CHANNEL3_NAME);

    // the channel is Stopped => go to the catch all channel
    channel = m_voChannelDAO->getBySites("cern.ch","in2p3.fr");
    CPPUNIT_ASSERT_MESSAGE("Channel Name not empty (CH4)", channel.empty() == true);
    channel = m_voChannelDAO->getBySites("cern.ch","*");
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Name (CH5)", channel == CHANNEL5_NAME);

    channel = m_voChannelDAO->getBySites("cern.ch","desy.de");
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Name (CH8)", channel == CHANNEL8_NAME);

    channel = m_voChannelDAO->getBySites("cern.ch","nowhere.com");
    CPPUNIT_ASSERT_MESSAGE("Channel Name Not empty (CH6)", channel.empty() == true);

    // Create VO ChannelDAO instance
    scoped_ptr<dao::vo::ChannelDAO> vo3_channelDao(dao::vo::DAOFactory::instance().createChannelDAO(VO3_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null VO ChannelDAO", vo3_channelDao.get() != 0);
    
    // Get By Sites
    channel = vo3_channelDao->getBySites("nowhere.com","nowhere.org");
    CPPUNIT_ASSERT_MESSAGE("Channel Name Not empty #0 (CH6)", channel.empty() == true);
    channel = vo3_channelDao->getBySites("nowhere.com","*");
    CPPUNIT_ASSERT_MESSAGE("Channel Name Not empty #1 (CH6)", channel.empty() == true);
    channel = vo3_channelDao->getBySites("*","nowhere.org");
    CPPUNIT_ASSERT_MESSAGE("Channel Name Not empty #2 (CH6)", channel.empty() == true);
    channel = vo3_channelDao->getBySites("*","*");
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Name (CH6)", channel == CHANNEL6_NAME);

}

/*
 * testGetBySites
 *
 * Test GetByDomains Method
 */
void ChannelDaoTestCase::testGetBySites_VO_2(){
    
    scoped_ptr<dao::vo::ChannelDAO> vo3_channelDao;

    // Create VO ChannelDAO instance
    vo3_channelDao.reset(dao::vo::DAOFactory::instance().createChannelDAO(VO3_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null VO ChannelDAO", vo3_channelDao.get() != 0);

    // Get By Sites
    
    std::string channel = vo3_channelDao->getBySites("cern.ch","ral.ac.uk");
    CPPUNIT_ASSERT_MESSAGE("Channel (CH1) shouldn't be reachable", channel.empty());

    channel = vo3_channelDao->getBySites("cern.ch","*");
    CPPUNIT_ASSERT_MESSAGE("Channel (CH5) shouldn't be reachable", channel.empty());

    channel = vo3_channelDao->getBySites("cern.ch","infn.it");
    CPPUNIT_ASSERT_MESSAGE("Channel (CH3) shouldn't be reachable", channel.empty());

    channel = vo3_channelDao->getBySites("cern.ch","in2p3.fr");
    CPPUNIT_ASSERT_MESSAGE("Channel (CH4) shouldn't be reachable", channel.empty());

    channel = vo3_channelDao->getBySites("cern.ch","desy.de");
    CPPUNIT_ASSERT_MESSAGE("Channel (CH8) shouldn't be reachable", channel.empty());

}

/*
 * testHasShare
 *
 * Test HasShare Method
 */
void ChannelDaoTestCase::testHasShare_VO(){
    bool has_share = m_voChannelDAO->hasShare(CHANNEL1_NAME);
    CPPUNIT_ASSERT_MESSAGE("Channel (CH1) should be authorized", has_share == true);

    has_share = m_voChannelDAO->hasShare(CHANNEL7_NAME);
    CPPUNIT_ASSERT_MESSAGE("Channel (CH7) should be denied", has_share == false);
}

/*
 * testIsBlacklisted
 *
 * Test HasShare Method
 */
void ChannelDaoTestCase::testIsBlacklisted_VO(){

    std::pair<bool,std::string> result = m_voChannelDAO->isBlacklisted(BLACKLISTED_HOST1,"CERN_PROD");
    CPPUNIT_ASSERT_MESSAGE("Host (HOST1) should be blacklisted", result.first == true);
    CPPUNIT_ASSERT_MESSAGE("Host (HOST1) should have a reason", result.second.empty() == false);

    result = m_voChannelDAO->isBlacklisted(GOOD_HOST1,BLACKLISTED_SITE1);
    CPPUNIT_ASSERT_MESSAGE("Site (SITE1) should be blacklisted", result.first == true);
    CPPUNIT_ASSERT_MESSAGE("Site (SITE1) should have a reason", result.second.empty() == false);

    result = m_voChannelDAO->isBlacklisted(GOOD_HOST1,"CERN_PROD");
    CPPUNIT_ASSERT_MESSAGE("Host (HOST2) should not be blacklisted", result.first == false);
    CPPUNIT_ASSERT_MESSAGE("Host (HOST2) should not have a reason", result.second.empty() == true);

}

/*
 * testGetSiteGroups_VO
 *
 * Test GetSiteGroups Method
 */
void ChannelDaoTestCase::testGetSiteGroups_VO(){ // [3.1.0]
    
    std::vector<std::string> groups;

    m_voChannelDAO->getSiteGroups(SITE_WITH_GROUPS1,groups);
    CPPUNIT_ASSERT_MESSAGE("Invalid number of groups (SITE1)", groups.size() == 2);
    CPPUNIT_ASSERT_MESSAGE("Invalid group name #1 (SITE1)", groups.at(0) == GROUP_2);
    CPPUNIT_ASSERT_MESSAGE("Invalid group name #2 (SITE1)", groups.at(1) == GROUP_1);

    groups.clear();
    m_voChannelDAO->getSiteGroups(SITE_WITHOUT_GROUPS,groups);
    CPPUNIT_ASSERT_MESSAGE("Invalid number of groups (SITE2)", groups.empty() == true);

}

/*
 * testGet
 *
 * Test Get Method
 */
void ChannelDaoTestCase::testGet_Ch(){

    // Get Channel instance
    scoped_ptr<model::Channel> channel(m_chChannelDAO->get(CHANNEL1_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL1)", channel.get() != 0);
    
    // Check that name is the same
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Name (CHANNEL1)", channel->name == CHANNEL1_NAME);
    
    // Check that the State is Active (Precondition)
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid State (CHANNEL1)", channel->state == Channel::S_ACTIVE);
    
    // Check that DomainA is cern.ch (Precondition)
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid SourceSite (CHANNEL1)", channel->sourceSite == "cern.ch");

    // Check that DomainB is ral.ac.uk (Precondition)
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid DestSite (CHANNEL1)", channel->destSite == "ral.ac.uk");

    // Check that Streams is 1
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Streams (CHANNEL1)", channel->streams == CHANNEL1_NOS);

    // Check that Files is 100
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Files (CHANNEL1)", channel->files == CHANNEL1_NOF);

}

/*
 * testGet
 *
 * Test Get Method
 */
void ChannelDaoTestCase::testGet_Ch_2(){

    // Get Channel instance
    scoped_ptr<model::Channel> channel(m_chChannelDAO->get(CHANNEL1_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL1)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL1)", channel->name == CHANNEL1_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid SE Limit (CHANNEL1)", channel->seLimit == 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Type (CHANNEL1)", channel->type == Channel::T_UNDEF);

    // Create Channel ChannelDAO instance
    scoped_ptr<dao::channel::ChannelDAO> chDAO(
        dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL11_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null Channel ChannelDAO", chDAO.get() != 0);

    // Get Channel instance
    channel.reset(chDAO->get(CHANNEL11_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL11)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL11)", channel->name == CHANNEL11_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid SE Limit (CHANNEL11)", channel->seLimit == 12);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Type (CHANNEL11)", channel->type == Channel::T_DEDICATED);

    // Create Channel ChannelDAO instance
    chDAO.reset(
        dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL12_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null Channel ChannelDAO", chDAO.get() != 0);

    // Get Channel instance
    channel.reset(chDAO->get(CHANNEL12_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL12)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL12)", channel->name == CHANNEL12_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid SE Limit (CHANNEL12)", channel->seLimit == 5);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Type (CHANNEL12)", channel->type == Channel::T_NON_DEDICATED);
}

/*
 * testGet
 *
 * Test Get Method (schema version 3.1.0)
 */
void ChannelDaoTestCase::testGet_Ch_3(){

    // Get Channel instance
    scoped_ptr<model::Channel> channel(m_chChannelDAO_3_1_0->get(CHANNEL20_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL20)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL20)", channel->name == CHANNEL20_NAME);

    CPPUNIT_ASSERT_MESSAGE("Invalid blocksize (CHANNEL20)", channel->blocksize == CHANNEL20_blocksize);
    CPPUNIT_ASSERT_MESSAGE("Invalid httpTimeout (CHANNEL20)", channel->httpTimeout == CHANNEL20_http_to);
    CPPUNIT_ASSERT_MESSAGE("Invalid txLoglevel (CHANNEL20)", channel->txLoglevel == CHANNEL20_tx_loglevel);

    CPPUNIT_ASSERT_MESSAGE("Invalid urlcopyPutTimeout (CHANNEL20)", channel->urlcopyPutTimeout == CHANNEL20_urlcopy_put_to);
    CPPUNIT_ASSERT_MESSAGE("Invalid urlcopyPutdoneTimeout (CHANNEL20)", channel->urlcopyPutdoneTimeout == CHANNEL20_urlcopy_putdone_to);
    CPPUNIT_ASSERT_MESSAGE("Invalid urlcopyGetTimeout (CHANNEL20)", channel->urlcopyGetTimeout == CHANNEL20_urlcopy_get_to);

    CPPUNIT_ASSERT_MESSAGE("Invalid getdoneTimeout (CHANNEL20)", channel->urlcopyGetdoneTimeout == CHANNEL20_urlcopy_getdone_to);
    CPPUNIT_ASSERT_MESSAGE("Invalid urlcopyTxTimeout (CHANNEL20)", channel->urlcopyTxTimeout == CHANNEL20_urlcopy_tx_to);
    CPPUNIT_ASSERT_MESSAGE("Invalid urlcopyTxmarksTimeout (CHANNEL20)", channel->urlcopyTxmarksTimeout == CHANNEL20_urlcopy_txmarks_to);

    CPPUNIT_ASSERT_MESSAGE("Invalid srmcopyDirection (CHANNEL20)", channel->srmcopyDirection == CHANNEL20_srmcopy_direction);
    CPPUNIT_ASSERT_MESSAGE("Invalid srmcopyTimeout (CHANNEL20)", channel->srmcopyTimeout == CHANNEL20_srmcopy_to);
    CPPUNIT_ASSERT_MESSAGE("Invalid srmcopyRefreshTimeout (CHANNEL20)", channel->srmcopyRefreshTimeout == CHANNEL20_srmcopy_refresh_to);
}

/*
 * testGet
 *
 * Test Get Method (schema version 3.1.0)
 */
void ChannelDaoTestCase::testGet_Ch_4(){

    // Get Channel instance
    scoped_ptr<model::Channel> channel(m_voChannelDAO->get(CHANNEL20_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL20)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL20)", channel->name == CHANNEL20_NAME);

    CPPUNIT_ASSERT_MESSAGE("Invalid blocksize (CHANNEL20)", channel->blocksize == CHANNEL20_blocksize);
    CPPUNIT_ASSERT_MESSAGE("Invalid httpTimeout (CHANNEL20)", channel->httpTimeout == CHANNEL20_http_to);
    CPPUNIT_ASSERT_MESSAGE("Invalid txLoglevel (CHANNEL20)", channel->txLoglevel == CHANNEL20_tx_loglevel);

    CPPUNIT_ASSERT_MESSAGE("Invalid urlcopyPutTimeout (CHANNEL20)", channel->urlcopyPutTimeout == CHANNEL20_urlcopy_put_to);
    CPPUNIT_ASSERT_MESSAGE("Invalid urlcopyPutdoneTimeout (CHANNEL20)", channel->urlcopyPutdoneTimeout == CHANNEL20_urlcopy_putdone_to);
    CPPUNIT_ASSERT_MESSAGE("Invalid urlcopyGetTimeout (CHANNEL20)", channel->urlcopyGetTimeout == CHANNEL20_urlcopy_get_to);

    CPPUNIT_ASSERT_MESSAGE("Invalid getdoneTimeout (CHANNEL20)", channel->urlcopyGetdoneTimeout == CHANNEL20_urlcopy_getdone_to);
    CPPUNIT_ASSERT_MESSAGE("Invalid urlcopyTxTimeout (CHANNEL20)", channel->urlcopyTxTimeout == CHANNEL20_urlcopy_tx_to);
    CPPUNIT_ASSERT_MESSAGE("Invalid urlcopyTxmarksTimeout (CHANNEL20)", channel->urlcopyTxmarksTimeout == CHANNEL20_urlcopy_txmarks_to);

    CPPUNIT_ASSERT_MESSAGE("Invalid srmcopyDirection (CHANNEL20)", channel->srmcopyDirection == CHANNEL20_srmcopy_direction);
    CPPUNIT_ASSERT_MESSAGE("Invalid srmcopyTimeout (CHANNEL20)", channel->srmcopyTimeout == CHANNEL20_srmcopy_to);
    CPPUNIT_ASSERT_MESSAGE("Invalid srmcopyRefreshTimeout (CHANNEL20)", channel->srmcopyRefreshTimeout == CHANNEL20_srmcopy_refresh_to);
}

/**
 * testGet_Ch_5
 */
void ChannelDaoTestCase::testGet_Ch_5(){
    scoped_ptr<dao::channel::ChannelDAO> channel_dao;
        
    channel_dao.reset(
            dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL21_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null Channel ChannelDAO", channel_dao.get() != 0);
    
    // Get Channel instance
    scoped_ptr<model::Channel> channel(channel_dao->get(CHANNEL21_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL21)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL21)", channel->name == CHANNEL21_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid targetDirCheck (CHANNEL21)", channel->targetDirCheck == CHANNEL21_TARGET_DIR_CHECK);
    CPPUNIT_ASSERT_MESSAGE("Invalid first tx mark to (CHANNEL21)", channel->urlcopyFirstTxmarkTimeout == CHANNEL21_FIRST_TXMARK_TO);

    channel_dao.reset(
            dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL22_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null Channel ChannelDAO", channel_dao.get() != 0);
    
    // Get Channel instance
    channel.reset(channel_dao->get(CHANNEL22_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL22)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL22)", channel->name == CHANNEL22_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid targetDirCheck (CHANNEL22)", channel->targetDirCheck == CHANNEL22_TARGET_DIR_CHECK);
    CPPUNIT_ASSERT_MESSAGE("Invalid first tx mark to (CHANNEL22)", channel->urlcopyFirstTxmarkTimeout == CHANNEL22_FIRST_TXMARK_TO);
}

/*
 * testGet
 *
 * Test Get Method
 */
void ChannelDaoTestCase::testGet_Ch_Error(){
    try{
        // Get Channel instance
        scoped_ptr<model::Channel> channel(m_chChannelDAO->get(CHANNEL2_NAME));
        CPPUNIT_FAIL("This Channel shouldn't be reachable (CHANNEL2)");
    } catch(const DAOException& exc){
        // OK
    }
}

/*
 * testGet
 *
 * Test Get no share Method
 */
void ChannelDaoTestCase::testGet_Ch_noshare(){
    // Create Channel ChannelDAO instance
    scoped_ptr<dao::channel::ChannelDAO> channelDAO(
        dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL7_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null Channel ChannelDAO", channelDAO.get() != 0);
    // Get Channel instance
    scoped_ptr<model::Channel> channel(channelDAO->get(CHANNEL7_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL7)", channel.get() != 0);
}

/*
 * testUpdateLastActive
 *
 * Test UpdateLastActive Method
 */
void ChannelDaoTestCase::testUpdateLastActive_Ch(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get Channel instance
        scoped_ptr<model::Channel> channel(m_chChannelDAO->get(CHANNEL1_NAME));
        CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL1)",channel.get() != 0);

        // Change The State
        Channel::State old_state = channel->state();
        channel->state = (old_state == Channel::S_ACTIVE)?Channel::S_INACTIVE:Channel::S_ACTIVE;

        // Set last Active Time
        time_t current;
        time(&current);
        channel->lastActive = current;
        // Update Transfer Channel
        m_chChannelDAO->updateLastActive(*channel);

        // Commit
        DaoHelper::ctx().commit();

        // Check instance        
        channel.reset(m_chChannelDAO->get(CHANNEL1_NAME));
        CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL1)",channel.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid TransferTime (CHANNEL1)",channel->lastActive == current);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (CHANNEL1)",channel->state == old_state);
        
    } catch(const DAOException& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetVONames_Ch
 *
 * Test GetVONames Method
 */
void ChannelDaoTestCase::testGetVONames_Ch(){
    // Get VO Names
    StringArray vo_names;
    m_chChannelDAO->getVONames(vo_names);
    CPPUNIT_ASSERT_MESSAGE("No VO Names", !vo_names.empty());
}

/*
 * testGetVOShare_Ch
 *
 * Test GetShare Method
 */
void ChannelDaoTestCase::testGetShare_Ch(){
    // Get By State
    double share = m_chChannelDAO->getShare(VO1_NAME);
    CPPUNIT_ASSERT_MESSAGE("Precondition check failed: Invalid Share (VO1)", share == ((double)VO1_SHARE)/100);
}

/*
 * testGetVOShare_Ch_2
 *
 * Test GetShare Method
 */
void ChannelDaoTestCase::testGetShare_Ch_2(){
    // Get By State
    double share = m_chChannelDAO->getShare(VO1_NAME,true);
    CPPUNIT_ASSERT_MESSAGE("Precondition check failed: Invalid Share (VO1)", share == ((double)VO1_SHARE)/(VO1_SHARE + VO2_SHARE));
}

/*
 * testGetVOShare_Ch_3
 *
 * Test GetShare Method
 */
void ChannelDaoTestCase::testGetShare_Ch_3(){
    // Get By State
    double share = m_chChannelDAO->getShare(VO1_NAME,true,true);
    CPPUNIT_ASSERT_MESSAGE("Precondition check failed: Invalid Share (VO1)", share == (double)1);
}

/*
 * testGetShare_Ch_Error
 *
 * Test GetShare Method
 */
void ChannelDaoTestCase::testGetShare_Ch_Error(){
    
    // Get By State
    try{    
        /*double share =*/ m_chChannelDAO->getShare("Invalid VO");
        CPPUNIT_FAIL("Invalid VO shoudn't be reachable");
    } catch(const DAOException& exc){
        //OK
    }
}

/*
 * testGetLimit_Ch
 *
 * Test GetLimit Method
 */
void ChannelDaoTestCase::testGetLimit_Ch(){
    
    // Get By State
    
    // Create Channel ChannelDAO instance
    scoped_ptr<dao::channel::ChannelDAO> ch_dao(
        dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL10_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null Channel ChannelDAO", ch_dao.get() != 0);
    
    std::pair<int,Channel::VOState> limit = ch_dao->getLimit(VO4_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Limit (VO4)"  , limit.first == VO4_LIMIT);
    CPPUNIT_ASSERT_MESSAGE("Invalid VOState (VO4)", limit.second == Channel::VS_ON);
    
    limit = ch_dao->getLimit(VO5_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Limit (VO5)"  , limit.first == VO5_LIMIT);
    CPPUNIT_ASSERT_MESSAGE("Invalid VOState (VO5)", limit.second == Channel::VS_OFF);

    limit = ch_dao->getLimit(VO6_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Limit (VO6)"  , limit.first == VO6_LIMIT);
    CPPUNIT_ASSERT_MESSAGE("Invalid VOState (VO6)", limit.second == Channel::VS_PAUSE);

    // Check default
    limit = m_chChannelDAO->getLimit(VO1_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Limit (VO1)"  , limit.first == 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid VOState (VO1)", limit.second == Channel::VS_UNDEF);

}

/*
 * testGetLimit_Ch_Error
 *
 * Test GetLimit Method
 */
void ChannelDaoTestCase::testGetLimit_Ch_Error(){
    
    // Get By State
    try{    
        std::pair<int,Channel::VOState> limit = m_chChannelDAO->getLimit("Invalid VO");
        CPPUNIT_FAIL("Invalid VO shoudn't be reachable");
    } catch(const DAOException& exc){
        //OK
    }
}

void ChannelDaoTestCase::testGet_Ch_TargetDirCheck_Default(){
    // verify that the default value for targetDirCheck is true
    scoped_ptr<model::Channel> channel(m_voChannelDAO->get(CHANNEL1_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL1)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL1)", channel->name == CHANNEL1_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid targetDirCheck (CHANNEL1)", channel->targetDirCheck == true);    
}

void ChannelDaoTestCase::testGet_Ch_Defaults(){
    // verify the default values for timeout properties (bug #38103)
    scoped_ptr<model::Channel> channel(m_voChannelDAO->get(CHANNEL1_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL1)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL1)", channel->name == CHANNEL1_NAME);
    
    // blocksize is expressed as a string (e.g. 1MB), the string is converted
    // with str_to_size. We want the converted value to be 0 (t-u-c does not)
    // use it if it's 0, letting globus choose the default
    unsigned long block_size = glite::data::agents::str_to_size(channel->blocksize());
    if (0 != block_size){
        std::stringstream ss;
        ss << "Invalid value for block_size : expected 0, got " << block_size;
        CPPUNIT_FAIL(ss.str().c_str());
    }
    // If 0, t-u-c uses default (40 sec)
    CHECK_MODEL_OBJECT_VALUE(channel,httpTimeout,-1);
    // txLoglevel
    CHECK_MODEL_OBJECT_VALUE(channel,txLoglevel,Channel::LL_UNDEF);
    // urlcopyPutTimeout 
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyPutTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyPutdoneTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyGetTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyGetdoneTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyTxTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyTxmarksTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,srmcopyDirection,Channel::SCD_UNDEF);
    CHECK_MODEL_OBJECT_VALUE(channel,srmcopyTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,srmcopyRefreshTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,targetDirCheck,true);
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyFirstTxmarkTimeout,-1);
        
}

void ChannelDaoTestCase::testGet_Ch_Defaults2(){
    // verify the default values for timeout properties (bug #38103)
    scoped_ptr<model::Channel> channel(m_voChannelDAO->get(CHANNEL23_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL23)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL23)", channel->name == CHANNEL23_NAME);
    
    // blocksize is expressed as a string (e.g. 1MB), the string is converted
    // with str_to_size. We want the converted value to be 0 (t-u-c does not)
    // use it if it's 0, letting globus choose the default
    unsigned long block_size = glite::data::agents::str_to_size(channel->blocksize());
    if (0 != block_size){
        std::stringstream ss;
        ss << "Invalid value for block_size : expected 0, got " << block_size;
        CPPUNIT_FAIL(ss.str().c_str());
    }
    // If 0, t-u-c uses default (40 sec)
    CHECK_MODEL_OBJECT_VALUE(channel,httpTimeout,-1);
    // txLoglevel
    CHECK_MODEL_OBJECT_VALUE(channel,txLoglevel,Channel::LL_INFO);
    // urlcopyPutTimeout 
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyPutTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyPutdoneTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyGetTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyGetdoneTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyTxTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyTxmarksTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,srmcopyDirection,Channel::SCD_PUSH);
    CHECK_MODEL_OBJECT_VALUE(channel,srmcopyTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,srmcopyRefreshTimeout,-1);
    CHECK_MODEL_OBJECT_VALUE(channel,targetDirCheck,true);
    CHECK_MODEL_OBJECT_VALUE(channel,urlcopyFirstTxmarkTimeout,-1);
        
}

/**
 * Schema 3.3.0
 */
void ChannelDaoTestCase::testGet_VO_schema_3_3_0(){
    static const char * CHANNEL24_NAME = "channel_24";
    static const char * CHANNEL25_NAME = "channel_25";
    
    // Get Channel instance
    scoped_ptr<model::Channel> channel(m_voChannelDAO->get(CHANNEL24_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL24)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL24)", channel->name == CHANNEL24_NAME);
    CHECK_MODEL_OBJECT_VALUE(channel,txTimeoutPerMb,            0);
    CHECK_MODEL_OBJECT_VALUE(channel,noTransferActivityTimeout, -1);
    CHECK_MODEL_OBJECT_VALUE(channel,preparingFilesRatio,       CHANNEL_DEFAULTPREPARINGFILESRATIO);
    CHECK_MODEL_OBJECT_VALUE(channel,transferType,              "URLCOPY");

    // Get Channel instance
    channel.reset(m_voChannelDAO->get(CHANNEL25_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL25)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL25)", channel->name == CHANNEL25_NAME);
    CHECK_MODEL_OBJECT_VALUE(channel,txTimeoutPerMb,            1.1);
    CHECK_MODEL_OBJECT_VALUE(channel,noTransferActivityTimeout, 3);
    CHECK_MODEL_OBJECT_VALUE(channel,preparingFilesRatio,       2.5);
    CHECK_MODEL_OBJECT_VALUE(channel,transferType,              "SRMCOPY");

}

void ChannelDaoTestCase::testGet_Ch_schema_3_3_0(){
    static const char * CHANNEL24_NAME = "channel_24";
    static const char * CHANNEL25_NAME = "channel_25";
    
    // Check default values
    scoped_ptr<dao::channel::ChannelDAO> channel_dao;
    channel_dao.reset(
            dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL24_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null Channel ChannelDAO", channel_dao.get() != 0);
    
    scoped_ptr<model::Channel> channel(channel_dao->get(CHANNEL24_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL24)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL24)", channel->name == CHANNEL24_NAME);
    CHECK_MODEL_OBJECT_VALUE(channel,txTimeoutPerMb,            0);
    CHECK_MODEL_OBJECT_VALUE(channel,noTransferActivityTimeout, -1);
    CHECK_MODEL_OBJECT_VALUE(channel,preparingFilesRatio,       CHANNEL_DEFAULTPREPARINGFILESRATIO);
    CHECK_MODEL_OBJECT_VALUE(channel,transferType,              "URLCOPY");

    // Check non-default values
    channel_dao.reset(
            dao::channel::DAOFactory::instance().createChannelDAO(CHANNEL25_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null Channel ChannelDAO", channel_dao.get() != 0);
    channel.reset(channel_dao->get(CHANNEL25_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null Channel (CHANNEL25)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Name (CHANNEL25)", channel->name == CHANNEL25_NAME);
    CHECK_MODEL_OBJECT_VALUE(channel,txTimeoutPerMb,            1.1);
    CHECK_MODEL_OBJECT_VALUE(channel,noTransferActivityTimeout, 3);
    CHECK_MODEL_OBJECT_VALUE(channel,preparingFilesRatio,       2.5);
    CHECK_MODEL_OBJECT_VALUE(channel,transferType,              "SRMCOPY");
        
}


