/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FileDAO Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: FileDaoTestCase.cpp,v 1.39 2008/12/11 13:24:44 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "DaoHelper.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/dao/vo/FileDAO.h"
#include "glite/data/transfer/agent/dao/channel/FileDAO.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"

#include <sys/time.h>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <algorithm>
#include <cstring>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using glite::data::agents::dao::DAOException;
using glite::data::agents::longlong;
using glite::data::agents::dao::StringArray;
using glite::data::transfer::agent::dao::StringArrayTable;

using boost::scoped_ptr;

/**
 * CPPUnit FileDaoTestCase
 */
class FileDaoTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( FileDaoTestCase);
    // VO View
    CPPUNIT_TEST( testGet_VO );
    CPPUNIT_TEST( testGet_VO_Error );
    CPPUNIT_TEST( testGet_VO_Stress );

    CPPUNIT_TEST( testGet_VO_2 );

    CPPUNIT_TEST( testUpdate_VO );
    CPPUNIT_TEST( testUpdate_VO_2 );
    CPPUNIT_TEST( testUpdate_VO_3 );
    CPPUNIT_TEST( testUpdate_VO_4 );
    CPPUNIT_TEST( testUpdate_reason );
    CPPUNIT_TEST( testUpdate_VO_5 );
    CPPUNIT_TEST( testUpdate_VO_6 );
    CPPUNIT_TEST( testUpdate_VO_Error );
    CPPUNIT_TEST( testUpdate_VO_Stress );

    CPPUNIT_TEST( testUpdate_VO_7 );

    CPPUNIT_TEST( testGetByJobId_VO );
    CPPUNIT_TEST( testGetByJobId_VO_Error );
    CPPUNIT_TEST( testGetWaiting_VO);
    CPPUNIT_TEST( testGetWaiting_VO_2);
    CPPUNIT_TEST( testGetWaitingCatalog_VO);
    CPPUNIT_TEST( testGetWaitingCatalog_VO_2);
    CPPUNIT_TEST( testGetFilesToCancel_Ch);
    // Channel View
    CPPUNIT_TEST( testGet_Ch );
    CPPUNIT_TEST( testGet_Ch_Error );

    CPPUNIT_TEST( testGet_Ch_2 );

    CPPUNIT_TEST( testUpdate_Ch );
    CPPUNIT_TEST( testUpdate_Ch_2 );
    CPPUNIT_TEST( testUpdate_Ch_3 );
    CPPUNIT_TEST( testUpdate_Ch_4 );
    CPPUNIT_TEST( testUpdate_Ch_Error );
    CPPUNIT_TEST( testUpdate_Ch_5 );
    CPPUNIT_TEST( testUpdate_Ch_6 );

    CPPUNIT_TEST( testUpdate_Ch_7 );

    CPPUNIT_TEST( testGetByJobId_Ch );
    CPPUNIT_TEST( testGetByJobId_Ch_Error );
    CPPUNIT_TEST( testGetNextJobToTransfer_Ch);
    CPPUNIT_TEST( testGetJobsToTransferPerSE_Ch);
    CPPUNIT_TEST_SUITE_END();

    // Test Methods VO View
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_VO();
    void testGet_VO_Error();
    void testGet_VO_Stress();
    void testGet_VO_2();
    /**
     * Test Update Method
     */
    void testUpdate_VO();
    void testUpdate_VO_2();
    void testUpdate_VO_3();
    void testUpdate_VO_4();
    void testUpdate_VO_5();
    void testUpdate_VO_6();
    void testUpdate_reason();
    void testUpdate_VO_Error();
    void testUpdate_VO_Stress();
    void testUpdate_VO_7();
    /**
     * Test GetByState Method
     */
    void testGetByJobId_VO();
    void testGetByJobId_VO_Error();
    /**
     * Test GetWaiting Method
     */
    void testGetWaiting_VO();
    void testGetWaiting_VO_2();
    /**
     * Test GetWaitingCatalog Method
     */
    void testGetWaitingCatalog_VO();
    void testGetWaitingCatalog_VO_2();

    // Test Methods Channel View
    // -------------------------------------------------------------------------
    
    /**
     * Test Get Method
     */
    void testGet_Ch();
    void testGet_Ch_Error();
    void testGet_Ch_2();
    /**
     * Test Update Method
     */
    void testUpdate_Ch();
    void testUpdate_Ch_2();
    void testUpdate_Ch_3();
    void testUpdate_Ch_4();
    void testUpdate_Ch_5();
    void testUpdate_Ch_6();
    void testUpdate_Ch_Error();
    void testUpdate_Ch_7();
    /**
     * Test GetByState Method
     */
    void testGetByJobId_Ch();
    void testGetByJobId_Ch_Error();
    /**
     * Test GetNextToTransfer Method
     */
    void testGetNextJobToTransfer_Ch();
    /**
     * Test GetNextToTransfer Method
     */
    void testGetJobsToTransferPerSE_Ch();
    /**
     * Test GetFilesToCancel Method
     */
    void testGetFilesToCancel_Ch();
    
private:
    /**
     * The VO FileDAO Instance
     */
    scoped_ptr<dao::vo::FileDAO> m_voFileDAO;
    /**
     * The Channel FileDAO Instance
     */
    scoped_ptr<dao::channel::FileDAO> m_chFileDAO;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( FileDaoTestCase, "FileDaoTestCase" );

namespace {
const char * const FILEDAO_TESTCASE_JOB1_ID           = "00000000-0000-0000-0000-000000000001";
const char * const FILEDAO_TESTCASE_JOB2_ID           = "00000000-0000-0000-0000-000000000002";
const char * const FILEDAO_TESTCASE_JOB3_ID           = "00000000-0000-0000-0000-000000000003";
const char * const FILEDAO_TESTCASE_JOB4_ID           = "00000000-0000-0000-0000-000000000004";
const char * const FILEDAO_TESTCASE_JOB5_ID           = "00000000-0000-0000-0000-000000000005";
const char * const FILEDAO_TESTCASE_JOB6_ID           = "00000000-0000-0000-0000-000000000006";
const char * const FILEDAO_TESTCASE_JOB7_ID           = "00000000-0000-0000-0000-000000000007";
const char * const FILEDAO_TESTCASE_JOB8_ID           = "00000000-0000-0000-0000-000000000008";
const char * const FILEDAO_TESTCASE_JOB9_ID           = "00000000-0000-0000-0000-000000000009";
const char * const FILEDAO_TESTCASE_JOB10_ID          = "00000000-0000-0000-0000-000000000010";
const char * const FILEDAO_TESTCASE_JOB11_ID          = "00000000-0000-0000-0000-000000000011";
const char * const FILEDAO_TESTCASE_JOB12_ID          = "00000000-0000-0000-0000-000000000012";
const char * const FILEDAO_TESTCASE_JOB13_ID          = "00000000-0000-0000-0000-000000000013";
const char * const FILEDAO_TESTCASE_JOB14_ID          = "00000000-0000-0000-0000-000000000014";
const char * const FILEDAO_TESTCASE_JOB15_ID          = "00000000-0000-0000-0000-000000000015";
const char * const FILEDAO_TESTCASE_JOB16_ID          = "00000000-0000-0000-0000-000000000016";
const char * const FILEDAO_TESTCASE_JOB17_ID          = "00000000-0000-0000-0000-000000000017";
const char * const FILEDAO_TESTCASE_JOB18_ID          = "00000000-0000-0000-0000-000000000018";
const char * const FILEDAO_TESTCASE_JOB101_ID         = "00000000-0000-0000-0000-000000000101";
const char * const FILEDAO_TESTCASE_JOB102_ID         = "00000000-0000-0000-0000-000000000102";
const char * const FILEDAO_TESTCASE_JOB103_ID         = "00000000-0000-0000-0000-000000000103";
const char * const FILEDAO_TESTCASE_JOB104_ID         = "00000000-0000-0000-0000-000000000104";
const char * const FILEDAO_TESTCASE_JOB105_ID         = "00000000-0000-0000-0000-000000000105";
const char * const FILEDAO_TESTCASE_JOB106_ID         = "00000000-0000-0000-0000-000000000106";
const char * const FILEDAO_TESTCASE_JOB107_ID         = "00000000-0000-0000-0000-000000000107";
const char * const FILEDAO_TESTCASE_JOB108_ID         = "00000000-0000-0000-0000-000000000108";
const char * const FILEDAO_TESTCASE_JOB109_ID         = "00000000-0000-0000-0000-000000000109";
const char * const FILEDAO_TESTCASE_JOB110_ID         = "00000000-0000-0000-0000-000000000110";
const char * const FILEDAO_TESTCASE_FILE1_ID          = "10000001";
const char * const FILEDAO_TESTCASE_FILE2_ID          = "10000002";
const char * const FILEDAO_TESTCASE_FILE3_ID          = "10000003";
const char * const FILEDAO_TESTCASE_FILE4_ID          = "10000004";
const char * const FILEDAO_TESTCASE_FILE5_ID          = "10000005";
const char * const FILEDAO_TESTCASE_FILE6_ID          = "10000006";
const char * const FILEDAO_TESTCASE_FILE7_ID          = "10000007";
const char * const FILEDAO_TESTCASE_FILE8_ID          = "10000008";
const char * const FILEDAO_TESTCASE_FILE9_ID          = "10000009";
const char * const FILEDAO_TESTCASE_FILE10_ID         = "10000010";
const char * const FILEDAO_TESTCASE_FILE11_ID         = "10000011";
const char * const FILEDAO_TESTCASE_FILE12_ID         = "10000012";
const char * const FILEDAO_TESTCASE_FILE13_ID         = "10000013";
const char * const FILEDAO_TESTCASE_FILE14_ID         = "10000014";
const char * const FILEDAO_TESTCASE_FILE15_ID         = "10000015";
const char * const FILEDAO_TESTCASE_FILE16_ID         = "10000016";
const char * const FILEDAO_TESTCASE_FILE17_ID         = "10000017";
const char * const FILEDAO_TESTCASE_FILE18_ID         = "10000018";
const char * const FILEDAO_TESTCASE_FILE19_ID         = "10000019";
const char * const FILEDAO_TESTCASE_FILE20_ID         = "10000020";
const char * const FILEDAO_TESTCASE_FILE21_ID         = "10000021";
const char * const FILEDAO_TESTCASE_FILE22_ID         = "10000022";
const char * const FILEDAO_TESTCASE_FILE23_ID         = "10000023";
const char * const FILEDAO_TESTCASE_FILE24_ID         = "10000024";
const char * const FILEDAO_TESTCASE_FILE25_ID         = "10000025";
const char * const FILEDAO_TESTCASE_FILE26_ID         = "10000026";
const char * const FILEDAO_TESTCASE_FILE27_ID         = "10000027";
const char * const FILEDAO_TESTCASE_FILE28_ID         = "10000028";
const char * const FILEDAO_TESTCASE_FILE29_ID         = "10000029";
const char * const FILEDAO_TESTCASE_FILE30_ID         = "10000030";
const char * const FILEDAO_TESTCASE_FILE31_ID         = "10000031";
const char * const FILEDAO_TESTCASE_FILE32_ID         = "10000032";
const char * const FILEDAO_TESTCASE_FILE33_ID         = "10000033";
const char * const FILEDAO_TESTCASE_FILE34_ID         = "10000034";
const char * const FILEDAO_TESTCASE_FILE35_ID         = "10000035";
const char * const FILEDAO_TESTCASE_FILE36_ID         = "10000036";
const char * const FILEDAO_TESTCASE_FILE37_ID         = "10000037";
const char * const FILEDAO_TESTCASE_FILE38_ID         = "10000038";
const char * const FILEDAO_TESTCASE_FILE39_ID         = "10000039";
const char * const FILEDAO_TESTCASE_FILE40_ID         = "10000040";
const char * const FILEDAO_TESTCASE_FILE101_ID        = "10000101";
const char * const FILEDAO_TESTCASE_FILE102_ID        = "10000102";
const char * const FILEDAO_TESTCASE_FILE103_ID        = "10000103";
const char * const FILEDAO_TESTCASE_FILE104_ID        = "10000104";
const char * const FILEDAO_TESTCASE_FILE105_ID        = "10000105";
const char * const FILEDAO_TESTCASE_FILE106_ID        = "10000106";
const char * const FILEDAO_TESTCASE_FILE107_ID        = "10000107";
const char * const FILEDAO_TESTCASE_FILE108_ID        = "10000108";
const char * const FILEDAO_TESTCASE_FILE109_ID        = "10000109";
const char * const FILEDAO_TESTCASE_FILE110_ID        = "10000110";
const char * const FILEDAO_TESTCASE_FILE111_ID        = "10000111";
const char * const FILEDAO_TESTCASE_FILE112_ID        = "10000112";
const char * const FILEDAO_TESTCASE_FILE113_ID        = "10000113";
const char * const FILEDAO_TESTCASE_FILE114_ID        = "10000114";
const char * const FILEDAO_TESTCASE_FILE115_ID        = "10000115";
const char * const FILEDAO_TESTCASE_FILE116_ID        = "10000116";
const char * const FILEDAO_TESTCASE_FILE117_ID        = "10000117";
const char * const FILEDAO_TESTCASE_FILE118_ID        = "10000118";
const char * const FILEDAO_TESTCASE_FILE119_ID        = "10000119";
const char * const FILEDAO_TESTCASE_FILE120_ID        = "10000120";
const char * const FILEDAO_TESTCASE_FILE121_ID        = "10000121";
const char * const FILEDAO_TESTCASE_FILE122_ID        = "10000122";
const char * const FILEDAO_TESTCASE_FILE123_ID        = "10000123";
const char * const FILEDAO_TESTCASE_FILE124_ID        = "10000124";
const char * const FILEDAO_TESTCASE_FILE125_ID        = "10000125";
const char * const FILEDAO_TESTCASE_FILE126_ID        = "10000126";
const char * const FILEDAO_TESTCASE_FILE127_ID        = "10000127";
const char * const FILEDAO_TESTCASE_FILE128_ID        = "10000128";
const char * const FILEDAO_TESTCASE_FILE129_ID        = "10000129";
const char * const FILEDAO_TESTCASE_FILE130_ID        = "10000130";
const char * const FILEDAO_TESTCASE_FILE6001_ID       = "60000001";
const char * const FILEDAO_TESTCASE_FILE6002_ID       = "60000002";
const char * const FILEDAO_TESTCASE_FILE6003_ID       = "60000003";
const char * const FILEDAO_TESTCASE_CHANNEL1_NAME     = "FILE_channel_1";
const char * const FILEDAO_TESTCASE_CHANNEL2_NAME     = "FILE_channel_2";
const char * const FILEDAO_TESTCASE_CHANNEL3_NAME     = "FILE_channel_3_SEL";
const char * const FILEDAO_TESTCASE_VO1_NAME          = "FILE_VO_1";
const char * const FILEDAO_TESTCASE_VO2_NAME          = "FILE_VO_2";
const char * const FILEDAO_TESTCASE_VO3_NAME          = "FILE_VO_3_SEL";

const int LOOP_ITERATIONS = 1000;

/**
 * Compare the first element of a std::pair
 */
template <typename T>
struct compare_key: std::unary_function<const std::pair<std::string,T>&, bool> {
    /**
     * Constructor
     */
    compare_key(const std::string& s):m_str(s){}
    /**
     * Call Operator
     */
    bool operator()(const std::pair<std::string,T>& p) const {
        return ( p.first == m_str )?true:false;
    }
private:
    const std::string& m_str;
};        
} // end global namespace

/*
 * setUp
 *
 * set-up test case
 */
void FileDaoTestCase::setUp(){
    
    try{
        // Create VO FileDAO instance
        m_voFileDAO.reset(
            dao::vo::DAOFactory::instance().createFileDAO(FILEDAO_TESTCASE_VO1_NAME, DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null VO FileDAO", m_voFileDAO.get() != 0);        
        // Create Channel FileDAO instance
        m_chFileDAO.reset(
            dao::channel::DAOFactory::instance().createFileDAO(FILEDAO_TESTCASE_CHANNEL1_NAME, DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null Channel FileDAO", m_chFileDAO.get() != 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void FileDaoTestCase::testGet_VO(){
    try{
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE1)", file.get() != 0);
        
        // Check the State is Pending (Precondition)
        CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid State (FILE1)", file->state == File::S_ACTIVE);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void FileDaoTestCase::testGet_VO_Error(){
    try{
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE3_ID));
        CPPUNIT_FAIL("This file shouldn't be reachable (File3)");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void FileDaoTestCase::testGet_VO_Stress(){
    try{
        
        struct timeval start_time,stop_time;
        long sec_diff,usec_diff;
                        
        // Perform iterations
        gettimeofday(&start_time,0);
        for(int i = 0; i < LOOP_ITERATIONS; ++i){
            // Get FileTransfer instance
            scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE1_ID));
            CPPUNIT_ASSERT_MESSAGE("Null File (FILE1)", file.get() != 0);
        }
        gettimeofday(&stop_time,0);
        if(start_time.tv_sec < stop_time.tv_sec){
            if(start_time.tv_usec < stop_time.tv_usec){
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec;
                usec_diff = stop_time.tv_usec - start_time.tv_usec;
            } else {
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec - 1;
                usec_diff = stop_time.tv_usec + 1000000 - start_time.tv_usec;
            }
        } else {
            sec_diff = 0;
            usec_diff = stop_time.tv_usec - start_time.tv_usec;
        }
        printf("File Get (VO) %d iterations in %d secs and %d usec\n",LOOP_ITERATIONS,(int)sec_diff,(int)usec_diff);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

void FileDaoTestCase::testGet_VO_2(){
    try{
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE6001_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE6001)", file.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (FILE6001)", file->state == File::S_FAILED);
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason (FILE6001)", file->reason == "http timeout");
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason Class (FILE6001)", file->reasonClass() == Error::HTTP_TIMEOUT);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Scope (FILE6001)", file->errorScope() == Error::S_SOURCE);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Phase (FILE6001)", file->errorPhase() == Error::P_TX_PREPARATION);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_VO(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE1)",file.get() != 0);
        
        // Update Some Properties
        file->checksum = "00000000";
        longlong file_size = 5000000000ll;
        file->fileSize = file_size;
        file->state = File::S_DONE;
        
        // Update File Transfer
        m_voFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_voFileDAO->get(FILEDAO_TESTCASE_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Checksum (FILE1)", file->checksum == "00000000");
        CPPUNIT_ASSERT_MESSAGE("Invalid Filesize (FILE1)", file->fileSize == (longlong)5000000000ll);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (FILE1)", file->state == File::S_DONE);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_VO_2(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE1)",file.get() != 0);
        
        // Update File Transfer
        m_voFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_voFileDAO->get(FILEDAO_TESTCASE_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE1)", file->id == FILEDAO_TESTCASE_FILE1_ID);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_VO_3(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE1)",file.get() != 0);
        
        time_t current;
        time(&current);
        file->currentFailures = 123;
        file->finishTime = current;
        // Update File Transfer
        m_voFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_voFileDAO->get(FILEDAO_TESTCASE_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE1)", file->id == FILEDAO_TESTCASE_FILE1_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Current Failures (FILE1)", file->currentFailures == 123);
        CPPUNIT_ASSERT_MESSAGE("Invalid File FinishTime (FILE1)", file->finishTime == current);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_VO_4(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE1)",file.get() != 0);
        
        time_t current;
        time(&current);
        file->state = File::S_WAITING_CATALOG_REGISTRATION;
        file->sourceSurl = "srm://srm.test.com/tmp/test.txt";
        file->destSurl = "srm://srm.test.com/tmp/test.txt-1";
        file->reason = "A Reason";
        file->reasonClass = Error::CATALOG_INTERNAL_ERROR;
        file->failures = 12345;
        file->currentFailures = 123;
        file->catalogFailures = 234;
        file->prestageFailures = 345;
        file->fileSize = 1234567890;
        file->checksum = "0x1212121212";
        file->finishTime = current;
        // Update File Transfer
        m_voFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_voFileDAO->get(FILEDAO_TESTCASE_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE1)", file->id == FILEDAO_TESTCASE_FILE1_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid File State (FILE1)", file->state == File::S_WAITING_CATALOG_REGISTRATION);
        CPPUNIT_ASSERT_MESSAGE("Invalid File SourceSurl (FILE1)", file->sourceSurl == "srm://srm.test.com/tmp/test.txt");
        CPPUNIT_ASSERT_MESSAGE("Invalid File DestSurl Failures (FILE1)", file->destSurl == "srm://srm.test.com/tmp/test.txt-1");
        CPPUNIT_ASSERT_MESSAGE("Invalid File Reason (FILE1)", file->reason == "A Reason");
        CPPUNIT_ASSERT_MESSAGE("Invalid File ReasonClass (FILE1)", file->reasonClass == Error::CATALOG_INTERNAL_ERROR);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Failures (FILE1)", file->failures == 12345);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Current Failures (FILE1)", file->currentFailures == 123);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Catalog Failures (FILE1)", file->catalogFailures == 234);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Prestage Failures (FILE1)", file->prestageFailures == 345);
        CPPUNIT_ASSERT_MESSAGE("Invalid File FileSize (FILE1)", file->fileSize == 1234567890);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Checksum (FILE1)", file->checksum == "0x1212121212");
        CPPUNIT_ASSERT_MESSAGE("Invalid File FinishTime (FILE1)", file->finishTime == current);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_VO_5(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE40_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE40)",file.get() != 0);
        
        file->internalParams = "test vo internal params";
        // Update File Transfer
        m_voFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_voFileDAO->get(FILEDAO_TESTCASE_FILE40_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE40)", file->id == FILEDAO_TESTCASE_FILE40_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid File InternalParams (FILE40)", file->internalParams == "test vo internal params");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_VO_6(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE40_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE40)",file.get() != 0);
        
        time_t current;
        time(&current);
        file->reason = "A Reason";
        file->reasonClass = Error::CATALOG_INTERNAL_ERROR;
        file->failures = 12345;
        file->currentFailures = 123;
        file->fileSize = 1234567890;
        file->finishTime = current;
        file->internalParams = "test another param";
        // Update File Transfer
        m_voFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_voFileDAO->get(FILEDAO_TESTCASE_FILE40_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File Reason (FILE40)", file->reason == "A Reason");
        CPPUNIT_ASSERT_MESSAGE("Invalid File ReasonClass (FILE40)", file->reasonClass == Error::CATALOG_INTERNAL_ERROR);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Failures (FILE40)", file->failures == 12345);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Current Failures (FILE40)", file->currentFailures == 123);
        CPPUNIT_ASSERT_MESSAGE("Invalid File FileSize (FILE40)", file->fileSize == 1234567890);
        CPPUNIT_ASSERT_MESSAGE("Invalid File FinishTime (FILE40)", file->finishTime == current);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Internal Params (FILE40)", file->internalParams == "test another param");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_reason(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE1)",file.get() != 0);
        
        const char * const r = "This is a long reason string, more than 1024 characters:"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789";
        
        file->reason = r;
        // Update File Transfer
        m_voFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_voFileDAO->get(FILEDAO_TESTCASE_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE1)", file->id == FILEDAO_TESTCASE_FILE1_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid File reason length (FILE1)", file->reason().length() < strlen(r));
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_VO_Error(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(new File(FILEDAO_TESTCASE_FILE3_ID,FILEDAO_TESTCASE_JOB2_ID,File::S_DONE));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE3)",file.get() != 0);
        
        // Update Some Properties
        file->checksum = "00000000";
        
        // Update File Transfer
        m_voFileDAO->update(*file);
        CPPUNIT_FAIL(" This file shouldn't be reachable (File3)");
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_VO_Stress(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE1)",file.get() != 0);
            
        // Update FileTransfer instance
        file->state = File::S_WAITING_CATALOG_REGISTRATION;
        file->sourceSurl = "srm://srm.test.com/tmp/test.txt";
        file->destSurl = "srm://srm.test.com/tmp/test.txt-1";
        file->reason = "A Reason";
        file->reasonClass = Error::CATALOG_INTERNAL_ERROR;
        file->failures = 12345;
        file->currentFailures = 123;
        file->catalogFailures = 234;
        file->prestageFailures = 345;
        file->fileSize = 1234567890;
        file->checksum = "0x1212121212";
                
        struct timeval start_time,stop_time;
        long sec_diff,usec_diff;
                        
        // Perform iterations
        gettimeofday(&start_time,0);
        for(int i = 0; i < LOOP_ITERATIONS; ++i){
            time_t current;
            time(&current);
            file->finishTime = current;
            
            // Update File Transfer
            m_voFileDAO->update(*file);
        }
        gettimeofday(&stop_time,0);
        if(start_time.tv_sec < stop_time.tv_sec){
            if(start_time.tv_usec < stop_time.tv_usec){
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec;
                usec_diff = stop_time.tv_usec - start_time.tv_usec;
            } else {
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec - 1;
                usec_diff = stop_time.tv_usec + 1000000 - start_time.tv_usec;
            }
        } else {
            sec_diff = 0;
            usec_diff = stop_time.tv_usec - start_time.tv_usec;
        }
        printf("File Update (VO) %d iterations in %d secs and %d usec\n",LOOP_ITERATIONS,(int)sec_diff,(int)usec_diff);
    
        // Commit Transaction
        DaoHelper::ctx().commit();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

void FileDaoTestCase::testUpdate_VO_7(){
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE6002_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE6002)",file.get() != 0);

        file->reason = "A Reason";
        file->reasonClass = Error::LOST;
        file->errorScope = Error::S_AGENT;
        file->errorPhase = Error::P_CATALOG_REGISTRATION;
        // Update File Transfer
        m_voFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_voFileDAO->get(FILEDAO_TESTCASE_FILE6002_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File Reason (FILE6002)", file->reason == "A Reason");
        CPPUNIT_ASSERT_MESSAGE("Invalid File Reason Class (FILE6002)", file->reasonClass == Error::LOST);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Error Scope (FILE6002)", file->errorScope() == Error::S_AGENT);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Error Phase (FILE6002)", file->errorPhase() == Error::P_CATALOG_REGISTRATION);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByJobId
 *
 * Test GetByJobId Method
 */
void FileDaoTestCase::testGetByJobId_VO(){
    
    StringArray ids;
    try{
        // Get By State
        m_voFileDAO->getByJobId(FILEDAO_TESTCASE_JOB1_ID,ids);
        CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: No File for the given job (JOB1)", !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid number of files (JOB1)", ids.size() == 2);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByJobId
 *
 * Test GetByJobId Method
 */
void FileDaoTestCase::testGetByJobId_VO_Error(){
    
    StringArray ids;
    try{
        // Get By State
        m_voFileDAO->getByJobId(FILEDAO_TESTCASE_JOB2_ID,ids);
        CPPUNIT_FAIL("This job shouldn't be reachable (JOB2)");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetWaiting_VO
 *
 * Test GetWaiting Method
 */
void FileDaoTestCase::testGetWaiting_VO(){
    StringArray ids;
    try{
        // Get Waiting Files
        m_voFileDAO->getWaitingFiles(ids);
        CPPUNIT_ASSERT_MESSAGE("No Waiting files", !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of Waiting files", ids.size() == 3);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Id (FILE2)", 
            std::find(ids.begin(),ids.end(),FILEDAO_TESTCASE_FILE2_ID) != ids.end());
        CPPUNIT_ASSERT_MESSAGE("Invalid File Id (FILE6)", 
            std::find(ids.begin(),ids.end(),FILEDAO_TESTCASE_FILE6_ID) != ids.end());
        CPPUNIT_ASSERT_MESSAGE("Invalid File Id (FILE9)", 
            std::find(ids.begin(),ids.end(),FILEDAO_TESTCASE_FILE9_ID) != ids.end());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetWaiting_VO
 *
 * Test GetWaiting Method
 */
void FileDaoTestCase::testGetWaiting_VO_2(){
    StringArray ids;
    try{
        // Get Waiting Files
        m_voFileDAO->getWaitingFiles(ids,2);
        CPPUNIT_ASSERT_MESSAGE("No Waiting files", !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of Waiting files", ids.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Id (FILE2)", 
            std::find(ids.begin(),ids.end(),FILEDAO_TESTCASE_FILE2_ID) != ids.end());
        CPPUNIT_ASSERT_MESSAGE("Invalid File Id (FILE6)", 
            std::find(ids.begin(),ids.end(),FILEDAO_TESTCASE_FILE6_ID) != ids.end());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetWaitingCatalog_VO
 *
 * Test GetCatalogWaiting Method
 */
void FileDaoTestCase::testGetWaitingCatalog_VO(){
    dao::StringArrayTable ids;
    try{
        // Get Catalog Waiting Files
        m_voFileDAO->getCatalogWaitingFiles(ids);
        CPPUNIT_ASSERT_MESSAGE("No CatalogWaiting jobs", !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of Waiting jobs", ids.size() == 6);
        
        typedef compare_key<std::vector<std::string> > compare_StringArrayTable_key;
        
        // Check Jobs
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id (JOB7)", 
            std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB7_ID)) != ids.end());
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id (JOB8)", 
            std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB8_ID)) != ids.end());
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id (JOB9)", 
            std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB9_ID)) != ids.end());
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id (JOB10)", 
            std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB10_ID)) != ids.end());
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id (JOB11)", 
            std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB11_ID)) != ids.end());
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id (JOB12)", 
            std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB12_ID)) != ids.end());
        
        // Check Files
        dao::StringArrayTable::const_iterator it;
        // Job 7
        it = std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB7_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Files for JOB7", (*it).second.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE12)", (*it).second.at(0) == FILEDAO_TESTCASE_FILE12_ID);
        // Job 8
        it = std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB8_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Files for JOB8", (*it).second.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE13)", (*it).second.at(0) == FILEDAO_TESTCASE_FILE13_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE14)", (*it).second.at(1) == FILEDAO_TESTCASE_FILE14_ID);
        // Job 9
        it = std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB9_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Files for JOB9", (*it).second.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE15)", (*it).second.at(0) == FILEDAO_TESTCASE_FILE15_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE16)", (*it).second.at(1) == FILEDAO_TESTCASE_FILE16_ID);
        // Job 10
        it = std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB10_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Files for JOB10", (*it).second.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE20)", (*it).second.at(0) == FILEDAO_TESTCASE_FILE20_ID);
        // Job 11
        it = std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB11_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Files for JOB11", (*it).second.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE21)", (*it).second.at(0) == FILEDAO_TESTCASE_FILE21_ID);        
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE22)", (*it).second.at(1) == FILEDAO_TESTCASE_FILE22_ID);
        // Job 12
        it = std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB12_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Files for JOB12", (*it).second.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE24)", (*it).second.at(0) == FILEDAO_TESTCASE_FILE24_ID);
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testGetWaitingCatalog_VO_2
 *
 * Test GetCatalogWaiting Method
 */
void FileDaoTestCase::testGetWaitingCatalog_VO_2(){
    dao::StringArrayTable ids;
    try{
        // Get Catalog Waiting Files
        m_voFileDAO->getCatalogWaitingFiles(ids,2);
        CPPUNIT_ASSERT_MESSAGE("No CatalogWaiting jobs", !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of Waiting jobs", ids.size() == 2);
        
        typedef compare_key<std::vector<std::string> > compare_StringArrayTable_key;
        
        // Check Jobs
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id (JOB7)", 
            std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB7_ID)) != ids.end());
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id (JOB8)", 
            std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB8_ID)) != ids.end());
        
        // Check Files
        dao::StringArrayTable::const_iterator it;
        // Job 7
        it = std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB7_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Files for JOB7", (*it).second.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE12)", (*it).second.at(0) == FILEDAO_TESTCASE_FILE12_ID);
        // Job 8
        it = std::find_if(ids.begin(),ids.end(),compare_StringArrayTable_key(FILEDAO_TESTCASE_JOB8_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Files for JOB8", (*it).second.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE13)", (*it).second.at(0) == FILEDAO_TESTCASE_FILE13_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE14)", (*it).second.at(1) == FILEDAO_TESTCASE_FILE14_ID);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void FileDaoTestCase::testGet_Ch(){
    try{
        // Get FileTransfer instance
        scoped_ptr<File> file(m_chFileDAO->get(FILEDAO_TESTCASE_FILE5_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE5)", file.get() != 0);
        
        // Check the State is Pending (Precondition)
        CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid State (FILE5)", file->state == File::S_PENDING);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void FileDaoTestCase::testGet_Ch_Error(){
    try{
        // Get FileTransfer instance
        scoped_ptr<File> file(m_chFileDAO->get(FILEDAO_TESTCASE_FILE3_ID));
        CPPUNIT_FAIL("This file shouldn't be reachable (File3)");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

void FileDaoTestCase::testGet_Ch_2(){
    try{
        // Get FileTransfer instance
        scoped_ptr<File> file(m_chFileDAO->get(FILEDAO_TESTCASE_FILE6001_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE6001)", file.get() != 0);
        
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE6001)", file.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (FILE6001)", file->state == File::S_FAILED);
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason (FILE6001)", file->reason == "http timeout");
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason Class (FILE6001)", file->reasonClass() == Error::HTTP_TIMEOUT);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Scope (FILE6001)", file->errorScope() == Error::S_SOURCE);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Phase (FILE6001)", file->errorPhase() == Error::P_TX_PREPARATION);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }    
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_Ch(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_chFileDAO->get(FILEDAO_TESTCASE_FILE6_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE6)",file.get() != 0);
        
        // Update Some Properties
        file->checksum = "00000000";
        longlong file_size = 5000000000ll;
        file->fileSize = file_size;
        file->state = File::S_PENDING;
        
        // Update File Transfer
        m_chFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_chFileDAO->get(FILEDAO_TESTCASE_FILE6_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Checksum (FILE6)", file->checksum == "00000000");
        CPPUNIT_ASSERT_MESSAGE("Invalid Filesize (FILE6)", file->fileSize == (longlong)5000000000ll);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (FILE6)", file->state == File::S_PENDING);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_Ch_2(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE6_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE6)",file.get() != 0);
        
        // Update File Transfer
        m_chFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_voFileDAO->get(FILEDAO_TESTCASE_FILE6_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE6)", file->id == FILEDAO_TESTCASE_FILE6_ID);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_Ch_3(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_voFileDAO->get(FILEDAO_TESTCASE_FILE6_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE6)",file.get() != 0);
        
        // Update Some Properties
        time_t current;
        time(&current);
        file->currentFailures = 321;
        file->finishTime = current;
        
        // Update File Transfer
        m_chFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_voFileDAO->get(FILEDAO_TESTCASE_FILE6_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE6)", file->id == FILEDAO_TESTCASE_FILE6_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid File current failures (FILE6)", file->currentFailures == 321);
        CPPUNIT_ASSERT_MESSAGE("Invalid File FinishTime (FILE6)", file->finishTime == current);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_Ch_4(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_chFileDAO->get(FILEDAO_TESTCASE_FILE6_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE6)",file.get() != 0);
        
        time_t current;
        time(&current);
        file->state = File::S_WAITING;
        file->reason = "Another Reason";
        file->reasonClass = Error::STORAGE_INTERNAL_ERROR;
        file->failures = 123450;
        file->currentFailures = 1230;
        file->fileSize = 12345678901LL;
        file->checksum = "0x12121212123";
        file->finishTime = current;
        // Update File Transfer
        m_chFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_chFileDAO->get(FILEDAO_TESTCASE_FILE6_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE6)", file->id == FILEDAO_TESTCASE_FILE6_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid File State (FILE6)", file->state == File::S_WAITING);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Reason (FILE6)", file->reason == "Another Reason");
        CPPUNIT_ASSERT_MESSAGE("Invalid File ReasonClass (FILE6)", file->reasonClass == Error::STORAGE_INTERNAL_ERROR);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Failures (FILE6)", file->failures == 123450);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Current Failures (FILE6)", file->currentFailures == 1230);
        CPPUNIT_ASSERT_MESSAGE("Invalid File FileSize (FILE6)", file->fileSize == 12345678901LL);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Checksum (FILE6)", file->checksum == "0x12121212123");
        CPPUNIT_ASSERT_MESSAGE("Invalid File FinishTime (FILE6)", file->finishTime == current);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_Ch_5(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_chFileDAO->get(FILEDAO_TESTCASE_FILE40_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE40)",file.get() != 0);
        
        file->internalParams = "test channel internal params";
        // Update File Transfer
        m_chFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_chFileDAO->get(FILEDAO_TESTCASE_FILE40_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE40)", file->id == FILEDAO_TESTCASE_FILE40_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid File internal Param (FILE40)", file->internalParams == "test channel internal params");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_Ch_6(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_chFileDAO->get(FILEDAO_TESTCASE_FILE40_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE40)",file.get() != 0);
        
        file->state = File::S_WAITING;
        file->reason = "Another Channel Reason";
        file->reasonClass = Error::STORAGE_INTERNAL_ERROR;
        file->failures = 4;
        file->currentFailures = 4;
        file->fileSize = 4444;
        file->checksum = "0x444444";
        file->internalParams = "test another channel internal params";
        // Update File Transfer
        m_chFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_chFileDAO->get(FILEDAO_TESTCASE_FILE40_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (FILE40)", file->id == FILEDAO_TESTCASE_FILE40_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid File State (FILE40)", file->state == File::S_WAITING);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Reason (FILE40)", file->reason == "Another Channel Reason");
        CPPUNIT_ASSERT_MESSAGE("Invalid File ReasonClass (FILE40)", file->reasonClass == Error::STORAGE_INTERNAL_ERROR);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Failures (FILE40)", file->failures == 4);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Current Failures (FILE40)", file->currentFailures == 4);
        CPPUNIT_ASSERT_MESSAGE("Invalid File FileSize (FILE40)", file->fileSize == 4444);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Checksum (FILE40)", file->checksum == "0x444444");
        CPPUNIT_ASSERT_MESSAGE("Invalid File InternalParams (FILE40)", file->internalParams == "test another channel internal params");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void FileDaoTestCase::testUpdate_Ch_Error(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(new File(FILEDAO_TESTCASE_FILE3_ID,FILEDAO_TESTCASE_JOB2_ID,File::S_DONE));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE3)",file.get() != 0);
        
        // Update Some Properties
        file->checksum = "00000000";
        
        // Update File Transfer
        m_chFileDAO->update(*file);
        CPPUNIT_FAIL(" This file shouldn't be reachable (File3)");
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

void FileDaoTestCase::testUpdate_Ch_7(){
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get FileTransfer instance
        scoped_ptr<File> file(m_chFileDAO->get(FILEDAO_TESTCASE_FILE6003_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE6003)",file.get() != 0);

        file->reason = "Another Reason";
        file->reasonClass = Error::NO_SPACE_LEFT;
        file->errorScope = Error::S_DESTINATION;
        file->errorPhase = Error::P_TX_PREPARATION;
        // Update File Transfer
        m_voFileDAO->update(*file);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again        
        file.reset(m_voFileDAO->get(FILEDAO_TESTCASE_FILE6003_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File Reason (FILE6003)", file->reason == "Another Reason");
        CPPUNIT_ASSERT_MESSAGE("Invalid File Reason Class (FILE6003)", file->reasonClass == Error::NO_SPACE_LEFT);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Error Scope (FILE6003)", file->errorScope() == Error::S_DESTINATION);
        CPPUNIT_ASSERT_MESSAGE("Invalid File Error Phase (FILE6003)", file->errorPhase() == Error::P_TX_PREPARATION);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }    
}

/*
 * testGetByJobId
 *
 * Test GetByJobId Method
 */
void FileDaoTestCase::testGetByJobId_Ch(){
    
    StringArray ids;
    try{
        // Get By State
        m_chFileDAO->getByJobId(FILEDAO_TESTCASE_JOB1_ID,ids);
        CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: No File for the given job (JOB1)", !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid number of files (JOB1)", ids.size() == 2);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByJobId
 *
 * Test GetByJobId Method
 */
void FileDaoTestCase::testGetByJobId_Ch_Error(){
    
    StringArray ids;
    try{
        // Get By State
        m_chFileDAO->getByJobId(FILEDAO_TESTCASE_JOB2_ID,ids);
        CPPUNIT_FAIL("This job shouldn't be reachable (JOB2)");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetNextToTransfer_Ch
 *
 * Test GetNextToTransfer Method
 */
void FileDaoTestCase::testGetNextJobToTransfer_Ch(){
    std::string job_id;
    glite::data::agents::dao::StringArray ids;
    try{
        // Get Next To Transfer
        m_chFileDAO->getNextJobToTransfer(FILEDAO_TESTCASE_VO1_NAME,job_id,ids);
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id (1)", job_id == FILEDAO_TESTCASE_JOB13_ID);
        CPPUNIT_ASSERT_MESSAGE("No files id returned (1)", !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid numbr of files id (1)", ids.size() == 3);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #0 (1)", ids.at(0) == FILEDAO_TESTCASE_FILE26_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #1 (1)", ids.at(1) == FILEDAO_TESTCASE_FILE27_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #2 (1)", ids.at(2) == FILEDAO_TESTCASE_FILE28_ID);

        job_id.clear();
        ids.clear();
        m_chFileDAO->getNextJobToTransfer(FILEDAO_TESTCASE_VO1_NAME,job_id,ids,2);
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id (2)", job_id == FILEDAO_TESTCASE_JOB13_ID);
        CPPUNIT_ASSERT_MESSAGE("No files id returned (2)", !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid numbr of files id (2)", ids.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #0 (2)", ids.at(0) == FILEDAO_TESTCASE_FILE26_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #1 (2)", ids.at(1) == FILEDAO_TESTCASE_FILE27_ID);

        job_id.clear();
        ids.clear();
        m_chFileDAO->getNextJobToTransfer(FILEDAO_TESTCASE_VO2_NAME,job_id,ids,3);
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id (3)", job_id == FILEDAO_TESTCASE_JOB14_ID);
        CPPUNIT_ASSERT_MESSAGE("No files id returned (3)", !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid numbr of files id (3)", ids.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #0 (3)", ids.at(0) == FILEDAO_TESTCASE_FILE30_ID);
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetFilesToCancel_VO
 *
 * Test GetFilesToCancel Method
 */
void FileDaoTestCase::testGetFilesToCancel_Ch(){
    dao::StringArrayTable ids;
    try{
        // Get File sto Cancel
        m_chFileDAO->getFilesToCancel(ids);
        CPPUNIT_ASSERT_MESSAGE("No files to cancel on Channel (1)", !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of files to cancel returned (1)", ids.size() == 3);
        CPPUNIT_ASSERT_MESSAGE("Invalid job id #0 (1)", ids.at(0).first == FILEDAO_TESTCASE_JOB15_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid job id #1 (1)", ids.at(1).first == FILEDAO_TESTCASE_JOB16_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid job id #2 (1)", ids.at(2).first == FILEDAO_TESTCASE_JOB17_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #0:0 (1)", ids.at(0).second.at(0) == FILEDAO_TESTCASE_FILE31_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #0:1 (1)", ids.at(0).second.at(1) == FILEDAO_TESTCASE_FILE32_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #1:0 (1)", ids.at(1).second.at(0) == FILEDAO_TESTCASE_FILE37_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #2:0 (1)", ids.at(2).second.at(0) == FILEDAO_TESTCASE_FILE38_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #2:1 (1)", ids.at(2).second.at(1) == FILEDAO_TESTCASE_FILE39_ID);
        
        ids.clear();
        m_chFileDAO->getFilesToCancel(ids,2);
        CPPUNIT_ASSERT_MESSAGE("No files to cancel on Channel (2)", !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of files to cancel returned (2)", ids.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid job id #0 (2)", ids.at(0).first == FILEDAO_TESTCASE_JOB15_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #0:0 (2)", ids.at(0).second.at(0) == FILEDAO_TESTCASE_FILE31_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #0:1 (2)", ids.at(0).second.at(1) == FILEDAO_TESTCASE_FILE32_ID);

    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


// #warning "possibly se_limit is buggy..."
/*

e.g. the query could be like this... i.e. also counting the 'active' files.

SELECT f_id,j_id FROM (
    SELECT t_file.file_id AS f_id, t_file.job_id AS j_id, ROW_NUMBER() OVER (PARTITION BY t_file.job_id ORDER BY t_file.file_id ) FileCount
    FROM t_file
    WHERE t_file.job_id IN (
        SELECT j_id FROM (
            SELECT j_id, JobCount, ROW_NUMBER() OVER (ORDER BY j_p DESC, j_st) NumJobs
            FROM (
                SELECT t_job.job_id AS j_id, t_job.source_se AS src_se, t_job.dest_se AS dst_se, t_job.priority AS j_p, t_job.submit_time AS j_st, 
                  ROW_NUMBER() OVER (PARTITION BY t_job.source_se,  t_job.dest_se ORDER BY t_job.priority DESC, SYS_EXTRACT_UTC(t_job.submit_time), t_job.job_id) JobCount
                FROM t_job
                WHERE t_job.channel_name = (
                    SELECT t_channel.channel_name
                    FROM t_channel
                    WHERE t_channel.channel_name = 'FILE_channel_3_SEL' -- 1 = channel_name
                    AND t_channel.channel_state IN ('Active','Drain')
                )
                AND t_job.vo_name = 'FILE_VO_3_SEL' -- 2 = vo_name
                AND t_job.job_state IN ('Ready','Active')
                AND ((t_job.cancel_job IS NULL) OR (t_job.cancel_job = 'N'))
                AND EXISTS (
                    SELECT NULL FROM t_file
                    WHERE t_file.job_id = t_job.job_id
                    AND t_file.file_state = 'Ready' OR t_file.file_state = 'Active' -- <= LOOK HERE!!!
                )
            ) WHERE JobCount = 1
        ) WHERE (0=0 OR NumJobs<=0) -- 3,4 = limit
    ) AND t_file.file_state = 'Ready'
) WHERE ( 0=2 OR FileCount<=2 ) -- 5,6 = se_limit

*/


/*
 * testGetJobsToTransferPerSE_Ch
 *
 * Test GetJobsToTransferPerSE Method
 */
void FileDaoTestCase::testGetJobsToTransferPerSE_Ch(){
    StringArrayTable ids;
    try{
        // Create Channel FileDAO instance
        scoped_ptr<dao::channel::FileDAO> chFileDAO(
            dao::channel::DAOFactory::instance().createFileDAO(FILEDAO_TESTCASE_CHANNEL3_NAME, DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null Channel FileDAO", chFileDAO.get() != 0);
        
        // Get Next To Transfer
        chFileDAO->getJobsToTransferPerSE(FILEDAO_TESTCASE_VO3_NAME,ids,2);
        CPPUNIT_ASSERT_MESSAGE("Invalid number of Jobs (1)"       , ids.size() == 5);
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id #0 (1)"            , ids.at(0).first == FILEDAO_TESTCASE_JOB101_ID);
        CPPUNIT_ASSERT_MESSAGE("No files id returned #0 (1)"      ,!ids.at(0).second.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of files id #0 (1)", ids.at(0).second.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #0.0 (1)"         , ids.at(0).second.at(0) == FILEDAO_TESTCASE_FILE101_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #0.1 (1)"         , ids.at(0).second.at(1) == FILEDAO_TESTCASE_FILE102_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id #1 (1)"            , ids.at(1).first == FILEDAO_TESTCASE_JOB103_ID);
        CPPUNIT_ASSERT_MESSAGE("No files id returned #1 (1)"      ,!ids.at(1).second.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of files id #1 (1)", ids.at(1).second.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #1.0 (1)"         , ids.at(1).second.at(0) == FILEDAO_TESTCASE_FILE107_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #1.1 (1)"         , ids.at(1).second.at(1) == FILEDAO_TESTCASE_FILE108_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id #2 (1)"            , ids.at(2).first == FILEDAO_TESTCASE_JOB105_ID);
        CPPUNIT_ASSERT_MESSAGE("No files id returned #2 (1)"      ,!ids.at(2).second.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of files id #2 (1)", ids.at(2).second.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #2.0 (1)"         , ids.at(2).second.at(0) == FILEDAO_TESTCASE_FILE113_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #2.1 (1)"         , ids.at(2).second.at(1) == FILEDAO_TESTCASE_FILE114_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id #3 (1)"            , ids.at(3).first == FILEDAO_TESTCASE_JOB107_ID);
        CPPUNIT_ASSERT_MESSAGE("No files id returned #3 (1)"      ,!ids.at(3).second.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of files id #3 (1)", ids.at(3).second.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #3.0 (1)"         , ids.at(3).second.at(0) == FILEDAO_TESTCASE_FILE119_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #3.1 (1)"         , ids.at(3).second.at(1) == FILEDAO_TESTCASE_FILE120_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id #4 (1)"            , ids.at(4).first == FILEDAO_TESTCASE_JOB109_ID);
        CPPUNIT_ASSERT_MESSAGE("No files id returned #4 (1)"      ,!ids.at(4).second.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of files id #4 (1)", ids.at(4).second.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #4.0 (1)"         , ids.at(4).second.at(0) == FILEDAO_TESTCASE_FILE125_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #4.1 (1)"         , ids.at(4).second.at(1) == FILEDAO_TESTCASE_FILE126_ID);

        ids.clear();
        // Get Next To Transfer
        chFileDAO->getJobsToTransferPerSE(FILEDAO_TESTCASE_VO3_NAME,ids,2,3); // se_limit = 2, limit = 3
        CPPUNIT_ASSERT_MESSAGE("Invalid number of Jobs (1)"       , ids.size() == 3);
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id #0 (1)"            , ids.at(0).first == FILEDAO_TESTCASE_JOB101_ID);
        CPPUNIT_ASSERT_MESSAGE("No files id returned #0 (1)"      ,!ids.at(0).second.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of files id #0 (1)", ids.at(0).second.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #0.0 (1)"         , ids.at(0).second.at(0) == FILEDAO_TESTCASE_FILE101_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #0.1 (1)"         , ids.at(0).second.at(1) == FILEDAO_TESTCASE_FILE102_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id #1 (1)"            , ids.at(1).first == FILEDAO_TESTCASE_JOB103_ID);
        CPPUNIT_ASSERT_MESSAGE("No files id returned #1 (1)"      ,!ids.at(1).second.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of files id #1 (1)", ids.at(1).second.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #1.0 (1)"         , ids.at(1).second.at(0) == FILEDAO_TESTCASE_FILE107_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #1.1 (1)"         , ids.at(1).second.at(1) == FILEDAO_TESTCASE_FILE108_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid Job Id #2 (1)"            , ids.at(2).first == FILEDAO_TESTCASE_JOB105_ID);
        CPPUNIT_ASSERT_MESSAGE("No files id returned #2 (1)"      ,!ids.at(2).second.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number of files id #2 (1)", ids.at(2).second.size() == 2);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #2.0 (1)"         , ids.at(2).second.at(0) == FILEDAO_TESTCASE_FILE113_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid file id #2.1 (1)"         , ids.at(2).second.at(1) == FILEDAO_TESTCASE_FILE114_ID);
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
