/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: JobDAO Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: JobDaoTestCase.cpp,v 1.32 2009/07/06 17:15:20 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "DaoHelper.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/dao/vo/JobDAO.h"
#include "glite/data/transfer/agent/dao/channel/JobDAO.h"
#include "glite/data/transfer/agent/dao/cred/JobDAO.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"
#include "glite/data/transfer/agent/dao/cred/DAOFactory.h"

#include <sys/time.h>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

#include "dao_test_utils.h"
#include <cstring>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::agents::dao;

using boost::scoped_ptr;

/**
 * CPPUnit JobDaoTestCase
 */
class JobDaoTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( JobDaoTestCase);
        CPPUNIT_TEST( testGet_VO );
        CPPUNIT_TEST( testGet_VO_2 );
        CPPUNIT_TEST( testGet_VO_3 );
        CPPUNIT_TEST( testGet_VO_Error );
        CPPUNIT_TEST( testGet_VO_Stress );
        CPPUNIT_TEST( testGet_VO_Finished );
        CPPUNIT_TEST( testUpdate_VO );
        CPPUNIT_TEST( testUpdate_VO_2 );
        CPPUNIT_TEST( testUpdate_VO_3 );
        CPPUNIT_TEST( testUpdate_reason );
        CPPUNIT_TEST( testUpdate_VO_4 );
        CPPUNIT_TEST( testUpdate_VO_5 );
        CPPUNIT_TEST( testUpdate_VO_Error );
        CPPUNIT_TEST( testUpdate_VO_Stress );
        CPPUNIT_TEST( testUpdateStates_VO );
        CPPUNIT_TEST( testUpdateStates_VO_2 );
        CPPUNIT_TEST( testUpdateStates_VO_Error );
        CPPUNIT_TEST( testGetSubmitted_VO );
        CPPUNIT_TEST( testGetSubmitted_VO_2 );
        CPPUNIT_TEST( testGetPending_VO );
        CPPUNIT_TEST( testGetPending_VO_2 );
        CPPUNIT_TEST( testGetDone_VO );
        CPPUNIT_TEST( testGetDone_VO_2 );
        CPPUNIT_TEST( testGetFinishing_VO );
        CPPUNIT_TEST( testGetFinishing_VO_2 );
        // Channel Tests
        CPPUNIT_TEST( testGet_Ch );
        CPPUNIT_TEST( testGet_Ch_2 );
        CPPUNIT_TEST( testGet_Ch_3 );
        CPPUNIT_TEST( testGet_Ch_Error );
        CPPUNIT_TEST( testGet_Ch_Finished );
        CPPUNIT_TEST( testUpdate_Ch );
        CPPUNIT_TEST( testUpdate_Ch_2 );
        CPPUNIT_TEST( testUpdate_Ch_3 );
        CPPUNIT_TEST( testUpdate_Ch_Error );
        CPPUNIT_TEST( testUpdateStates_Ch );
        CPPUNIT_TEST( testUpdateStates_Ch_Error );
        CPPUNIT_TEST( testGetJobsToFix_Ch );
        // Channel Tests
        CPPUNIT_TEST( testGetLast_Cred );
        CPPUNIT_TEST( testGetLast_Cred_Error );
        CPPUNIT_TEST( testGetLast_Cred_Stress );
        // schema version 3.3.0
        CPPUNIT_TEST( testGet_VO_schema_3_3_0 );
        CPPUNIT_TEST( testGet_Ch_schema_3_3_0 );
        // schema version 3.4.1
        CPPUNIT_TEST( testGet_compare_checksum_3_4_1 );
    CPPUNIT_TEST_SUITE_END();

    // VO Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_VO();
    void testGet_VO_2();
    void testGet_VO_3();
    void testGet_VO_Error();
    void testGet_VO_Stress();
    void testGet_VO_Finished();
    /**
     * Test Update Method
     */
    void testUpdate_VO();
    void testUpdate_VO_2();
    void testUpdate_VO_3();
    void testUpdate_VO_4();
    void testUpdate_VO_5();
    void testUpdate_reason();
    void testUpdate_VO_Error();
    void testUpdate_VO_Stress();
    /**
     * Test UpdateStates Method
     */
    void testUpdateStates_VO();
    void testUpdateStates_VO_2();
    void testUpdateStates_VO_Error();
    /**
     * Test GetSubmittedJobs Method
     */
    void testGetSubmitted_VO();
    void testGetSubmitted_VO_2();
    /**
     * Test GetPendingJobs Method
     */
    void testGetPending_VO();
    void testGetPending_VO_2();
    /**
     * Test GetDoneJobs Method
     */
    void testGetDone_VO();
    void testGetDone_VO_2();
    /**
     * Test GetFinishingJobs Method
     */
    void testGetFinishing_VO();
    void testGetFinishing_VO_2();

    // Channel Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_Ch();
    void testGet_Ch_2();
    void testGet_Ch_3();
    void testGet_Ch_Error();
    void testGet_Ch_Finished();
    /**
     * Test Update Method
     */
    void testUpdate_Ch();
    void testUpdate_Ch_2();
    void testUpdate_Ch_3();
    void testUpdate_Ch_Error();
    /**
     * Test UpdateStates Method
     */
    void testUpdateStates_Ch();
    void testUpdateStates_Ch_Error();

    /**
     * Test UpdateStates Method
     */
    void testGetJobsToFix_Ch();

    // Cred Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGetLast_Cred();
    void testGetLast_Cred_Error();
    void testGetLast_Cred_Stress();

    /**
     * Schema 3.3.0
     */
    void testGet_VO_schema_3_3_0();
    void testGet_Ch_schema_3_3_0();

    /**
     * Schema 3.4.1
     */
    void testGet_compare_checksum_3_4_1();

private:
    /**
     * The VO JobDAO Instance
     */
    scoped_ptr<dao::vo::JobDAO> m_voJobDAO;
    /**
     * The Channel JobDAO Instance
     */
    scoped_ptr<dao::channel::JobDAO> m_chJobDAO;
    /**
     * The Cred JobDAO Instance
     */
    scoped_ptr<dao::cred::JobDAO> m_credJobDAO;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( JobDaoTestCase, "JobDaoTestCase" );

namespace {
const char * const JOBDAO_TESTCASE_JOB1_ID           = "10000000-0000-0000-0000-000000000001";
const char * const JOBDAO_TESTCASE_JOB2_ID           = "10000000-0000-0000-0000-000000000002";
const char * const JOBDAO_TESTCASE_JOB3_ID           = "10000000-0000-0000-0000-000000000003";
const char * const JOBDAO_TESTCASE_JOB4_ID           = "10000000-0000-0000-0000-000000000004";
const char * const JOBDAO_TESTCASE_JOB5_ID           = "10000000-0000-0000-0000-000000000005";
const char * const JOBDAO_TESTCASE_JOB6_ID           = "10000000-0000-0000-0000-000000000006";
const char * const JOBDAO_TESTCASE_JOB7_ID           = "10000000-0000-0000-0000-000000000007";
const char * const JOBDAO_TESTCASE_JOB8_ID           = "10000000-0000-0000-0000-000000000008";
const char * const JOBDAO_TESTCASE_JOB9_ID           = "10000000-0000-0000-0000-000000000009";
const char * const JOBDAO_TESTCASE_JOB10_ID          = "10000000-0000-0000-0000-000000000010";
const char * const JOBDAO_TESTCASE_JOB11_ID          = "10000000-0000-0000-0000-000000000011";
const char * const JOBDAO_TESTCASE_JOB12_ID          = "10000000-0000-0000-0000-000000000012";
const char * const JOBDAO_TESTCASE_JOB13_ID          = "10000000-0000-0000-0000-000000000013";
const char * const JOBDAO_TESTCASE_JOB14_ID          = "10000000-0000-0000-0000-000000000014";
const char * const JOBDAO_TESTCASE_JOB15_ID          = "10000000-0000-0000-0000-000000000015";
const char * const JOBDAO_TESTCASE_JOB16_ID          = "10000000-0000-0000-0000-000000000016";
const char * const JOBDAO_TESTCASE_JOB17_ID          = "10000000-0000-0000-0000-000000000017";
const char * const JOBDAO_TESTCASE_JOB18_ID          = "10000000-0000-0000-0000-000000000018";
const char * const JOBDAO_TESTCASE_JOB19_ID          = "10000000-0000-0000-0000-000000000019";
const char * const JOBDAO_TESTCASE_JOB20_ID          = "10000000-0000-0000-0000-000000000020";
const char * const JOBDAO_TESTCASE_JOB21_ID          = "10000000-0000-0000-0000-000000000021";
const char * const JOBDAO_TESTCASE_JOB22_ID          = "10000000-0000-0000-0000-000000000022";
const char * const JOBDAO_TESTCASE_JOB23_ID          = "10000000-0000-0000-0000-000000000023";
const char * const JOBDAO_TESTCASE_JOB24_ID          = "10000000-0000-0000-0000-000000000024";
const char * const JOBDAO_TESTCASE_JOB25_ID          = "10000000-0000-0000-0000-000000000025";
const char * const JOBDAO_TESTCASE_JOB26_ID          = "10000000-0000-0000-0000-000000000026";
const char * const JOBDAO_TESTCASE_JOB27_ID          = "10000000-0000-0000-0000-000000000027";
const char * const JOBDAO_TESTCASE_JOB28_ID          = "10000000-0000-0000-0000-000000000028";
const char * const JOBDAO_TESTCASE_JOB29_ID          = "10000000-0000-0000-0000-000000000029";
const char * const JOBDAO_TESTCASE_JOB30_ID          = "10000000-0000-0000-0000-000000000030";
const char * const JOBDAO_TESTCASE_JOB31_ID          = "10000000-0000-0000-0000-000000000031";
const char * const JOBDAO_TESTCASE_JOB601_ID         = "60000000-0000-0000-0000-000000000001";
const char * const JOBDAO_TESTCASE_FILE1_ID          = "20000001";
const char * const JOBDAO_TESTCASE_FILE2_ID          = "20000002";
const char * const JOBDAO_TESTCASE_FILE3_ID          = "20000003";
const char * const JOBDAO_TESTCASE_FILE4_ID          = "20000004";
const char * const JOBDAO_TESTCASE_FILE5_ID          = "20000005";
const char * const JOBDAO_TESTCASE_FILE6_ID          = "20000006";
const char * const JOBDAO_TESTCASE_FILE7_ID          = "20000007";
const char * const JOBDAO_TESTCASE_FILE8_ID          = "20000008";
const char * const JOBDAO_TESTCASE_FILE9_ID          = "20000009";
const char * const JOBDAO_TESTCASE_FILE10_ID         = "20000010";
const char * const JOBDAO_TESTCASE_FILE11_ID         = "20000011";
const char * const JOBDAO_TESTCASE_CHANNEL1_NAME     = "JOB_channel_1";
const char * const JOBDAO_TESTCASE_CHANNEL2_NAME     = "JOB_channel_2";
const char * const JOBDAO_TESTCASE_VO1_NAME          = "JOB_VO_1";
const char * const JOBDAO_TESTCASE_VO2_NAME          = "JOB_VO_2";
const char * const JOBDAO_TESTCASE_SUBMITHOST        = "fts.cern.ch";
const char * const JOBDAO_TESTCASE_USER_DN_1         = "DN=A User";
const char * const JOBDAO_TESTCASE_CRED_ID_1         = "1234567";
const unsigned int JOBDAO_TESTCASE_MAXTIMEINQUEUE = 123456;

const int LOOP_ITERATIONS = 1000;
}

/*
 * setUp
 *
 * set-up test case
 */
void JobDaoTestCase::setUp(){

    // Create VO FileDAO instance
    m_voJobDAO.reset(
        vo::DAOFactory::instance().createJobDAO(JOBDAO_TESTCASE_VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null VO JobDAO", m_voJobDAO.get() != 0);
    // Create Channel JobDAO instance
    m_chJobDAO.reset(
        channel::DAOFactory::instance().createJobDAO(JOBDAO_TESTCASE_CHANNEL1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null Channel JobDAO", m_chJobDAO.get() != 0);
    // Create Cred JobDAO instance
    m_credJobDAO.reset(
        cred::DAOFactory::instance().createJobDAO(DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null Cred JobDAO", m_chJobDAO.get() != 0);

}

/*
 * testGet
 *
 * Test Get Method
 */
void JobDaoTestCase::testGet_VO(){

    // Get TransferJob instance
    scoped_ptr<Job> job(m_voJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB1)", job.get() != 0);

    // Check the State is Active(Precondition)
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid State (JOB1)", job->state == Job::S_READY);
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Submit host (JOB1)", job->submitHost == JOBDAO_TESTCASE_SUBMITHOST);
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Max time in queue (JOB1)", job->maxTimeInQueue == JOBDAO_TESTCASE_MAXTIMEINQUEUE);

}

/*
 * testGet
 *
 * Test Get Method
 */
void JobDaoTestCase::testGet_VO_2(){

    // Get TransferJob instance
    scoped_ptr<Job> job(m_voJobDAO->get(JOBDAO_TESTCASE_JOB30_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB30)", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Overwrite Flag (JOB30)", job->overwriteFlag == false);

    // Get TransferJob instance
    job.reset(m_voJobDAO->get(JOBDAO_TESTCASE_JOB31_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB31)", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Overwrite Flag (JOB31)", job->overwriteFlag == true);

    // Get TransferJob instance
    job.reset(m_voJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB1)", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Overwrite Flag (JOB30)", job->overwriteFlag == false);

}

/**
 * testGet_VO_3
 */
void JobDaoTestCase::testGet_VO_3(){

    static const char * job_id = "80000000-0000-0000-0000-000000000001";
    static const char * sst = "someSourceSpaceToken";

    // Get TransferJob instance
    scoped_ptr<Job> job(m_voJobDAO->get(job_id));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid sourceSpaceToken", job->sourceSpaceToken() == sst);

}

/*
 * testGet
 *
 * Test Get Method
 */
void JobDaoTestCase::testGet_VO_Error(){
    try{
        // Get Job instance
        scoped_ptr<Job> job(m_voJobDAO->get(JOBDAO_TESTCASE_JOB3_ID));
        CPPUNIT_FAIL("This Job shouldn't be reachable (JOB3)");
    } catch(const DAOException& exc){
        // OK
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void JobDaoTestCase::testGet_VO_Stress(){
    try{
        struct timeval start_time,stop_time;
        long sec_diff,usec_diff;

        // Perform iterations
        gettimeofday(&start_time,0);
        for(int i = 0; i < LOOP_ITERATIONS; ++i){
            // Get FileTransfer instance
            scoped_ptr<Job> job(m_voJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
            CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB1)", job.get() != 0);
        }
        gettimeofday(&stop_time,0);
        if(start_time.tv_sec < stop_time.tv_sec){
            if(start_time.tv_usec < stop_time.tv_usec){
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec;
                usec_diff = stop_time.tv_usec - start_time.tv_usec;
            } else {
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec - 1;
                usec_diff = stop_time.tv_usec + 1000000 - start_time.tv_usec;
            }
        } else {
            sec_diff = 0;
            usec_diff = stop_time.tv_usec - start_time.tv_usec;
        }
        printf("Job Get (VO) %d iterations in %d secs and %d usec\n",LOOP_ITERATIONS,(int)sec_diff,(int)usec_diff);
    } catch(const DAOException& exc){
        // OK
    }
}

void JobDaoTestCase::testGet_VO_Finished(){
    try{
        // Get TransferJob instance
        scoped_ptr<Job> job(m_voJobDAO->get(JOBDAO_TESTCASE_JOB601_ID));
        CPPUNIT_FAIL("This operation should fail");
    } catch(const DAOException& exc){
        // OK
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void JobDaoTestCase::testUpdate_VO(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get TransferJob instance
        scoped_ptr<Job> job(m_voJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB1)",job.get() != 0);

        // Update Some Properties
        job->state = Job::S_ACTIVE;

        // Update Transfer Job
        m_voJobDAO->update(*job);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Job Again
        job.reset(m_voJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid State (JOB1)", job->state == Job::S_ACTIVE);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void JobDaoTestCase::testUpdate_VO_2(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get FileTransfer instance
        scoped_ptr<Job> job(m_voJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (JOB1)",job.get() != 0);

        // Update Job
        m_voJobDAO->update(*job);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again
        job.reset(m_voJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (JOB1)", job->id == JOBDAO_TESTCASE_JOB1_ID);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void JobDaoTestCase::testUpdate_VO_3(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get TransferJob instance
        scoped_ptr<Job> job(m_voJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB1)",job.get() != 0);

        time_t current;
        time(&current);
        // Update Some Properties
        job->state          = Job::S_READY;
        job->channelName    = JOBDAO_TESTCASE_CHANNEL1_NAME;
        job->source         = "SOURCE_SITE";
        job->dest           = "DEST_SITE";
        job->reason         = "A Reason";
        job->finishTime     = current;
        job->myproxyServer  = "myproxy.nowhere.com";
        job->srcCatalog     = "http://catalog.nowhere.com/Catalog";
        job->srcCatalogType = "DummyCatalog";
        job->destCatalog    = "http://catalog.somewhere.com/Catalog";
        job->destCatalogType= "AnotherDummyCatalog";
        job->internalParams = "internal";

        // Update Transfer Job
        m_voJobDAO->update(*job);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Job Again
        job.reset(m_voJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid State          (JOB1)", job->state == Job::S_READY);
        CPPUNIT_ASSERT_MESSAGE("Invalid ChannelName    (JOB1)", job->channelName == JOBDAO_TESTCASE_CHANNEL1_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid Source         (JOB1)", job->source == "SOURCE_SITE");
        CPPUNIT_ASSERT_MESSAGE("Invalid Dest           (JOB1)", job->dest == "DEST_SITE");
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason         (JOB1)", job->reason == "A Reason");
        CPPUNIT_ASSERT_MESSAGE("Invalid FinishTime     (JOB1)", job->finishTime == current);
        CPPUNIT_ASSERT_MESSAGE("Invalid MyproxyServer  (JOB1)", job->myproxyServer  == "myproxy.nowhere.com");
        CPPUNIT_ASSERT_MESSAGE("Invalid SrcCatalog     (JOB1)", job->srcCatalog     == "http://catalog.nowhere.com/Catalog");
        CPPUNIT_ASSERT_MESSAGE("Invalid SrcCatalogType (JOB1)", job->srcCatalogType == "DummyCatalog");
        CPPUNIT_ASSERT_MESSAGE("Invalid DestCatalog    (JOB1)", job->destCatalog    == "http://catalog.somewhere.com/Catalog");
        CPPUNIT_ASSERT_MESSAGE("Invalid DestCatalogType(JOB1)", job->destCatalogType== "AnotherDummyCatalog");
        CPPUNIT_ASSERT_MESSAGE("Invalid InternalParams (JOB1)", job->internalParams == "internal");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void JobDaoTestCase::testUpdate_VO_4(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get TransferJob instance
        scoped_ptr<Job> job(m_voJobDAO->get(JOBDAO_TESTCASE_JOB30_ID));
        CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB30)",job.get() != 0);

        // Update Some Properties
        job->sourceSE      = "host.source.site";
        job->destSE        = "host.dest.site";

        // Update Transfer Job
        m_voJobDAO->update(*job);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Job Again
        job.reset(m_voJobDAO->get(JOBDAO_TESTCASE_JOB30_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Source SE      (JOB30)", job->sourceSE == "host.source.site");
        CPPUNIT_ASSERT_MESSAGE("Invalid Dest SE        (JOB30)", job->destSE == "host.dest.site");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void JobDaoTestCase::testUpdate_VO_5(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get TransferJob instance
        scoped_ptr<Job> job(m_voJobDAO->get(JOBDAO_TESTCASE_JOB30_ID));
        CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB30)",job.get() != 0);

        // Update Some Properties
        job->source         = "SOURCE_SITE";
        job->dest           = "DEST_SITE";
        job->reason         = "A Reason";
        job->internalParams = "internal";
        job->sourceSE       = "host.source.site";
        job->destSE         = "host.dest.site";

        // Update Transfer Job
        m_voJobDAO->update(*job);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Job Again
        job.reset(m_voJobDAO->get(JOBDAO_TESTCASE_JOB30_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Source         (JOB30)", job->source == "SOURCE_SITE");
        CPPUNIT_ASSERT_MESSAGE("Invalid Dest           (JOB30)", job->dest == "DEST_SITE");
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason         (JOB30)", job->reason == "A Reason");
        CPPUNIT_ASSERT_MESSAGE("Invalid InternalParams (JOB30)", job->internalParams == "internal");
        CPPUNIT_ASSERT_MESSAGE("Invalid Source SE      (JOB30)", job->sourceSE == "host.source.site");
        CPPUNIT_ASSERT_MESSAGE("Invalid Dest SE        (JOB30)", job->destSE == "host.dest.site");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate_reason
 *
 * Test Update Method
 */
void JobDaoTestCase::testUpdate_reason(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get FileTransfer instance
        scoped_ptr<Job> job(m_voJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (JOB1)",job.get() != 0);

        const char * const r = "This is a long reason string, more than 1024 characters:"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789"\
                "0123456789012345678901234567890123456789012345678901234567890123456789";
        job->reason = r;

        // Update Job
        m_voJobDAO->update(*job);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again
        job.reset(m_voJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Job id (JOB1)", job->id == JOBDAO_TESTCASE_JOB1_ID);
        CPPUNIT_ASSERT_MESSAGE("Invalid Job reason length (JOB1)", job->reason().length() < strlen(r));
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
/*
 * testUpdate
 *
 * Test Update Method
 */
void JobDaoTestCase::testUpdate_VO_Error(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get FileTransfer instance
        scoped_ptr<Job> job(new Job(JOBDAO_TESTCASE_JOB3_ID,Job::S_PENDING,JOBDAO_TESTCASE_VO2_NAME));
        CPPUNIT_ASSERT_MESSAGE("Null Job (JOB3)",job.get() != 0);

        // Update Some Properties
        job->reason = "A Reason";

        // Update File Transfer
        m_voJobDAO->update(*job);
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("This Job shouldn't be reachable (JOB3)");
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void JobDaoTestCase::testUpdate_VO_Stress(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get TransferJob instance
        scoped_ptr<Job> job(m_voJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB1)",job.get() != 0);

        // Update Some Properties
        job->state          = Job::S_READY;
        job->channelName    = JOBDAO_TESTCASE_CHANNEL1_NAME;
        job->source         = "SOURCE_SITE";
        job->dest           = "DEST_SITE";
        job->reason         = "A Reason";
        job->myproxyServer  = "myproxy.nowhere.com";
        job->srcCatalog     = "http://catalog.nowhere.com/Catalog";
        job->srcCatalogType = "DummyCatalog";
        job->destCatalog    = "http://catalog.somewhere.com/Catalog";
        job->destCatalogType= "AnotherDummyCatalog";
        job->internalParams = "internal";

        struct timeval start_time,stop_time;
        long sec_diff,usec_diff;

        // Perform iterations
        gettimeofday(&start_time,0);
        for(int i = 0; i < LOOP_ITERATIONS; ++i){
            time_t current;
            time(&current);
            job->finishTime     = current;

            // Update Transfer Job
            m_voJobDAO->update(*job);
        }
        gettimeofday(&stop_time,0);
        if(start_time.tv_sec < stop_time.tv_sec){
            if(start_time.tv_usec < stop_time.tv_usec){
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec;
                usec_diff = stop_time.tv_usec - start_time.tv_usec;
            } else {
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec - 1;
                usec_diff = stop_time.tv_usec + 1000000 - start_time.tv_usec;
            }
        } else {
            sec_diff = 0;
            usec_diff = stop_time.tv_usec - start_time.tv_usec;
        }
        printf("Job Update (VO) %d iterations in %d secs and %d usec\n",LOOP_ITERATIONS,(int)sec_diff,(int)usec_diff);

        // Commit Transaction
        DaoHelper::ctx().commit();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdateStates
 *
 * Test UpdateStates Method
 */
void JobDaoTestCase::testUpdateStates_VO(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        StringArray ids;
        ids.push_back(JOBDAO_TESTCASE_JOB1_ID);
        ids.push_back(JOBDAO_TESTCASE_JOB2_ID);

        // Update Job State
        m_voJobDAO->updateStates(ids);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Check Job 1
        scoped_ptr<Job> job(m_voJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Job (JOB1)",job.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (JOB1)", job->state == Job::S_ACTIVE);

        // Check Job 2
        job.reset(m_voJobDAO->get(JOBDAO_TESTCASE_JOB2_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Job (JOB2)",job.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (JOB2)", job->state == Job::S_READY);

    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdateStates
 *
 * Test UpdateStates Method
 */
void JobDaoTestCase::testUpdateStates_VO_2(){
    try{
        // Start Transaction
        DaoHelper::ctx().start();

        StringArray ids;
        ids.push_back(JOBDAO_TESTCASE_JOB8_ID);

        // Update Job State
        m_voJobDAO->updateStates(ids);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Check Job 8
        scoped_ptr<Job> job(m_voJobDAO->getFullSearch(JOBDAO_TESTCASE_JOB8_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Job (JOB8)",job.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (JOB8)", job->state == Job::S_FINISHED);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdateStates
 *
 * Test UpdateStates Method
 */
void JobDaoTestCase::testUpdateStates_VO_Error(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        StringArray ids;
        ids.push_back(JOBDAO_TESTCASE_JOB3_ID);
        ids.push_back(JOBDAO_TESTCASE_JOB4_ID);

        // Update Job State
        m_voJobDAO->updateStates(ids);
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("This Job shouldn't be reachable (JOB3)");
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetSubmitted_VO
 *
 * Test GetSubmittedJobs Method
 */
void JobDaoTestCase::testGetSubmitted_VO(){

    StringArray jobIds;

    // Get Jobs To allocate
    m_voJobDAO->getSubmittedJobs(jobIds);
    CPPUNIT_ASSERT_MESSAGE("No Submitted jobs", !jobIds.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid Number of Submitted jobs", jobIds.size() == 2);
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB5)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB5_ID) != jobIds.end());
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB10)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB10_ID) != jobIds.end());

}

/*
 * testGetSubmitted_VO
 *
 * Test GetSubmittedJobs Method
 */
void JobDaoTestCase::testGetSubmitted_VO_2(){

    StringArray jobIds;

    // Get Jobs To allocate
    m_voJobDAO->getSubmittedJobs(jobIds,1);
    CPPUNIT_ASSERT_MESSAGE("No Submitted jobs", !jobIds.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid Number of Submitted jobs", jobIds.size() == 1);
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB9)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB9_ID) == jobIds.end());

}

/*
 * testGetPending_VO
 *
 * Test GetPendingJobs Method
 */
void JobDaoTestCase::testGetPending_VO(){

    StringArray jobIds;

    // Get Jobs To allocate
    m_voJobDAO->getPendingJobs(jobIds);
    CPPUNIT_ASSERT_MESSAGE("No Submitted jobs", !jobIds.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid Number of Pending jobs", jobIds.size() == 5);
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB18)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB18_ID) != jobIds.end());
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB20)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB20_ID) != jobIds.end());
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB21)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB21_ID) != jobIds.end());
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB24)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB24_ID) != jobIds.end());
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB25)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB25_ID) != jobIds.end());

}

/*
 * testGetPending_VO
 *
 * Test GetPndingJobs Method
 */
void JobDaoTestCase::testGetPending_VO_2(){

    StringArray jobIds;

    // Get Jobs To allocate
    m_voJobDAO->getPendingJobs(jobIds,1);
    CPPUNIT_ASSERT_MESSAGE("No Submitted jobs", !jobIds.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid Number of Submitted jobs", jobIds.size() == 1);
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB19)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB19_ID) == jobIds.end());

}

/*
 * testGetDone_VO
 *
 * Test GetSubmittedJobs Method
 */
void JobDaoTestCase::testGetDone_VO(){

    StringArray jobIds;

    // Get Jobs To allocate
    m_voJobDAO->getDoneJobs(jobIds);
    CPPUNIT_ASSERT_MESSAGE("No Done jobs", !jobIds.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid Number of Done jobs", jobIds.size() == 2);
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB11)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB11_ID) != jobIds.end());
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB12)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB12_ID) != jobIds.end());

}

/*
 * testGetDone_VO
 *
 * Test GetDoneJobs Method
 */
void JobDaoTestCase::testGetDone_VO_2(){

    StringArray jobIds;

    // Get Jobs To allocate
    m_voJobDAO->getDoneJobs(jobIds,1);
    CPPUNIT_ASSERT_MESSAGE("No Done jobs", !jobIds.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid Number of Done jobs", jobIds.size() == 1);

}

/*
 * testGetFinishing_VO
 *
 * Test GetSubmittedJobs Method
 */
void JobDaoTestCase::testGetFinishing_VO(){
    StringArray jobIds;

    // Get Jobs To allocate
    m_voJobDAO->getFinishingJobs(jobIds);
    CPPUNIT_ASSERT_MESSAGE("No Finishing jobs", !jobIds.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid Number of Finishing jobs", jobIds.size() == 4);
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB13)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB13_ID) != jobIds.end());
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB15)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB15_ID) != jobIds.end());
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB16)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB16_ID) != jobIds.end());
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB17)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB17_ID) != jobIds.end());

}

/*
 * testGetDone_VO
 *
 * Test GetDoneJobs Method
 */
void JobDaoTestCase::testGetFinishing_VO_2(){
    StringArray jobIds;

    // Get Jobs To allocate
    m_voJobDAO->getFinishingJobs(jobIds,1);
    CPPUNIT_ASSERT_MESSAGE("No Finishing jobs", !jobIds.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid Number of Finishing jobs", jobIds.size() == 1);
    CPPUNIT_ASSERT_MESSAGE("Invalid jobs Id (JOB14)",
                           std::find(jobIds.begin(),jobIds.end(),JOBDAO_TESTCASE_JOB14_ID) == jobIds.end());

}

/*
 * testGet
 *
 * Test Get Method
 */
void JobDaoTestCase::testGet_Ch(){

    // Get TransferJob instance
    scoped_ptr<Job> job(m_chJobDAO->get(JOBDAO_TESTCASE_JOB6_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB6)", job.get() != 0);

    // Check the State is Active(Precondition)
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid State (JOB6)", job->state == Job::S_SUBMITTED);
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Submit host (JOB6)", job->submitHost == JOBDAO_TESTCASE_SUBMITHOST);
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Max time in queue (JOB6)", job->maxTimeInQueue == JOBDAO_TESTCASE_MAXTIMEINQUEUE);

}

/**
 * testGet
 * Test Get Method
 */
void JobDaoTestCase::testGet_Ch_2(){

    // Get TransferJob instance
    scoped_ptr<Job> job(m_chJobDAO->get(JOBDAO_TESTCASE_JOB30_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB30)", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Overwrite Flag(JOB30)", job->overwriteFlag == false);

    // Get TransferJob instance
    job.reset(m_chJobDAO->get(JOBDAO_TESTCASE_JOB31_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB31)", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Overwrite Flag(JOB31)", job->overwriteFlag == true);

    // Get TransferJob instance
    job.reset(m_chJobDAO->get(JOBDAO_TESTCASE_JOB6_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB6)", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Precondition Check Failed: Invalid Overwrite Flag(JOB6)", job->overwriteFlag == false);

}

/**
 * testGet_Ch_3
 */
void JobDaoTestCase::testGet_Ch_3(){

    static const char * job_id  = "80000000-0000-0000-0000-000000000001";
    static const char * sst     = "someSourceSpaceToken";
    static const char * sstd    = "someSourceTokenDescription";

    // Get TransferJob instance
    scoped_ptr<Job> job(m_chJobDAO->get(job_id));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob", job.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid sourceSpaceToken", job->sourceSpaceToken == sst);
    CHECK_MODEL_OBJECT_VALUE(job,sourceTokenDescription,sstd);

}

/*
 * testGet
 *
 * Test Get Method
 */
void JobDaoTestCase::testGet_Ch_Error(){
    try{
        // Get Job instance
        scoped_ptr<Job> job(m_chJobDAO->get(JOBDAO_TESTCASE_JOB4_ID));
        CPPUNIT_FAIL("This Job shouldn't be reachable (JOB4)");
    } catch(const DAOException& exc){
        // OK
    }
}

void JobDaoTestCase::testGet_Ch_Finished(){
    try{
        // Get TransferJob instance
        scoped_ptr<Job> job(m_chJobDAO->get(JOBDAO_TESTCASE_JOB601_ID));
        CPPUNIT_FAIL("This operation should fail");
    } catch(const DAOException& exc){
        // OK
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void JobDaoTestCase::testUpdate_Ch(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get TransferJob instance
        scoped_ptr<Job> job(m_chJobDAO->get(JOBDAO_TESTCASE_JOB2_ID));
        CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB2)",job.get() != 0);

        // Update Some Properties
        job->state = Job::S_PENDING;

        // Update Transfer Job
        m_chJobDAO->update(*job);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Job Again
        job.reset(m_chJobDAO->get(JOBDAO_TESTCASE_JOB2_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid State (JOB2)", job->state == Job::S_PENDING);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void JobDaoTestCase::testUpdate_Ch_2(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get FileTransfer instance
        scoped_ptr<Job> job(m_chJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null File (JOB1)",job.get() != 0);

        // Update Job
        m_chJobDAO->update(*job);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get File Again
        job.reset(m_chJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File id (JOB1)", job->id == JOBDAO_TESTCASE_JOB1_ID);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void JobDaoTestCase::testUpdate_Ch_3(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get TransferJob instance
        scoped_ptr<Job> job(m_chJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB1)",job.get() != 0);

        time_t current;
        time(&current);
        // Update Some Properties
        job->state  = Job::S_READY;
        job->reason = "Another Reason";
        job->finishTime     = current;
        job->internalParams = "internal2";

        // Update Transfer Job
        m_voJobDAO->update(*job);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Job Again
        job.reset(m_chJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid State          (JOB1)", job->state == Job::S_READY);
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason         (JOB1)", job->reason == "Another Reason");
        CPPUNIT_ASSERT_MESSAGE("Invalid FinishTime     (JOB1)", job->finishTime == current);
        CPPUNIT_ASSERT_MESSAGE("Invalid InternalParams (JOB1)", job->internalParams == "internal2");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void JobDaoTestCase::testUpdate_Ch_Error(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get FileTransfer instance
        scoped_ptr<Job> job(new Job(JOBDAO_TESTCASE_JOB4_ID,Job::S_PENDING,JOBDAO_TESTCASE_VO1_NAME,JOBDAO_TESTCASE_CHANNEL2_NAME));
        CPPUNIT_ASSERT_MESSAGE("Null File (FILE4)",job.get() != 0);

        // Update Some Properties
        job->reason = "A Reason";

        // Update File Transfer
        m_chJobDAO->update(*job);
        CPPUNIT_FAIL(" This Job shouldn't be reachable (JOB4)");
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdateStates
 *
 * Test UpdateStates Method
 */
void JobDaoTestCase::testUpdateStates_Ch(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        StringArray ids;
        ids.push_back(JOBDAO_TESTCASE_JOB1_ID);
        ids.push_back(JOBDAO_TESTCASE_JOB2_ID);

        // Update Job State
        m_chJobDAO->updateStates(ids);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Check Job 1
        scoped_ptr<Job> job(m_chJobDAO->get(JOBDAO_TESTCASE_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Job (JOB1)",job.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (JOB1)", job->state == Job::S_ACTIVE);

        // Check Job 2
        job.reset(m_chJobDAO->get(JOBDAO_TESTCASE_JOB2_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Job (JOB2)",job.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (JOB2)", job->state == Job::S_READY);

    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdateStates
 *
 * Test UpdateStates Method
 */
void JobDaoTestCase::testUpdateStates_Ch_Error(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        StringArray ids;
        ids.push_back(JOBDAO_TESTCASE_JOB3_ID);
        ids.push_back(JOBDAO_TESTCASE_JOB4_ID);

        // Update Job State
        m_chJobDAO->updateStates(ids);
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("This Job shouldn't be reachable (JOB4)");
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetJobsToFix_Ch
 *
 * Test GetJobsToFix Method
 */
void JobDaoTestCase::testGetJobsToFix_Ch(){

    StringArray jobIds;

    // Get Jobs To Fix
    m_chJobDAO->getJobsToFix(jobIds);
    CPPUNIT_ASSERT_MESSAGE("No jobs To Fix (1)", !jobIds.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid Number of jobs to Fix (1)", jobIds.size() == 2);

    // Get Jobs To Fix
    jobIds.clear();
    m_chJobDAO->getJobsToFix(jobIds,1);
    CPPUNIT_ASSERT_MESSAGE("No jobs To Fix (2)", !jobIds.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid Number of jobs to fix (2)", jobIds.size() == 1);

}

/*
 * testGetLast_Cred
 *
 * Test GetLast Method
 */
void JobDaoTestCase::testGetLast_Cred(){

    // Get Job instance
    scoped_ptr<Job> job(m_credJobDAO->getLast(JOBDAO_TESTCASE_CRED_ID_1, JOBDAO_TESTCASE_USER_DN_1));
    CPPUNIT_ASSERT_MESSAGE("Null Job (JOB1)", job.get() != 0);

    // Check the Preconditions
    CPPUNIT_ASSERT_MESSAGE("Invalid Job ID (JOB1)", job->id == JOBDAO_TESTCASE_JOB27_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job Cred ID (JOB1)", job->credId == JOBDAO_TESTCASE_CRED_ID_1);
    CPPUNIT_ASSERT_MESSAGE("Invalid Job User DN (JOB1)", job->userDn == JOBDAO_TESTCASE_USER_DN_1);

}
/*
 * testGetLast_Cred_Error
 *
 * Test GetLast Method
 */
void JobDaoTestCase::testGetLast_Cred_Error(){
    try{
        // Get Job instance
        scoped_ptr<Job> job(m_credJobDAO->getLast("Invalid Cred id", JOBDAO_TESTCASE_USER_DN_1));
        CPPUNIT_FAIL("This Job shouldn't be reachable (JOB1)");
    } catch(const DAOException& exc){
        // OK
    }
}

/**
 * testGetLast_Cred_Stress
 * Test GetLast Method
 */
void JobDaoTestCase::testGetLast_Cred_Stress(){

    struct timeval start_time,stop_time;
    long sec_diff,usec_diff;

    // Perform iterations
    gettimeofday(&start_time,0);
    for(int i = 0; i < LOOP_ITERATIONS; ++i){
        // Get FileTransfer instance
        scoped_ptr<Job> job(m_credJobDAO->getLast(JOBDAO_TESTCASE_CRED_ID_1, JOBDAO_TESTCASE_USER_DN_1));
        CPPUNIT_ASSERT_MESSAGE("Null Job (JOB1)", job.get() != 0);
    }
    gettimeofday(&stop_time,0);
    if(start_time.tv_sec < stop_time.tv_sec){
        if(start_time.tv_usec < stop_time.tv_usec){
            sec_diff  = stop_time.tv_sec  - start_time.tv_sec;
            usec_diff = stop_time.tv_usec - start_time.tv_usec;
        } else {
            sec_diff  = stop_time.tv_sec  - start_time.tv_sec - 1;
            usec_diff = stop_time.tv_usec + 1000000 - start_time.tv_usec;
        }
    } else {
        sec_diff = 0;
        usec_diff = stop_time.tv_usec - start_time.tv_usec;
    }
    printf("Job Get (Cred) %d iterations in %d secs and %d usec\n",LOOP_ITERATIONS,(int)sec_diff,(int)usec_diff);

}

/**
 * Schema 3.3.0
 */
void JobDaoTestCase::testGet_VO_schema_3_3_0(){
    static const char * JOB101_ID = "00000010-0000-0000-0000-000000000001";
    static const char * JOB102_ID = "00000010-0000-0000-0000-000000000002";
    static const char * JOB103_ID = "00000010-0000-0000-0000-000000000003";

    // Get TransferJob instance
    scoped_ptr<Job> job(m_voJobDAO->get(JOB101_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB101)", job.get() != 0);
    CHECK_MODEL_OBJECT_VALUE(job,copyPinLifetime,0);
    CHECK_MODEL_OBJECT_VALUE(job,lanConnection,false);
    CHECK_MODEL_OBJECT_VALUE(job,failNearline,false);

    job.reset(m_voJobDAO->get(JOB102_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB102)", job.get() != 0);
    CHECK_MODEL_OBJECT_VALUE(job,copyPinLifetime,100);
    CHECK_MODEL_OBJECT_VALUE(job,lanConnection,true);
    CHECK_MODEL_OBJECT_VALUE(job,failNearline,true);

    job.reset(m_voJobDAO->get(JOB103_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB103)", job.get() != 0);
    CHECK_MODEL_OBJECT_VALUE(job,copyPinLifetime,100);
    CHECK_MODEL_OBJECT_VALUE(job,lanConnection,false);
    CHECK_MODEL_OBJECT_VALUE(job,failNearline,false);
}
void JobDaoTestCase::testGet_Ch_schema_3_3_0(){
    static const char * JOB101_ID = "00000010-0000-0000-0000-000000000001";
    static const char * JOB102_ID = "00000010-0000-0000-0000-000000000002";
    static const char * JOB103_ID = "00000010-0000-0000-0000-000000000003";

    // Get TransferJob instance
    scoped_ptr<Job> job(m_chJobDAO->get(JOB101_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB101)", job.get() != 0);
    CHECK_MODEL_OBJECT_VALUE(job,copyPinLifetime,0);
    CHECK_MODEL_OBJECT_VALUE(job,lanConnection,false);
    CHECK_MODEL_OBJECT_VALUE(job,failNearline,false);

    job.reset(m_chJobDAO->get(JOB102_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB102)", job.get() != 0);
    CHECK_MODEL_OBJECT_VALUE(job,copyPinLifetime,100);
    CHECK_MODEL_OBJECT_VALUE(job,lanConnection,true);
    CHECK_MODEL_OBJECT_VALUE(job,failNearline,true);

    job.reset(m_chJobDAO->get(JOB103_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB103)", job.get() != 0);
    CHECK_MODEL_OBJECT_VALUE(job,copyPinLifetime,100);
    CHECK_MODEL_OBJECT_VALUE(job,lanConnection,false);
    CHECK_MODEL_OBJECT_VALUE(job,failNearline,false);
}

void JobDaoTestCase::testGet_compare_checksum_3_4_1()
{
    static const char * JOB101_ID = "00000011-0000-0000-0000-000000000001";
    static const char * JOB102_ID = "00000011-0000-0000-0000-000000000002";
    static const char * JOB103_ID = "00000011-0000-0000-0000-000000000003";

    // Get TransferJob instance
    scoped_ptr<Job> job(m_chJobDAO->get(JOB101_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB101)", job.get() != 0);
    CHECK_MODEL_OBJECT_VALUE(job,checksum_method,false);

    job.reset(m_chJobDAO->get(JOB102_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB102)", job.get() != 0);
    CHECK_MODEL_OBJECT_VALUE(job,checksum_method,false);

    job.reset(m_chJobDAO->get(JOB103_ID));
    CPPUNIT_ASSERT_MESSAGE("Null TransferJob (JOB103)", job.get() != 0);
    CHECK_MODEL_OBJECT_VALUE(job,checksum_method,true);
}

