/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: StageReqDAO Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: StageReqDaoTestCase.cpp,v 1.6 2008/12/11 13:24:44 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "DaoHelper.h"
#include "glite/data/transfer/agent/model/StageReq.h"
#include "glite/data/transfer/agent/dao/vo/StageReqDAO.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cstring>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using glite::data::agents::dao::DAOException;
using glite::data::agents::longlong;
using glite::data::agents::dao::StringArray;

using boost::scoped_ptr;

/**
 * CPPUnit StageReqDaoTestCase
 */
class StageReqDaoTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( StageReqDaoTestCase);
    // VO View
    CPPUNIT_TEST( testGet_VO );
    CPPUNIT_TEST( testGet_VO_2 );
    CPPUNIT_TEST( testGet_VO_Error );
    CPPUNIT_TEST( testGetByFileId_VO );
    CPPUNIT_TEST( testGetByFileId_VO_2 );
    CPPUNIT_TEST( testGetByFileId_VO_3 );
    CPPUNIT_TEST( testGetByFileId_VO_Error );
    CPPUNIT_TEST( testGetByRequestId_VO );
    CPPUNIT_TEST( testGetByRequestId_VO_Error );
    CPPUNIT_TEST( testUpdate_VO );
    CPPUNIT_TEST( testUpdate_VO_2 );
    CPPUNIT_TEST( testUpdate_VO_3 );
    CPPUNIT_TEST( testUpdate_VO_4 );
    CPPUNIT_TEST( testUpdate_reason );
    CPPUNIT_TEST( testUpdate_VO_Error );
    CPPUNIT_TEST( testCreate_VO );
    CPPUNIT_TEST( testCountActiveRequests_VO );
    CPPUNIT_TEST( testCountActiveRequests_VO_2 );
    CPPUNIT_TEST_SUITE_END();

    // VO Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_VO();
    void testGet_VO_2();
    void testGet_VO_Error();
    /**
     * Test GetByFileID Method
     */
    void testGetByFileId_VO();
    void testGetByFileId_VO_2();
    void testGetByFileId_VO_3();
    void testGetByFileId_VO_Error();
    /**
     * Test GetBySites Method
     */
    void testGetByRequestId_VO();    
    void testGetByRequestId_VO_Error();    

    /**
     * Test Update Method
     */
    void testUpdate_VO();
    void testUpdate_reason();
    void testUpdate_VO_2();
    void testUpdate_VO_3();
    void testUpdate_VO_4();
    void testUpdate_VO_Error();
    /**
     * Test Create Method
     */
    void testCreate_VO();
    /**
     * Test CountActiveStageReq Method
     */
    void testCountActiveRequests_VO();
    void testCountActiveRequests_VO_2();
    
private:
    /**
     * The VO FileDAO Instance
     */
    scoped_ptr<dao::vo::StageReqDAO> m_voStageReqDAO;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( StageReqDaoTestCase, "StageReqDaoTestCase" );

namespace {
const char * const STAGEREQDAO_TESTCASE_JOB1_ID        = "40000000-0000-0000-0000-000000000001";
const char * const STAGEREQDAO_TESTCASE_JOB2_ID        = "40000000-0000-0000-0000-000000000002";
const char * const STAGEREQDAO_TESTCASE_JOB3_ID        = "40000000-0000-0000-0000-000000000003";
const char * const STAGEREQDAO_TESTCASE_JOB4_ID        = "40000000-0000-0000-0000-000000000004";
const char * const STAGEREQDAO_TESTCASE_JOB5_ID        = "40000000-0000-0000-0000-000000000005";
const char * const STAGEREQDAO_TESTCASE_JOB6_ID        = "40000000-0000-0000-0000-000000000006";
const char * const STAGEREQDAO_TESTCASE_JOB7_ID        = "40000000-0000-0000-0000-000000000007";
const char * const STAGEREQDAO_TESTCASE_FILE1_ID       = "50000001";
const char * const STAGEREQDAO_TESTCASE_FILE2_ID       = "50000002";
const char * const STAGEREQDAO_TESTCASE_FILE3_ID       = "50000003";
const char * const STAGEREQDAO_TESTCASE_FILE4_ID       = "50000004";
const char * const STAGEREQDAO_TESTCASE_FILE5_ID       = "50000005";
const char * const STAGEREQDAO_TESTCASE_FILE6_ID       = "50000006";
const char * const STAGEREQDAO_TESTCASE_FILE7_ID       = "50000007";
const char * const STAGEREQDAO_TESTCASE_FILE8_ID       = "50000008";
    
const char * const STAGEREQDAO_TESTCASE_FILE9_ID       = "50000005";
    
const char * const STAGEREQDAO_TESTCASE_FILE_NEW_ID    = "50001000";
const char * const STAGEREQDAO_TESTCASE_REQ1_ID        = "req_5_1";
const char * const STAGEREQDAO_TESTCASE_REQ2_ID        = "req_5_2";
const char * const STAGEREQDAO_TESTCASE_REQ3_ID        = "req_5_3";
const char * const STAGEREQDAO_TESTCASE_REQ4_ID        = "req_5_4";
const char * const STAGEREQDAO_TESTCASE_REQ5_ID        = "req_5_5";
const char * const STAGEREQDAO_TESTCASE_REQ6_ID        = "req_5_6";
const char * const STAGEREQDAO_TESTCASE_REQ_NEW_ID     = "req_5_new";

const char * const STAGEREQDAO_TESTCASE_REQ7_ID        = "req_5_7";
const char * const STAGEREQDAO_TESTCASE_REQ8_ID        = "req_5_8";

const char * const STAGEREQDAO_TESTCASE_VO1_NAME       = "STAGEREQ_VO_1";
const char * const STAGEREQDAO_TESTCASE_VO2_NAME       = "STAGEREQ_VO_2";
const char * const STAGEREQDAO_TESTCASE_FILE1_SOURCE   = "file:///tmp/src.txt";
const char * const STAGEREQDAO_TESTCASE_FILE1_DEST     = "file:///tmp/dst.txt";
const char * const STAGEREQDAO_TESTCASE_CHANNEL1_NAME  = "STAGEREQ_channel_1";
const char * const STAGEREQDAO_TESTCASE_CHANNEL2_NAME  = "STAGEREQ_channel_2";
const char * const STAGEREQDAO_TESTCASE_CHANNEL3_NAME  = "STAGEREQ_channel_3";
}

/*
 * setUp
 *
 * set-up test case
 */
void StageReqDaoTestCase::setUp(){
    
    try{
        // Create VO StageReqDAO instance
        m_voStageReqDAO.reset(
            dao::vo::DAOFactory::instance().createStageReqDAO(STAGEREQDAO_TESTCASE_VO1_NAME, DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null VO StageReqDAO", m_voStageReqDAO.get() != 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void StageReqDaoTestCase::testGet_VO(){
    try{
        // Get StageReq instance
        scoped_ptr<model::StageReq> sr(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE1_ID,STAGEREQDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ1)", sr.get() != 0);
        
        // Check Some Precondition
        CPPUNIT_ASSERT_MESSAGE("Invalid TURL (STAGEREQ1)", sr->turl == STAGEREQDAO_TESTCASE_FILE1_SOURCE);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (STAGEREQ1)", sr->state == StageReq::S_SUBMITTED);

        // Get StageReq instance
        sr.reset(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE2_ID,STAGEREQDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ2)", sr.get() != 0);
        // Check Some Precondition
        CPPUNIT_ASSERT_MESSAGE("Invalid State (STAGEREQ2)", sr->state == StageReq::S_COMPLETED);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

void StageReqDaoTestCase::testGet_VO_2(){
    try{
        // Get StageReq instance
        scoped_ptr<model::StageReq> sr(m_voStageReqDAO->get(
            STAGEREQDAO_TESTCASE_FILE9_ID,
            STAGEREQDAO_TESTCASE_REQ7_ID));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ7)", sr.get() != 0);
        // Check Properties
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason (STAGEREQ7)",sr->reason() == "staging failed");
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason Class (STAGEREQ7)",sr->reasonClass() == Error::STORAGE_INTERNAL_ERROR);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Scope (STAGEREQ7)",sr->errorScope() == Error::S_SOURCE);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Phase (STAGEREQ7)",sr->errorPhase() == Error::P_PRESTAGING);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testGet
 *
 * Test Get Method
 */
void StageReqDaoTestCase::testGet_VO_Error(){
    try{
        // Get StageReq instance
        scoped_ptr<model::StageReq> sr(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE3_ID,STAGEREQDAO_TESTCASE_REQ2_ID));
        CPPUNIT_FAIL("This StageReq shouldn't be reachable (STAGEREQ3)");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
    try{
        // Get StageReq instance
        scoped_ptr<model::StageReq> sr(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE1_ID,STAGEREQDAO_TESTCASE_REQ3_ID));
        CPPUNIT_FAIL("This StageReq shouldn't be reachable (Invalid file_id-req_id pair)");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void StageReqDaoTestCase::testGetByFileId_VO(){
    
    StringArray reqIds;
    try{
        m_voStageReqDAO->getByFileId(STAGEREQDAO_TESTCASE_FILE4_ID,reqIds);
        CPPUNIT_ASSERT_MESSAGE("Invalid Files ID", !reqIds.empty());
        
        reqIds.clear();
        m_voStageReqDAO->getByFileId(STAGEREQDAO_TESTCASE_FILE4_ID,reqIds,1);
        CPPUNIT_ASSERT_MESSAGE("Too many transfers", reqIds.size() == 1);
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void StageReqDaoTestCase::testGetByFileId_VO_2(){
    
    StringArray reqIds;
    try{
        m_voStageReqDAO->getByFileId(STAGEREQDAO_TESTCASE_FILE6_ID,reqIds);
        CPPUNIT_ASSERT_MESSAGE("Pending File (FILE6)", reqIds.empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void StageReqDaoTestCase::testGetByFileId_VO_3(){
    
    StringArray reqIds;
    try{
        m_voStageReqDAO->getByFileId(STAGEREQDAO_TESTCASE_FILE5_ID,reqIds,1);
        CPPUNIT_ASSERT_MESSAGE("Too many transfers", reqIds.size() == 1);

        reqIds.clear();
        m_voStageReqDAO->getByFileId(STAGEREQDAO_TESTCASE_FILE5_ID,reqIds,2);
        CPPUNIT_ASSERT_MESSAGE("Too many transfers", reqIds.size() == 2);
        
        // Get StageReq instances
        scoped_ptr<model::StageReq> sr1(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE5_ID,reqIds.at(0)));
        CPPUNIT_ASSERT_MESSAGE("Null pointer returned #0", sr1.get() != 0);
        scoped_ptr<model::StageReq> sr2(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE5_ID,reqIds.at(1)));
        CPPUNIT_ASSERT_MESSAGE("Null pointer returned #1", sr2.get() != 0);

        // Check the transfer times
        CPPUNIT_ASSERT_MESSAGE("Invalid order", sr1->requestTime() >= sr2->requestTime());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void StageReqDaoTestCase::testGetByFileId_VO_Error(){
    
    StringArray reqIds;
    try{
        m_voStageReqDAO->getByFileId(STAGEREQDAO_TESTCASE_FILE3_ID,reqIds);
        CPPUNIT_FAIL("VO_2 StageReqs shoudn't be reachable");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByRequestId
 *
 * Test GetByRequestId Method
 */
void StageReqDaoTestCase::testGetByRequestId_VO(){
    
    StringArray reqIds;
    try{
        m_voStageReqDAO->getByRequestId(STAGEREQDAO_TESTCASE_REQ1_ID,reqIds);
        CPPUNIT_ASSERT_MESSAGE("Invalid Request ID", !reqIds.empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void StageReqDaoTestCase::testGetByRequestId_VO_Error(){
    
    StringArray reqIds;
    try{
        m_voStageReqDAO->getByRequestId(STAGEREQDAO_TESTCASE_REQ2_ID,reqIds);
        CPPUNIT_FAIL("VO_2 StageReqs shoudn't be reachable");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void StageReqDaoTestCase::testUpdate_VO(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get StageReq instance
        scoped_ptr<model::StageReq> sr(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE1_ID,STAGEREQDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ1)",sr.get() != 0);

        time_t current;
        time(&current);
        // Set StageReq Properties        
        sr->turl = "file:///tmp/source.txt-1";
        sr->requestTime = current;
        sr->duration = 1234.56;
        // Update StageReq
        m_voStageReqDAO->update(*sr);
        
        // Commit StageReq
        DaoHelper::ctx().commit();
        
        // Get StageReq Instance
        sr.reset(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE1_ID,STAGEREQDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ1)",sr.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid SourceTURL (STAGEREQ1)",sr->turl == "file:///tmp/source.txt-1");
        CPPUNIT_ASSERT_MESSAGE("Invalid StageReqTime (STAGEREQ1)",sr->requestTime == current);
        CPPUNIT_ASSERT_MESSAGE("Invalid Duration (STAGEREQ1)",sr->duration == 1234.56);
        
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void StageReqDaoTestCase::testUpdate_VO_2(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get StageReq instance
        scoped_ptr<model::StageReq> sr(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE1_ID,STAGEREQDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ1)",sr.get() != 0);
        
        // Update StageReq
        m_voStageReqDAO->update(*sr);
        // Commit Transactyion
        DaoHelper::ctx().commit();

        // Get StageReq Instance
        sr.reset(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE1_ID,STAGEREQDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ1)",sr.get() != 0);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testUpdate
 *
 * Test Update Method
 */
void StageReqDaoTestCase::testUpdate_VO_3(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get StageReq instance
        scoped_ptr<model::StageReq> sr(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE1_ID,STAGEREQDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ1)",sr.get() != 0);

        time_t current;
        time(&current);
        // Set StageReq Properties        
        sr->state  = model::StageReq::S_STAGED;
        sr->turl = "file:///tmp/source.txt-2";
        sr->requestTime = current;
        sr->duration = 1.23;
        sr->finalDuration = 0.1;
        sr->reasonClass = model::Error::STORAGE_INTERNAL_ERROR;
        sr->reason = "Another reason";
        sr->fileSize = 50111000222LL;
        
        // Update StageReq
        m_voStageReqDAO->update(*sr);
        
        // Commit StageReq
        DaoHelper::ctx().commit();
        
        // Get StageReq Instance
        sr.reset(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE1_ID,STAGEREQDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ1)",sr.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (STAGEREQ1)",sr->state == model::StageReq::S_STAGED);
        CPPUNIT_ASSERT_MESSAGE("Invalid SourceTURL (STAGEREQ1)",sr->turl == "file:///tmp/source.txt-2");
        CPPUNIT_ASSERT_MESSAGE("Invalid StageReqTime (STAGEREQ1)",sr->requestTime == current);
        CPPUNIT_ASSERT_MESSAGE("Invalid Duration (STAGEREQ1)",sr->duration == 1.23);
        CPPUNIT_ASSERT_MESSAGE("Invalid Dst F Duration (STAGEREQ1)",sr->finalDuration == 0.1);
        CPPUNIT_ASSERT_MESSAGE("Invalid reasonClass (STAGEREQ1)",sr->reasonClass == model::Error::STORAGE_INTERNAL_ERROR);
        CPPUNIT_ASSERT_MESSAGE("Invalid reason (STAGEREQ1)",sr->reason == "Another reason");
        CPPUNIT_ASSERT_MESSAGE("Invalid filesize (STAGEREQ1)",sr->fileSize == 50111000222LL);
        
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

void StageReqDaoTestCase::testUpdate_VO_4(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get StageReq instance
        scoped_ptr<model::StageReq> sr(m_voStageReqDAO->get(
            STAGEREQDAO_TESTCASE_FILE9_ID,
            STAGEREQDAO_TESTCASE_REQ8_ID));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ8)",sr.get() != 0);

        time_t current;
        time(&current);
        
        sr->reason = "lorem ipsum";
        sr->reasonClass = Error::INVALID_SIZE;
        sr->errorScope = Error::S_AGENT;
        sr->errorPhase = Error::P_PRESTAGING;
        sr->finishTime = current;
        
        
        // Update Transfer
        m_voStageReqDAO->update(*sr);
        // Commit Transfer
        DaoHelper::ctx().commit();
        
        // Check Properties
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason (STAGEREQ8)",sr->reason() == "lorem ipsum");
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason Class (STAGEREQ8)",sr->reasonClass() == Error::INVALID_SIZE);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Scope (STAGEREQ8)",sr->errorScope() == Error::S_AGENT);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Phase (STAGEREQ8)",sr->errorPhase() == Error::P_PRESTAGING);
        CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (STAGEREQ8)",sr->finishTime() == current);
        
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testUpdate_reason
 *
 * Test Update Method
 */
void StageReqDaoTestCase::testUpdate_reason(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get StageReq instance
        scoped_ptr<model::StageReq> sr(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE1_ID,STAGEREQDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ1)",sr.get() != 0);
        
        const char * const r = "This is a long reason string, more than 1024 characters:"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789";
        sr->reason = r;
        
        // Update StageReq
        m_voStageReqDAO->update(*sr);
        // Commit Transactyion
        DaoHelper::ctx().commit();

        // Get StageReq Instance
        sr.reset(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE1_ID,STAGEREQDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ1)",sr.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid StageReq reason length (STAGEREQ1)", sr->reason().length() < strlen(r));
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void StageReqDaoTestCase::testUpdate_VO_Error(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get StageReq instance 
        scoped_ptr<model::StageReq> sr(new model::StageReq(
                    STAGEREQDAO_TESTCASE_REQ5_ID,
                    STAGEREQDAO_TESTCASE_FILE7_ID,
                    0,
                    STAGEREQDAO_TESTCASE_JOB5_ID,
                    model::StageReq::S_SUBMITTED));
        CPPUNIT_ASSERT_MESSAGE("Null new StageReq",sr.get() != 0);
        
        sr->state = model::StageReq::S_COMPLETED;
        // Update StageReq
        m_voStageReqDAO->update(*sr);
        CPPUNIT_FAIL("This StageReq shouldn't be updatable (STAGEREQ7)");
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCreate_VO
 *
 * Test Create Method
 */
void StageReqDaoTestCase::testCreate_VO(){
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get StageReq instance 
        scoped_ptr<model::StageReq> sr(new model::StageReq(
                    STAGEREQDAO_TESTCASE_REQ_NEW_ID,
                    STAGEREQDAO_TESTCASE_FILE_NEW_ID,
                    0,
                    STAGEREQDAO_TESTCASE_JOB7_ID,
                    model::StageReq::S_SUBMITTED));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ_NEW)",sr.get() != 0);
        
        // Create StageReq
        m_voStageReqDAO->create(*sr);
        // Commit Transactyion
        DaoHelper::ctx().commit();

        // Get StageReq Instance
        sr.reset(m_voStageReqDAO->get(STAGEREQDAO_TESTCASE_FILE_NEW_ID,STAGEREQDAO_TESTCASE_REQ_NEW_ID));
        CPPUNIT_ASSERT_MESSAGE("Null StageReq (STAGEREQ_NEW)",sr.get() != 0);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCountActiveRequests_VO
 *
 * Test CountActiveStageReq Method
 */
void StageReqDaoTestCase::testCountActiveRequests_VO(){
    try{
        unsigned long count = m_voStageReqDAO->countActiveRequests();
        CPPUNIT_ASSERT_MESSAGE("Invalid count returned",count != 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCountActiveRequests_VO
 *
 * Test CountActiveStageReq Method
 */
void StageReqDaoTestCase::testCountActiveRequests_VO_2(){
    try{
        unsigned long count = m_voStageReqDAO->countActiveRequests(STAGEREQDAO_TESTCASE_CHANNEL1_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid count returned (VO_1)",count != 0);
        
        count = m_voStageReqDAO->countActiveRequests(STAGEREQDAO_TESTCASE_CHANNEL3_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid count returned (VO_3)",count == 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
