/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  Authors:  Zsolt.Molnar <zsolt.molnar@cern.ch>
 *  
 *  Version info: $Id: TestSeStatements.cpp,v 1.1 2010/02/09 14:31:02 molnarzs Exp $
 *  Release: $Name:  $
 *
 */

#include "glite/data/transfer/agent/model/Agent.h"
#include "TestAgentStatements.h"
#include "AgentStatements.h"
#include "dao-utility.h"
#include "DaoHelper.h"

using namespace glite::data::agents::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using namespace glite::data::transfer::agent;
using glite::data::agents::dao::oracle::OracleDAOContext;

void TestAgentStatements::test65661WrongSqlQueryGeneration()
{
    OracleDAOContext* ctx = dynamic_cast<OracleDAOContext*>(&DaoHelper::ctx());
    CPPUNIT_ASSERT(ctx);
    AgentStatements& statements = AgentStatements::instance();
    // Test the case when Agent::state::isSet() returns true
    std::string fixture_state = "UPDATE t_agent SET t_agent.agent_state = :1, t_agent.last_active = SYSTIMESTAMP WHERE t_agent.agent_name = :2";
    model::Agent agent_state("type", model::Agent::S_STARTED, "version", "host");
    agent_state.state = model::Agent::S_STARTED;
    CPPUNIT_ASSERT(agent_state.state.isSet());
    CPPUNIT_ASSERT(!agent_state.lastActive.isSet());
    boost::scoped_ptr< ::oracle::occi::Statement > stm(statements.createUpdate(*ctx, agent_state, "testAgent"));
    CPPUNIT_ASSERT(stm.get());
    CPPUNIT_ASSERT_EQUAL(fixture_state, stm->getSQL());

    // Test the case when Agent::lastActive::isSet() returns true
    std::string fixture_lastActive = "UPDATE t_agent SET t_agent.last_active = :1 WHERE t_agent.agent_name = :2";
    model::Agent agent_lastActive("type", model::Agent::S_STARTED, "version", "host");
    agent_lastActive.lastActive = (time_t) -1;
    CPPUNIT_ASSERT(!agent_lastActive.state.isSet());
    CPPUNIT_ASSERT(agent_lastActive.lastActive.isSet());
    stm.reset(statements.createUpdate(*ctx, agent_lastActive, "testAgent"));
    CPPUNIT_ASSERT(stm.get());
    CPPUNIT_ASSERT_EQUAL(fixture_lastActive, stm->getSQL());
}

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(TestAgentStatements, "TestAgentStatements");

