/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FetchTransfers Test Case
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: TestCase30271.cpp,v 1.3 2007/12/17 13:12:37 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Fetch.h"
#include "ChannelFlags.h"
#include "DaoHelper.h"
#include "actions/channel/ActionsConfig.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/Channel.h"
#include "glite/data/transfer/agent/dao/channel/FileDAO.h"
#include "glite/data/transfer/agent/dao/channel/JobDAO.h"
#include "glite/data/transfer/agent/dao/channel/TransferDAO.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"
#include "glite/data/transfer/agent/ts/TransferService.h"
#include "glite/data/transfer/agent/ts/TransferServiceFactory.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::data::transfer::agent::model;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class TestCase30271 : public CppUnit::TestCase{

private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( TestCase30271 );
        CPPUNIT_TEST( test_bug30271 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------
    void test_bug30271();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( TestCase30271, "TestCase30271" );

namespace {
const char * const CHANNEL_NAME     = "channel_30271";
const char * const VO1_NAME          = "VO_1";
const char * const VO2_NAME          = "VO_2";
}

/**
 * test_bug30271
 */
void TestCase30271::test_bug30271(){
    try{
        scoped_ptr<Fetch> m_action;
        // Reset cache
        ActionsConfig::instance().activeTxCache().reset();
        // Create Action instance
        std::auto_ptr<Action> a(ActionFactory::instance().create("glite:Fetch"));
        CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Fetch*>(a.get()) != 0);
        // Store Action        
        m_action.reset(dynamic_cast<Fetch*>(a.release()));
        CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);
        // Set Context
        m_action->context(CHANNEL_NAME,DaoHelper::ctx());
        // execute
        m_action->execute();
        // reset cache and action
        ActionsConfig::instance().activeTxCache().reset();
        m_action.reset();
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
