/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FakeAction Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: DummyAction.cpp,v 1.4 2006/01/30 15:19:35 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "DummyAction.h"
#include "FakeActionConfig.h"

#include <unistd.h>

using namespace glite::data::transfer::agent::action;


/*
 * Register ActionFactory Method 
 */
REGISTER_FACTORY_METHOD( DummyAction );
REGISTER_FACTORY_METHOD( SleepAction );
        
/*
 * DummyAction
 *
 * Constructor
 */
DummyAction::DummyAction(){
}

/*
 * ~DummyAction
 *
 * Destructor
 */
DummyAction::~DummyAction(){
}

/*
 * execute
 *
 * Execute the Action
 */
void DummyAction::execute(const std::string& params) throw (ExecuteException){
    // Don't do anything
}


/*
 * SleepAction
 *
 * Constructor
 */
SleepAction::SleepAction(){
}

/*
 * ~SleepAction
 *
 * Destructor
 */
SleepAction::~SleepAction(){
}

/*
 * execute
 *
 * Execute the Action
 */
void SleepAction::execute(const std::string& params) throw (ExecuteException){
    // Sleep for a while
    unsigned int remaining = sleep(200);
    if(remaining != 0){
        throw ExecuteException("I need more time to rest !!!");
    }
}

