/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: DummyAction Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: DummyAction.h,v 1.6 2006/08/14 08:39:07 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_TEST_DUMMYACTION_H_
#define GLITE_DATA_TRANSFER_AGENTS_TEST_DUMMYACTION_H_

#include "glite/data/agents/dao/DAOContext.h"
#include "glite/data/transfer/agent/action/Action.h"
#include "glite/data/transfer/agent/action/ActionFactoryMethod.h"

using namespace glite::data::agents;

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {

/**
 * Dummy Action
 */
class DummyAction : public Action {
    /*
     * Declare Action Factory Method
     */
    DECLARE_FACTORY_METHOD( DummyAction , "Dummy");

public:
    
    /**
     * Constructor
     */
    DummyAction();

    /**
     * Destructor
     */
    virtual ~DummyAction();

    /**    
     * set DAOContext
     */
    virtual void context(const std::string& params, class dao::DAOContext& ctx){}
    
    /**    
     * Get DAOContext
     */
    virtual class dao::DAOContext& context() /*throw (ExecuteException)*/{throw ExecuteException("Not Needed");}
     
    /**
     * Execute The Action
     */
    virtual void init() throw (ExecuteException){}

    /**
     * Execute The Action
     */
    virtual void execute(const std::string& params = "") throw (ExecuteException);

private:
        
    /**
     * Copy Constructor. Not Implemented
     */
    DummyAction(const DummyAction&);

    /**
     * Assignment Operator. Not Implemented
     */
    DummyAction& operator=(const DummyAction&);    
};

/**
 * Sleep Action
 */
class SleepAction : public Action {
    /*
     * Declare Action Factory Method
     */
    DECLARE_FACTORY_METHOD( SleepAction , "Sleep");

public:
    
    /**
     * Constructor
     */
    SleepAction();

    /**
     * Destructor
     */
    virtual ~SleepAction();

    /**    
     * set DAOContext
     */
    virtual void context(const std::string& params, class dao::DAOContext& ctx){}
    
    /**    
     * Get DAOContext
     */
    virtual class dao::DAOContext& context() /*throw (ExecuteException)*/{throw ExecuteException("Not Needed");}
     
    /**
     * Execute The Action
     */
    virtual void init() throw (ExecuteException){}

    /**
     * Execute The Action
     */
    virtual void execute(const std::string& params = "") throw (ExecuteException);

private:
        
    /**
     * Copy Constructor. Not Implemented
     */
    SleepAction(const SleepAction&);

    /**
     * Assignment Operator. Not Implemented
     */
    SleepAction& operator=(const SleepAction&);    
};

} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_TEST_DUMMYACTION_H_
