/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FakeActionConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: FakeActionConfig.h,v 1.5 2006/08/23 14:31:52 badino Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_TEST_FAKEACTIONCONFIG_H_
#define GLITE_DATA_TRANSFER_AGENTS_TEST_FAKEACTIONCONFIG_H_

#include "glite/data/config/service/ComponentConfiguration.hxx"
#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {

/**
 * The ActionsConfig Class is used to retrieve the Configuration Parameters from 
 * the ServiceConfigurator
 * @streotype singleton
 */    
class FakeActionConfig :  public glite::config::ComponentConfiguration {
public:
    static const char * const NAME;
                                                                                                                           
    /**
     * Initialize the Component
     */
    virtual int init(const Params& params){ return 0;}
                                                                                                                           
    /**
     * Configure the Component
     */
    virtual int config(const Params& params){ return 0;} 
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start(){return 0;}
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}
                                                                                                                           
    /**
     * Finailize the Component
     */
    virtual int fini(){return 0;}
                                                                                                                           
    /**
     * Return Singleton Instance
     */
    static FakeActionConfig * instance();
                                                                                                                           
    /**
     * Finalize Singleton Instance
     */
    static void finalize();

    // FakeAction counter
    unsigned long count() const {return m_count;}
    void count(unsigned long c) {m_count = c;}
    void incremenet() {m_count++;}
    
    // FakeAction counter
    bool isDisabled() const {return m_disabled;}
    void disable(bool d) {m_disabled = d;}
    
protected:
    /**
     * Constructor
     */
    FakeActionConfig();
                                                                                                                           
    /**
     * Destructor
     */
    virtual ~FakeActionConfig();
                                                                                                                           
private:
        
    /**
     * Copy Constructor. Not Implemented
     */
    FakeActionConfig(const FakeActionConfig&);

    /**
     * Assignment Operator. Not Implemented
     */
    FakeActionConfig& operator=(const FakeActionConfig&);

private:
    /**
     * Singleton Instance
     */
    static FakeActionConfig * s_instance;

    /**
     * Fake Action Counter
     */
    unsigned long m_count;

    /**
     * Disable The Action
     */
    bool m_disabled;
};

} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_TEST_FAKEACTIONCONFIG_H_
