/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch>
 *  Version info: $Id: TestProperties.cpp,v 1.1 2009/12/11 11:27:04 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "TestProperties.h"
#include "Properties.h"
#include <glite/data/config/service/ParamReader.hxx>
#include <glite/data/config/service/ParamValue.hxx>

using namespace glite::config;

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {
namespace Property {

void TestProperties::testTransferType()
{
    CPPUNIT_ASSERT_EQUAL(TransferType::TYPE_URLCOPY, TransferType::Default());
    CPPUNIT_ASSERT_EQUAL(std::string("TransferType"), TransferType::Name());
    TransferType v;
    CPPUNIT_ASSERT_EQUAL(TransferType::TYPE_URLCOPY, v.value());
    // Test getting value with reader
    ParamValue pval("TransferType", "srmcopy");
    ComponentConfiguration::Params params;
    params["TransferType"] = &pval;
    ParamReader reader("TestProperties", params);
    v.read(reader);
    CPPUNIT_ASSERT_EQUAL(TransferType::TYPE_SRMCOPY, v.value());
}

void TestProperties::testMaxTransfers()
{
    MaxTransfers::Type defaultVal = 50;
    CPPUNIT_ASSERT_EQUAL(defaultVal, MaxTransfers::Default());
    CPPUNIT_ASSERT_EQUAL(std::string("MaxTransfers"), MaxTransfers::Name());
    MaxTransfers v;
    CPPUNIT_ASSERT_EQUAL(defaultVal, v.value());
    // Test getting value with reader
    MaxTransfers::Type fixVal = 123456;
    ParamValue pval("MaxTransfers", "123456");
    ComponentConfiguration::Params params;
    ParamReader reader("TestProperties", params);
    params["MaxTransfers"] = &pval;
    v.read(reader);
    CPPUNIT_ASSERT_EQUAL(fixVal, v.value());
}

void TestProperties::testMaxBulkSize()
{
    MaxBulkSize::Type defaultVal = 100;
    CPPUNIT_ASSERT_EQUAL(defaultVal, MaxBulkSize::Default());
    CPPUNIT_ASSERT_EQUAL(std::string("MaxBulkSize"), MaxBulkSize::Name());
    MaxBulkSize v;
    CPPUNIT_ASSERT_EQUAL(defaultVal, v.value());
    // Test getting value with reader
    MaxBulkSize::Type fixVal = 123456;
    ParamValue pval("MaxBulkSize", "123456");
    ComponentConfiguration::Params params;
    ParamReader reader("TestProperties", params);
    params["MaxBulkSize"] = &pval;
    v.read(reader);
    CPPUNIT_ASSERT_EQUAL(fixVal, v.value());
}

void TestProperties::testSRMGridFTPSplit()
{
    SRMGridFTPSplit::Type defaultVal = false;
    CPPUNIT_ASSERT_EQUAL(defaultVal, SRMGridFTPSplit::Default());
    CPPUNIT_ASSERT_EQUAL(std::string("SRMGridFTPSplit"), SRMGridFTPSplit::Name());
    SRMGridFTPSplit v;
    CPPUNIT_ASSERT_EQUAL(defaultVal, v.value());
    // Test getting value with reader
    SRMGridFTPSplit::Type fixVal = true;
    ParamValue pval("SRMGridFTPSplit", "true");
    ComponentConfiguration::Params params;
    ParamReader reader("TestProperties", params);
    params["SRMGridFTPSplit"] = &pval;
    v.read(reader);
    CPPUNIT_ASSERT_EQUAL(fixVal, v.value());
}

void TestProperties::testSyslogFacility()
{
    SyslogFacility::Type defaultVal = SyslogFacility::FTP;
    CPPUNIT_ASSERT_EQUAL(defaultVal, SyslogFacility::Default());
    CPPUNIT_ASSERT_EQUAL(std::string("SyslogFacility"), SyslogFacility::Name());
    SyslogFacility v;
    CPPUNIT_ASSERT_EQUAL(defaultVal, v.value());
    // Test getting value with reader
    SyslogFacility::Type fixVal = SyslogFacility::USER;
    ParamValue pval("SyslogFacility", "LOG_USER");
    ComponentConfiguration::Params params;
    ParamReader reader("TestProperties", params);
    params["SyslogFacility"] = &pval;
    v.read(reader);
    CPPUNIT_ASSERT_EQUAL(fixVal, v.value());
}

void TestProperties::testSyslogIdent()
{
    SyslogIdent::Type defaultVal = "FTS";
    CPPUNIT_ASSERT_EQUAL(defaultVal, SyslogIdent::Default());
    CPPUNIT_ASSERT_EQUAL(std::string("SyslogIdent"), SyslogIdent::Name());
    SyslogIdent v;
    CPPUNIT_ASSERT_EQUAL(defaultVal, v.value());
    // Test getting value with reader
    SyslogIdent::Type fixVal = "USER";
    ParamValue pval("SyslogIdent", "USER");
    ComponentConfiguration::Params params;
    ParamReader reader("TestProperties", params);
    params["SyslogIdent"] = &pval;
    v.read(reader);
    CPPUNIT_ASSERT_EQUAL(fixVal, v.value());
}

void TestProperties::testSyslogDisable()
{
    SyslogDisable::Type defaultVal = false;
    CPPUNIT_ASSERT_EQUAL(defaultVal, SyslogDisable::Default());
    CPPUNIT_ASSERT_EQUAL(std::string("SyslogDisable"), SyslogDisable::Name());
    SyslogDisable v;
    CPPUNIT_ASSERT_EQUAL(defaultVal, v.value());
    // Test getting value with reader
    SyslogDisable::Type fixVal = true;
    ParamValue pval("SyslogDisable", "true");
    ComponentConfiguration::Params params;
    ParamReader reader("TestProperties", params);
    params["SyslogDisable"] = &pval;
    v.read(reader);
    CPPUNIT_ASSERT_EQUAL(fixVal, v.value());
}

void TestProperties::testStringConverter_Property_Base()
{
    CPPUNIT_ASSERT_EQUAL(size_t(2), StringConverter_Property_Base<TransferType>::_FacilityNames().size());
    CPPUNIT_ASSERT_EQUAL(size_t(3), StringConverter_Property_Base<SyslogFacility>::_FacilityNames().size());
}

void TestProperties::testStringConverter_TransferType()
{
    StringConverter<TransferType::Type> sc;
    CPPUNIT_ASSERT_EQUAL(std::string("srmcopy"), sc.to_string(TransferType::TYPE_SRMCOPY));
    CPPUNIT_ASSERT_EQUAL(TransferType::TYPE_SRMCOPY, sc.from_string("srmcopy"));

    CPPUNIT_ASSERT_EQUAL(std::string("urlcopy"), sc.to_string(TransferType::TYPE_URLCOPY));
    CPPUNIT_ASSERT_EQUAL(TransferType::TYPE_URLCOPY, sc.from_string("urlcopy"));

    CPPUNIT_ASSERT_EQUAL(size_t(2), sc.m_StringToEnum.size());
    CPPUNIT_ASSERT(sc.m_StringToEnum.find("urlcopy") != sc.m_StringToEnum.end());
    CPPUNIT_ASSERT_EQUAL(TransferType::TYPE_URLCOPY, sc.m_StringToEnum["urlcopy"]);
}

void TestProperties::testStringConverter_SysLogFacility()
{
    StringConverter<SyslogFacility::Type> sc;
    CPPUNIT_ASSERT_EQUAL(std::string("LOG_DAEMON"), sc.to_string(SyslogFacility::DAEMON));
    CPPUNIT_ASSERT_EQUAL(SyslogFacility::DAEMON, sc.from_string("LOG_DAEMON"));

    CPPUNIT_ASSERT_EQUAL(std::string("LOG_FTP"), sc.to_string(SyslogFacility::FTP));
    CPPUNIT_ASSERT_EQUAL(SyslogFacility::FTP, sc.from_string("LOG_FTP"));

    CPPUNIT_ASSERT_EQUAL(std::string("LOG_USER"), sc.to_string(SyslogFacility::USER));
    CPPUNIT_ASSERT_EQUAL(SyslogFacility::USER, sc.from_string("LOG_USER"));
}

} // end namespace Property
} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite
