/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: UrlCopyTransferFactory tests
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: TestSrmCopyTransferService.cpp,v 1.1 2009/12/11 16:14:25 molnarzs Exp $
 *  Release: $Name:  $
 *
 *  NOTE: this is a test of the STUB IMPLEMENTATION of the transfer service interface
 */
#include "TestSrmCopyTransferService.h"
#include "UrlCopyTransferFactory.h"
#include "SrmCopyTransferService.h"
#include <glite/data/config/service/ParamValue.hxx>
#include <syslog.h>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {

using namespace glite::config;

void TestSrmCopyTransferService::testIfPropertiesPassed()
{
    // Set up paremeters for SRM copy. Only the required minimum...
    ComponentConfiguration::Params params;
    ParamValue transferType_pval("TransferType", "srmcopy");
    params["TransferType"] = &transferType_pval;
    ParamValue syslogFacility_pval("SyslogFacility", "LOG_USER");
    params["SyslogFacility"] = &syslogFacility_pval;
    ParamValue syslogIdent_pval("SyslogIdent", "USER");
    params["SyslogIdent"] = &syslogIdent_pval;
    ParamValue syslogDisable_pval("SyslogDisable", "true");
    params["SyslogDisable"] = &syslogDisable_pval;

    UrlCopyTransferFactory factory;
    factory.init(params);

    boost::scoped_ptr<SrmCopyTransferService> service(
        dynamic_cast<SrmCopyTransferService*>(
            factory.create("testChannel")
        )
    );

    CPPUNIT_ASSERT(service.get());
    // Inject the test itself
    service->injectTscStarterDependency_<TestSrmCopyTransferService::TestSrmCopyStarter_>();
    TransferService::TransferRequest  req;
    TransferService::TransferResponse out;
    // Set up only the mininum...
    req.surls.resize(1);
    req.surls[0].source = "file:///SOURCE.txt";
    req.surls[0].destination = "file:///DESTINATION.txt";
    service->submit(req, out, "testValue");
}

urlcopy::TransferSrmCopy::Ptr TestSrmCopyTransferService::TestSrmCopyStarter_::start(
    const urlcopy::TransferSrmCopy::Args& args,
    const std::string& prefix)
{
    // Check if the default parameters defined in test::CreateTestParameters()
    // passed to TransferUrlCopy. We check the arguments coming from config
    // only.
    CPPUNIT_ASSERT_EQUAL(true, args.syslog_disable);
    CPPUNIT_ASSERT_EQUAL(LOG_USER, args.syslog_facility);
    CPPUNIT_ASSERT_EQUAL(std::string("LOG_USER"), args.syslog_facility_name);
    CPPUNIT_ASSERT_EQUAL(std::string("USER"), args.syslog_ident);
    return urlcopy::TransferSrmCopy::start(args, prefix);
}

} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite
