/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: main function
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TestTransferService.cpp,v 1.4 2009/12/11 11:27:04 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "testSuite.h"
#include "glite/data/config/service/glite-config.h"
#include "glite/data/config/service/glite-config-register.hxx"

#include <stdlib.h>
#include <string>
#include <iostream>
#include <unistd.h>
#include <boost/scoped_ptr.hpp>

using std::cout;
using std::cerr;
using std::cin;
using std::endl;
using boost::scoped_ptr;

namespace {
const char * const PROGNAME    = "glite_data_transfer_agent_test_ts";
const char * const PROGVERSION = "0.0";
const char * const PROGAUTHOR  = "(C) EGEE";
    
const char * const DEFAULT_CONFIG_NAME = "glite-transfer-agent-test-ts";
}

/*
 * print_usage_and_die
 *
 * Print Program Usage and ther exit
 */
void print_usage_and_die(){
    cout << endl;
    cout << "<" << PROGNAME << "> Version " << PROGVERSION << " by " << PROGAUTHOR;
    cout << endl;
    cout << "usage: " << PROGNAME << "[-l <glite_location_user>][-c <config_name>][-s] " << endl;
    cout << " Where:" << endl;
    cout << " -l : specify the glite location. Configuration files should be located under the etc folder on that path" << endl;
    cout << " -s : Enable the Compiler Outputter" << endl;
    cout << " -c : Configuration Name" << endl;
    cout << " Where:" << endl;
    cout << " Optional parameters:" << endl;
    cout << "  -h : print this screen" << endl;
    exit(-1);
}

/*
 * main
 *
 * Transfer FTS Main function
 */
int main(int argc, char* argv[]) {

    // Get Options
    const char * location = 0;
    const char * config_name = DEFAULT_CONFIG_NAME;
    bool selfTest = false;
    int flag;
    while ((flag = getopt (argc, argv, "l:c:sh")) != -1) {
        switch (flag) {
            case 'l':
                location = optarg;
            break;
            case 'c':
                config_name = optarg;
            break;
            case 's':
                selfTest = true;
            break;
            case 'h':
            default:
                print_usage_and_die();
            break;
        } // End switch
    } // End while

    if(0 != location){
        // Set "GLite Location User"
        setenv("GLITE_LOCATION_USER",location,1);
    }
    
    // Initialize the Application
    int initres = glite_config_initialize(config_name);
    if(initres < 0){
        cerr << "Initialization failed " << endl;
     //   return -1;
    }
    
    CPPUNIT_NS :: TestResult testresult;
    CPPUNIT_NS :: TestResultCollector collectedresults;
    testresult.addListener (&collectedresults);
    CPPUNIT_NS :: BriefTestProgressListener progress;
    testresult.addListener (&progress);
    CPPUNIT_NS :: TestRunner testrunner;
    testrunner.addTest (CPPUNIT_NS :: TestFactoryRegistry :: getRegistry ().makeTest ());
    testrunner.run (testresult);
    CPPUNIT_NS :: CompilerOutputter compileroutputter (&collectedresults, std::cerr);
    compileroutputter.write ();
    bool wasSucessful = collectedresults.wasSuccessful();
    // Finalize the Application
    glite_config_finalize();
    return wasSucessful ? 0 : 1;
}
