/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: UrlCopyTransferFactory tests
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: TestUrlCopyTransferFactory.cpp,v 1.2 2009/12/11 16:14:25 molnarzs Exp $
 *  Release: $Name:  $
 *
 *  NOTE: this is a test of the STUB IMPLEMENTATION of the transfer service interface
 */
#include "TestUrlCopyTransferFactory.h"
#include "UrlCopyTransferFactory.h"
#include "UrlCopyTransferService.h"
#include "SrmCopyTransferService.h"

#include <glite/data/config/service/ParamValue.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {

using namespace glite::config;

void TestUrlCopyTransferFactory::testIfPropertiesPassed()
{
    /** Create test parameters. In production, the configuration service reads
     * them from a config file, and fill them in. */
    ComponentConfiguration::Params params;
    ParamValue transferType_pval("TransferType", "srmcopy");
    params["TransferType"] = &transferType_pval;
    ParamValue maxTransfers_pval("MaxTransfers", "123456");
    params["MaxTransfers"] = &maxTransfers_pval;
    ParamValue maxBulkSize_pval("MaxBulkSize", "123456");
    params["MaxBulkSize"] = &maxBulkSize_pval;
    ParamValue sRMGridFTPSplit_pval("SRMGridFTPSplit", "true");
    params["SRMGridFTPSplit"] = &sRMGridFTPSplit_pval;
    ParamValue syslogFacility_pval("SyslogFacility", "LOG_USER");
    params["SyslogFacility"] = &syslogFacility_pval;
    ParamValue syslogIdent_pval("SyslogIdent", "USER");
    params["SyslogIdent"] = &syslogIdent_pval;
    ParamValue syslogDisable_pval("SyslogDisable", "true");
    params["SyslogDisable"] = &syslogDisable_pval;
    UrlCopyTransferFactory factory;
    factory.init(params);

    CPPUNIT_ASSERT_EQUAL(TransferType::Type(TransferType::TYPE_SRMCOPY), factory.m_properties.transferType.value());
    CPPUNIT_ASSERT_EQUAL(MaxTransfers::Type(123456), factory.m_properties.maxTransfers.value());
    CPPUNIT_ASSERT_EQUAL(MaxBulkSize::Type(123456), factory.m_properties.maxBulkSize.value());
    CPPUNIT_ASSERT_EQUAL(SyslogFacility::Type(SyslogFacility::USER), factory.m_properties.syslogFacility.value());
    CPPUNIT_ASSERT_EQUAL(SyslogIdent::Type("USER"), factory.m_properties.syslogIdent.value());
    CPPUNIT_ASSERT_EQUAL(SyslogDisable::Type(true), factory.m_properties.syslogDisable.value());

    // In case of srmcopy, the sRMGridFTPSplit is ignored. Test it with urlcopy as well.
    ComponentConfiguration::Params params_forSrmSplit;
    ParamValue transferType_pval_urlcopy("TransferType", "urlcopy");
    params_forSrmSplit["TransferType"] = &transferType_pval_urlcopy;
    params_forSrmSplit["SRMGridFTPSplit"] = &sRMGridFTPSplit_pval;
    UrlCopyTransferFactory factory_forSrmSplit;
    factory_forSrmSplit.init(params_forSrmSplit);

    CPPUNIT_ASSERT_EQUAL(TransferType::Type(TransferType::TYPE_URLCOPY), factory_forSrmSplit.m_properties.transferType.value());
    CPPUNIT_ASSERT_EQUAL(SRMGridFTPSplit::Type(true), factory_forSrmSplit.m_properties.sRMGridFTPSplit.value());
}

void TestUrlCopyTransferFactory::testIfProperServiceCreated_SrmCopy()
{
    testIfProperServiceCreated_<SrmCopyTransferService>("srmcopy");
}

void TestUrlCopyTransferFactory::testIfProperServiceCreated_UrlCopy()
{
    testIfProperServiceCreated_<UrlCopyTransferService>("urlcopy");
}

template<class SERVICE_TYPE>
void TestUrlCopyTransferFactory::testIfProperServiceCreated_(const std::string& serviceName)
{
    // Set up paremeters for SRM copy. Only the required minimum...
    ComponentConfiguration::Params params;
    ParamValue transferType_pval("TransferType", serviceName.c_str());
    params["TransferType"] = &transferType_pval;

    UrlCopyTransferFactory factory;
    factory.init(params);
    boost::scoped_ptr<TransferService> service(factory.create("testChannel"));
    SERVICE_TYPE* realService = dynamic_cast<SERVICE_TYPE*>(service.get());
    CPPUNIT_ASSERT(realService);
}

} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite
