/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: UrlCopyTransferFactory tests
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: TransferServiceTestCase.h,v 1.1 2009/12/11 11:27:04 molnarzs Exp $
 *  Release: $Name:  $
 *
 *  NOTE: this is a test of the STUB IMPLEMENTATION of the transfer service interface
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_TS_TEST_TRANSFERSERVICETESTCASE_H_
#define GLITE_DATA_TRANSFER_AGENTS_TS_TEST_TRANSFERSERVICETESTCASE_H_

#include "testSuite.h"
#include <glite/data/config/service/Logger.hxx>
#include "glite/data/transfer/agent/ts/TransferService.h"
#include <boost/scoped_ptr.hpp>

using namespace glite::data::transfer::agent::ts;

class TransferServiceTestCase : public CppUnit::TestCase
{
public:
    TransferServiceTestCase() : m_logger("agent-ts-unittest"){}
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( TransferServiceTestCase);
        CPPUNIT_TEST( testSubmit_single );
        CPPUNIT_TEST( testSubmit_single_Error );
        CPPUNIT_TEST( testSubmit_bulk );
        CPPUNIT_TEST( testSubmit_bulk_Error );
        CPPUNIT_TEST( testCount );
        CPPUNIT_TEST( testState_single_Error );
        CPPUNIT_TEST( testState_bulk_Error );
        CPPUNIT_TEST( testState_single );
        CPPUNIT_TEST( testState_bulk );
        CPPUNIT_TEST( testClean );
        CPPUNIT_TEST( testTrace );
        CPPUNIT_TEST( test_name_rule );
        CPPUNIT_TEST( test_transfer_properties );
        CPPUNIT_TEST( testChecksum );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Submit Method
     */
    void testSubmit_single();
    void testSubmit_single_Error();
    void testSubmit_bulk();
    void testSubmit_bulk_Error();
    /**
     * Test State Method
     */
    void testState_single();
    void testState_single_Error();
    void testState_bulk();
    void testState_bulk_Error();
    /**
     * Test Count Method
     */
    void testCount();
    /**
     * Test Clean Method
     */
    void testClean();
    /**
     * Test Trace Method
     */
    void testTrace();

    /**
     * test_name_rule
     */
    void test_name_rule();

    /**
     * check additional properties of t_transfer are filled
     */
    void test_transfer_properties();

    void testChecksum();
private:
    glite::config::Logger m_logger;
    /**
     * The Transfer Service Instance
     */
    boost::scoped_ptr<TransferService> m_transferService;

    typedef std::pair<std::string,unsigned int> TxId;
    typedef std::vector<std::string> ReqIdArray;
    typedef std::vector<TxId> TxIdArray;

    static ReqIdArray s_requests;
    static TxIdArray  s_transfers;
};

#endif // #ifndef GLITE_DATA_TRANSFER_AGENTS_TS_TEST_TRANSFERSERVICETESTCASE_H_
