/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: StubTransferFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: StubTransferFactory.h,v 1.6 2008/11/17 10:17:39 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_TS_TEST_STUBTRANSFERFACTORY_H_
#define GLITE_DATA_TRANSFER_AGENTS_TS_TEST_STUBTRANSFERFACTORY_H_

#include "glite/data/config/service/ComponentConfiguration.hxx"
#include "glite/data/transfer/agent/ts/TransferServiceFactory.h"
#include "glite/data/transfer/agent/ts/TransferException.h"
#include "glite/data/transfer/agent/ts/TransferService.h"
#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {
    
/**
 * Stub implementation of the Transfer Service Factory
 * @factory
 */
class StubTransferFactory : public TransferServiceFactory , 
                            public glite::config::ComponentConfiguration{
public:

    /**
     * Constructor
     */
    StubTransferFactory();
        
    /**
     * Destructor
     */
    virtual ~StubTransferFactory();

    // =========================================================================
    // Transfer Service Method
    // =========================================================================

    /**
     * Create a TransferService
     * @param channel_name [IN] the name of the channel the TransferService is
     * responsible for
     * @return a new instance of a TransferService
     * @throws TransferException in case of error
     */
    virtual TransferService * create(const std::string& channel_name) /*throw (LogicError)*/;

    // =========================================================================
    // ComponentConfiguration Methods
    // =========================================================================

    /**
     * Initialize the Component
     */
    virtual int init(const Params& params);
                                                                                                                           
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start(){return 0;}
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}
                                                                                                                           
    /**
     * Finailize the Component
     */
    virtual int fini(){return 0;}

private:

    /**
     * The Logger
     */
    glite::config::Logger m_logger;
    
    /**
     * The TransferService Type
     */
    std::string m_type;
    
    /**
     * Max Parallel Transfers
     */
    unsigned long m_maxTransfers;
    
    /**
     * Bulk Size
     */
    unsigned long m_maxRequestSize;
    
    /**
     * flag signaling if transfers should be stopped after the preparation phase
     */
    bool m_stopAfterPreparation;
        
};

} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_TS_TEST_STUBTRANSFERFACTORY_H_
